@@ -10,12 +10,12 @@ an addFieldInfos method (note the plural).  These then get called,
 which causes one of the classes in adql.fieldinfos to be constructed
 and assigned to the node's fieldInfos attribute.  The source for
 these infos is either an AnnotationContext (and thus typically
-the user-supplied retrieveFieldInfos function) or derived annotations.
+the user-supplied FieldInfoGetter function) or derived annotations.
 These are computed by the nodes themselves, using their addFieldInfo
 (singular!) method.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -48,9 +48,9 @@ class AnnotationContext(object):
 	at any time, gives a list of the current node's ancestor nodes.
 	"""
 	def __init__(self,
-			retrieveFieldInfos,
+			fieldInfoGetter,
 			equivalencePolicy=stc.defaultPolicy):
-		self.retrieveFieldInfos = retrieveFieldInfos
+		self.fieldInfoGetter = fieldInfoGetter
 		self.policy = equivalencePolicy
 		self.withTables = {}
 		self.colResolvers = []
@@ -73,7 +73,7 @@ class AnnotationContext(object):
 
 		withTable must already be annotated for this to work.
 		"""
-		self.retrieveFieldInfos.addExtraFieldInfos(
+		self.fieldInfoGetter.addExtraFieldInfos(
 			withTable.name,
 			withTable.fieldInfos.seq)
 
@@ -87,6 +87,15 @@ class AnnotationContext(object):
 				" %s.%s.  Please report this to the gavo@ari.uni-heidelberg.de"
 				" together with the failed query."%(tableName, colName))
 		return res
+	
+	def retrieveFieldInfos(self, tableName):
+		"""returns a sequence of (column name, adql.FieldInfo) pairs for
+		the table tableName or raises a NotFoundError.
+
+		Really, this calls just the FieldInfoGetter that was passed into
+		the annotation context and hence shares its interface.
+		"""
+		return self.fieldInfoGetter(tableName)
 
 
 def _annotateTraverse(node, context):
