@@ -2,7 +2,7 @@
 Exceptions and helper functions for ADQL processing.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -135,7 +135,7 @@ def computeCommonColumns(tableNode):
 		return set()
 
 
-class FieldInfoGetter(object):
+class FieldInfoGetter:
 	"""An abstract class to retrieve table metadata.
 
 	A subclass of this must be passed into adql.parseAnnotating.
