@@ -123,7 +123,7 @@ To make the whole thing work, I added the generalLiteral to the
 characterPrimary production.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -368,6 +368,9 @@ def getADQLGrammarCopy(nodes):
 			+ Optional( "." + g.identifier )
 			+ Optional( "." + g.identifier ))
 		g.tableName = g.qualifier("tableName")
+		g.uploadTableName = (CaselessKeyword("tap_user")
+			+ Optional( "." + g.identifier )
+			+ Optional( "." + g.identifier ))("tableName")
 		g.columnReferenceByUCD = (
 			CaselessKeyword("UCDCOL")
 			+ '(' + g.characterStringLiteral + ')')
@@ -780,7 +783,7 @@ def getADQLGrammarCopy(nodes):
 			- g.unsignedNumericLiteral
 			- Literal(')'))
 		g.possiblyAliasedTable = (
-			g.tableName
+			(g.uploadTableName | g.tableName)
 			+ Optional(g.correlationSpecification)
 			+ Optional(g.tablesample)("tablesample"))
 		g.setGeneratingFunction = (
@@ -879,7 +882,7 @@ def getADQLGrammarCopy(nodes):
 			+ g.optionalAll)
 
 		# I had to rewrite a few of the following ones to avoid pyparse
-		# endless recursions.
+		# infinite recursions.
 		g.querySetPrimary = MatchFirstWithSaneError([
 			g.selectQuery,
 			'(' + g.selectExpression + ')'])
@@ -898,7 +901,14 @@ def getADQLGrammarCopy(nodes):
 
 		g.querySpecification = Optional(g.withSpecification) + g.selectExpression
 
-		g.statement = g.querySpecification + Optional(White()) + StringEnd()
+		g.createAs = (
+			CaselessKeyword("CREATE")
+			- CaselessKeyword("TABLE")
+			- g.uploadTableName
+			- CaselessKeyword("AS"))
+		g.statement = (
+			Optional(g.createAs)
+			+ g.querySpecification + Suppress(Optional(White()) + StringEnd()))
 
 		g.statement.ignore(g.sqlComment)
 		return g.getSymbols_(), g.statement
@@ -950,7 +960,26 @@ if __name__=="__main__":  # pragma: no cover
 	enableTree(syms)
 	res = syms["statement"].parseString(
 		"""
-select * from (select * from z) as q
+SELECT ivoid, short_name, res_title, updated, content_level,
+  res_description, reference_url, creator_seq, waveband,
+  detail_paths, detail_values, access_url, last_checked
+FROM rr.resource
+NATURAL JOIN rr.capability
+NATURAL JOIN rr.interface
+NATURAL JOIN (
+	SELECT ivoid, cap_index,
+		string_agg(detail_xpath, '#') as detail_paths,
+		string_agg(detail_value, '#SEP#') as detail_values
+	FROM rr.res_detail
+	GROUP BY ivoid, cap_index) AS details
+NATURAL LEFT OUTER JOIN (
+	SELECT ivoid, date_value as last_checked
+	FROM rr.res_date
+	WHERE value_role='inspected') as dates
+WHERE
+	res_type='doc:document'
+	AND intf_role='rendered'
+ORDER BY creator_seq, ivoid, cap_index
 		"""
 		, parseAll=True)
 	pprint.pprint(res.asList(), stream=sys.stderr)
