@@ -10,7 +10,7 @@ for full ADQL geometry support, we'll phase this out (although it's a good
 deal faster when it works).
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -507,7 +507,7 @@ def _morphArrayMap(node, state):
 _miscMorphers = {
 	"numericValueFunction": _adqlFunctionToPG,
 	"timestampFunction": _morphTimestamp,
-	"arrayMapFunction": _morphArrayMap
+	"arrayMapFunction": _morphArrayMap,
 }
 
 
