@@ -2,7 +2,7 @@
 Node classes and factories used in ADQL tree processing.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -540,12 +540,14 @@ class TableName(ADQLNode):
 		# Implementation detail: We map tap_upload to temporary tables
 		# here; therefore, we can just nil out anything called tap_upload.
 		# If we need more flexibility, this probably is the place to implement
-		# the mapping.
+		# the mapping (though on tap_user, we don't and create a different
+		# node).
 		if self.schema and self.schema.lower()=="tap_upload":
 			self.schema = None
 
 		self.qName = ".".join(flatten(n)
 			for n in (self.cat, self.schema, self.name) if n)
+		self.physicalName = self.qName
 
 	@classmethod
 	def _getInitKWs(cls, _parseResult):
@@ -577,6 +579,43 @@ class TableName(ADQLNode):
 			if p is not None)
 
 
+class UploadTableName(TableName):
+	"""a reference to a table in tap_upload.
+
+	In DaCHS these are mapped to tables in a magic schema, and their
+	interpretation depends on the current user.  Here, using the class
+	we just give a sentinel to later components that something needs to be
+	done, specifically, establish the physical table name during anntation
+	and then making sure the node is morphed into that.
+	"""
+	type = "uploadTableName"
+	fieldInfo = None
+
+	def addFieldInfo(self, context):
+		# we don't actually have field infos on this, but we need to find
+		# our physical name using the context's fieldInfoGetter.
+		# We have to call getInfosFor explicitly before we can do that,
+		# because the DaCHS' FieldInfoGetter only then retrieves that
+		# information from the database.
+		# By the way, I can't just query tap_user.tables here directly because
+		# down here I have no idea which user is running the query.  Perhaps
+		# I should keep that info in the annotation context and get rid
+		# of this slightly insane indirection?
+		innerName = self.qName.lower()
+		context.fieldInfoGetter.getInfosFor(innerName)
+		self.physicalName = context.fieldInfoGetter.physicalNameMap[innerName]
+
+	def flatten(self):
+		# we flatten to our physicalName, because that's what the database
+		# will evaluate; note that before annotation, this will be the
+		# same as the ADQL-visible name.  Yes, this is confusing behaviour,
+		# but I don't have a good idea on how to fix this.
+		if not re.match(r"tap_user\.[A-Za-z_]+$", self.physicalName):
+			raise NotImplementedError(
+				f"Not a regular table reference: {self.physicalName}")
+		return self.physicalName
+
+
 class PlainTableRef(ColumnBearingNode):
 	"""A reference to a simple table.
 	
@@ -587,6 +626,7 @@ class PlainTableRef(ColumnBearingNode):
 	type = "possiblyAliasedTable"
 	_a_tableName = None      # a TableName instance
 	_a_originalTable = None  # a TableName instance
+	_a_wasAliased = False
 	_a_sampling = None
 
 	@classmethod
@@ -594,9 +634,11 @@ class PlainTableRef(ColumnBearingNode):
 		if _parseResult.get("alias"):
 			tableName = TableName(name=_parseResult.get("alias"))
 			originalTable = _parseResult.get("tableName")
+			wasAliased = True
 		else:
-			tableName = getChildOfType(_parseResult, "tableName")
+			tableName = getChildOfClass(_parseResult, TableName)
 			originalTable = tableName  #noflake: locals returned
+			wasAliased = False
 
 		if _parseResult.get("tablesample"):
 			sampling = float(_parseResult.get("tablesample")[2])
@@ -611,10 +653,10 @@ class PlainTableRef(ColumnBearingNode):
 
 	def flatten(self):
 		origTable = flatten(self.originalTable)
-		if origTable!=self.qName:
+		if self.wasAliased:
 			literal = "%s AS %s"%(origTable, flatten(self.tableName))
 		else:
-			literal = self.qName
+			literal = origTable
 
 		if self.sampling:
 			# TODO: Postgres dependency; this should be in morphpg
@@ -1189,6 +1231,36 @@ class QuerySpecification(TransparentNode):
 		return getattr(self.children[-1], attrName)
 
 
+class Statement(FieldInfoedNode):
+	type = "statement"
+	_a_createAs = None
+	_a_query = None
+
+	@classmethod
+	def _getInitKWs(cls, _parseResult):
+		if len(_parseResult)==1:
+			createAs, query = None, _parseResult[0]
+		elif len(_parseResult)==2:
+			createAs, query = _parseResult
+		else:
+			raise NotImplementedError(f"Extra tokens in statement {_parseResult}.")
+		return locals()
+
+	def flatten(self):
+		if self.createAs:
+			return "{} {}".format(flatten(self.createAs), flatten(self.query))
+		else:
+			return self.query.flatten()
+
+	def addFieldInfos(self, context):
+		self.query.addFieldInfos(context)
+		if self.createAs:
+			self.createAs.allocateTAPUpload(context)
+
+	def __getattr__(self, attrName):
+		return getattr(self.query, attrName)
+
+
 class _BaseColumnReference(FieldInfoedNode):
 	# normal column references will be handled by the dispatchColumnReference
 	# function below; there we look at the columnReference type to figure
@@ -1338,7 +1410,12 @@ class FromClause(ADQLNode):
 		"""returns a ColumnReference object for a name, colInfo pair from a
 		table's fieldInfos.
 		"""
-		cr = ColumnReference(name=colPair[0], refName=sourceTableName)
+		if isinstance(sourceTableName, TableName):
+			cr = ColumnReference(
+				name=colPair[0],
+				refName=sourceTableName.physicalName)
+		else:
+			cr = ColumnReference(name=colPair[0], refName=sourceTableName)
 		cr.fieldInfo = colPair[1]
 		return cr
 
@@ -1551,7 +1628,7 @@ class ArrayReference(FieldInfoedNode, TransparentMixin):
 			raise common.Error("Cannot subscript a typeless thing in %s"%(
 				self.flatten()))
 	
-		lastSubscript = re.search("\[[0-9]*\]$", childInfo.type)
+		lastSubscript = re.search(r"\[[0-9]*\]$", childInfo.type)
 		if lastSubscript is None:
 			raise common.Error("Cannot subscript a non-array in %s"%(
 				self.flatten()))
@@ -1973,7 +2050,7 @@ class CastSpecification(FieldInfoedNode, TransparentMixin):
 	def _getInitKWs(cls, _parseResult):
 		value = _parseResult["value"]
 		newType = _parseResult["newType"].lower()
-		if newType.startswith("char ("):
+		if newType.startswith("char (") or newType.startswith("varchar ("):
 			newType = "text"
 		elif newType.startswith("national char"):
 			newType = "unicode"
@@ -2586,6 +2663,57 @@ class Area(FunctionNode):
 			userData=collectUserData(self._getInfoChildren())[0])
 
 
+############################### the odd one: DDL, CREATE TABLE AS
+
+class CreateAs(ADQLNode):
+	"""this must basically work like the embedded query specification,
+	except that it morphs into a create table statement.
+
+	Well, it also has to arrange for name mangling of the uploaded
+	table and for having it added to the management tables.  All this
+	happens during annotation time, as we don't have the necessary
+	information while we are parsing.  Yes, that's a bit more
+	brittle than I would like it to be, but exposing the requesting
+	users to the parsing code seems dramatically overdoing it, too.
+	"""
+	type = "createAs"
+
+	_a_destName = None
+
+	@classmethod
+	def _getInitKWs(cls, parseResult):
+		return {"destName": parseResult[2]}
+
+	def flatten(self):
+		return "CREATE TABLE {} AS ".format(flatten(self.destName))
+
+	def allocateTAPUpload(self, context):
+		"""creates a tap_upload table for the table we are selecting into.
+
+		This is called from statement's addFieldInfos.  It is important
+		to not hand through the addFieldInfos call to our table, because
+		we don't have that at this point.  Instead, we poke our
+		child's physicalName attribute.
+		"""
+		# this is all a DaCHS extension, so we're liberally importing
+		# from DaCHS' guts
+		from gavo.web import tap_uploads
+		from gavo import base
+
+		requestedName = self.destName.name.lower()
+		if not re.match("[a-z_]+$", requestedName):
+			raise NotImplementedError("Non-regular upload table name")
+
+		with base.getWritableUntrustedConn() as conn:
+			self.destName.physicalName = tap_uploads.uploadUserTable(
+				conn,
+				requestedName,
+				None,
+				context.fieldInfoGetter.forUser)
+
+
+############################## Bookkeeping
+
 _ADDITIONAL_NODES = []
 def registerNode(node):
 	"""registers a node class or a symbolAction from a module other than node.
