@@ -6,7 +6,7 @@ http://docs.g-vo.org/DaCHS/ref.html#api-function-reference for what is
 available here.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -95,7 +95,7 @@ def reloadLocal():
 	a constant error regardless of what you try to getattr from it.
 	"""
 	global local
-	modpath = base.getConfig("configDir")+"/local"
+	modpath = base.getConfig("configDir") / "local"
 	try:
 		local, _ = loadPythonModule(modpath)
 	except (FileNotFoundError, ImportError):
