@@ -22,7 +22,7 @@ There is a non-expanding version of STREAM, too, but that has to take
 special precautions.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -216,6 +216,15 @@ class RecordingBase(structure.Structure):
 			if ev[0]==_EXPANDED_VALUE:
 				self.events_[ind] = ("value",)+ev[1:]
 
+	def dump(self):
+		indent = ""
+		for type, name, val, pos in self.events_:
+			print(f"{indent}{type}\t{name}\t{val}")
+			if type=="start":
+				indent += "  "
+			elif type=="end":
+				indent = indent[:-2]
+
 	# This lets us feedFrom these
 	iterEvents = getEventSource
 
@@ -247,9 +256,9 @@ class RawEventStream(EventStream):
 	With non-expanded streams, you can do that::
 
 		<NXSTREAM id="cols">
-			<LOOP listItems="\stuff">
+			<LOOP listItems="\\stuff">
 				<events>
-					<column name="\\item"/>
+					<column name="\\\\item"/>
 				</events>
 			</LOOP>
 		</NXSTREAM>
@@ -417,7 +426,7 @@ class ReplayBase(ActiveTag, structure.Structure, macros.StandardMacroMixin):
 		typeOfExpandedValues = _EXPANDED_VALUE
 		if isinstance(self.source, RawEventStream):
 			typeOfExpandedValues = "value"
-		
+	
 		with ctx.replaying():
 			for type, name, val, pos in events:
 				if (self._expandMacros
@@ -527,8 +536,8 @@ class DelayedReplayBase(ReplayBase, GhostMixin):
 class ReplayedEventsWithFreeAttributesBase(DelayedReplayBase):
 	"""An active tag that takes arbitrary attributes as macro definitions.
 	"""
-	def __init__(self, *args, **kwargs):
-		DelayedReplayBase.__init__(self, *args, **kwargs)
+	def __init__(self, parent, *args, **kwargs):
+		DelayedReplayBase.__init__(self, parent, *args, **kwargs)
 		# managedAttrs in general is a class attribute.  Here, we want
 		# to add values for the macros, and these are instance-local.
 		self.managedAttrs = self.managedAttrs.copy()
@@ -677,6 +686,26 @@ class Loop(ReplayedEventsWithFreeAttributesBase):
 			self._replayer()
 
 
+class Debug(ActiveTag, GhostMixin):
+	"""Enter a debugger when parsing to here.
+
+	This is probably only interesting for DaCHS developers.
+	"""
+	name_ = "DEBUG"
+
+	attrSeq = []
+
+	def __init__(self, parent, *args, **kwargs):
+		self.parent = parent
+
+	def feedEvent(self, ctx, type, name, value):
+		if type=="end":
+			print("DEBUG elemeent: the parent element is in self.parent")
+			print("  Also, use ctx.getById(...) to get elements by id")
+			import pdb;pdb.Pdb(nosigint=True).set_trace()
+		return self.parent
+
+
 getActiveTag = utils.buildClassResolver(ActiveTag, list(globals().values()),
 	key=lambda obj: getattr(obj, "name_", None))
 
