@@ -54,7 +54,7 @@ but I don't think this distinction is necessary here).
 See structure on how to use all these.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -274,7 +274,7 @@ class NWUnicodeAttribute(UnicodeAttribute):
 		value = UnicodeAttribute.parse(self, value)
 		if value is None:
 			return value
-		return re.sub("\s+", " ", value.strip())
+		return re.sub(r"\s+", " ", value.strip())
 
 
 class FunctionRelativePathAttribute(UnicodeAttribute):
