@@ -2,7 +2,7 @@
 Definition of DC config options and their management including I/O.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -221,7 +221,7 @@ class ProfileParser:
 		elif token=="\n":
 			return self._state_init
 
-		if not re.match("[A-Za-z][\w]+$", token):
+		if not re.match(r"[A-Za-z][\w]+$", token):
 			self._raiseError("invalid identifier %s"%repr(token))
 		self.tokenStack.append(token)
 		return self._state_waitForEqual
@@ -375,13 +375,13 @@ _config = Configuration(
 			" accessible under from the outside if that is not 80; nonstandard"
 			" ports are not supported for https. If you offer"
 			" both http and https, use your preferred protocol here."
-			"  If you change this on a running server, you *must* run"
+			"  If you change this on an existing server, you *must* run"
 			" dachs pub -a to update internal and external links."),
 		ListConfigItem("alternateHostnames", default="",
 			description="A comma-separated list of hostnames this server is"
 				" also known under.  Only set this if you're running https."
 				" With this, you can handle a situation where your data"
-				" center can be reached as both example.org and www.example.org."),
+				" centre can be reached as both example.org and www.example.org."),
 		StringConfigItem("bindAddress", default="127.0.0.1", description=
 			"Interface to bind to"),
 		IntConfigItem("serverPort", default="8080",
@@ -420,7 +420,8 @@ _config = Configuration(
 		IntConfigItem("maxPreviewWidth", "300", "Ignored, only present"
 			" for backward compatibility"),
 		ListConfigItem("graphicMimes",
-			"image/fits,image/jpeg,application/x-votable+xml;content=datalink",
+			"image/fits,image/jpeg,application/x-votable+xml;content=datalink,"
+			"application/fits",
 			"Media types considered as graphics (for SIAP, mostly)"),
 		IntConfigItem("maxUploadSize",
 			"20000000",
@@ -526,6 +527,11 @@ _config = Configuration(
 			" stateDir/system_tables once a day?"),
 		IntConfigItem("poolSize", "2",
 			"Number of connections in DaCHS' postgres connection pools."),
+		BooleanConfigItem("q3cPlannerHack",
+			"True",
+			"Disable seqscanning when running queries using q3c.  This"
+			" used to unconditionally help in ancient times but may actually"
+			" hurt with newer postgreses.")
 	),
 	
 	Section('ivoa', 'The interface to the Greater VO.',
@@ -561,7 +567,10 @@ _config = Configuration(
 			"ivo_managed, vosi, local",
 			"Comma-separated list of OAI sets DaCHS should accept for publication."
 			"  This must always include ivo_managed and vosi, and you will"
-			" want local if you run a web interface.")
+			" want local if you run a web interface."),
+		IntConfigItem("userTableDays", "7",
+			"Number of days that user-uploaded TAP tables are kept before"
+			" garbage collection"),
 	),
 )
 
@@ -578,13 +587,15 @@ def loadConfig():
 	
 	# also set XDG directories so astropy and friends look for their
 	# configuration in DaCHS' directories
-	os.environ["XDG_CONFIG_HOME"] = _config.get("configDir")
-	os.environ["XDG_CACHE_HOME"] = _config.get("cacheDir")
+	os.environ["XDG_CONFIG_HOME"] = str(_config.get("configDir"))
+	os.environ["XDG_CACHE_HOME"] = str(_config.get("cacheDir"))
+	from astropy.config import reload_config
+	reload_config("astropy")
 
 	# also configure matplotlib to pull their configuration from
 	# DaCHS' configuration (rather than any random matplotlibrc people
 	# may have left).
-	os.environ["MATPLOTLIBRC"] = _config.get("configDir")
+	os.environ["MATPLOTLIBRC"] = str(_config.get("configDir"))
 
 loadConfig()
 
