@@ -5,7 +5,7 @@ between them.
 Basically all of this should be taken over by stc and astropysics.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -24,9 +24,6 @@ from gavo.utils import DEG
 from gavo.utils import pgsphere
 from gavo.utils import pyfits
 
-# temporary measure to shut up astropy's configuration parser.
-import builtins
-builtins._ASTROPY_SETUP_ = True
 wcs = utils.DeferredImport("wcs", "from astropy import wcs")
 
 
