@@ -9,7 +9,7 @@ This is usually used in conjunction with
 base.typesystems.ToPythonCodeConverter.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -171,15 +171,15 @@ def parseCooPair(soup):
 	soup = soup.strip()
 
 	def parseFloatPair(soup):
-		mat = re.match("(%s)\s*[\s,/]\s*(%s)$"%(utils.floatRE,
+		mat = re.match(r"(%s)\s*[\s,/]\s*(%s)$"%(utils.floatRE,
 			utils.floatRE), soup)
 		if mat:
 			return float(mat.group(1)), float(mat.group(2))
 
 	def parseSexa(soup):
 		timeangleRE = r"(?:\d+\s+)?(?:\d+\s+)?\d+(?:\.\d*)?"
-		dmsRE = "[+-]?\s*(?:\d+\s+)?(?:\d+\s+)?\d+(?:\.\d*)?"
-		mat = re.match("(%s)\s*[\s,/]?\s*(%s)$"%(timeangleRE, dmsRE), soup)
+		dmsRE = r"[+-]?\s*(?:\d+\s+)?(?:\d+\s+)?\d+(?:\.\d*)?"
+		mat = re.match(r"(%s)\s*[\s,/]?\s*(%s)$"%(timeangleRE, dmsRE), soup)
 		if mat:
 			try:
 				return utils.hmsToDeg(mat.group(1)), utils.dmsToDeg(
@@ -189,8 +189,8 @@ def parseCooPair(soup):
 
 	def parseSexaColon(soup):
 		timeangleRE = r"(?:\d+:)?(?:\d+:)?\d+(?:\.\d*)?"
-		dmsRE = "[+-]?\s*(?:\d+:)?(?:\d+:)?\d+(?:\.\d*)?"
-		mat = re.match("(%s)\s*[\s,/]?\s*(%s)$"%(timeangleRE, dmsRE), soup)
+		dmsRE = r"[+-]?\s*(?:\d+:)?(?:\d+:)?\d+(?:\.\d*)?"
+		mat = re.match(r"(%s)\s*[\s,/]?\s*(%s)$"%(timeangleRE, dmsRE), soup)
 		if mat:
 			try:
 				return (utils.hmsToDeg(mat.group(1), sepChar=":"),
