@@ -4,7 +4,7 @@ Basic OS interface/utility functions that depend on our configuration.
 (everything that doesn't need getConfig is somewhere in gavo.utils)
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -24,10 +24,12 @@ from email.header import Header
 from email.parser import Parser
 from email.mime.nonmultipart import MIMENonMultipart
 
-
 import pkg_resources
 
+from twisted import copyright as twcopyright
+
 from gavo.base import config
+from gavo.base import parsecontext
 from gavo import utils
 
 
@@ -178,6 +180,35 @@ def makeAbsoluteURL(path, canonical=False):
 	return str(serverURL+makeSitePath(path))
 
 
+def makeLinkForIvoid(ivoid):
+	"""returns an http URL for ivoid.
+
+	This is using GAVO's ivoid resolver it it's not our authority, and
+	it will guess an info link otherwise.  If ivoid does not look like an
+	ivoid, this will raise a ValueError.
+	"""
+	mat = re.match("ivo://([^/]+)(/.*)?", ivoid)
+	if not mat:
+		raise ValueError(f"Not an ivoid: '{ivoid}'")
+
+	authority = mat.group(1)
+	if authority.lower()==config.get("ivoa", "authority"):
+		# it's ours; see if we can resolve the local part to something
+		# that has a reference URL
+		parts = mat.group(2).strip("/").split("/")
+		try:
+			regOb = parsecontext.resolveCrossId(
+				"/".join(parts[:-1])+"#"+parts[-1])
+		except utils.NotFoundError:
+			# fall through to making a landing page
+			pass
+		else:
+			return str(regOb.getMeta("referenceURL"))
+
+	# it's someone else's, so link to our landing page generator
+	return "https://dc.g-vo.org/LP/"+ivoid
+	
+
 def getBinaryName(baseName):
 	"""returns the name of a binary it thinks is appropriate for the platform.
 
@@ -223,6 +254,12 @@ def getVersion():
 	return pkg_resources.require("gavodachs")[0].version
 
 
+# Our software id as per https://ivoa.net/documents/Notes/softid/
+SERVER_SOFTWARE = "DaCHS/%s twistedWeb/%s"%(
+	getVersion(),
+	twcopyright.version)
+
+
 def formatMail(mailText):
 	"""returns a mail with headers and content properly formatted as
 	a bytestring and MIME.
@@ -306,7 +343,7 @@ def tryRemoteReload(rdId):
 
 	try:
 		f = utils.urlopenRemote(makeAbsoluteURL("/seffe/%s"%rdId),
-			{"__nevow_form__": "adminOps", "submit": "Reload RD"},
+			data={"__nevow_form__": "adminOps", "submit": "Reload RD"},
 			creds=("gavoadmin", pw))
 		f.read()
 	except IOError:
