@@ -6,7 +6,7 @@ This is currently very postgres specific.  If we really wanted to
 support some other database, this would need massive refactoring.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -1226,14 +1226,20 @@ def _makeConnectionManager(profileName, autocommitted=True, singleton=False):
 			# no exception raised, commit if not autocommitted
 			if not autocommitted:
 				conn.commit()
-		
+	
 		try:
 			pool[0].putconn(conn, close=conn.closed)
 		except InterfaceError:
 			# Connection already closed
 			pass
 
-	return contextlib.contextmanager(getConnFromPool)
+	res = contextlib.contextmanager(getConnFromPool)
+	# I want to look at the pools in web.ifpages.CurReaders, so I
+	# better expose them somewhere where they can be found in the
+	# hour of need.
+	res.pool = pool
+
+	return res
 
 
 getUntrustedConn = _makeConnectionManager("untrustedquery")
