@@ -9,7 +9,7 @@ Use the meta.forKeys class decorator to teach meta.py where to
 use your new class.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -55,7 +55,7 @@ class MetaURL(meta.MetaValue):
 	look like this::
 		
 		<meta name="_related" title="Aspec SSAP"
-			>\internallink{aspec/q/ssa/info}</meta>
+			>\\internallink{aspec/q/ssa/info}</meta>
 
 	"""
 	def __init__(self, url, format="plain", title=None):
@@ -138,7 +138,7 @@ class RelatedResourceMeta(meta.MetaValue):
 
 @meta.forKeys("_news")
 class NewsMeta(meta.MetaValue):
-	"""A meta value representing a "news" items.
+	"""A meta value representing a "news" item.
 
 	The content is the body of the news.  In addition, they have
 	date, author, and role children.  In plain text, you would write::
