@@ -17,7 +17,7 @@ linuga franca of our type systems is SQL+extensions as laid
 down in utils.typeconversions.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -346,6 +346,8 @@ class ToPgTypeValidatorConverter(FromSQLConverter):
 		"smap": makeChecker("smap"),
 		"smoc": makeChecker("smoc"),
 		"unicode": makeChecker("text"),
+		"json": makeChecker("json"),
+		"jsonb": makeChecker("jsonb"),
 	}
 
 	def mapComplex(self, type, length):
