@@ -13,7 +13,7 @@ Unit tests for this are at
 http://svn.ari.uni-heidelberg.de/svn/gavo/python/trunk/tests/unitconvtest.py
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -35,6 +35,7 @@ class BadUnit(utils.Error):
 PLANCK_H = 6.62607015e-34
 LIGHT_C = 299792458.0
 BOLTZMANN_K = 1.380649e-23
+ELECTRON_CHARGE = 1.602176634e-19
 
 
 # We can't yet restructure the tree, so we don't do SI-base-casting for
@@ -106,6 +107,7 @@ PLAIN_UNITS = units = {
 	"W": (1, "W"),        # kg.m2/s3 or A.V -- that's going to be a tough one
 	"Wb": (1, "Wb"),
 	"yr": (3600*24*365.25, "s"), # This is the SI/julian year!
+	"%": (1, "%"), # This is the SI/julian year!
 }
 
 
@@ -121,7 +123,7 @@ EXCEPTIONAL_CONVERSIONS = {
 NON_PREFIXABLE = frozenset([
 	"AU", "au", "D", "Ry", "arcmin", "beam", "bin", "chan", "d", "h", "mas",
 	"min", "ph", "photon", "pix", "pixel", "solLum", "solMass", "solRad",
-	"voxel"])
+	"voxel", "%"])
 
 
 PREFIXES = prefixes = {"d": 1e-1, "c": 1e-2, "m":1e-3, "u":1e-6,
@@ -512,7 +514,7 @@ def getUnitGrammar(debugging=False):
 		Suppress, Forward, pyparsingWhitechars, ParserElement)
 
 	with pyparsingWhitechars(''):
-		unit_atom = Word(alphas).addParseAction(UnitNode.fromToks)
+		unit_atom = (Word(alphas) | '%').addParseAction(UnitNode.fromToks)
 		unit_atom.setName("atomic unit")
 		quoted_unit_atom = ("'" + Word(alphas) + "'"
 			).addParseAction(QuotedUnitNode.fromToks)
@@ -526,7 +528,7 @@ def getUnitGrammar(debugging=False):
 		SIGNED_INTEGER = SIGN + UNSIGNED_INTEGER
 		FLOAT = Regex(r"[+-]?([0-9]+(\.[0-9]*)?)")
 		VOFLOAT = Regex(r"0.[0-9]+([eE][+-]?[0-9]+)?"
-			"|[1-9][0-9]*(\.[0-9]+)?([eE][+-]?[0-9]+)?")
+			r"|[1-9][0-9]*(\.[0-9]+)?([eE][+-]?[0-9]+)?")
 
 		integer = (SIGNED_INTEGER | UNSIGNED_INTEGER).setName("integer")
 		power_operator = Literal('**')
