@@ -11,7 +11,7 @@ utils.serializers has once been a part of this module.  To save migration
 effort, for now it reproduces that module's interface.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -230,7 +230,7 @@ class VOTNameMaker(object):
 		self.knownNames, self.index = set(getNameBlacklist(True)), 0
 
 	def makeName(self, field):
-		preName = re.sub("[^\w]+", "x", (getattr(field, "name", None)
+		preName = re.sub(r"[^\w]+", "x", (getattr(field, "name", None)
 			or getattr(field, "ID", None)
 			or "field%02d"%self.index))
 		if not re.match("[A-Za-z_]", preName):
