@@ -1,19 +1,20 @@
 """
 Code to parse structures from XML sources.
 
-The purpose of much of the mess here is to symmetrized XML attributes
+The purpose of much of the mess here is to symmetrize XML attributes
 and values.  Basically, we want start, value, end events whether
 or not a piece of data comes in an element with a certain tag name or
 via a named attribute.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
 import io
+import pathlib
 import re
 
 from gavo import utils
@@ -22,7 +23,7 @@ from gavo.base import common
 from gavo.base import parsecontext
 
 
-ALL_WHITESPACE = re.compile("\s*$")
+ALL_WHITESPACE = re.compile(r"\s*$")
 
 
 class EventProcessor(object):
@@ -215,3 +216,64 @@ def parseFromString(rootStruct, inputString, context=None):
 		parseFromString(rscdef.Column, "<column name='foo'/>")
 	"""
 	return parseFromStream(rootStruct, io.StringIO(inputString), context)
+
+
+def structToETree(aStruct):
+	"""returns an ElementTree for the copyable content of aStruct.
+
+	Note that due to manipulations at parse time and non-copyable content,
+	this will, in general, not reproduce the original XML trees.
+	"""
+	nodeStack = [utils.ElementTree.Element(aStruct.name_)]
+	for evType, elName, value in aStruct.iterEvents():
+		# data model horror.  let me hack around it for now
+		if elName=='_synthesizedRoles':
+			continue
+		try:
+			if evType=="start":
+				nodeStack.append(utils.ElementTree.SubElement(nodeStack[-1], elName))
+			elif evType=="end":
+				nodeStack.pop()
+			elif evType=="value":
+
+				if value is None or value is common.NotGiven:
+					# do not serialise empty or missing attributes at all
+					pass
+
+				elif elName=="content_":
+					# This will become an actual "attribute" in an element.
+					# Let's see how much we will regret blindly stringifying
+					# these.
+					nodeStack[-1].text = str(value)
+
+				elif hasattr(value, "payload"):
+					# it's an _AttBox, which ought to go soon anyway; skip
+					# these, it's only hit stupid legacy STC annotation
+					continue
+
+				else:
+					if isinstance(value, pathlib.Path):
+						value = str(value)
+					if hasattr(value, "id"):
+						# I blindly assume that's a reference
+						value = value.id
+
+					# silently swallow everything that's not a string by now.
+					# should we do better?
+					if isinstance(value, str):
+						nodeStack[-1].set(elName, value)
+			else:
+				raise utils.Error("Invalid struct event: %s"%evType)
+		except Exception as exc:
+			exc.dachsNote = ("Badness occurred in element %s, event %s,"
+				" value %s\n"%(elName, evType, value))
+			raise
+	return nodeStack[-1]
+
+
+def structToXML(aStruct):
+	"""returns RD XML for a structure as a native string.
+	"""
+	dest = io.BytesIO()
+	utils.ElementTree.ElementTree(structToETree(aStruct)).write(dest, "utf-8")
+	return dest.getvalue().decode("utf-8")
