@@ -3,7 +3,7 @@ SIL, the Simple Instance Language, is an attempt to allow
 data model instances written in a simple, JSON-like language.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -20,7 +20,7 @@ from gavo.dm import common
 class Atom(str):
 	"""a sentinel class for atomic values of roles
 	"""
-	noQuotesOkRE = re.compile("[\w_.]+$")
+	noQuotesOkRE = re.compile(r"[\w_.]+$")
 
 	def asSIL(self):
 		if self.noQuotesOkRE.match(self):
