@@ -11,13 +11,12 @@ does not support this, the data's primary table is written to the file
 instance.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
-import cgi
 import io
 import os
 import mimetypes
@@ -30,7 +29,7 @@ from gavo import utils
 EXTENSION_FALLBACKS = {
 	".vot": base.votableType,
 	".fits": "application/fits",
-	".fz": "image/fits",
+	".fz": "application/fits",
 }
 
 # image/fits is probably not quite legal (it's not in Debian's
@@ -65,7 +64,7 @@ def getMIMEKey(contentType):
 	nonsense, anything else will get really ugly.
 	"""
 	contentType = contentType.lower()
-	media_type, paramdict = cgi.parse_header(contentType)
+	media_type, paramdict = utils.parseMediaType(contentType)
 	try:
 		major, minor = media_type.split("/")
 	except (ValueError, TypeError):
@@ -186,13 +185,16 @@ class FORMATS_REGISTRY(object):
 		if formatName in cls.writerRegistry:
 			return formatName
 
-		parsed = getMIMEKey(formatName)
-		if parsed in cls.mimeToKey:
-			return cls.mimeToKey[parsed]
-		parsed = (parsed[0], parsed[1], frozenset())
-		if parsed in cls.mimeToKey:
-			return cls.mimeToKey[parsed]
-
+		try:
+			parsed = getMIMEKey(formatName)
+			if parsed in cls.mimeToKey:
+				return cls.mimeToKey[parsed]
+			parsed = (parsed[0], parsed[1], frozenset())
+			if parsed in cls.mimeToKey:
+				return cls.mimeToKey[parsed]
+		except Exception:
+			raise base.ui.logOldExc(CannotSerializeIn(formatName))
+		
 		raise CannotSerializeIn(formatName)
 
 	@classmethod
