@@ -9,7 +9,7 @@ VOTable FIELDs and the corresponding values.
 You should access this module through formats.votable.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -78,7 +78,7 @@ class VOTableContext(utils.IdManagerMixin):
 			suppressNamespace=False, overflowElement=None):
 		self.mfRegistry = mfRegistry
 		self.tablecoding = tablecoding
-		self.version = version or (1,4)
+		self.version = version or (1,5)
 		self.acquireSamples = acquireSamples
 		self.suppressNamespace = suppressNamespace
 		self.overflowElement = overflowElement
@@ -88,7 +88,7 @@ class VOTableContext(utils.IdManagerMixin):
 		self._pushedRefs = {}
 
 		# state for VO-DML serialisation
-		self.produceVODML = self.version[0]>1 or self.version[1]>4
+		self.produceVODML = self.version[0]>1 or self.version[1]>5
 		# group-serialising annotations must enter their python ids when they
 		# end up in the tree so they get only included once when
 		# referenced by multiple other annotations.
@@ -115,7 +115,7 @@ class VOTableContext(utils.IdManagerMixin):
 		"""adds VODML annotation to this VOTable.
 
 		Note that it will only be rendered if produceVODML is true
-		(in general, for target versions >1.4).
+		(in general, for target versions >1.5).
 		"""
 		self.vodmlTemplates[stuff]
 
@@ -465,9 +465,9 @@ def _makeValuesForColDesc(colDesc):
 	if colDesc.get("max") is None:
 		colDesc["max"] = getattr(colDesc.original.values, "max", None)
 
-	if colDesc["max"] is utils.Infimum:
+	if colDesc["max"] is utils.Infimum or colDesc["max"]=="":
 		colDesc["max"] = None
-	if colDesc["min"] is utils.Supremum:
+	if colDesc["min"] is utils.Supremum or colDesc["min"]=="":
 		colDesc["min"] = None
 
 	if colDesc["min"] is not None:
@@ -564,10 +564,13 @@ def _makeVOTParam(ctx, param):
 	# note that we're usually accessing the content, i.e., the string
 	# serialization we got.  The only exception is when we're seeing
 	# nulls or null-equivalents.
-	if param.content_ is base.NotGiven or param.value is None:
-		content = None
-	else:
-		content = param.content_
+	try:
+		if param.content_ is base.NotGiven or param.value is None:
+			content = None
+		else:
+			content = param.content_
+	except Exception as ex:
+		content = f"not serialisable ({ex})"
 
 	el = V.PARAM()
 	defineField(ctx, el, valuemappers.AnnotatedColumn(param))
@@ -1086,6 +1089,8 @@ def makeVOTable(data, ctx=None, **kwargs):
 		vot = V.VOTABLE()
 	elif ctx.version==(1,5):
 		vot = V.VOTABLE()
+	elif ctx.version==(1,6):
+		vot = V.VOTABLE()
 	else:
 		raise votable.VOTableError("No toplevel element for VOTable version %s"%
 			repr(ctx.version))
@@ -1179,12 +1184,12 @@ common.registerDataWriter("votabletd1.2", functools.partial(
 	".vot2",
 	"text/xml")
 common.registerDataWriter("vodml", functools.partial(
-	format, tablecoding="td", version=(1,5)),
-	"application/x-votable+xml;serialization=TABLEDATA;version=1.5",
-	"VOTable version 1.5, tabledata",
-	".vot5")
+	format, tablecoding="td", version=(1,6)),
+	"application/x-votable+xml;serialization=TABLEDATA;version=1.6",
+	"VOTable version 1.6 beta, tabledata",
+	".vot6")
 common.registerDataWriter("vodmlb", functools.partial(
-	format, version=(1,5)),
-	"application/x-votable+xml;version=1.5",
-	"VOTable version 1.5",
-	".vot5")
+	format, version=(1,6)),
+	"application/x-votable+xml;version=1.6",
+	"VOTable version 1.6 beta",
+	".vot6")
