@@ -3,7 +3,7 @@ A grammar that just splits the source into input lines and then
 lets you name character ranges.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -11,7 +11,8 @@ lets you name character ranges.
 
 from gavo import base
 from gavo import utils
-from gavo.grammars.common import Grammar, FileRowIterator, FileRowAttributes
+from gavo.grammars.common import (
+	Grammar, FileRowIterator, FileRowAttributes, REAttribute)
 from gavo.utils import parsetricks
 
 
@@ -28,6 +29,9 @@ class SplitLineIterator(FileRowIterator):
 			inputLine = self.inputFile.readline()
 			if not inputLine:
 				break
+			if self.grammar.endMarker:
+				if self.grammar.endMarker.match(inputLine):
+					break
 
 			if (self.grammar.commentIntroducer is not base.NotGiven
 					and inputLine.startswith(self.grammar.commentIntroducer)):
@@ -122,6 +126,10 @@ class ColumnGrammar(Grammar, FileRowAttributes):
 		" ignored", copyable=True)
 	_strip = base.BooleanAttribute("strip", default=True,
 		description="Strip all parsed strings?", copyable=True)
+	_endMarker = REAttribute("endMarker",
+		description="A regular expression that, when it matches an"
+			" entire line read, stops processing.  This is for when"
+			" there is trailing junk in files.", copyable=True)
 
 	def _getColDefGrammar(self):
 		with parsetricks.pyparsingWhitechars("\n\t\r "):
