@@ -6,7 +6,7 @@ rscdef.builtingrammars.GRAMMAR_REGISTRY (we don't want to import all
 the mess in this package just to make that).
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -27,7 +27,7 @@ from gavo.rscdef import rowtriggers
 
 
 class REAttribute(base.UnicodeAttribute):
-	"""is an attribute containing (compiled) RE
+	"""An attribute containing (compiled) RE
 	"""
 	def parse(self, value):
 		if value is None or not value:
@@ -196,16 +196,21 @@ class Rowfilter(procdef.ProcApp):
 		return rscdef.replaceProcDefAt(procdef.ProcApp.getFuncCode(self), "row")
 
 
+def _identityRowfilter(row, rowIter):
+	yield row
+
+
 def compileRowfilter(filters):
 	"""returns an iterator that "pipes" the rowfilters in filters.
 
 	This means that the output of filters[0] is used as arguments to
 	filters[1] and so on.
 
-	If filters is empty, None is returned.
+	If filters is empty, an identity function is returned.
 	"""
 	if not filters:
-		return
+		return _identityRowfilter
+
 	iters = [f.compile() for f in filters] #noflake: code gen
 	src = [
 		"def iterPipe(row, rowIter):",
@@ -245,7 +250,7 @@ class MapKeys(base.Structure):
 	mapKeys is necessary in grammars like keyValueGrammar or fitsProdGrammar.
 	In these, the source files themselves give key names.  Within the GAVO
 	DC, keys are required to be valid python identifiers (i.e., match
-	``[A-Za-z\_][A-Za-z\_0-9]*``).  If keys coming in do not have this form,
+	``[A-Za-z_][A-Za-z_0-9]*``).  If keys coming in do not have this form,
 	mapping can force proper names.
 
 	mapKeys could also be used to make incoming names more suitable for
@@ -261,6 +266,7 @@ class MapKeys(base.Structure):
 		copyable=True)
 
 	def _parseShortenedMap(self, literal):
+		literal = literal.strip().rstrip(",")
 		try:
 			for dest, src in (p.split(":") for p in literal.split(",")):
 				src = src.strip()
@@ -329,10 +335,8 @@ class RowIterator(object):
 	def __iter__(self):
 		if self.notify:
 			base.ui.notifyNewSource(self.sourceToken)
-		if hasattr(self, "rowfilter"):
-			baseIter = self._iterRowsProcessed()
-		else:
-			baseIter = self._iterRows()
+		baseIter = self._iterRowsProcessed()
+
 		if self.grammar.ignoreOn:
 			rowSource = self._filteredIter(baseIter)
 		else:
@@ -348,9 +352,6 @@ class RowIterator(object):
 						d = row
 
 					if isinstance(d, dict):
-						# else it could be a sentinel like FLUSH, which we leave alone
-						if self.sourceRow:
-							d.update(self.sourceRow)
 						d["parser_"] = self
 
 					yield row
@@ -372,11 +373,20 @@ class RowIterator(object):
 
 	def _iterRowsProcessed(self):
 		if self.grammar.isDispatching:
-			for dest, row in self._iterRows():
-				for procRow in self.rowfilter(row, self):
-					yield dest, procRow
+			for item in self._iterRows():
+				if not isinstance(item, tuple):
+					# That's mainly for rsc.FLUSH
+					yield item
+				else:
+					dest, row = item
+					if self.sourceRow and isinstance(row, dict):
+						row.update(self.sourceRow)
+					for procRow in self.rowfilter(row, self):
+						yield dest, procRow
 		else:
 			for row in self._iterRows():
+				if self.sourceRow and isinstance(row, dict):
+					row.update(self.sourceRow)
 				for procRow in self.rowfilter(row, self):
 					yield procRow
 
@@ -534,8 +544,8 @@ class GrammarMacroMixin(base.StandardMacroMixin):
 		In rowmakers fed by grammars with //products#define, better use
 		@prodtblAccref.
 		"""
-		return ('utils.getRelativePath(rowIter.sourceToken,'
-			' base.getConfig("inputsDir"), liberalChars=%s)'%(
+		return ('str(utils.getRelativePath(rowIter.sourceToken,'
+			' base.getConfig("inputsDir"), liberalChars=%s))'%(
 				base.parseBooleanLiteral(liberalChars)))
 
 	def macro_fullDLURL(self, dlService):
@@ -706,8 +716,7 @@ class Grammar(base.Structure, GrammarMacroMixin):
 	def parse(self, sourceToken, targetData=None):
 		ri = self.rowIterator(self, sourceToken,
 			sourceRow=self.getSourceFields(sourceToken, targetData))
-		if self.rowfilters:
-			ri.rowfilter = compileRowfilter(self.rowfilters)
+		ri.rowfilter = compileRowfilter(self.rowfilters)
 		return ri
 
 
