@@ -9,7 +9,7 @@ the rest of the DC software.
 Currently, only one kind of DirectGrammar is supported: C boosters.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -623,7 +623,7 @@ class _NumpyMetaCodeGenerator(_CodeGenerator):
 
 		All others raise errors.
 		"""
-		mat = re.match("(\d*)(.)$", format)
+		mat = re.match(r"(\d*)(.)$", format)
 		if not mat:
 			raise base.ReportableError("FITS type code '%s' of %s not handled"
 				" by gavo mkboost; add handling if you can."%(format, colName))
@@ -735,7 +735,7 @@ class FITSCodeGenerator(_NumpyMetaCodeGenerator):
 
 		All others raise errors.
 		"""
-		mat = re.match("(\d*)(.)$", format)
+		mat = re.match(r"(\d*)(.)$", format)
 		if not mat:
 			raise base.ReportableError("FITS type code '%s' of %s not handled"
 				" by gavo mkboost; add handling if you can."%(format, colName))
