@@ -2,7 +2,7 @@
 A grammar defined by code embedded in the RD.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -81,11 +81,17 @@ class EmbeddedGrammar(common.Grammar, base.RestrictionMixin):
 	If you need to raise an error from within an embeddedGrammar, use
 	a SourceParseError, somewhat like this::
 
-		raise utils.SourceParseError(
+		raise base.SourceParseError(
 			"Bad line",
 			offending=inputLine,
 			location=str(lineNumber+1),
 			source=inputFile.name)
+	
+	To furnish other exceptions with information on the location
+	they occurred at (rather than something like "unknown position -- locator
+	missing"), since DaCHS 2.9.3 you can set ``self.location``.  As the
+	error reporting code includes the sourceToken itself, there is generally
+	no need to include it in the location.  Make it a string, though.
 	"""
 	name_ = "embeddedGrammar"
 	_iterator = base.StructAttribute("iterator", default=base.Undefined,
@@ -100,14 +106,18 @@ class EmbeddedGrammar(common.Grammar, base.RestrictionMixin):
 	_notify = base.BooleanAttribute("notify", default=False,
 		description="Enable notification of begin/end of processing (as"
 			" for other grammars; embedded grammars often have odd source"
-			" tokens for which you don't want that).  Note that the -M option"
-			" of gavo imp will only work if you set it to true.", copyable=True)
+			" tokens for which you don't want that.  Note that the -M option"
+			" of gavo imp will only work if you set this to true.", copyable=True)
 	
 	def onElementComplete(self):
 		super().onElementComplete()
 		class RowIterator(common.RowIterator):
 			_iterRows = self.iterator.compile()
 			notify = self.notify
+			location = "unknown location (set self.location to see something here)"
+
+			def getLocator(self):
+				return self.location
 
 		if self.pargetter:
 			RowIterator.getParameters = self.pargetter.compile()
