@@ -2,11 +2,12 @@
 A grammar taking rows from a FITS table.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+import itertools
 
 from gavo import base
 from gavo.grammars import common
@@ -18,19 +19,26 @@ class FITSTableIterator(common.RowIterator):
 	"""The row iterator for FITSTableGrammars.
 	"""
 	def _iterRows(self):
-		data = table.Table.read(self.sourceToken)
+		data = table.Table.read(
+			self.sourceToken,
+			format="fits",
+			hdu=self.grammar.hdu)
 		names = [n for n in data.dtype.names]
 
 		if self.grammar.lowerKeys:
 			names = [n.lower() for n in names]
 
-		for raw_row, mask in zip(data, data.mask):
+		mask = data.mask
+		if mask is None:
+			mask = itertools.cycle([[False]*len(names)])
+
+		for raw_row, mask in zip(data, mask):
 			row = {}
 			for i in range(len(raw_row)):
 				val = raw_row[i]
 				if mask[i]:
 					row[names[i]] = None
-				elif val!=val and self.grammar.nanIsNULL:
+				elif self.grammar.nanIsNULL and val!=val:
 					row[names[i]] = None
 				else:
 					row[names[i]] = val
