@@ -2,7 +2,7 @@
 A q'n'd grammar for reading MySQL dumps of moderate size.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -18,7 +18,7 @@ def guessFieldNames(dump):
 	"""returns the table name and the column names for the first
 	CREATE TABLE statement in a MySQL dump.
 	"""
-	mat = re.search("CREATE TABLE `([^`]*)` \(", dump)
+	mat = re.search(r"CREATE TABLE `([^`]*)` \(", dump)
 	if not mat:
 		raise base.DataError("No proper CREATE TABLE statement found")
 	tableName = mat.group(1)
