@@ -6,7 +6,7 @@ them can be passed in via the solverParameters argument to getWCSFieldsFor,
 a dictionary with keys including:
 
 indices
-  (default: "index-\*.fits", meaning all indices in your default index dir)
+  (default: ``index-*.fits``, meaning all indices in your default index dir)
   The file names from anet's index directory you want to have used.
   glob patterns are expanded, but no recursion takes place.
 
@@ -57,7 +57,7 @@ downsample
 	and it should be something like 2, 3, or 4.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
