@@ -2,7 +2,7 @@
 Helpers for pulling data from CDs, DVDs and similar media.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -71,7 +71,7 @@ class CDHandler(object):
 
 
 def _getCurMax(destBase):
-	"""returns the highest index of ??\d+-formed names in destBase.
+	"""returns the highest index of ??\\d+-formed names in destBase.
 	"""
 	def brutalInt(val):
 		try:
