@@ -15,7 +15,7 @@ can be used by processors.  If these use custom templates, they should
 register them (or regret it later).  See registerTemplate's docstring.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -371,7 +371,7 @@ def getTemplateNameFromHistory(hdr):
 	A ReportableError is raised if the info signature is missing.
 	"""
 	for card in hdr["HISTORY"]:
-		mat = re.search("GAVO DaCHS template used: (\w+)", card)
+		mat = re.search(r"GAVO DaCHS template used: (\w+)", card)
 		if mat:
 			return mat.group(1)
 	raise base.ReportableError("DaCHS template signature not found.",
