@@ -6,7 +6,7 @@ WARNING: This messes up some global state.  DO NOT import into modules
 doing regular work.  testtricks is the module for that kind for stuff.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -17,6 +17,7 @@ import contextlib
 import gc
 import io
 import os
+import pathlib
 import pickle
 import re
 import subprocess
@@ -66,11 +67,10 @@ if "GAVO_OOTTEST" in os.environ:
 	from gavo import base
 
 else:
-	TEST_BASE = os.getcwd()
+	TEST_BASE = pathlib.Path.cwd()
 	originalEnvironment = os.environ.copy()
 	os.environ["GAVOCUSTOM"] = "/invalid"
-	os.environ["GAVOSETTINGS"] = os.path.join(TEST_BASE,
-		"test_data", "test-gavorc")
+	os.environ["GAVOSETTINGS"] = str(TEST_BASE / "test_data" / "test-gavorc")
 	if not os.path.exists(os.environ["GAVOSETTINGS"]):
 		warnings.warn("testhelpers imported from non-test directory.  This"
 			" is almost certainly not what you want (or set GAVO_OOTTEST).")
@@ -81,7 +81,7 @@ else:
 	from gavo import base #noflake: import above is conditional
 	del os.environ["GAVO_INIT_RUNNING"]
 
-	if os.path.exists(base.getConfig("rootDir")):
+	if base.getConfig("rootDir").exists():
 		# we're assuming the test environment is complete then
 		# (i.e., including the test database)
 		from gavo.base import sqlsupport
@@ -93,25 +93,21 @@ else:
 		dsn = initdachs.DSN(dbname)
 		initdachs.createFSHierarchy(dsn, "test")
 
-		with open(os.path.join(
-				base.getConfig("configDir"), "defaultmeta.txt"), "a",
+		with open(base.getConfig("configDir") / "defaultmeta.txt", "a",
 				encoding="utf-8") as f:
 			f.write("!organization.description: Mein wüster Club\n")
 			f.write("!contact.email: invalid@wherever.else\n")
 
-		with open(os.path.join(
-				base.getConfig("configDir"), "vanitynames.txt"), "w") as f:
+		with open(base.getConfig("configDir") / "vanitynames.txt", "w") as f:
 			f.write('http://sofo-hd.de sofo !redirect\n'
 				'data/cores/scs/scs.xml fromvanitynames\n')
 
 		from gavo.base import config, meta
 		meta.makeFallbackMeta(reload=True)
-		os.symlink(os.path.join(TEST_BASE, "test_data"),
-			os.path.join(base.getConfig("inputsDir"), "data"))
-		os.rmdir(os.path.join(base.getConfig("inputsDir"), "__system"))
-		os.symlink(os.path.join(TEST_BASE, "test_data", "__system"),
-			os.path.join(base.getConfig("inputsDir"), "__system"))
-		os.mkdir(os.path.join(base.getConfig("inputsDir"), "test"))
+		os.symlink(TEST_BASE / "test_data",
+			base.getConfig("inputsDir") / "data")
+		(base.getConfig("inputsDir") / "test").mkdir(exist_ok=True)
+		(base.getConfig("inputsDir") / "__system").mkdir(exist_ok=True)
 
 		subprocess.check_call(["createdb", "--template=template0",
 			"--encoding=UTF-8", "--locale=C", dbname])
@@ -133,11 +129,20 @@ else:
 				publication.updateServiceList([base.caches.getRD("//rds")])
 				publication.updateServiceList([base.caches.getRD("//services")])
 				publication.updateServiceList([base.caches.getRD("//tap")])
+				publication.updateServiceList([base.caches.getRD("data/cores")])
+				publication.updateServiceList([base.caches.getRD("data/ssatest")])
 
 				# Import some resources necessary in trial tests
 				from gavo.user import importing
+				importing.process(rsc.getParseOptions(), ["//tap_user"])
 				importing.process(rsc.getParseOptions(), ["//obscore"])
 				importing.process(rsc.getParseOptions(), ["//biblinks"])
+
+				# clear the RD cache so we see the new publications
+				base.caches.clearCaches()
+				from gavo.rscdef.common import getIvoidsForRDId
+				getIvoidsForRDId.cache_clear()
+
 				with base.getWritableAdminConn() as conn:
 					rsc.makeData(base.resolveCrossId("//uws#enable_useruws"),
 						connection=conn)
@@ -161,6 +166,7 @@ from gavo.base import config  #noflake: previous import conditional
 from gavo.base import sqlsupport
 from gavo.adql import ufunctions
 
+
 class FakeSimbad(object):
 	"""we monkeypatch simbadinterface such that we don't query simbad during
 	tests.
@@ -286,7 +292,8 @@ class TestResource(testresources.TestResource):
 
 
 from gavo.helpers.testtricks import (  #noflake: exported names
-	XSDTestMixin, testFile, getMemDiffer, getXMLTree, collectedEvents)
+	XSDTestMixin, testFile, getMemDiffer, getXMLTree, collectedEvents,
+	pickSingle)
 
 
 class ForkingSubprocess(subprocess.Popen):
@@ -521,6 +528,15 @@ class VerboseTest(testresources.ResourcedTestCase):
 				ex.__class__.__name__, str(ex)))
 
 	def assertAlmostEqualVector(self, first, second, places=7, msg=None):
+		def getnulls(vec):
+			return [val is None or val!=val for val in vec]
+
+		if getnulls(first)!=getnulls(second):
+			raise AssertionError(f"{first} and {second} have a different null mask")
+
+		first = [val for val in first if val is not None and val==val]
+		second = [val for val in second if val is not None and val==val]
+
 		try:
 			for f, s in zip(first, second):
 				self.assertAlmostEqual(f, s, places)
@@ -576,7 +592,6 @@ class VerboseTest(testresources.ResourcedTestCase):
 		retcode = p.wait()
 
 		try:
-			
 			if isinstance(expectedStderr, (bytes, str)):
 				self.assertEqual(err, utils.bytify(expectedStderr))
 			elif isinstance(expectedStderr, list):
@@ -617,9 +632,9 @@ class VerboseTest(testresources.ResourcedTestCase):
 
 
 _xmlJunkPat = re.compile("|".join([
-	'(xmlns(:[a-z0-9]+)?="[^"]*"\s*)',
-	'((frame_|coord_system_)?id="[^"]*")',
-	'(xsi:schemaLocation="[^"]*"\s*)']))
+	r'(xmlns(:[a-z0-9]+)?="[^"]*"\s*)',
+	r'((frame_|coord_system_)?id="[^"]*")',
+	r'(xsi:schemaLocation="[^"]*"\s*)']))
 
 
 def cleanXML(aString):
@@ -633,7 +648,7 @@ def cleanXML(aString):
 	aString can be bytes, in which case it will be interpreted as ASCII.
 	cleanXML returns a string in any case.
 	"""
-	return re.sub("\s+", " ",
+	return re.sub(r"\s+", " ",
 		_xmlJunkPat.sub('',
 			utils.debytify(aString))).strip(
 				).replace(" />", "/>").replace(" >", ">")
@@ -891,18 +906,19 @@ def userconfigContent(content):
 
 	content are RD elements without the root (resource) tag.
 	"""
-	userConfigPath = os.path.join(
-		base.getConfig("configDir"), "userconfig.rd")
-	base.caches.clearForName(userConfigPath[:-3])
-	with open(userConfigPath, "w") as f:
+	from gavo import rscdesc
+
+	userConfigPath = rscdesc.USERCONFIG_RD_PATH
+	base.caches.clearForName(str(userConfigPath))
+	with open(userConfigPath.with_suffix(".rd"), "w") as f:
 		f.write('<resource schema="__system">\n'
 			+content
 			+'\n</resource>\n')
 	try:
 		yield
 	finally:
-		os.unlink(userConfigPath)
-	base.caches.clearForName(userConfigPath[:-3])
+		os.unlink(userConfigPath.with_suffix(".rd"))
+		base.caches.clearForName(str(userConfigPath))
 
 
 @contextlib.contextmanager
