@@ -6,7 +6,7 @@ testhelpers, too.  Unit test modules should not be forced to import
 this.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -42,6 +42,15 @@ def _nukeNamespaces(xmlString):
 		re.sub(b'xmlns="[^"]*"', b"", xmlString))
 
 
+def pickSingle(seq):
+	"""gets the first item for a 1-item sequence and raises an AssertionError
+	if there's more.
+	"""
+	if len(seq)!=1:
+		raise AssertionError(f"!= one item in {seq}")
+	return seq[0]
+
+
 class _WrappedEtree:
 	"""a wrapper adding a few methods to an lxml etree.
 
