@@ -2,7 +2,7 @@
 Helpers for trial-based tests, in particular retrieving pages.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -100,20 +100,28 @@ def makeRequestArgs(aDict=None, **kwargs):
 class FakeRequest(testing.FakeRequest, web.Request):
 	# our request class has a few frills that we want to exercise, too,
 	# so we mix it into the standard fake request
-	pass
+	class host:
+		host = 'this.is.no.ip.address'
 testing.FakeRequest = FakeRequest
 
 # for convenience, expose FakeFile, too
 FakeFile = testing.FakeFile
 
+
+def addCreds(request, user="testing", password="testing"):
+	# testhelpers sets up testing in the users table
+	request.user = user
+	request.password = password
+
+
 def runSvcWith(service, renderer, args):
 	"""runs svc through renderers, passing a dict args.
 
 	args maps to single values, *not* to lists as in t.w.  Unless, of course,
 	you actually have lists to begin with.
 	"""
-	args = makeRequestArgs(args)
-	queryMeta = svcs.QueryMeta.fromRequestArgs(args)
+	req = FakeRequest(args=args)
+	queryMeta = svcs.QueryMeta.fromRequest(req)
 	return service.run(renderer, args, queryMeta)
 
 
