@@ -2,7 +2,7 @@
 Common code supporting functionality described in DALI.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -154,22 +154,23 @@ def mangleUploads(request):
 	request.strargs["upload"] = newUploads
 			
 
-def getDALIError(errInfo, queryStatus="ERROR", request=None):
-	"""returns  a DALI-compliant error VOTable from an error info.
+def getDALIError(errInfo, *, queryStatus="ERROR", request=None):
+	"""returns  a DALI-compliant error VOTable from an error info and
+	a suggested http status code for the delivered document.
 
 	errInfo can either be an exception, a UWS error info (a
-	dict with keys msg, type, and hint), or a pre-formatted string.
+	dict with keys msg and possibly hint and httpCode), or a
+	pre-formatted string.
 	"""
 	if isinstance(errInfo, Exception):
 		errInfo = {
 			"msg": str(errInfo),
-			"type": errInfo.__class__.__name__,
-			"hint": getattr(errInfo, "hint", None)}
+			"hint": getattr(errInfo, "hint", None),
+			"httpCode": getattr(errInfo, "httpCode", 500)}
 	
 	elif isinstance(errInfo, str):
 		errInfo = {
 			"msg": errInfo,
-			"type": "ReportableError",
 			"hint": None}
 
 	res = V.RESOURCE(type="results") [
@@ -178,14 +179,14 @@ def getDALIError(errInfo, queryStatus="ERROR", request=None):
 	if request:
 		res[V.INFO(name="request", value=utils.debytify(request.uri))]
 
-	if errInfo["hint"]:
+	if errInfo.get("hint"):
 		res[V.INFO(name="HINT", value="HINT")[
 			errInfo["hint"]]]
 
-	return V.VOTABLE[res]
+	return V.VOTABLE[res], errInfo.get("httpCode", 500)
 
 
-def serveDALIError(request, errInfo, httpStatus=200, queryStatus="ERROR"):
+def serveDALIError(request, errInfo, *, httpStatus=None, queryStatus="ERROR"):
 	"""serves a DALI-compliant error message from errInfo.
 
 	See getDALIError for what errInfo can be.
@@ -193,9 +194,11 @@ def serveDALIError(request, errInfo, httpStatus=200, queryStatus="ERROR"):
 	This closes the request and returns NOT_DONE_YET, so you
 	can write ``return serverDALIError`` in normal render functions
 
-	DALI isn't quite clear on what httpCode ought to be, and protocols
-	sometimes actually require it to be 200 even of errors (sigh).
-	To return non-200, pass an httpCode
+	Regrettably, various protocols at times required 200 HTTP statuses
+	even for error messages.  If you need "odd" http statuses, pass
+	one in here.  Otherwise, we will try to infer something suitable
+	from errInfo (httpCode key in a dict, httpCode attribute on an
+	Exception, 500 for strings).
 	"""
 	if request._disconnected:
 		# the other end has hung up; don't touch the connection any more,
@@ -204,7 +207,11 @@ def serveDALIError(request, errInfo, httpStatus=200, queryStatus="ERROR"):
 
 	try:
 		request.setHeader("content-type", "text/xml")
-		errVOT = getDALIError(errInfo, queryStatus, request)
+		errVOT, errStatus = getDALIError(errInfo,
+			queryStatus=queryStatus, request=request)
+		if httpStatus is None:
+			httpStatus = errStatus
+
 		request.setResponseCode(httpStatus)
 		request.write(
 			utils.xmlrender(errVOT,
@@ -227,8 +234,14 @@ class DALIErrorResource(resource.Resource):
 	function.  Otherwise, just use serveDALIError.
 	"""
 	isLeaf = True
-	def __init__(self, errInfo):
+	def __init__(self, errInfo, *, httpStatus=None, queryStatus="ERROR"):
 		self.errInfo = errInfo
+		self.httpStatus = httpStatus
+		self.queryStatus = queryStatus
 	
 	def render(self, request):
-		return serveDALIError(request, self.errInfo)
+		return serveDALIError(
+			request,
+			self.errInfo,
+			httpStatus=self.httpStatus,
+			queryStatus=self.queryStatus)
