@@ -4,7 +4,7 @@ The datalink core and its numerous helper classes.
 More on this in "Datalink Cores" in the reference documentation.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -158,7 +158,7 @@ class ProductDescriptor(DatalinkDescriptorMixin):
 
 	def estimateSize(self):
 		if isinstance(self.accessPath, str):
-			candPath = os.path.join(base.getConfig("inputsDir"), self.accessPath)
+			candPath = base.getConfig("inputsDir") / self.accessPath
 			try:
 				return os.path.getsize(candPath)
 			except:
@@ -195,14 +195,12 @@ class FITSProductDescriptor(ProductDescriptor):
 		self.imageExtind = 0
 
 		if qnd and self.imageExtind==0:
-			with open(os.path.join(
-					base.getConfig("inputsDir"), self.accessPath), "rb") as f:
+			with open(base.getConfig("inputsDir") / self.accessPath, "rb") as f:
 				self.hdr = utils.readPrimaryHeaderQuick(f,
 					maxHeaderBlocks=100)
 
 		else:
-			hdus = pyfits.open(os.path.join(
-				base.getConfig("inputsDir"), self.accessPath))
+			hdus = pyfits.open(base.getConfig("inputsDir") / self.accessPath)
 			self.imageExtind = fitstools.fixImageExtind(
 				hdus, self.imageExtind)
 			self.hdr = hdus[self.imageExtind].header
@@ -240,9 +238,8 @@ class DLFITSProductDescriptor(FITSProductDescriptor):
 	has a datalink as the product.
 	"""
 	def __init__(self, *args, **kwargs):
-		kwargs["accessPath"] = os.path.join(
-			base.getConfig("inputsDir"),
-			kwargs["accref"])
+		kwargs["accessPath"] = str(
+			base.getConfig("inputsDir") / kwargs["accref"])
 		FITSProductDescriptor.__init__(self, *args, **kwargs)
 
 
@@ -507,39 +504,68 @@ class ProcLinkDef:
 		return self.serviceDescriptor.asVOT(ctx)
 
 
-class _ServiceDescriptor(object):
-	"""An internal descriptor for one of our services.
+class ExternalProcLinkDef:
+	"""a description of a non-DaCHS processing service.
 
-	These are serialized into service resources in VOTables.
-	Basically, these collect input keys, a pubDID, as well as any other
-	data we might need in service definition.
+	This results in a datalink #proc line with a service descriptor
+	that has:
+
+	* standardId and accessURL as global params (where standardID is optional)
+	* inputKeys as future members of the inputParams GROUP
+	* description and local_semantics as the respective fields in the
+		datalink table
+	* title for use in the descriptor block.
 	"""
-	def __init__(self, pubDID, inputKeys, rendName, dlSvc):
-		self.pubDID, self.inputKeys = pubDID, inputKeys
-		self.rendName, self.dlSvc = rendName, dlSvc
+	def __init__(self,
+			pubDID,
+			inputKeys,
+			accessURL,
+			title,
+			description,
+			localSemantics="",
+			standardId=None):
+		self.pubDID = pubDID
+		self.accessURL = accessURL
+		self.localSemantics = localSemantics
+		self.description = description
 
-		if self.pubDID:
-		# if we're fixed to a specific pubDID, reflect that in the ID
-		# field -- this is how clients know which dataset to pull
-		# from datalink documents.
-			for index, ik in enumerate(self.inputKeys):
-				if ik.name=="ID":
-					ik = ik.copy(None)
-					ik.set(pubDID)
-					self.inputKeys[index] = ik
+		self.serviceDescriptor = _ExternalServiceDescriptor(
+			self.pubDID,
+			inputKeys,
+			accessURL,
+			"external service",
+			title,
+			self.description,
+			standardId)
 
-	@property
-	def description(self):
-		return base.getMetaText(self.dlSvc,
-			"description",
-			default="An interactive service on this dataset.")
+	def asDict(self):
+		"""returns a datalink row for this service.
 
-	@property
-	def title(self):
-		return base.getMetaText(self.dlSvc,
-			"title",
-			default="Processing service with id {}".format(self.dlSvc.id))
+		This is called by //datalink#make_response for the table row.
+		"""
+		if self.svcId is None:
+			raise base.Error("You need to arrange for asVOT to be called first"
+				f" on this {self.__class__}.")
+		return {
+			"ID": self.pubDID,
+			"service_def": self.svcId,
+			"description": self.description,
+			"semantics": "#proc",
+			"local_semantics": self.localSemantics}
 
+	def asVOT(self, ctx):
+		"""returns a VOTable fragment for the service descriptor
+		
+		This called by runForMeta.
+		"""
+		self.svcId = ctx.getOrMakeIdFor(
+			self.serviceDescriptor, suggestion="ext_svc")
+		return self.serviceDescriptor.asVOT(ctx)
+
+
+class _SerialisableServiceDescriptor:
+	"""something that implements the getVOT method for _XServiceDescriptor-s.
+	"""
 	def asVOT(self, ctx, linkIdTo=None):
 		"""returns VOTable stanxml for a description of this service.
 
@@ -559,9 +585,9 @@ class _ServiceDescriptor(object):
 			colRef.toParam = True
 			return ctx.getOrMakeIdFor(paramsByName[colRef.dest],
 				suggestion=colRef.dest)
-	
+
 		res = V.RESOURCE(ID=ctx.getOrMakeIdFor(self, suggestion="proc_svc"),
-				name=self.dlSvc.id,
+				name=self.name,
 				type="meta",
 				utype="adhoc:service")[
 			V.DESCRIPTION[self.description],
@@ -570,17 +596,12 @@ class _ServiceDescriptor(object):
 				for ast in stcSpecs],
 			V.PARAM(arraysize="*", datatype="char",
 				name="accessURL", ucd="meta.ref.url",
-				value=self.dlSvc.getURL(self.rendName))]
+				value=self.accessURL)]
 
-		standardId = base.getMetaText(self.dlSvc, "standardID", default=None)
-		if standardId is None:
-			standardId = {
-				"dlasync": ASYNC_STANDARD_ID,
-				"dlget": SYNC_STANDARD_ID}.get(self.rendName)
-		if standardId:
+		if self.standardId:
 			res[
 				V.PARAM(arraysize="*", datatype="char",
-					name="standardID", value=standardId)]
+					name="standardID", value=self.standardId)]
 
 		inputParams = V.GROUP(name="inputParams")
 		res = res[inputParams]
@@ -601,6 +622,76 @@ class _ServiceDescriptor(object):
 		return res
 
 
+class _ExternalServiceDescriptor(_SerialisableServiceDescriptor):
+	"""A descriptor for a non-Datalink service.
+
+	These are for when there is no dlget renederer on the service or all
+	you have is a URL and some parameters.
+
+	See ExternalProcLinkDef for what this is constructed with.
+	"""
+	def __init__(self,
+			pubDID,
+			inputKeys,
+			accessURL,
+			name,
+			title,
+			description,
+			standardId):
+		self.pubDID, self.inputKeys = pubDID, inputKeys,
+		self.accessURL, self.description = accessURL, description
+		self.title, self.name = title, name
+		self.standardId = standardId
+
+
+class _ServiceDescriptor(_SerialisableServiceDescriptor):
+	"""An internal descriptor for one of our services.
+
+	These are serialized into service resources in VOTables.
+	Basically, these collect input keys, a pubDID, as well as any other
+	data we might need in service definition.
+	"""
+	def __init__(self, pubDID, inputKeys, rendName, dlSvc):
+		self.pubDID, self.inputKeys = pubDID, inputKeys
+		self.dlSvc, self.rendName = dlSvc, rendName
+		self.name = self.dlSvc.id
+
+		if self.pubDID:
+		# if we're fixed to a specific pubDID, reflect that in the ID
+		# field -- this is how clients know which dataset to pull
+		# from datalink documents.
+			for index, ik in enumerate(self.inputKeys):
+				if ik.name=="ID":
+					ik = ik.copy(None)
+					ik.set(pubDID)
+					self.inputKeys[index] = ik
+
+		self.standardId = base.getMetaText(self.dlSvc, "standardID", default=None)
+		if self.standardId is None:
+			self.standardId = {
+				"dlasync": ASYNC_STANDARD_ID,
+				"dlget": SYNC_STANDARD_ID}.get(rendName)
+
+	@property
+	def description(self):
+		return base.getMetaText(self.dlSvc,
+			"description",
+			default="An interactive service on this dataset.")
+
+	@property
+	def title(self):
+		return base.getMetaText(self.dlSvc,
+			"title",
+			default="Processing service with id {}".format(self.dlSvc.id))
+
+	@property
+	def accessURL(self):
+		# this is a property because we may not have an RD when the
+		# descriptor is constructed, and hence we want to defer this
+		# computation
+		return self.dlSvc.getURL(self.rendName)
+
+
 class MetaMaker(rscdef.ProcApp):
 	"""A procedure application that generates metadata for datalink services.
 
@@ -646,7 +737,7 @@ class MetaMaker(rscdef.ProcApp):
 				<code>
 					 yield descriptor.makeLink(
 						 "http://example.org/flats/master-flat.fits",
-						 contentType="image/fits",
+						 contentType="application/fits",
 						 description="The master flat for this epoch",
 						 contentLength=2048x2048*2)
 				</code>
@@ -667,6 +758,8 @@ class MetaMaker(rscdef.ProcApp):
 	- LinkDef -- a class to define further links within datalink services.
 	- ProcLinkDef -- a class to reference dlget services outside of the current
 	  service
+	- ExternalProcLinkDef -- a class to reference arbitrary services
+		(typically outside outside of the local DaCHS system).
 	- DatalinkFault -- a container of datalink error generators
 	- soda -- the soda module.
 	"""
@@ -681,6 +774,7 @@ class MetaMaker(rscdef.ProcApp):
 		"Option": rscdef.Option,
 		"LinkDef": LinkDef,
 		"ProcLinkDef": ProcLinkDef,
+		"ExternalProcLinkDef": ExternalProcLinkDef,
 		"DatalinkFault": DatalinkFault,
 		"soda": soda,
 	}
@@ -703,7 +797,7 @@ class DataFunction(rscdef.ProcApp):
 	attribute (or raise some appropriate exception), or a server error will
 	be returned to the client.
 
-	What is returned depends on the service, but typcially it's going to
+	What is returned depends on the service, but typically it's going to
 	be a table or products.*Product instance.
 
 	Data functions can shortcut if it's evident that further data functions
@@ -873,7 +967,7 @@ class DatalinkCoreBase(svcs.Core, base.ExpansionDelegator):
 			with descriptor.currentSemantics(metaMaker.semantics):
 				try:
 					for item in metaMaker.compile(self)(self, descriptor):
-						if isinstance(item, (LinkDef, ProcLinkDef)):
+						if isinstance(item, (LinkDef, ProcLinkDef, ExternalProcLinkDef)):
 							linkDefs.append(item)
 						elif isinstance(item, DatalinkFault):
 							errors.append(item)
@@ -905,10 +999,7 @@ class DatalinkCoreBase(svcs.Core, base.ExpansionDelegator):
 		# if the accref is a datalink document, go through dlget itself.
 		if descriptor.mime=="application/x-votable+xml;content=datalink":
 			if isinstance(descriptor, DLFITSProductDescriptor):
-				# this is perhaps a bit insane, but I like image/fits
-				# for non-tabular FITS files, and that's what DLFITS
-				# deals with.
-				mediaType = "image/fits"
+				mediaType = "application/fits"
 			else:
 				mediaType = formats.guessMediaType(descriptor.accref),
 			yield LinkDef(descriptor.pubDID,
