@@ -13,7 +13,7 @@ write the RD as the first resource; for good measure, we also mark
 it by having a dachs-resource-descriptor name.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -43,7 +43,7 @@ def makeBasicMeta(rd:rscdesc.RD) -> dict:
 	"""
 	res = {
 		"dachs-resdir":
-			utils.getRelativePath(rd.resdir, base.getConfig("inputsDir")),
+			utils.getRelativePath(str(rd.resdir), base.getConfig("inputsDir")),
 		"name": rd.schema,
 		"homepage": base.makeAbsoluteURL("/browse/"+rd.sourceId),
 		# TODO: We'd like to have an custom profile for DaCHS.  We'll
