@@ -5,12 +5,13 @@ This includes both some high-level functions and rudimentary parsers
 that can serve as bases for more specialized parsers.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
+import functools
 import io
 import hashlib
 import os
@@ -52,7 +53,7 @@ class CanonicalPrefixes(object):
 	stuff as instance method here, not as class method.
 	"""
 	def __init__(self, pickleName):
-		self.pickleName = pickleName
+		self.pickleName = str(pickleName)
 		self._registry = {}
 		self._reverseRegistry = {}
 		self._loadData()
@@ -143,9 +144,10 @@ class CanonicalPrefixes(object):
 				msg)
 
 
+@functools.cache
 def getCanonicalPrefixes():
-	return CanonicalPrefixes(os.path.join(base.getConfig("cacheDir"),
-		"rrOaiPrefixes.pickle"))
+	return CanonicalPrefixes(
+		base.getConfig("cacheDir") / "rrOaiPrefixes.pickle")
 
 
 class OAIErrorMixin(object):
@@ -386,7 +388,10 @@ class OAIRecordsParser(sax.ContentHandler, OAIErrorMixin):
 	def _start_oai_record(self, name, attrs):
 		self._errorOccurred = False
 		self.curXML = io.BytesIO()
-		self.writer = saxutils.XMLGenerator(self.curXML, "utf-8")
+		self.writer = saxutils.XMLGenerator(
+			self.curXML,
+			"utf-8",
+			short_empty_elements=True)
 		self.writer.startDocument()
 		self.ivoid, self.updated = None, None
 		self.metadataSeen = False
@@ -656,10 +661,9 @@ def getRecords(registry, startDate=None, endDate=None, set=None,
 	return q.talkOAI(RecordParser)
 
 
-def _addCanonicalNSDecls(xmlLiteral):
-	"""adds XML namespace declarations for namespace prefixes we
-	suspect in xmlLiteral.
-	
+def guessPrefixesUsed(xmlLiteral):
+	"""returns a set of xml namespace prefixes probably used in xmlLiteral.
+
 	This is an ugly hack based on REs necessary because in the OAIRecordsParser
 	we discard the namespace declarations.  It won't work with CDATA
 	sections, and it'll make a hash of things if namespace declarations are
@@ -670,19 +674,31 @@ def _addCanonicalNSDecls(xmlLiteral):
 	Without an XML schema and a full parse (which of course is impossible
 	without the necessary declarations), this is, really, not possible.  But
 	the whole idea of canonical namespace prefixes is a mess, and so we
-	hack along; in particular, we accept any string of the form \w+: within
+	hack along; in particular, we accept any string of the form \\w+: within
 	what looks like an XML tag as a namespace.  Oh my.
 	"""
 	prefixesUsed = set()
 	for elementContent in re.finditer("<[^>]+>", xmlLiteral):
-		prefixesUsed |= set(re.findall("([a-zA-Z_-][\w-]*):[a-zA-Z_]",
+		prefixesUsed |= set(re.findall(r"([a-zA-Z_-][\w-]*):[a-zA-Z_]",
 			elementContent.group()))
+	return prefixesUsed
+
+
+def _addCanonicalNSDecls(xmlLiteral):
+	"""adds XML namespace declarations for namespace prefixes we
+	suspect in xmlLiteral.
+
+	See guessPrefixesUsed for why this a rough approximation that in
+	particular won't work when a document re-binds ns prefixes.  Well,
+	let's make it so that doesn't happen in the Registry.
+	"""
+	prefixesUsed = guessPrefixesUsed(xmlLiteral)
 
 	cp = getCanonicalPrefixes()
 	nsDecls = " ".join('xmlns:%s=%s'%(
 			pref, utils.escapeAttrVal(cp.getNSForPrefix(pref)))
 		for pref in prefixesUsed)
-	return re.sub("<([\w:-]+)", r"<\1 "+nsDecls, xmlLiteral, 1)
+	return re.sub(r"<([\w:-]+)", r"<\1 "+nsDecls, xmlLiteral, 1)
 	
 
 def getRecord(registry, identifier):
