@@ -23,7 +23,7 @@ it if we don't, and for now it doesn't seem we'll support a different framework
 in the foreseeable future.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -129,7 +129,7 @@ def computePreviewFor(product):
 		return threads.deferToThread(product.makePreview)
 
 	sourceMime = product.pr["mime"]
-	if sourceMime=='image/fits':
+	if sourceMime.endswith('/fits'):
 		return threads.deferToThread(makePreviewFromFITS, product)
 	elif sourceMime in _PIL_COMPATIBLE_MIMES:
 		return threads.deferToThread(makePreviewWithPIL, product)
@@ -430,7 +430,7 @@ class StaticPreview(FileProduct):
 		else:
 			# remote URLs can't happen here as RemotePreview is checked
 			# before us.
-			localName = os.path.join(base.getConfig("inputsDir"), previewPath)
+			localName = base.getConfig("inputsDir") / previewPath
 
 		if localName is None:
 			return None
@@ -613,7 +613,7 @@ class CutoutProduct(ProductBase):
 	@classmethod
 	def fromRAccref(cls, rAccref, authGroups=None):
 		if (len(set(rAccref.params.keys())&cls._myKeySet)==4
-				and rAccref.productsRow["mime"]=="image/fits"):
+				and rAccref.productsRow["mime"].endswith("/fits")):
 			return cls(rAccref)
 
 	def _getCutoutHDU(self):
@@ -655,7 +655,7 @@ class CutoutProduct(ProductBase):
 			destF.write(chunk)
 
 	def render(self, request):
-		request.setHeader("content-type", "image/fits")
+		request.setHeader("content-type", "application/fits")
 		return streaming.streamOut(self._writeStuffTo, request)
 	
 	def makePreview(self):
@@ -680,7 +680,7 @@ class ScaledFITSProduct(ProductBase):
 	@classmethod
 	def fromRAccref(cls, rAccref, authGroups=None):
 		if ("scale" in rAccref.params
-				and rAccref.productsRow["mime"]=="image/fits"):
+				and rAccref.productsRow["mime"].endswith("/fits")):
 			return cls(rAccref)
 
 	def _makeName(self):
@@ -702,7 +702,7 @@ class ScaledFITSProduct(ProductBase):
 			destF.write(chunk)
 
 	def render(self, request):
-		request.setHeader("content-type", "image/fits")
+		request.setHeader("content-type", "application/fits")
 		return streaming.streamOut(self._writeStuffTo, request)
 
 
@@ -789,17 +789,19 @@ class ProductCore(svcs.DBCore):
 
 		return keys
 
-	def _getGroups(self, user, password):
-		if user is None:
+	def _getGroups(self, authUser):
+		if authUser is None:
 			return set()
 		else:
-			return creds.getGroupsForUser(user, password)
+			return creds.getGroupsForAuthUser(authUser)
 
 	def run(self, service, inputTable, queryMeta):
 		"""returns a list of {"source": product} dicts for products matching
 		the inputTable.
 		"""
-		authGroups = self._getGroups(queryMeta["user"], queryMeta["password"])
+		authGroups = frozenset()
+		if queryMeta["user_pretend"]:
+			authGroups = self._getGroups(queryMeta.getAuthUser())
 
 		return [getProductForRAccref(r, authGroups)
 			for r in self._getRAccrefs(inputTable)]
@@ -859,17 +861,6 @@ class RAccref(object):
 			if len(value)>0:
 				inputDict[key] = value[-1]
 
-		# Save old URLs: if no (real) path was passed, try to get it
-		# from key.  Remove this ca. 2014, together with
-		# RaccrefTest.(testPathFromKey|testKeyMandatory)
-		if not path.strip("/").strip():
-			if "key" in inputDict:
-				path = inputDict["key"]
-			else:
-				raise base.ValidationError(
-					"Must give key when constructing RAccref",
-					"accref")
-
 		return cls(path, inputDict)
 
 	@classmethod
@@ -967,8 +958,8 @@ class RAccref(object):
 		try:
 			return self._localpathCache
 		except AttributeError:
-			self._localpathCache = os.path.join(base.getConfig("inputsDir"),
-				self.productsRow["accessPath"])
+			self._localpathCache = \
+				base.getConfig("inputsDir") / self.productsRow["accessPath"]
 		return self._localpathCache
 
 	def previewIsCacheable(self):
