@@ -10,7 +10,7 @@ for SODA manipulations that are generic enough to be kept outside of
 the RD but not generic enough to go do base.coords.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -19,6 +19,7 @@ the RD but not generic enough to go do base.coords.
 from gavo import base
 from gavo import rscdef
 from gavo import svcs
+from gavo import utils
 from gavo.base import coords
 from gavo.utils import fitstools
 
@@ -79,7 +80,7 @@ class DatalinkFault(object):
 		raise self.exceptionClass(self.message+" (pubDID: %s)"%self.pubDID)
 
 for errName, exClass in [
-		("AuthenticationFault", svcs.ForbiddenURI), 
+		("AuthenticationFault", svcs.ForbiddenURI),
 		("AuthorizationFault", svcs.ForbiddenURI),
 		("NotFoundFault", svcs.UnknownURI),
 		("UsageFault", svcs.BadMethod),
@@ -103,7 +104,7 @@ class DeliverNow(base.ExecutiveAction):
 
 
 def ensureSkyWCS(descriptor):
-	"""furnishes a soda fits descriptor with skyWCS and spatialAxes attributes 
+	"""furnishes a soda fits descriptor with skyWCS and spatialAxes attributes
 	if necessary.
 
 	This is usually called by the functions needing this automatically, and
@@ -145,7 +146,7 @@ def iterSpatialAxisKeys(descriptor, axisMetaOverrides):
 			"pos.eq.ra", "WCSLONG")]:
 		if name:
 			vertexCoos = footprint[:,colInd]
-			paramArgs = {"name": name, "unit": "deg", 
+			paramArgs = {"name": name, "unit": "deg",
 					"description": description,
 					"ucd": baseUCD}
 
@@ -172,6 +173,10 @@ def iterOtherAxisKeys(descriptor, axisMetaOverrides):
 	"""yields inputKeys for all non-spatial WCS axes.
 
 	descriptor must be a FITSDescriptor.
+
+	Note that this will not generate params unless the fits axis index
+	is mentioned in axisMetaOverrides.  This is because typically, there
+	are pre-defined SODA parameters for such other axes.
 	"""
 	ensureSkyWCS(descriptor)
 	if descriptor.skyWCS is None:
@@ -182,12 +187,17 @@ def iterOtherAxisKeys(descriptor, axisMetaOverrides):
 		fitsAxis = axIndex+1
 		if fitsAxis in descriptor.spatialAxes:
 			continue
+		if fitsAxis not in axisMetaOverrides:
+			continue
 		if length==1:
 			# no cutouts along degenerate axes
 			continue
 		
 		try:
-			ax = fitstools.WCSAxis.fromHeader(descriptor.hdr, fitsAxis)
+			ax = fitstools.WCSAxis.fromHeader(
+				descriptor.hdr,
+				fitsAxis,
+				forceSeparable=True)
 		except ValueError:
 			# probably botched WCS, or an inseparable axis.
 			# Just ignore this axis, operators can add it manually
@@ -198,11 +208,10 @@ def iterOtherAxisKeys(descriptor, axisMetaOverrides):
 		minPhys, maxPhys = ax.getLimits()
 
 		# FIXME: ucd inference
-		paramArgs = {"name": ax.name, "unit": ax.cunit, 
+		paramArgs = {"name": ax.name, "unit": ax.cunit,
 			"description": "Coordinate along axis number %s"%fitsAxis,
 			"ucd": None}
-		if fitsAxis in axisMetaOverrides:
-			paramArgs.update(axisMetaOverrides[fitsAxis])
+		paramArgs.update(axisMetaOverrides[fitsAxis])
 
 		yield MS(svcs.InputKey,  multiplicity="single",
 			type="double precision[2]", xtype="interval",
@@ -259,10 +268,11 @@ def doAxisCutout(descriptor, args):
 		else:
 			# 1-d axis
 			transform = fitstools.WCSAxis.fromHeader(descriptor.hdr, fitsAxis)
-			axMin, axMax = args[parName]
+			limits1 = [transform.physToPix(v) for v in args[parName]]
 			descriptor.changingAxis(fitsAxis, parName)
-			slices.append((fitsAxis, 
-				transform.physToPix(axMin), transform.physToPix(axMax)))
+			slices.append((fitsAxis,
+				utils.floorInt(min(limits1)),
+				utils.ceilInt(max(limits1))))
 
 	if limitsChangedName:
 		for axisInd, lower, upper in coords.getPixelLimits([
@@ -271,6 +281,9 @@ def doAxisCutout(descriptor, args):
 			descriptor.changingAxis(axisInd, limitsChangedName)
 			slices.append((axisInd, lower, upper))
 
+	if descriptor.data==["This is test instrumentation"]:
+		return
+
 	if slices:
 		for axis, lower, upper in slices:
 			if lower==upper:  # Sentinel for empty data
