@@ -13,7 +13,7 @@ query to the DB server.  When done, the job's state is one of COMPLETED,
 ABORTED or ERROR.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -234,7 +234,7 @@ def getQTableFromJob(parameters, job, queryProfile, timeout):
 		connectionForQuery)
 
 	return adqlglue.runTAPQuery(query, timeout, connectionForQuery,
-		tdsForUploads, maxrec)
+		tdsForUploads, maxrec, forUser=job.owner or "anonymous")
 
 
 def runTAPJobNoState(parameters, jobId, queryProfile, timeout):
