@@ -2,7 +2,7 @@
 Support classes for the universal worker service.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -263,9 +263,7 @@ class UWS(object):
 			request.uwsArgs,
 			self.parameterGrammar.parseStrargs(
 				request.strargs, ensureRequired=False),
-			request.getUser()
-				if request.getAuthUser()
-				else None)
+			request.getAuthUser())
 
 	def _getJob(self, jobId, conn, writable=False):
 		"""helps getJob and getNewJob.
@@ -800,7 +798,6 @@ class BaseUWSJob(object, metaclass=UWSJobType):
 			return None
 		if not isinstance(value, dict):
 			value = {
-				"type": value.__class__.__name__,
 				"msg": str(value),
 				"hint": getattr(value, "hint", None),
 			}
@@ -890,6 +887,12 @@ class BaseUWSJob(object, metaclass=UWSJobType):
 						LocalFile(self, self.getWD(), value[1].split("/")[-1]
 						).getURL())
 
+			elif isinstance(value, list):
+				# that's supposed to be an array; the following really only works
+				# for numbers; let's see what we do when we do need string arrays
+				# (they don't properly work in VOTable either...)
+				yield key, " ".join(str(v) for v in value)
+
 			else:
 				yield key, "" if value is None else str(value)
 
