@@ -18,7 +18,7 @@ in asyncrender, which lets asyncrender factor out a bit of common
 functionality.  Hm.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -221,7 +221,7 @@ class ErrorAction(JobAction):
 		if job.error is None:
 			return b""
 
-		return dali.serveDALIError(request, job.error)
+		return dali.serveDALIError(request, job.error, httpStatus=200)
 
 	doPOST = doGET
 JobActions.addStandardAction(ErrorAction)
