@@ -13,7 +13,7 @@ using common.getResType; this means the resType meta is tried first,
 using resob.resType as a fallback.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -70,7 +70,7 @@ def _build_dateFromNews(children, localattrs={}):
 # really useful to specify dates.  So, contrary to our principle, there's
 # an underscore-starting meta here, anyway.
 	if utils.dateRE.match(localattrs.get("date", "")):
-		return [VOR.date(role=localattrs.get("role", "updated"))[
+		return [VOR.date(role=localattrs.get("role", "Updated"))[
 			localattrs["date"]]]
 
 
@@ -83,12 +83,13 @@ _vrResourceBuilder = meta.ModelBasedBuilder([
 		('publisher', SF(VOR.publisher), (), {
 				"ivoId": "ivoId"}),
 		('creator', SF(VOR.creator), [
-			('name', SF(VOR.name)),
-			('altIdentifier', SF(VOR.altIdentifier)),
+			('name', SF(VOR.name), (), {
+				'altIdentifier': "altIdentifier"}),
 			('logo', SF(VOR.logo)),]),
 		('contributor', SF(VOR.contributor), (), {
-				"ivoId": "ivoId"}),
-		('_dataUpdated', SF(VOR.date, role="updated")),
+				"ivoId": "ivoId",
+				'altIdentifier': "altIdentifier"}),
+		('_dataUpdated', SF(VOR.date, role="Updated")),
 		('date', SF(VOR.date), (), {
 				"role": "role"}),
 		('_news', _build_dateFromNews, (), {
@@ -167,7 +168,8 @@ _oaiHeaderBuilder = meta.ModelBasedBuilder([
 
 
 _orgMetaBuilder = meta.ModelBasedBuilder([
-	('facility', SF(VOR.facility)),# XXX TODO: look up ivo-ids?
+	('facility', SF(VOR.facility), (),
+		{"altIdentifier": "altIdentifier", "ivoId": "ivoId"}),
 	('instrument', SF(VOR.instrument), (),
 		{"altIdentifier": "altIdentifier", "ivoId": "ivoId"}),
 ])
@@ -229,7 +231,8 @@ _coverageMetaBuilder = meta.ModelBasedBuilder([
 		('spatial', _build_footprintURL),
 		('waveband', SF(VS.waveband)),
 		('regionOfRegard', SF(VS.regionOfRegard)),
-	])])
+	]),
+	('productTypeServed', SF(VS.productTypeServed))])
 
 
 def getResourceArgs(resob):
