@@ -7,7 +7,7 @@ Thus, in the module we mostly deal with publication objects.  If you
 need the service object, use publication.parent.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -547,11 +547,17 @@ class TAPCapabilityMaker(CapabilityMaker):
 									"REGION", "CENTROID", "COORD1", "COORD2",
 									"DISTANCE", "CONTAINS", "INTERSECTS", "AREA")]],
 
+						TR.languageFeatures(
+								type="ivo://ivoa.net/std/TAPRegExt#features-adql-conditional")[
+							[TR.feature[
+								TR.form[funcName]]
+							for funcName in ("COALESCE",)]],
+
 						TR.languageFeatures(
 								type="ivo://ivoa.net/std/TAPRegExt#features-adql-string")[
 							[TR.feature[
 								TR.form[funcName]]
-							for funcName in ("LOWER", "ILIKE")]],
+							for funcName in ("LOWER", "ILIKE", "UPPER")]],
 
 						TR.languageFeatures(
 								type="ivo://ivoa.net/std/TAPRegExt#features-adql-offset")[
@@ -593,10 +599,6 @@ class TAPCapabilityMaker(CapabilityMaker):
 									"  It either takes a string argument with an ASCII"
 									" MOC ('4/13 17-18 8/3002'), or an order and another"
 									" geometry."]],
-							TR.feature[
-								TR.form["COALESCE"],
-								TR.description["This is the standard SQL COALESCE"
-									" for providing defaults in case of NULL values."]],
 							TR.feature[
 								TR.form["VECTORMATH"],
 								TR.description["You can compute with vectors here. See"
@@ -754,7 +756,7 @@ class BiblinksCapabilityMaker(CapabilityMaker):
 	renderer = "biblinks.json"
 
 	class capabilityClass(VOR.capability):
-		_a_standardID = "ivo://ivoa.net/std/BibVO#biblink-harvest-0.1"
+		_a_standardID = "ivo://ivoa.net/std/BibVO#biblink-harvest-1.0"
 
 
 _getCapabilityMaker = utils.buildClassResolver(CapabilityMaker,
