@@ -8,7 +8,7 @@ Thus, all renderers of a given service have the same ivo-id, which is
 to say, they are all just capabilities on the same record.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -28,7 +28,7 @@ def computeIdentifierFromRestup(restup):
 	return restup["ivoid"]
 
 
-_idPattern = re.compile("ivo://(\w[^!;:@%$,/]+)(/[^?#]*)?")
+_idPattern = re.compile(r"ivo://(\w[^!;:@%$,/]+)(/[^?#]*)?")
 
 def parseIdentifier(identifier):
 	"""returns a pair of authority, resource key for identifier.
