@@ -2,7 +2,7 @@
 The schema and XML namespaces for OAI/VOR documents.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -228,7 +228,7 @@ class VOR:
 
 	class instrument(ResourceName): pass
 	
-	class name(VORElement): pass
+	class name(ResourceName): pass
 	
 	class address(VORElement): pass
 	
@@ -469,6 +469,8 @@ class VS:
 
 	class temporal(VSElement): pass
 
+	class productTypeServed(VSElement): pass
+
 	class format(VSElement):
 		_a_isMIMEType = None
 	
