@@ -2,7 +2,7 @@
 Resources that are not services.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -27,13 +27,17 @@ class NonServiceResource(
 		else:
 			localPart = "/%s/%s"%(self.rd.sourceId, self.id)
 		return "ivo://%s%s"%(base.getConfig("ivoa", "authority"), localPart)
-		
+	
 
-class ResRec(rscdef.IVOMetaMixin, NonServiceResource):
+def _nodata(instance, request, tag):
+	return []
+
+
+class ResRec(rscdef.IVOMetaMixin, svcs.GetURLMixin, NonServiceResource):
 	"""A resource for pure registration purposes.
 
-	A Resource without DaCHS defined behaviour.  This can be
-	Organizations or Instruments, but possibly also external services
+	A Resource without (much) DaCHS-defined behaviour.  This can be
+	Organizations or Instruments, but possibly also external services.
 	
 	All resources must either have an id (which is used in the construction of
 	their IVOID), or you must give an identifier meta item.
@@ -62,13 +66,34 @@ class ResRec(rscdef.IVOMetaMixin, NonServiceResource):
 	name_ = "resRec"
 	_rd = rscdef.RDAttribute()
 
+	metaModel = ("resType(1), title(1), subject, description(1),"
+		" referenceURL(1), creationDate(1),"
+		" contentLevel(? voc:voresource/content_level)")
+
 	_publications = base.StructListAttribute("publications",
 		childFactory=svcs.Publication,
 		description="Capabilities the record should have (this is empty"
 			" for standards, organisations, instruments, etc.)")
 
+	# we add a few "constant" attributes to make this work with the
+	# info renderer
+	limitTo = None
+	nevowRenderers = {}
+	nevowDataFunctions = {
+		"inputFields": _nodata,
+		"htmlOutputFields": _nodata,
+		"votableOutputFields": _nodata}
+
+	@property
+	def allowed(self):
+		# for services, we enumerate usable renderers here; that doesn't
+		# make sense for resRecs.  Instead, we list the renderers mentioned
+		# in publications.
+		return [p.render for p in self.publications]
+
 	def onElementComplete(self):
 		super().onElementComplete()
+		self.core = base.makeStruct(svcs.NullCore, parent_=self)
 		self.setMetaParent(self.rd)
 
 	def getPublicationsForSet(self, names):
@@ -84,6 +109,9 @@ class ResRec(rscdef.IVOMetaMixin, NonServiceResource):
 		for relatedTable in self.iterMeta("tableset"):
 			yield base.resolveCrossId(relatedTable.getContent("text"))
 
+	def getBrowserURL(self, fq=True):
+		return None
+
 	def _meta_sets(self):
 		# This is a copy of the corresponding code in svcs.Service.
 		# See there for more info.
