@@ -2,7 +2,7 @@
 The (DC-internal) service list: querying, adding records, etc.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -101,6 +101,17 @@ def getChunkedServiceList(setName=None):
 		lambda srec: srec.get("title", ".")[0].upper())
 
 
+def getRankedServiceList(setName=None):
+	setName = setName or 'local'
+	rscs = queryServicesList("setName=%(setName)s and not deleted",
+		{"setName": setName})
+	# pre-sort by title; python sort is stable, so we will have alphabetic
+	# ordering within each rank
+	rscs.sort(key=lambda r: r["title"])
+	rscs.sort(key=lambda r: r["rank"] or 10000)
+	return rscs
+
+
 def cleanServiceTablesFor(rd, connection):
 	"""removes/invalidates all entries originating from rd from the service
 	tables.
