@@ -7,7 +7,7 @@ want to use these names quoted.  Quoted identifiers match regular identifiers
 only if case-normalized (i.e., all-lower in DaCHS).
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -93,11 +93,6 @@ def getTableColumnFromColumn(column, typeFactory):
 	This module contains simpleDataTypeFactory, voTableDataTypeFactory,
 	and should soon contain tapTypeFactory.
 	"""
-	if isinstance(column.name, utils.QuotedName):
-		colName = str(column.name)
-	else:
-		colName = column.name.lower()
-
 	flags = []
 	if column.isIndexed():
 		flags.append("indexed")
@@ -107,7 +102,7 @@ def getTableColumnFromColumn(column, typeFactory):
 		flags.append("nullable")
 
 	return VS.column[
-		VS.name[colName],
+		VS.name[column.getTAPName()],
 		VS.description[column.description],
 		VS.unit[column.unit],
 		VS.ucd[column.ucd],
@@ -173,6 +168,27 @@ def _getObscoreStub():
 	return td
 
 
+def _getSchemaRank(rdAndSchema):
+	"""returns the rank of a schemas entry.
+
+	This is for sorting the tableset.  There's a default of 100000 when
+	there is no rank.
+	"""
+	return rdAndSchema[0].getSchemaRank()
+
+
+def _getTableRank(td):
+	"""returns the rank of a table definition.
+
+	This is for sorting the tableset.  There's a default of 100000 when
+	there is no rank.
+	"""
+	m = td.getMeta("table-rank")
+	if m:
+		return int(m.getContent())
+	return 100000
+
+
 def getTablesetForSchemaCollection(
 		schemas, rootElement=VS.tableset, suppressBodies=False):
 	"""returns a vs:tableset element from a sequence of (rd, tables) pairs.
@@ -208,14 +224,14 @@ def getTablesetForSchemaCollection(
 			schemas.append((base.caches.getRD('//obscore'), [_getObscoreStub()]))
 
 	res = rootElement()
-	for rd, tables in schemas:
+	for rd, tables in sorted(schemas, key=_getSchemaRank):
 		res[VS.schema[
 			VS.name[rd.schema],
 			VS.title[base.getMetaText(rd, "title")],
 			VS.description[base.getMetaText(rd, "description")],
 			VS.utype[base.getMetaText(rd, "utype", None)],
 			[getTableForTableDef(td, namesInSet, suppressBodies=suppressBodies)
-				for td in tables]]]
+				for td in sorted(tables, key=_getTableRank)]]]
 	return res
 
 
@@ -229,6 +245,9 @@ class _InternalSchemaStandin(base.MetaMixin):
 		self.setMeta("title", "Not a Schema")
 		self.setMeta("description",
 			"A helper for virtual and temporary tables")
+	
+	def getSchemaRank(self):
+		return 10000
 
 _INTERNAL_SCHEMA = _InternalSchemaStandin()
 
