@@ -2,7 +2,7 @@
 Tables on disk
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -264,23 +264,28 @@ class DBMethodsMixin(sqlsupport.QuerierMixin):
 			self.setSchemaPrivileges(self.tableDef.rd)
 		return self
 
-	def ensureOnDiskMatches(self):
+	def ensureOnDiskMatches(self, tapSchemaColumns=None):
 		"""raises a DataError if the on-disk structure of a table doesn't
 		match DaCHS' idea of it.
 
 		If the table doesn't exist on disk, this will raise a NotFoundError.
+
+		tapSchemaColumns, if given must be a mapping from column names
+		to dicts giving unit, ucd, datatype, arraysize and xtype for
+		the column.  We will then also ensure that that metadata is
+		up to date.
 		"""
 		dbCols = self.getColumnsFromDB(self.tableDef.getQName())
 		mismatches = []
 		if len(self.tableDef.columns)<len(dbCols):
 			mismatches.append("extra columns in DB (%s)"%", ".join(
 				name for name, _ in dbCols[len(self.tableDef.columns):]))
-		
+	
 		for index, col in enumerate(self.tableDef):
 			try:
 				name, type = dbCols[index]
 			except IndexError:
-				mismatches.append("from column %s on: No matches in DB"%col.name)
+				mismatches.append(f"column {col.name}: No matches in DB")
 				break
 
 			if isinstance(col.name, utils.QuotedName):
@@ -295,6 +300,11 @@ class DBMethodsMixin(sqlsupport.QuerierMixin):
 			except TypeError as ex:
 				mismatches.append("type mismatch in column %s (%s)"%(
 					col.name, utils.safe_str(ex)))
+		
+			if tapSchemaColumns:
+				mismatches.extend(
+					col.validateAgainstTAPSchema(
+						tapSchemaColumns.get(col.getTAPName())))
 
 		if mismatches:
 			raise base.DataError("Table %s: %s"%(
@@ -671,7 +681,7 @@ class DBTable(DBMethodsMixin, table.BaseTable, MetaTableMixin):
 		"""runs query within this table's connection.
 
 		query is macro-expanded within the table definition (i.e., you can,
-		e.g., write \qName to obtain the table's qualified name).
+		e.g., write \\qName to obtain the table's qualified name).
 
 		Don't use this in new code; use t.connection.query or execute
 		as required.
