@@ -9,7 +9,7 @@ Or at least give up on the extra connection and have the clients use
 base.getTableDefForTable(connection, tableName) directly.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -18,7 +18,7 @@ base.getTableDefForTable(connection, tableName) directly.
 from gavo import base
 
 
-class MetaTableHandler(object):
+class MetaTableHandler:
 	"""an interface to DaCHS meta tables.
 
 	This used to be a fairly complex interface to all sorts for DC-related
