@@ -2,11 +2,12 @@
 Description of columns (and I/O fields).
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+
 import functools
 
 from gavo import base
@@ -135,7 +136,7 @@ class TableManagedAttribute(base.AttributeDef):
 	
 	That's stc and stcUtype here, currently.
 
-	Do not use this in new code.  This should to when the stc element
+	Do not use this in new code.  This should go when the stc element
 	can safely be replaced by gavo-dm-based stuff (here: DmRoles).
 	"""
 	typeDesc_ = "non-settable internally used value"
@@ -733,6 +734,15 @@ class ColumnBase(base.Structure, base.MetaMixin):
 			return self.tablehead
 		return str(self.key).capitalize()
 
+	def getTAPName(self):
+		"""returns self's name lowercased or with quotes, as is necessary
+		for inclusion into the TAP schema.
+		"""
+		if isinstance(self.name, utils.QuotedName):
+			return str(self.name)
+		else:
+			return self.name.lower()
+
 	def _getVOTableType(self):
 		"""returns the VOTable type, arraysize and xtype for this
 		column-like thing.
@@ -752,7 +762,7 @@ class Column(ColumnBase):
 	table or a VOTable (the exceptions are for column properties that may
 	span several columns, most notably indices).
 
-	Note that the type system adopted by the DC software is a subset
+	Note that the type system adopted by the DaCHS is a subset
 	of postgres' type system.  Thus when defining types, you have to
 	specify basically SQL types.  Types for other type systems (like
 	VOTable, XSD, or the software-internal representation in python values)
@@ -873,6 +883,39 @@ class Column(ColumnBase):
 		
 		return res
 
+	def validateAgainstTAPSchema(self, tapDesc):
+		"""yields mismatches between self's metdatadata and
+		what is given in tapDesc.
+
+		tapDesc is a dict mapping to TAP_SCHEMA.columns-compliant
+		unit, ucd, datatype, arraysize and xtype.
+
+		Special rule: if tapDesc is None, it will just report a missing column.
+		"""
+		if self.hidden:
+			# we are not in TAP_SCHEMA, so there is nothing to compare.
+			return
+
+		if self.xtype=='adql:REGION':
+			# That's pathological. I'm hoping it'll die out and hence not
+			# monkey around with it.
+			return
+
+		if not tapDesc:
+			yield f"Column {self.name} missing in TAP_SCHEMA"
+			return
+
+		type, arraysize, xtype = base.sqltypeToVOTable(self.type)
+		for toCheck, attrName in [
+				(type, "datatype"),
+				(arraysize, "arraysize"),
+				(xtype, "xtype"),
+				(self.ucd, "ucd"),
+				(self.unit, "unit")]:
+			if toCheck!=tapDesc[attrName]:
+				yield "Column {}: tap_schema {}!={}".format(
+					self.name, attrName, repr(toCheck))
+
 
 class ParamBase(ColumnBase):
 	"""A basic parameter.
