@@ -2,18 +2,20 @@
 Common items used by resource definition objects.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
+import functools
 import os
 import re
 import urllib.parse
 
 from gavo import base
 from gavo import utils
+from gavo.base import meta
 
 
 # The following is a flag for initdachs (and initdachs exclusively)
@@ -151,6 +153,15 @@ class PrivilegesMixin(base.StructCallbacks):
 		defaultSource="[db]maintainers")
 
 
+@functools.lru_cache(None)
+def getIvoidsForRDId(rdId):
+	with base.getTableConn() as conn:
+		res = [res[0] for res in conn.query(
+			"SELECT ivoid FROM dc.resources WHERE sourcerd=%(rdId)s",
+			locals())]
+		return res
+
+
 class IVOMetaMixin(base.StructCallbacks):
 	"""A mixin for resources aspiring to have IVO ids.
 
@@ -179,10 +190,23 @@ class IVOMetaMixin(base.StructCallbacks):
 	def _meta_published_identifier(self):
 		"""returns identifier if we believe we are dealing with a published
 		resource.
+
+		There is a hitch here: there are many situations in which the "object"
+		itself has no publications even though there is a published resource,
+		for instance, when a table is published through a service or
+		a service as a secondary service.  Hence, we don't look at our
+		metadata here but instead at what's actually published, and
+		we return all identifiers associated with the current RD.  This
+		may be overdoing things a bit, but in this particular case that's
+		better than the other way round.
 		"""
-		if (getattr(self, "publications", None)
-				or getattr(self, "registration", None)):
-			return self._meta_identifier()
+		if not getattr(self, "rd", None):
+			return
+		try:
+			return [meta.MetaValue(i) for i in getIvoidsForRDId(self.rd.sourceId)
+				] or None
+		except base.DBError:
+			return None
 
 	def _meta_status(self):
 		return "active"
@@ -572,7 +596,7 @@ def getStandardPubDID(path):
 	# getInputsRelativePath in the loop since it does some validation
 	# and may, at some point, do more.
 	if path[0]!="/":
-		path = os.path.join(base.getConfig("inputsDir"), path)
+		path = base.getConfig("inputsDir") / path
 
 	auth = base.getConfig("ivoa", "authority")
 	if auth=="x-unregistred":
@@ -638,7 +662,11 @@ def getInputsRelativePath(absPath, liberalChars=True):
 	``liberalChars``, we see the `function getRelativePath`_.
 
 	In rowmakers and rowfilters, you'll usually use the macro
-	``\inputRelativePath`` that inserts the appropriate code.
+	``\\inputRelativePath`` that inserts the appropriate code.
 	"""
-	return utils.getRelativePath(absPath,
-		base.getConfig("inputsDir"), liberalChars=liberalChars)
+	res = str(utils.getRelativePath(absPath,
+		base.getConfig("inputsDir"), liberalChars=liberalChars))
+	if res==".":
+		return ""
+	return res
+
