@@ -2,7 +2,7 @@
 The execute element and related stuff.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -153,13 +153,13 @@ class Execute(base.Structure, base.ExpansionDelegator):
 		copyable=False,)
 
 	_at = base.StringListAttribute("at",
+		default=base.NotGiven,
 		description="One or more hour:minute pairs at which to run"
 			" the code each day.  This conflicts with every.  Optionally,"
 			" you can prefix each time by one of m<dom> or w<dow> for"
 			" jobs only to be executed at some day of the month or week, both"
 			" counted from 1.  So, 'm22 7:30, w3 15:02' would execute on"
 			" the 22nd of each month at 7:30 UTC and on every wednesday at 15:02.",
-		default=base.NotGiven,
 		copyable=True,)
 
 	_every = base.IntAttribute("every",
@@ -174,6 +174,14 @@ class Execute(base.Structure, base.ExpansionDelegator):
 		" then run every abs(every) seconds",
 		copyable=True,)
 
+	_on = base.EnumeratedUnicodeAttribute("on",
+		default=base.NotGiven,
+		validValues=["loaded"],
+		description="Run the job when a certain event happens.  Right now,"
+			" only 'loaded' is supported: the code is executed when the RD"
+			" has finished parsing.",
+		copyable=True)
+
 	_job = base.StructAttribute("job",
 		childFactory=CronJob,
 		default=base.Undefined,
@@ -263,15 +271,16 @@ class Execute(base.Structure, base.ExpansionDelegator):
 		return (dom, dow, hour, minute)
 
 	def completeElement(self, ctx):
-		super().completeElement(ctx)
-		if len([s for s in [self.at, self.every] if s is base.NotGiven])!=1:
-			raise base.StructureError("Exactly one of at and every required"
-				" for Execute", pos=ctx.pos)
-
 		if self.at is not base.NotGiven:
 			self.parsedAt = []
 			for literal in self.at:
 				self.parsedAt.append(self._parseAt(literal, ctx))
+	
+		# we need the parse context for on, which onElementComplete does
+		# not normally have.  We cheat it in there for now; on the long
+		# run, I think we want to unify completeElement, validate,
+		# onElementComplete, and the callCompletedCallbacks anyway.
+		self.ctx = ctx
 
 	def onElementComplete(self):
 		super().onElementComplete()
@@ -283,5 +292,31 @@ class Execute(base.Structure, base.ExpansionDelegator):
 
 		if self.at is not base.NotGiven:
 			cron.repeatAt(self.parsedAt, self.jobName, self.callable)
-		else:
+
+		elif self.every is not base.NotGiven:
 			cron.runEvery(self.every, self.jobName, self.callable)
+
+		elif self.on=="loaded":
+			# self.ctx set in completeElement
+			del self.callable
+			self.ctx.addExitFunc(lambda rd, ctx: self.job.compile()(rd, self))
+			
+		else:
+			raise NotImplementedError("This cannot happen")
+
+		del self.ctx
+
+	def validate(self):
+		super().validate()
+		if len([s for s in [self.at, self.every, self.on]
+				if s is not base.NotGiven])!=1:
+			raise base.StructureError("Exactly one of {at, on, every} required"
+				" for execute")
+
+	def runImmediate(self):
+		"""compiles and executes the job without any scaffolding.
+
+		That's for debugging only, don't do it in a running server.
+		"""
+		callable = self.job.compile()
+		callable(self.rd, self)
