@@ -6,7 +6,7 @@ defining tests for a given piece of data.	These things can then be
 run while (or rather, after) executing dachs val.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -139,7 +139,7 @@ class Keywords(argparse.Action):
 		setattr(namespace, self.dest, self)
 
 	def _normalise(self, s):
-		return set(re.sub("[^\w\s]+", "", s).lower().split())
+		return set(re.sub(r"[^\w\s]+", "", s).lower().split())
 
 	def match(self, other):
 		if not self.keywords:
@@ -329,7 +329,8 @@ class DataURL(base.Structure):
 
 	_postMediaType = base.UnicodeAttribute("httpPostMediaType",
 		default="application/octet-stream",
-		description="The media type of postPayload",
+		description="The media type of postPayload (which is also applicable"
+		" to other methods)",
 		copyable=True)
 
 	_parset = base.EnumeratedUnicodeAttribute("parSet",
@@ -410,7 +411,7 @@ class DataURL(base.Structure):
 			"user-agent": "DaCHS regression tester"}
 		headers.update(self.httpHeader)
 
-		if self.httpMethod=="POST":
+		if self.httpMethod not in ["HEAD", "GET"]:
 			if self.postPayload:
 				headers["content-type"] = self.httpPostMediaType
 				with open(self.postPayload, "rb") as f:
@@ -470,8 +471,8 @@ class DataURL(base.Structure):
 			if self.getParams():
 				raise base.StructureError("No parameters (or parSets) are"
 					" possible with postPayload")
-			if self.httpMethod!="POST":
-				raise base.StructureError("Only POST is allowed as httpMethod"
+			if self.httpMethod in ["GET", "HEAD"]:
+				raise base.StructureError("We don't let you do GET or HEAD"
 					" together with postPayload")
 				
 		if self.uploads:
@@ -711,6 +712,10 @@ class RegTest(procdef.ProcApp, unittest.TestCase):
 		is an issue) also honor _DBOPTIONS_ORDER).
 		"""
 		data, metadata = votable.loads(self.data)
+		if metadata is None:
+			raise AssertionError("Response did not contain a TABLE -- is it"
+				" an error message?")
+
 		rows = metadata.iterDicts(data)
 		result = next(rows)
 
@@ -731,6 +736,9 @@ class RegTest(procdef.ProcApp, unittest.TestCase):
 		as a sequence of dictionaries.
 		"""
 		data, metadata = votable.loads(self.data)
+		if metadata is None:
+			raise AssertionError("Response did not contain a TABLE -- is it"
+				" an error message?")
 		return list(metadata.iterDicts(data))
 
 	@utils.document
@@ -744,6 +752,16 @@ class RegTest(procdef.ProcApp, unittest.TestCase):
 		self.assertEqual(len(seq), 1)
 		return seq[0]
 
+	@utils.document
+	def datalinkBySemantics(self):
+		"""returns a mapping of semantics -> datalink row for a datalink
+		table.
+		"""
+		bySemantics = {}
+		for r in self.getVOTableRows():
+			bySemantics.setdefault(r["semantics"], []).append(r)
+		return bySemantics
+
 
 class RegTestSuite(base.Structure):
 	"""A suite of regression tests.
