@@ -5,7 +5,7 @@ rowmakers are objects that take a dictionary of some kind and emit
 a row suitable for inclusion into a table.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -282,8 +282,8 @@ class RowmakerMacroMixin(base.StandardMacroMixin):
 	def macro_inputRelativePath(self, liberalChars="True"):
 		"""see grammars.common.GrammarMacroMixin
 		"""
-		return ('getInputsRelativePath('
-			'vars["parser_"].sourceToken, liberalChars=%s)'
+		return ('str(getInputsRelativePath('
+			'vars["parser_"].sourceToken, liberalChars=%s))'
 			)%base.parseBooleanLiteral(liberalChars)
 	
 	def macro_rowsProcessed(self):
