@@ -10,13 +10,14 @@ In the DaCHS, a resource descriptor typically sets up a schema in
 the database.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
 import datetime
+import pathlib
 import os
 import pkg_resources
 import time
@@ -53,8 +54,7 @@ class ResdirAttribute(base.UnicodeAttribute):
 		if parsedValue==".":
 			parsedValue = "/".join(instance.sourceId.split("/")[:-1])
 
-		parsedValue = os.path.join(
-			base.getConfig("inputsDir"), parsedValue)
+		parsedValue = base.getConfig("inputsDir") / parsedValue
 		super().feedObject(instance, parsedValue)
 
 
@@ -66,10 +66,23 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 	collection of FITS images, a catalogue, or whatever), about the database
 	tables the data ends up in, and the services used to access them.
 
-	In DaCHS' RD XML serialisation, they correspond to the root element.
+	This is the root element of all RDs.
+
+	To give your schema a utype, set a ``utype`` meta on resource.
+
+	To set the schema_index in TAP_SCHEMA, put some integer to
+	a schema-rank meta; lower-ranked schemas are displayed further
+	up in supporting clients (since version 2.9.3).
 	"""
 	name_ = "resource"
 
+	metaModel = ("creationDate(1), title(1), description(1), subject,"
+		" coverage.waveband (? voc:messenger),"
+		" content.level (? voc:voresource/content_level),"
+		" content.type (? voc:voresource/content_type),"
+		" _news.role (? voc:voresource/date_role)"
+		)
+
 	# this is set somewhere below once parsing has proceeded far enough
 	# such that caching the RD make sense
 	cacheable = False
@@ -79,15 +92,15 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 	# scripts for a while
 	acceptedScriptTypes = {}
 
-	_resdir = ResdirAttribute("resdir", 
-		default=None, 
+	_resdir = ResdirAttribute("resdir",
+		default=None,
 		description="Base directory for source files and everything else"
 			" belonging to the resource.  Use a single dot (.) to say 'the"
 			" directory the RD resides in', which is recommended in modern"
-			" DaCHS.", 
+			" DaCHS.",
 		copyable=True)
 
-	_schema = base.UnicodeAttribute("schema", 
+	_schema = base.UnicodeAttribute("schema",
 		default=base.Undefined,
 		description="Database schema for tables defined here.  Follow the rule"
 		" 'one schema, one RD' if at all possible.  If two RDs share the same"
@@ -97,41 +110,41 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		copyable=True,
 		callbacks=["_inferResdir"])
 
-	_dds = base.StructListAttribute("dds", 
+	_dds = base.StructListAttribute("dds",
 		childFactory=rscdef.DataDescriptor,
 		description="Descriptors for the data generated and/or published"
-		" within this resource.", 
-		copyable=True, 
+		" within this resource.",
+		copyable=True,
 		before="outputTables")
 
 	_tables = base.StructListAttribute("tables",
-		childFactory=rscdef.TableDef, 
-		description="A table used or created by this resource", 
-		copyable=True, 
+		childFactory=rscdef.TableDef,
+		description="A table used or created by this resource",
+		copyable=True,
 		before="dds")
 
 	_outputTables = base.StructListAttribute("outputTables",
-		childFactory=svcs.OutputTableDef, 
-		description="Canned output tables for later reference.", 
+		childFactory=svcs.OutputTableDef,
+		description="Canned output tables for later reference.",
 		copyable=True)
 
 	_rowmakers = base.StructListAttribute("rowmakers",
-		childFactory=rscdef.RowmakerDef, 
+		childFactory=rscdef.RowmakerDef,
 		description="Transformations for going from grammars to tables."
 			" If specified in the RD, they must be referenced from make"
 			" elements to become active.",
-		copyable=True, 
+		copyable=True,
 		before="dds")
 
-	_procDefs = base.StructListAttribute("procDefs", 
+	_procDefs = base.StructListAttribute("procDefs",
 		childFactory=rscdef.ProcDef,
 		description="Procedure definintions (rowgens, rowmaker apply-s)",
 		copyable=True, before="rowmakers")
 
-	_condDescs = base.StructListAttribute("condDescs", 
+	_condDescs = base.StructListAttribute("condDescs",
 		childFactory=svcs.CondDesc,
-		description="Global condition descriptors for later reference", 
-		copyable=True, 
+		description="Global condition descriptors for later reference",
+		copyable=True,
 		before="cores")
 
 	_resRecs = base.StructListAttribute("resRecs",
@@ -139,25 +152,25 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		description="Non-service resources for the IVOA registry.  They will"
 			" be published when gavo publish is run on the RD.")
 
-	_services = base.StructListAttribute("services", 
-		childFactory=svcs.Service, 
-		description="Services exposing data from this resource.", 
+	_services = base.StructListAttribute("services",
+		childFactory=svcs.Service,
+		description="Services exposing data from this resource.",
 		copyable=True)
 
-	_macDefs = base.MacDefAttribute(before="tables", 
+	_macDefs = base.MacDefAttribute(before="tables",
 		description="User-defined macros available on this RD")
 
 	_mixinDefs = base.StructListAttribute("mixdefs",
 		childFactory=rscdef.MixinDef,
 		description="Mixin definitions (usually not for users)")
 
-	_require = base.ActionAttribute("require", 
+	_require = base.ActionAttribute("require",
 		methodName="importModule",
-		description="Import the named gavo module (for when you need something"
-		" registered)")
+		description="Deprecated, don't use this any more."
+			" May go away by DaCHS 2.12")
 
-	_cores = base.MultiStructListAttribute("cores", 
-		childFactory=svcs.getCore, 
+	_cores = base.MultiStructListAttribute("cores",
+		childFactory=svcs.getCore,
 		childNames=list(svcs.CORE_REGISTRY.keys()),
 		description="Cores available in this resource.", copyable=True,
 		before="services")
@@ -259,7 +272,17 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		return iter(self.dds)
 
 	def getService(self, id):
-		return self.serviceIndex.get(id, None)
+		"""returns a service or resRec object with id.
+
+		This will return None if neiter exists.
+		"""
+		try:
+			res = self.getById(id)
+			if isinstance(res, (svcs.Service, registry.ResRec)):
+				return res
+		except base.NotFoundError:
+			pass  # Fall through when there's no id.
+		return None
 
 	def getTableDefById(self, id):
 		return self.getById(id, rscdef.TableDef)
@@ -292,7 +315,7 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 	def openRes(self, relPath, mode="rb"):
 		"""returns a file object for relPath within self's resdir.
 
-		Deprecated.  This is going to go away, use getAbsPath and a context 
+		Deprecated.  This is going to go away, use getAbsPath and a context
 		manager.
 		"""
 		return open(self.getAbsPath(relPath), mode)
@@ -326,7 +349,7 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		See the grammar in //rds for what you can give for reason.
 		"""
 		rsc.makeData(
-			base.resolveCrossId("//rds#update-for-rd"), 
+			base.resolveCrossId("//rds#update-for-rd"),
 			connection=connection,
 			forceSource=(reason, self))
 
@@ -348,6 +371,15 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 
 		self.__class__ = BrokenClass
 
+	def getSchemaRank(self):
+		"""returns the value of the schema-rank meta parsed and with a
+		fallback.
+		"""
+		m = self.getMeta("schema-rank")
+		if m:
+			return int(m.getContent())
+		return 100000
+
 	def _meta__metadataUpdated(self):
 		"""falls back to utcnow in case we don't know the metadata update
 		time.
@@ -373,8 +405,8 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		instead of this, as that also sets the rights URI.
 		"""
 		return ("%s is licensed under the `Creative Commons Attribution"
-			" Share-Alike 3.0"
-			" License <http://creativecommons.org/licenses/by-sa/3.0/>`_\n\n"
+			" Share-Alike 4.0"
+			" License <http://creativecommons.org/licenses/by-sa/4.0/>`_\n\n"
 			".. image:: /static/img/ccbysa.png\n   :alt: [CC-BY-SA]\n"
 			)%stuffDesignation
 
@@ -388,8 +420,8 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		You'll probably want to use the `//procs#license-cc-by`_ stream
 		instead of this, as that also sets the rights URI.
 		"""
-		return ("%s is licensed under the `Creative Commons Attribution 3.0"
-			" License <http://creativecommons.org/licenses/by/3.0/>`_\n\n"
+		return ("%s is licensed under the `Creative Commons Attribution 4.0"
+			" License <http://creativecommons.org/licenses/by/4.0/>`_\n\n"
 			".. image:: /static/img/ccby.png\n   :alt: [CC-BY]\n\n"
 			)%stuffDesignation
 
@@ -419,7 +451,7 @@ class RDParseContext(base.ParseContext):
 
 	It defines a couple of attributes that structures can ask for (however,
 	it's good practice not to rely on their presence in case someone wants
-	to parse XML snippets with a standard parse context, so use 
+	to parse XML snippets with a standard parse context, so use
 	getattr(ctx, "doQueries", True) or somesuch.
 	"""
 	def __init__(self, doQueries=True, restricted=False, forRD=None):
@@ -460,6 +492,7 @@ def canonicalizeRDId(srcId):
 	// is expanded to __system__/.  The path to built-in RDs,
 	/resources/inputs, is treated analogous to inputsDir.
 	"""
+	srcId = str(srcId)
 	if srcId.startswith("//"):
 		srcId = "__system__"+srcId[1:]
 
@@ -478,7 +511,7 @@ def canonicalizeRDId(srcId):
 			cleanedPath.append(segment)
 	srcId = "/".join(cleanedPath)
 
-	for inputsDir in (base.getConfig("inputsDir"), "/resources/inputs"):
+	for inputsDir in (str(base.getConfig("inputsDir")), "/resources/inputs"):
 		if srcId.startswith(inputsDir):
 			srcId = srcId[len(inputsDir):].lstrip("/")
 	
@@ -492,12 +525,12 @@ def _getFilenamesForId(srcId):
 	"""helps getRDInputStream by iterating over possible files for srcId.
 	"""
 	if srcId.startswith("/"):
-		yield srcId+".rd"
-		yield srcId
+		yield pathlib.Path(srcId+".rd")
+		yield pathlib.Path(srcId)
 	else:
 		inputsDir = base.getConfig("inputsDir")
-		yield os.path.join(inputsDir, srcId)+".rd"
-		yield os.path.join(inputsDir, srcId)
+		yield inputsDir / (srcId+".rd")
+		yield inputsDir / srcId
 		yield "/resources/inputs/%s.rd"%srcId
 		yield "/resources/inputs/%s"%srcId
 
@@ -515,29 +548,34 @@ def getRDInputStream(srcId):
 	override built-in system RDs).
 	"""
 	for fName in _getFilenamesForId(srcId):
-		if os.path.isfile(fName):
-			# We don't want RDs from outside of inputs and config, as 
-			# these make referencing really messy.
-			filePath = os.path.abspath(fName)
-			if not (
-					filePath.startswith(base.getConfig("inputsDir"))
-					or filePath.startswith(base.getConfig("configDir"))):
-				raise base.ReportableError("%s: Only RDs below inputsDir (%s) are"
-					" allowed."%(fName, base.getConfig("inputsDir")))
+		if isinstance(fName, pathlib.Path):
+			# a user-controlled file
+			if fName.is_file():
+				# We don't want RDs from outside of inputs and config, as
+				# these make referencing really messy.
+				filePath = str(fName.absolute())
+				if not (
+						filePath.startswith(str(base.getConfig("inputsDir")))
+						or filePath.startswith(str(base.getConfig("configDir")))):
+					raise base.ReportableError("%s: Only RDs below inputsDir (%s) are"
+						" allowed."%(fName, base.getConfig("inputsDir")))
 
-			return fName, open(fName, "rb")
+				return fName, open(fName, "rb")
+
+		else:
+			# a pkg_resources key
+			if (pkg_resources.resource_exists('gavo', fName)
+					and not pkg_resources.resource_isdir('gavo', fName)):
+				return (PkgResourcePath(fName),
+					pkg_resources.resource_stream('gavo', fName))
 
-		if (pkg_resources.resource_exists('gavo', fName)
-				and not pkg_resources.resource_isdir('gavo', fName)):
-			return (PkgResourcePath(fName), 
-				pkg_resources.resource_stream('gavo', fName))
 	raise base.RDNotFound(srcId)
 
 
 def setRDDateTimes(ctx, rd, inputFile):
 	"""sets the _dataUpdated and _metadataUpdated meta items on rd.
 	"""
-	rd.setMeta("_metadataUpdated", 
+	rd.setMeta("_metadataUpdated",
 		datetime.datetime.utcfromtimestamp(
 			utils.fgetmtime(inputFile)))
 
@@ -548,7 +586,7 @@ def setRDDateTimes(ctx, rd, inputFile):
 		pass
 
 
-USERCONFIG_RD_PATH = os.path.join(base.getConfig("configDir"), "userconfig")
+USERCONFIG_RD_PATH = base.getConfig("configDir") / "userconfig"
 
 
 class _UserConfigFakeRD(object):
@@ -575,7 +613,7 @@ class _UserConfigFakeRD(object):
 		try:
 			try:
 				return base.caches.getRD(
-					os.path.join(base.getConfig("configDir"), "userconfig.rd")
+					base.getConfig("configDir") / "userconfig.rd"
 					).getById(id, forceType=forceType)
 			except base.NotFoundError:
 				pass
@@ -586,7 +624,7 @@ class _UserConfigFakeRD(object):
 			return base.caches.getRD("//userconfig"
 				).getById(id, forceType=forceType)
 		except base.NotFoundError:
-			raise base.NotFoundError(id, "Element with id", 
+			raise base.NotFoundError(id, "Element with id",
 				"etc/userconfig.rd")
 
 
@@ -595,7 +633,7 @@ def refuseBlacklisted(srcId, rdInputPath):
 	one of the strings in [general]rdblacklist.
 	"""
 	for blacklistedName in base.getConfig("rdblacklist"):
-		if (rdInputPath.endswith(blacklistedName)
+		if (str(rdInputPath).endswith(blacklistedName)
 				or blacklistedName==srcId):
 			raise utils.Error(
 				f"RD {srcId} is blacklisted in gavo.rc.  Not loading.")
@@ -635,7 +673,7 @@ def getRD(srcId, doQueries=True, restricted=False, useRD=None):
 			utils.stealVar("getRD_context"), forRD=rd.sourceId)
 	except ValueError:
 		# no getRD_context variable in the stack
-		getRD_context = RDParseContext(doQueries=doQueries, 
+		getRD_context = RDParseContext(doQueries=doQueries,
 			restricted=restricted, forRD=rd.sourceId)
 	
 	rdinj.injectIntoContext(getRD_context, rd.sourceId)
@@ -688,7 +726,7 @@ class CachedException(object):
 			# the RD hasn't been there.  Since we can't tell if the
 			# file has appreared in the mean time, we'll have to re-check
 			return False
-		if not os.path.exists(self.sourcePath): 
+		if not os.path.exists(self.sourcePath):
 			# someone has removed the file, kill cache
 			return True
 		return os.path.getmtime(self.sourcePath)>self.loadedAt
@@ -716,7 +754,7 @@ def _loadRDIntoCache(canonicalRDId, cacheDict):
 
 	if justWait:
 		# Someone else is already parsing.  If it's the current thread,
-		# go on (lock is an RLock!) so we can resolve self-references 
+		# go on (lock is an RLock!) so we can resolve self-references
 		# (as long as they are backward references).  All other threads
 		# just wait for the parsing thread to finish
 		lock.acquire()
@@ -730,7 +768,7 @@ def _loadRDIntoCache(canonicalRDId, cacheDict):
 			# Importing failed, invalidate the RD (in case other threads still
 			# see it from _CURRENTLY_PARSING)
 			if getattr(ex, "cacheable", False):
-				cacheDict[canonicalRDId] = CachedException(ex, 
+				cacheDict[canonicalRDId] = CachedException(ex,
 					getattr(rd, "srcPath", None))
 			rd.invalidate()
 			raise
@@ -770,7 +808,7 @@ def _makeRDCache():
 		clean until it's being changed again.  This will avoid re-parsing
 		RDs all the time when they're broken for a while.
 		"""
-		if (srcId in rdCache 
+		if (srcId in rdCache
 				and getattr(rdCache[srcId], "isDirty", lambda: False)()):
 			try:
 				# limit re-check rate to 10 seconds to avoid getRD storms
@@ -793,10 +831,10 @@ def _makeRDCache():
 					"  Will retry loading when it has been edited."%srcId)
 				# the source path of the cached exception isn't necessarily
 				# right (for system and userconfig RDs); but it ought
-				# to be good enough for what users typcially are confronted with.
+				# to be good enough for what users typically are confronted with.
 				if isinstance(rdCache[srcId], CachedException):
-					rdCache[srcId] = CachedException(ex, 
-						os.path.join(base.getConfig("inputsDir")+srcId+".rd"))
+					rdCache[srcId] = CachedException(ex,
+						str(os.path.join(base.getConfig("inputsDir"))+srcId+".rd"))
 
 				else:
 					rdCache[srcId].loadedAt = time.time()
