@@ -6,7 +6,7 @@ TAP's semi-sanitation of STC needs some special handling anyway,
 and this is much faster than the full-blown mess.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -233,7 +233,7 @@ def getSimpleSTCSParser():
 				frameRE, refposRE, flavorRE)
 		coordsRE = r"(?P<coords>(%s\s*)+)"%utils.floatRE
 
-		simpleStatement = Regex("(?i)\s*"
+		simpleStatement = Regex(r"(?i)\s*"
 			"(?P<shape>position|circle|box|polygon)"
 			+systemRE
 			+coordsRE)
