@@ -24,7 +24,7 @@ Support code is in common.  Most importantly, this is QueryMeta, a data
 structure carrying lots of information on the query being processed.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -34,7 +34,7 @@ structure carrying lots of information on the query being processed.
 
 from gavo.svcs.common import (Error, UnknownURI, ForbiddenURI, WebRedirect,
 	RedirectBase, SeeOther, Authenticate, BadMethod, Found,
-	RequestEntityTooLarge,
+	RequestEntityTooLarge, BadRequest,
 	QueryMeta, emptyQueryMeta, getTemplatePath, loadSystemTemplate)
 
 from gavo.svcs.core import getCore, Core, CORE_REGISTRY
@@ -53,9 +53,10 @@ from gavo.svcs.outputdef import OutputField, OutputTableDef
 
 from gavo.svcs.renderers import RENDERER_REGISTRY, getRenderer
 
-from gavo.svcs.service import (Service, Publication, PreparsedInput)
+from gavo.svcs.service import (Service, Publication, PreparsedInput,
+	GetURLMixin)
 
-from gavo.svcs.standardcores import (DBCore, CondDesc,
+from gavo.svcs.standardcores import (DBCore, CondDesc, NullCore,
 	mapDBErrors)
 
 from gavo.svcs.uploadcores import UploadCore
