@@ -3,7 +3,7 @@ User-defined cores, either with an external file or with code within
 the RD.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -72,7 +72,7 @@ class CustomCore(core.Core):
 	
 
 class CoreProc(rscdef.ProcApp):
-	"""A definition of a pythonCore's functionalty.
+	"""A definition of a pythonCore's functionality.
 
 	This is a procApp complete with setup and code; you could inherit
 	between these.
