@@ -8,7 +8,7 @@ Renderers are actually defined in web.grend, but we need some way to
 get at them from svcs and above, so the registry is kept here.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -25,6 +25,7 @@ RENDERER_REGISTRY = {
 	'api': ("web.vodal", "APIRenderer"),
 	'async': ("web.asyncrender", "DALIAsyncRenderer"),
 	'availability': ("web.vosi", "VOSIAvailabilityRenderer"),
+	'biblinks.json': ("web.vodal","BiblinksRenderer"),
 	'capabilities': ("web.vosi", "VOSICapabilityRenderer"),
 	'coverage': ("web.metarender", "CoverageRenderer"),
 	'custom': ("web.customrender", "CustomRenderer"),
@@ -57,6 +58,7 @@ RENDERER_REGISTRY = {
 	'tableMetadata': ("web.vosi", "VOSITablesetRenderer"),
 	'tables': ("web.vosi", "VOSITablesetRenderer"),
 	'tablenote': ("web.metarender", "TableNoteRenderer"),
+	'user_tables': ("web.tap_uploads", "PersistentUploadRenderer"),
 	'upload': ("web.uploadservice", "Uploader"),
 	'uws.xml': ("web.asyncrender", "DALIAsyncRenderer"),
 	'volatile': ("web.constantrender", "VolatilePageRenderer"),
