@@ -5,7 +5,7 @@ See the __init__.py docstring for a bit more on the general
 architecture.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -13,7 +13,7 @@ architecture.
 
 import functools
 import os
-import urllib.request, urllib.parse, urllib.error
+import urllib.parse
 
 from twisted.web.template import tags as T #noflake: namespace registration
 
@@ -35,7 +35,6 @@ from gavo.svcs import renderers
 MS = base.makeStruct
 
 
-
 class PreparsedInput(dict):
 	"""a sentinel class signalling to the service that its input already
 	is parsed.
@@ -301,8 +300,37 @@ def getDALIServiceKeys():
 	return res
 
 
+class GetURLMixin:
+	"""A mixin for classes with rd and id, letting them compute
+	URLs working with the DaCHS root.py URL dispatcher.
+	"""
+	def getURL(self, rendName, absolute=True, canonical=False, **kwargs):
+		"""returns the full canonical access URL of this service together
+		with renderer.
+
+		rendName is the name of the intended renderer in the registry
+		of renderers.
+
+		With absolute, a fully qualified URL is being returned.
+
+		Further keyword arguments are translated into URL parameters in the
+		query part.
+		"""
+		basePath = "%s%s/%s"%(base.getConfig("web", "nevowRoot"),
+			self.rd.sourceId, self.id)
+		if absolute:
+			basePath = base.makeAbsoluteURL(basePath, canonical=canonical)
+		res = renderers.getRenderer(rendName
+			).makeAccessURL(basePath)
+
+		if kwargs:
+			res = res+"?"+urllib.parse.urlencode(kwargs)
+		return res
+
+
 class Service(base.Structure, base.ComputedMetaMixin,
-		base.StandardMacroMixin, rscdef.IVOMetaMixin):
+		base.StandardMacroMixin, rscdef.IVOMetaMixin,
+		GetURLMixin):
 	"""A service definition.
 
 	A service is a combination of a core and one or more renderers.  They
@@ -368,7 +396,9 @@ class Service(base.Structure, base.ComputedMetaMixin,
 	_original = base.OriginalAttribute()
 
 	metaModel = ("title(1), creationDate(1), description(1),"
-		"subject, referenceURL(1), shortName(!)")
+		" subject, referenceURL(1), shortName(!),"
+		" productTypeServed(? voc:product-type),"
+		" contentLevel(? voc:voresource/content_level)")
 
 	# formats that should query the same fields as HTML (the others behave
 	# like VOTables and offer a "verbosity" widget in forms).
@@ -437,6 +467,15 @@ class Service(base.Structure, base.ComputedMetaMixin,
 
 		self._computeResourceType()
 
+		if hasattr(self.core, "queriedTable"):
+			# Copy over any served-by relationships from our table if
+			# we query one.  This is important, for instance, for
+			# obscore-published tables, where we want to let our
+			# clients know they could be using obscore instead.
+			for m in self.core.queriedTable.iterMeta("isServedBy"):
+				self.addMeta("isServedBy", m)
+
+
 	def _compileCustomPage(self):
 		if self.customPage:
 			try:
@@ -669,7 +708,9 @@ class Service(base.Structure, base.ComputedMetaMixin,
 			# add publications for everything allowed but not yet marked
 			# as published (for VOSI)
 			alreadyPublished = set(p.render for p in result
-				) | frozenset(["sync", "async"]) # these never get capabilities
+				# some TAP subrenderers never get capabilities for now
+				# (but then all this needs to change anyway; cf. caproles)
+				) | frozenset(["sync", "async", "user_tables"])
 
 			for rendName in self.allowed:
 				if not rendName in alreadyPublished:
@@ -682,29 +723,6 @@ class Service(base.Structure, base.ComputedMetaMixin,
 
 		return result
 
-	def getURL(self, rendName, absolute=True, canonical=False, **kwargs):
-		"""returns the full canonical access URL of this service together
-		with renderer.
-
-		rendName is the name of the intended renderer in the registry
-		of renderers.
-
-		With absolute, a fully qualified URL is being returned.
-
-		Further keyword arguments are translated into URL parameters in the
-		query part.
-		"""
-		basePath = "%s%s/%s"%(base.getConfig("web", "nevowRoot"),
-			self.rd.sourceId, self.id)
-		if absolute:
-			basePath = base.makeAbsoluteURL(basePath, canonical=canonical)
-		res = renderers.getRenderer(rendName
-			).makeAccessURL(basePath)
-
-		if kwargs:
-			res = res+"?"+urllib.parse.urlencode(kwargs)
-		return res
-
 	# used by getBrowserURL; keep external higher than form as long as
 	# we have mess like Potsdam CdC.
 	_browserScores = {"form": 10, "external": 12, "fixed": 15,
@@ -1058,7 +1076,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 	def _meta_sets(self):
 		# this is the union of the sets of the publications; this
 		# is a bit lame, but it's a consequence of our mismatch
-		# betwenn publications, capabilities, and services.
+		# between publications, capabilities, and services.
 		# In practice, these sets are meaningless where they are
 		# pulled from meta (everywhere except in service selection
 		# and in list identifiers, both of which don't use this).
