@@ -6,7 +6,7 @@ parameters in the doc rec) and returns an output thing, usually a table
 (but see the src package docstring).
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -436,7 +436,7 @@ class TableBasedCore(core.Core):
 class FancyQueryCore(TableBasedCore, base.RestrictionMixin):
 	"""A core executing a pre-specified query with fancy conditions.
 
-	Unless you select \*, you *must* define the outputTable here;
+	Unless you select \\*, you *must* define the outputTable here;
 	Weird things will happen if you don't.
 
 	The queriedTable attribute is mandatory but only used for name resolution
