@@ -2,14 +2,12 @@
 Parsing and maintaining URL shortcuts.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
-import os
-
 from gavo import base
 from gavo import utils
 
@@ -104,8 +102,8 @@ def _loadVanityMap():
 	vm = _VanityMap()
 	vm.addFromString(BUILTIN_VANITY)
 
-	fSrc = os.path.join(base.getConfig("configDir"), "vanitynames.txt")
-	if os.path.exists(fSrc):
+	fSrc = base.getConfig("configDir") / "vanitynames.txt"
+	if fSrc.exists():
 		with open(fSrc) as f:
 			vm.addFromFile(f)
 
