@@ -2,7 +2,7 @@
 DC administration interface.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -282,16 +282,13 @@ def makedelrec(querier, args):
 
 
 @exposedFunction([], help="Update the TAP_SCHEMA metadata for all"
-	" RDs mentioned in TAP_SCHEMA.")
+	" RDs mentioned in TAP_SCHEMA (deprecated; use"
+	" dachs imp //tap refresh instead)")
 def updateTAPSchema(querier, args):
-	from gavo.protocols import tap
-	for rdId, in querier.connection.query(
-			"select sourcerd from TAP_SCHEMA.tables"):
-		try:
-			rd = base.caches.getRD(rdId)
-			tap.publishToTAP(rd, querier.connection)
-		except base.NotFoundError as msg:
-			base.ui.notifyWarning("Stale records in TAP_SCHEMA: %s"%msg)
+	from gavo import rsc
+	rsc.makeData(
+		base.resolveCrossId("//tap#refresh"),
+		connection=querier.connection)
 
 
 @exposedFunction([Arg(help="Password to hash", dest="pw")],
@@ -344,7 +341,7 @@ def hipsgen(querier, args):
 
 
 def _normalizeWhitespace(s):
-	return re.sub("\s+", " ", s)
+	return re.sub(r"\s+", " ", s)
 
 
 def _getBoundsFromIntervals(intervals):
@@ -397,7 +394,7 @@ def _editProp(propLn, fillers):
 	Unedited lines are returned stripped but otherwise unchanged.
 	"""
 	propLn = propLn.strip()
-	mat = re.match("(#?)(\w+)\s+=\s+(.*)", propLn)
+	mat = re.match(r"(#?)(\w+)\s+=\s+(.*)", propLn)
 	if not mat:
 		raise base.ReportableError(f"Invalid properties line: {propLn}")
 	
