@@ -3,7 +3,7 @@ Generation of system docs by introspection and combination with static
 docs.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -264,11 +264,17 @@ class MacroDoc(object):
 	def makeDoc(self, content):
 		"""adds documentation of macFunc to the RSTFragment content.
 		"""
-		# macros have args in {}, of course there's no self, and null-arg
-		# macros have not {}...
 		sig = inspect.signature(self.macFunc)
-		args = str(sig).replace("(", "{").replace(")", "}").replace("{}", ""
+
+		# macros have args in {}, of course there's no self, and null-arg
+		# macros have no {}...
+		args = str(sig).replace("self, ", ""
+			).replace("(", "{"
+			).replace(")", "}"
+			).replace("{self}", ""
+			).replace("{}", ""
 			).replace(", ", "}{")
+
 		content.addRaw("::\n\n  \\%s%s\n\n"%(self.name, args))
 		content.addRaw(utils.fixIndentation(
 			self.macFunc.__doc__ or "undocumented", "", 1).replace("\\", "\\\\"))
@@ -375,7 +381,8 @@ def getTriggerDocs(docStructure):
 def _documentParameters(content, pars):
 	content.makeSpace()
 	for par in sorted(pars, key=lambda p: p.key):
-		if par.late:
+		# not all procs have the concept of late, so be defensive here
+		if getattr(par, "late", False):
 			doc = ["Late p"]
 		else:
 			doc = ["P"]
@@ -523,8 +530,8 @@ def getMetaKeyDocs():
 	return content.content
 
 
-NO_API_SYMBOLS = frozenset(["__builtins__", "AdhocQuerier",
-	"__doc__", "__file__", "__name__", "__package__", "addCartesian",
+NO_API_SYMBOLS = frozenset(["AdhocQuerier",
+	"addCartesian",
 	"combinePMs", "getBinaryName", "getDefaultValueParsers", "getHTTPPar",
 	"makeProc", "ParseOptions"])
 
@@ -533,6 +540,9 @@ def getAPIDocs(docStructure):
 	content = RSTFragment()
 
 	for name in sorted(dir(api)):
+		if name.startswith("_"):
+			continue
+
 		obj = getattr(api, name)
 		if (name in NO_API_SYMBOLS
 				or type(obj)==type(re)
