@@ -2,7 +2,7 @@
 Common error handling facilities for user interface components.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -21,7 +21,7 @@ from gavo.base import sqlsupport
 class _Reformatter(object):
 	"""A helper class for reformatMessage.
 	"""
-	verbatimRE = re.compile("\s")
+	verbatimRE = re.compile(r"\s")
 
 	def __init__(self):
 		self.inBuffer, self.outBuffer = [], []
@@ -139,7 +139,6 @@ def raiseAndCatch(opts=None, output=outputError):
 	elif isinstance(msg, (base.ValidationError, base.ReportableError,
 			base.LiteralParseError, base.StructureError, base.NotFoundError,
 			base.MetaValidationError)):
-
 		if not getattr(msg, "posInMsg", False):
 			if getattr(msg, "pos", None):
 				messages.append("At or near %s:"%msg.pos)
@@ -170,7 +169,7 @@ def raiseAndCatch(opts=None, output=outputError):
 			messages.append("")
 		messages.append("Oops.  Unhandled exception %s.\n"%msg.__class__.__name__)
 		messages.append("Exception payload: %s"%utils.safe_str(msg))
-		base.ui.notifyError("Uncaught exception at toplevel")
+		base.ui.notifyError(f"Uncaught exception at toplevel: {msg}")
 
 	if getattr(opts, "enablePDB", False):
 		raise
