@@ -2,7 +2,7 @@
 The user interface to importing resources into the VO.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -115,8 +115,7 @@ def process(opts, args):
 	if base.UnmanagedQuerier(connection).getTableType("dc.resources"):
 		# skip this during bootstrap
 		connection.execute("UPDATE dc.resources SET"
-			" dateupdated=CURRENT_TIMESTAMP,"
-			" rectimestamp=CURRENT_TIMESTAMP"
+			" dateupdated=CURRENT_TIMESTAMP"
 			" WHERE sourcerd=%(rdid)s", {"rdid": rd.sourceId})
 
 	# We're committing here so that we don't lose all importing
