@@ -6,7 +6,7 @@ as possible.  Take care to give sensible error messages -- much can go wrong
 here, and it's nice if the user has a way to figure out what's wrong.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -21,8 +21,11 @@ import warnings
 
 import psycopg2
 
+# keep sqlsupport from initialisation before we've set up the profiles
+os.environ["GAVO_INIT_RUNNING"] = "1"
 from gavo import base
 from gavo import utils
+del os.environ["GAVO_INIT_RUNNING"]
 
 
 def bailOut(msg, hint=None):
@@ -38,7 +41,7 @@ def unindentString(s):
 
 def makeRoot():
 	rootDir = base.getConfig("rootDir")
-	if os.path.isdir(rootDir):
+	if rootDir.is_dir():
 		return
 	try:
 		os.makedirs(rootDir)
@@ -67,17 +70,13 @@ def makeDirVerbose(path, setGroupTo, makeWritable):
 				os.chown(path, -1, setGroupTo)
 				if makeWritable:
 					os.chmod(path, stats.st_mode | 0o060)
-			except Exception as msg:
-				bailOut("Cannot set %s to group ownership %s, group writable"%(
-					path, setGroupTo),
-					hint="Certain directories must be writable by multiple user ids."
-					"  They must therefore belong to the group %s and be group"
-					" writeable.  The attempt to make sure that's so just failed"
-					" with the error message %s."
-					"  Either grant the directory in question to yourself, or"
-					" fix permissions manually.  If you own the directory and"
-					" sill see permission errors, try 'newgrp %s'"%(
-						base.getConfig("group"), msg, base.getConfig("group")))
+			except Exception as ex:
+				# don't fail hard because something is wrong here; there
+				# *are* configurations in which things may still work.
+				sys.stderr.write(f"Cannot set {path} to group ownership {setGroupTo},"
+						f" group writable.\n{ex}\n"
+						"This will probably be trouble later.  Hence,"
+						" please fix it manually.\n")
 
 
 _GAVO_WRITABLE_DIRS = set([
@@ -94,7 +93,7 @@ def makeDirForConfig(configKey, gavoGrpId):
 
 
 def makeDefaultMeta():
-	destPath = os.path.join(base.getConfig("configDir"), "defaultmeta.txt")
+	destPath = base.getConfig("configDir") / "defaultmeta.txt"
 	if os.path.exists(destPath):
 		return
 	rawData = unindentString(r"""publisher: Your organisation's name
@@ -118,23 +117,31 @@ def makeDefaultMeta():
 		f.write(rawData)
 	
 	# load new new default meta
-	from gavo.base import config
-	config.makeFallbackMeta()
+	from gavo.base import meta
+	meta.makeFallbackMeta()
 
 
 def makeMatplotlibCfg():
-	destPath = os.path.join(base.getConfig("configDir"), "matplotlibrc")
-	if os.path.exists(destPath):
+	destPath = base.getConfig("configDir") / "matplotlibrc"
+	if destPath.exists():
 		return
 	with open(destPath, "w") as f:
-		f.write("backend: Agg\n")
+		f.write("backend: Agg\nsvg.fonttype: path\n")
+
+
+def makeAstropyCfg():
+	"""creates an astropy directory in configDir.
+
+	This should prevent astropy from trying to read from home once DaCHS
+	is properly up.
+	"""
+	destPath = base.getConfig("configDir") / "astropy"
+	makeDirVerbose(destPath, base.getGroupId(), True)
 
 
 def prepareWeb(groupId):
-	makeDirVerbose(os.path.join(base.getConfig("webDir"), "nv_static"),
-		groupId, False)
-	makeDirVerbose(os.path.join(base.getConfig("webDir"), "templates"),
-		groupId, False)
+	makeDirVerbose(base.getConfig("webDir") / "nv_static", groupId, False)
+	makeDirVerbose(base.getConfig("webDir") / "templates", groupId, False)
 
 
 def _genPW():
@@ -174,8 +181,8 @@ def makeProfiles(dsn, userPrefix=""):
 				userPrefix, _genPW())),
 			("untrustedquery", "include dsn\nuser = %suntrusted\npassword = %s\n"%(
 				userPrefix, _genPW())),]:
-		destPath = os.path.join(profilePath, fName)
-		if not os.path.exists(destPath):
+		destPath = profilePath / fName
+		if not destPath.exists():
 			with open(destPath, "w") as f:
 				f.write(content)
 
@@ -190,10 +197,10 @@ def createFSHierarchy(dsn, userPrefix=""):
 	for configKey in ["configDir", "inputsDir", "cacheDir", "logDir",
 			"tempDir", "webDir", "stateDir"]:
 		makeDirForConfig(configKey, grpId)
-	makeDirVerbose(os.path.join(base.getConfig("inputsDir"), "__system"),
-		grpId, False)
+	makeDirVerbose(base.getConfig("inputsDir") / "__system", grpId, False)
 	makeDefaultMeta()
 	makeMatplotlibCfg()
+	makeAstropyCfg()
 	makeProfiles(dsn, userPrefix)
 	prepareWeb(grpId)
 
@@ -412,17 +419,12 @@ def _importBasicResources():
 	for rdId in ["//dc_tables", "//services", "//users",
 			"//uws", "//adql", "//tap", "//products",
 			"//datalink"]:
+		print(f"({rdId})", end=" ", flush=True)
 		base.ui.notifyInfo("Importing %s"%rdId)
 		importing.process(rsc.getParseOptions(), [rdId])
+	print()
 	common._BOOTSTRAPPING = False
 
-	# We need to raise extra_float_digits; gavoadmin has sufficient
-	# privileges for doing that system-wide
-	with base.getWritableAdminConn() as conn:
-		for profileName in ["untrustedquery", "trustedquery", "admin"]:
-			profile = base.getDBProfile(profileName)
-			conn.execute("alter role %s set extra_float_digits=3"%profile.user)
-
 
 def initDB(dsn):
 	"""creates users and tables expected by DaCHS in the database described
@@ -434,11 +436,8 @@ def initDB(dsn):
 	_doLocalSetup(dsn)
 	_readDBScripts(dsn)
 
-	# Un-init the db connection to re-read any functions that may
-	# have been created by anything up to here.
 	from gavo.base import sqlsupport
-	sqlsupport._PSYCOPG_INITED = False
-	sqlsupport._initPsycopg()
+	sqlsupport.initPsycopg()
 	_importBasicResources()
 
 
@@ -457,14 +456,26 @@ def parseCommandLine():
 	parser.add_argument("--nodb", help="Inhibit initialization of the"
 		" database (you may want to use this when refreshing the file system"
 		" hierarchy)", action="store_false", dest="initDB")
+	parser.add_argument("--debug", help="Dump a traceback when init"
+		"fails", action="store_true", dest="showTB")
+
 	return parser.parse_args()
 
 
 def main():
 	"""initializes the DaCHS environment (where that's not already done).
 	"""
-	opts = parseCommandLine()
-	dsn = DSN(opts.dsn)
-	createFSHierarchy(dsn)
-	if opts.initDB:
-		initDB(dsn)
+	# do all error handling here; you cannot rely on the rest of DaCHS
+	# when init hasn't run.
+	try:
+		opts = parseCommandLine()
+		dsn = DSN(opts.dsn)
+		createFSHierarchy(dsn)
+		if opts.initDB:
+			initDB(dsn)
+	except Exception as exc:
+		sys.stderr.write(f"*** DaCHS init failed: {exc}.\n")
+		if opts.showTB:
+			import traceback; traceback.print_exc()
+		else:
+			sys.stderr.write("Pass --debug for a traceback.\n")
