@@ -12,7 +12,7 @@ commented RD templates, fills out a thing or two and leaves the rest to
 the operator.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -58,14 +58,14 @@ def getTypeForFTFormat(format, name):
 	typeCode = None
 
 	# variable-length array FITS codes
-	mat = re.match("(\d*)P(.)\(\d*\)$", format)
+	mat = re.match(r"(\d*)P(.)\(\d*\)$", format)
 	if mat:
 		typeCode = mat.group(2)
 		arrSize = "100"  # just a sentinel
 
 	# "conventional FITS codes
 	else:
-		mat = re.match("(\d*)(.)$", format)
+		mat = re.match(r"(\d*)(.)$", format)
 		if mat:
 			arrSize, typeCode = mat.group(1), mat.group(2)
 
@@ -90,7 +90,7 @@ ignoredFITSHeaders = set(["COMMENT", "SIMPLE", "BITPIX", "EXTEND",
 	"NEXTEND", "SOFTNAME", "SOFTVERS", "SOFTDATE", "SOFTAUTH", "SOFTINST",
 	"HISTORY", "BZERO", "BSCALE", "DATAMIN", "DATAMAX"])
 wcsKey = re.compile("CD.*|CRVAL.*|CDELT.*|NAXIS.*|CRPIX.*|CTYPE.*|CUNIT.*"
-	"|CROTA.*|RADECSYS|AP?_\d_\d|BP?_\d_\d|LATPOLE|LONPOLE")
+	r"|CROTA.*|RADECSYS|AP?_\d_\d|BP?_\d_\d|LATPOLE|LONPOLE")
 
 
 def isIgnoredKeyword(kw):
@@ -102,39 +102,6 @@ def isIgnoredKeyword(kw):
 	return kw in ignoredFITSHeaders or wcsKey.match(kw)
 
 
-def structToETree(aStruct):
-	"""returns an ElementTree for the copyable content of aStruct.
-
-	Note that due to manipulations at parse time and non-copyable content,
-	this will, in general, not reproduce the original XML trees.
-	"""
-	nodeStack = [ElementTree.Element(aStruct.name_)]
-	for evType, elName, value in aStruct.iterEvents():
-		try:
-			if evType=="start":
-				nodeStack.append(ElementTree.SubElement(nodeStack[-1], elName))
-			elif evType=="end":
-				nodeStack.pop()
-			elif evType=="value":
-				if value is None or value is base.NotGiven:
-					continue
-				if elName=="content_":
-					nodeStack[-1].text = value
-				else:
-					if not isinstance(value, str):
-						# TODO: figure out if something is a reference by inspecting
-						# the attribute definition; meanwhile, just assume it is:
-						value = value.id
-					nodeStack[-1].set(elName, value)
-			else:
-				raise base.Error("Invalid struct event: %s"%evType)
-		except:
-			base.ui.notifyError("Badness occurred in element %s, event %s,"
-				" value %s\n"%(elName, evType, value))
-			raise
-	return nodeStack[-1]
-
-
 def makeTableFromFT(rd, srcName, opts):
 	from gavo.utils import pyfits
 	cols, nameMaps = [], {}
@@ -354,7 +321,7 @@ def main():
 	opts, args = parseCommandLine()
 	rd = makeRD(args, opts)
 	rd._metaAttr.copyable = True
-	eTree = structToETree(rd)
+	eTree = base.structToETree(rd)
 	writePrettyPrintedXML(eTree)
 
 # ======================= End deprecated ui and implementation ===========
@@ -379,12 +346,22 @@ def iterColAttrsFITS(table):
 		yield dict((k,v) for k,v in colMeta if v is not None)
 
 
-def iterColAttrsVOTable(inFile):
+def iterColAttrsVOTable(inFile, tableInd):
 	"""tries to parse (binary) inFile as a VOTable and yields colAttrs
 	dictionaries for all FIELDs in it.
 	"""
-	for el in votable.parse(inFile, watchset=[votable.V.FIELD]):
-		if isinstance(el, votable.V.FIELD):
+	yielding = tableInd==1
+
+	for el in votable.parse(inFile, watchset=[votable.V.TABLE, votable.V.FIELD]):
+		if isinstance(el, votable.V.TABLE):
+			if yielding:
+				break  # done parsing our table
+			else:
+				tableInd -= 1
+				if tableInd==1:
+					yielding = True
+
+		if yielding and isinstance(el, votable.V.FIELD):
 			colMeta = [
 				("name", el.name),
 				("unit", el.unit),
@@ -434,7 +411,7 @@ class VizBBBParser(object):
 			self.state = "bbb_colhead"
 			self._parse_bbb_colhead(ln, mat)
 
-		elif self.tableBorderPat.match(ln) or re.match("\s*Bytes", ln):
+		elif self.tableBorderPat.match(ln) or re.match(r"\s*Bytes", ln):
 			pass
 
 		else:
@@ -585,6 +562,10 @@ def parseGenColCommandLine():
 		choices=["fits", "vot", "viz"],
 		help="Format of the input table: FITS binary, VOTable, or Vizier"
 			" byte-by-byte.")
+	parser.add_argument("-x", "--extension", dest="extension", action="store",
+		type=int, default=1,
+		help="Extension to work on, for VOTables the table to work on."
+			" In VOTables, counting starts at one.")
 		
 	return parser.parse_args()
 
@@ -598,7 +579,7 @@ def gencol():
 
 	if args.table.endswith(".fits") or args.format=="fits":
 		try:
-			table = pyfits.open(args.table)[1]
+			table = pyfits.open(args.table)[args.extension]
 			colIterator = iterColAttrsFITS(table)
 		except Exception as exc:
 			base.ui.notifyError(f"Cannot open {args.table} as a FITS binary table: "
@@ -610,7 +591,7 @@ def gencol():
 			or args.format=="vot"):
 		try:
 			with open(args.table, "rb") as f:
-				colIterator = list(iterColAttrsVOTable(f))
+				colIterator = list(iterColAttrsVOTable(f, args.extension))
 		except Exception as exc:
 			base.ui.notifyError(f"Cannot open {args.table} as a VOTable: "
 				+str(exc))
