@@ -2,7 +2,7 @@
 A wrapper script suitable for starting the server.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -39,6 +39,12 @@ from gavo.utils import exposedFunction, makeCLIParser, Arg
 from gavo.web import root
 
 
+def onSystemd():
+	"""returns true if it's likely that our init is a systemd.
+	"""
+	return os.path.realpath("/sbin/init").endswith("systemd")
+
+
 def setupServer(rootPage, runIntervalJobs=True):
 	"""sets the process up for running a server.
 
@@ -76,13 +82,13 @@ class _PIDManager(object):
 	There's a single instance of this below.
 	"""
 	def __init__(self):
-		self.path = os.path.join(base.getConfig("stateDir"), "web.pid")
+		self.path = base.getConfig("stateDir") / "web.pid"
 	
 	def getPID(self):
 		"""returns the PID of the currently running server, or None.
 		"""
 		try:
-			with open(self.path) as f:
+			with self.path.open() as f:
 				pidString = f.readline()
 		except IOError: # PID file does not exist (or we're beyond repair)
 			return None
@@ -102,7 +108,7 @@ class _PIDManager(object):
 		people won't start two competing daemons.
 		"""
 		try:
-			with open(self.path, "w") as f:
+			with self.path.open("w") as f:
 				f.write(str(os.getpid()))
 		except IOError: # Cannot write PID.  This would suggest that much else
 		                # is broken as well, so we bail out
@@ -114,7 +120,7 @@ class _PIDManager(object):
 		"""removes the PID file.
 		"""
 		try:
-			os.unlink(self.path)
+			self.path.unlink()
 		except os.error as ex:
 			if ex.errno==2: # ENOENT, we don't have to do anything
 				pass
@@ -131,6 +137,8 @@ def _reloadConfig():
 	"""should clear as many caches as we can get hold of.
 	"""
 	base.caches.clearCaches()
+	from gavo.rscdef.common import getIvoidsForRDId
+	getIvoidsForRDId.cache_clear()
 
 	root.loadUserVanity(root.ArchiveService)
 	config.makeFallbackMeta(reload=True)
@@ -220,37 +228,40 @@ def daemonize(logFile, callable):
 		os._exit(0)
 
 
-def _configureTwistedLog(theLog=None):
+def _configureTwistedLog(additionalLog=None):
 	"""sets up the destination of the twisted web.log.
 
-	theLog can be a file to write to rather than the default web.log.  No
+	additionalLog can be a file to write to in addition to web.log.  No
 	rotation will take place on that.
 
 	Note that dcErrors and dcInfos are set up in user.logui.
 	"""
-	if theLog is None:
-		# create the file with the right permissions before passing it on
-		logFile = getLogFile("web.log")
-		logPath = logFile.name
-		logFile.close()
+	# create the file with the right permissions before passing it on
+	logFile = getLogFile("web.log")
+	logPath = logFile.name
+	logFile.close()
 
-		theLog = logfile.LogFile(
-			os.path.basename(logPath),
-			os.path.dirname(logPath))
+	theLog = logfile.LogFile(
+		os.path.basename(logPath),
+		os.path.dirname(logPath))
+
+	log.startLogging(theLog, setStdout=False)
+	def rotator():
+		theLog.shouldRotate()
+		reactor.callLater(86400, rotator)
+	rotator()
+
+	if additionalLog is not None:
+		from twisted.logger import globalLogPublisher, textFileLogObserver
+		observer = textFileLogObserver(additionalLog)
+		globalLogPublisher.addObserver(observer)
 
-		log.startLogging(theLog, setStdout=False)
-		def rotator():
-			theLog.shouldRotate()
-			reactor.callLater(86400, rotator)
-		rotator()
-	else:
-		log.startLogging(theLog, setStdout=False)
 
 
 def getLogFile(baseName):
 	"""returns a log file group-writable by gavo.
 	"""
-	fName = os.path.join(base.getConfig("logDir"), baseName)
+	fName = base.getConfig("logDir") / baseName
 	f = open(fName, "a")
 	try:
 		os.chmod(fName, 0o664)
@@ -306,8 +317,6 @@ class _Scheduler(object):
 		"""puts job on the reactor's queue for execution in wakeTime seconds.
 		"""
 		if cls.lastDelayedCall is not None and cls.lastDelayedCall.active():
-			base.ui.notifyWarning("Cancelling schedule at %s"%
-				cls.lastDelayedCall.getTime())
 			cls.lastDelayedCall.cancel()
 
 		cls.lastDelayedCall = reactor.callLater(wakeTime, job)
@@ -373,8 +382,7 @@ def _perhapsEnableSSL(factory, sslPort=443):
 	If anything goes wrong here, we're just emitting a diagnostic and don't
 	fail.
 	"""
-	certPath = os.path.join(
-		base.getConfig("rootDir"), "hazmat", "bundle.pem")
+	certPath = base.getConfig("rootDir") / "hazmat" / "bundle.pem"
 	if not os.path.isfile(certPath):
 		return
 
@@ -407,7 +415,7 @@ def _startServer(managePID=True):
 			" got lost somehow.  It's more likely that some"
 			" other server is already taking up this port; you may want to change"
 			" the [web] serverPort setting in that case.")
-	os.chdir(base.getConfig("statedir"))
+	os.chdir(base.getConfig("stateDir"))
 	_dropPrivileges()
 	root.site.webLog = _configureTwistedLog()
 
@@ -437,6 +445,10 @@ def _startServer(managePID=True):
 	)
 def start(args):
 	if not getattr(args, "foreground", False):
+		if onSystemd():
+			raise base.ReportableError("Do not use dachs serve start on systemd"
+				" machines.  Say ``systemctl start dachs`` instead.")
+
 		oldPID = PIDManager.getPID()
 		if oldPID is not None:  # Server could already be running,.. .
 			if os.path.exists(f"/proc/{oldPID}"):
@@ -477,6 +489,10 @@ def _waitForServerExit(timeout=5):
 
 
 def _stopServer():
+	if onSystemd():
+		raise base.ReportableError("Do not use dachs serve stop on systemd"
+			" machines.  Say ``systemctl stop dachs`` instead.")
+
 	pid = PIDManager.getPID()
 	if pid is None:  # No server running, nothing to do
 		base.ui.notifyWarning("No running DaCHS server found."
@@ -538,6 +554,11 @@ def restart(args):
 
 @exposedFunction(help="reload server configuration (incomplete)")
 def reload(args):
+	if onSystemd():
+		raise base.ReportableError("dachs serve reload does"
+			" not work on systemd machines.  Use"
+			" ``systemctl restart dachs`` instead.")
+
 	pid = PIDManager.getPID()
 	if pid is None:
 		raise base.ReportableError("No DaCHS server appears to be running."
@@ -599,7 +620,7 @@ def updateCertificate(args):
 	from gavo.web import ifpages
 	acmeChallengeDir = ifpages.ACMEChallenge.acmeChallengeDir
 
-	os.chdir(os.path.join(base.getConfig("rootDir"), "hazmat"))
+	os.chdir(base.getConfig("rootDir") / "hazmat")
 
 	# to make a CSR for the host names we're known as, first write an
 	# openssl cfg...
