@@ -13,13 +13,12 @@ upgrade does everything required to bring the what's in the database in sync
 with the code (or so I hope).
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
-import os
 import re
 import sys
 
@@ -32,7 +31,7 @@ from gavo import rscdesc  #noflake: for cache registration
 from gavo import utils
 
 
-CURRENT_SCHEMAVERSION = 34
+CURRENT_SCHEMAVERSION = 36
 
 
 class _COMMIT(object):
@@ -60,26 +59,9 @@ def _updateTAP_SCHEMA(connection):
 	"""re-ingests TAP_SCHEMA metadata for all RDs already mentioned
 	in TAP_SCHEMA.
 	"""
-	from gavo.protocols import tap
-
-	toDo = [r[0] for r in
-		connection.query(
-			"SELECT DISTINCT sourceRD FROM tap_schema.tables")]
-
-	for rdId in toDo:
-		if rdId=="__system__/tap":
-			# TAP_SCHEMA needs special handling because of bootstrapping
-			tap._insertRDIntoTAP_SCHEMA(base.caches.getRD(rdId), connection)
-			continue
-
-		try:
-			tap.publishToTAP(
-				base.caches.getRD(rdId, doQueries=False), connection)
-		except Exception as msg:
-			base.ui.notifyWarning("RD %s couldn't be loaded or ingested to"
-				" TAP_SCHEMA (%s).  Fix and run dachs imp -m on it to have"
-				" up-to-date metadata in TAP_SCHEMA"%(rdId, msg))
-			continue
+	rsc.makeData(
+		base.resolveCrossId("//tap#refresh"),
+		connection=connection)
 
 
 def iterEPNTAPTables(connection):
@@ -907,7 +889,7 @@ class To32Upgrader(Upgrader):
 
 	@classmethod
 	def u_10_update_obscore_defs(cls, connection):
-		"""updating obscore definitions to use reals for spectral"""
+		"""updating obscore definitions to use reals for em_x"""
 		if relationExists("ivoa._obscoresources", connection):
 			dd = base.caches.getRD("//obscore", doQueries=False
 				).getById("refreshAfterSchemaUpdate")
@@ -939,6 +921,74 @@ class To34Upgrader(Upgrader):
 			connection=connection, runCommit=False)
 
 
+class To35Upgrader(Upgrader):
+	version = 34
+	@classmethod
+	def s_000_add_rank_column(cls, connection):
+		"""add rank column to the resource list"""
+		if "rank" not in getColumnNamesFor("//services#resources", connection):
+			connection.execute("ALTER TABLE dc.resources"
+				" ADD COLUMN rank INTEGER")
+
+	@classmethod
+	def s_010_refresh_resources_join(cls, connection):
+		"""refreshing the dc.resources_join view"""
+		rsc.makeData(
+			base.resolveCrossId("//services#views"),
+			connection=connection)
+
+		# we will rely on the upsert trigger, and some versions of DaCHS
+		# did this in a way that is no longer reliable.  So: update
+		# this at least on //services#resources before going on
+		rsc.TableForDef(
+			base.resolveCrossId("//services#resources"),
+			connection=connection).createUniquenessRules()
+
+	@classmethod
+	def u_000_fill_rank(cls, connection):
+		"""filling rank column for published RDs"""
+		from gavo.registry import publication
+		publication.updateServiceList(
+			publication.getRDs(
+				publication.findPublishedRDs()),
+			keepTimestamp=True)
+
+
+class To36Upgrader(Upgrader):
+	version = 35
+	@classmethod
+	def u_000_fitsInProducts(cls, connection):
+		"""updating FITS media type in dc.products"""
+		connection.execute("update dc.products set mime='application/fits'"
+			" where mime='image/fits'")
+	
+	@classmethod
+	def u_010_fitsInInstanceTables(cls, connection):
+		"""updating FITS media type in products tables"""
+		for rdId in registry.findPublishedRDs():
+			try:
+				rd = base.caches.getRD(rdId)
+			except:
+				showProgress(f"<{rdId} broken, skipping>")
+
+			for table in rd.tables:
+				if not table.onDisk or table.viewStatement:
+					continue
+
+				tableName = table.getQName()
+				if not relationExists(tableName, connection):
+					continue
+
+				if "mime" in table:
+					showProgress(" "+table.getQName())
+					# it's surprisingly hard to figure out whether a table mixes
+					# in //products#table because mixesIn does not work
+					# transitively.  Having a column mime, however, ought to
+					# be about as good.
+					connection.execute(f"update {table.getQName()}"
+						" set mime='application/fits' where mime='image/fits'")
+
+
 def iterStatements(startVersion, endVersion=CURRENT_SCHEMAVERSION,
 		upgraders=None):
 	"""yields all upgraders from startVersion to endVersion in sequence.
@@ -948,7 +998,7 @@ def iterStatements(startVersion, endVersion=CURRENT_SCHEMAVERSION,
 			upgraders or list(globals().values())):
 		if startVersion<=upgrader.version<endVersion:
 			toRun.append(upgrader)
-	toRun.sort(key=lambda upgrader:upgrader.version)
+	toRun.sort(key=lambda upgrader: upgrader.version)
 
 	schemaUpgraded = False
 	for upgrader in toRun:
@@ -1009,11 +1059,11 @@ def upgrade(forceDBVersion=None):
 def clearThirdPartyCache():
 	from gavo.web import ifpages
 	tpcDir = ifpages.ThirdPartyCachePage.cacheDir
-	if os.path.isdir(tpcDir):
-		for fName in os.listdir(tpcDir):
-			path = os.path.join(tpcDir)
-			if os.path.isfile(path):
-				os.unlink(path)
+	if tpcDir.is_dir():
+		for fName in tpcDir.iterdir():
+			path = tpcDir / fName
+			if path.is_file():
+				path.unlink()
 
 
 def parseCommandLine():
