@@ -12,7 +12,7 @@ to anything expensive, they're advised to have something like::
 		(don't do the expensive thing)
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -298,65 +298,82 @@ def validateOtherCode(rd, args):
 	return retval
 
 
+def validateTable(td, compareDB):
+	"""sanity-checks the table described by td.
+
+	It is not an error if the table does not exist.  The function will
+	yield diagnostics as ([warning, error], message).
+	"""
+	tn = td.getQName()
+	identifierSymbol = adql.getSymbols()["identifier"]
+
+	try:
+		base.validateStructure(td)
+	except api.MetaValidationError as ex:
+		yield "error", f"Missing metadata in table {td.id}:\n{ex}"
+		return
+
+	for col in td:
+		try:
+			if col.unit:
+				parsedUnit = api.parseUnit(col.unit)
+				if not parsedUnit.isValid:
+					yield ("warning",
+						f"Column {tn}.{col.name}: Unit {col.unit} is"
+							" not interoperable (in VOUnit's list of recognised units)")
+					
+		except api.BadUnit:
+			yield "error", f"Bad unit for table {tn}.{col.name}: {col.unit}"
+
+		if col.ucd and not col.ucd in UCD_WHITELIST:
+			try:
+				astropyucd.parse_ucd(col.ucd,
+					check_controlled_vocabulary=True, has_colon=True)
+			except ValueError as msg:
+				yield ("warning",
+					f"Column {tn}.{col.name}: UCD {col.ucd}"
+					f" not accepted by astropy ({msg}).")
+
+		try:
+			identifierSymbol.parseString(str(col.name), parseAll=True)
+		except base.ParseException:
+			yield ("warning", "Column %s.%s: Name is not a regular"
+				" ADQL identifier."%(td.id, col.name))
+
+	if td.onDisk and compareDB:
+		with base.getTableConn() as conn:
+			q = base.UnmanagedQuerier(conn)
+			if q.getTableType(tn) is not None:
+				tapSchemaColumns = None
+				if td.adql:
+					tapSchemaColumns = dict((r["column_name"], r)
+						for r in conn.queryToDicts(
+						"SELECT column_name, unit, ucd, datatype, arraysize, xtype"
+						" FROM TAP_SCHEMA.columns"
+						" WHERE table_name=%(table_name)s",
+						{"table_name": tn}))
+
+				t = api.TableForDef(td, connection=conn)
+				try:
+					t.ensureOnDiskMatches(tapSchemaColumns)
+				except api.DataError as msg:
+					yield "error", utils.makeEllipsis(str(msg), 160)
+
+
 def validateTables(rd, args):
 	"""does some sanity checks on the (top-level) tables within rd.
 	"""
 	valid = True
 
-	identifierSymbol = adql.getSymbols()["identifier"]
-
 	for td in rd.tables:
-		curTableName = td.getQName()
-
-		try:
-			base.validateStructure(td)
-		except api.MetaValidationError as ex:
-			valid = False
-			outputError(rd.sourceId, "Missing metadata in"
-				" table %s:\n%s"%(td.id, str(ex)))
-			continue # further checks will just add verbosity
-
-		for col in td:
-			try:
-				if col.unit:
-					parsedUnit = api.parseUnit(col.unit)
-					if not parsedUnit.isValid:
-						outputWarning(rd.sourceId,
-							f"Column {curTableName}.{col.name}: Unit {col.unit} is"
-							" not interoperable (in VOUnit's list of recognised units)")
-					
-			except api.BadUnit:
+		for type, msg in validateTable(td, args.compareDB):
+			if type=="error":
 				valid = False
-				outputError(rd.sourceId, "Bad unit in table %s, column %s: %s"%(
-					curTableName, col.name, repr(col.unit)))
-
-			if col.ucd and not col.ucd in UCD_WHITELIST:
-				try:
-					astropyucd.parse_ucd(col.ucd,
-						check_controlled_vocabulary=True,
-						has_colon=True)
-				except ValueError as msg:
-					outputWarning(rd.sourceId,
-						f"Column {curTableName}.{col.name}: UCD {col.ucd}"
-						f" not accepted by astropy ({msg}).")
-
-			try:
-				identifierSymbol.parseString(str(col.name), parseAll=True)
-			except base.ParseException:
-				outputWarning(rd.sourceId, "Column %s.%s: Name is not a regular"
-					" ADQL identifier."%(td.id, col.name))
-
-		if td.onDisk and args.compareDB:
-			with base.getTableConn() as conn:
-				q = base.UnmanagedQuerier(conn)
-				if q.getTableType(curTableName) is not None:
-					t = api.TableForDef(td, connection=conn)
-					try:
-						t.ensureOnDiskMatches()
-					except api.DataError as msg:
-						valid = False
-						outputError(rd.sourceId,
-							utils.makeEllipsis(utils.safe_str(msg), 160))
+				outputError(rd.sourceId, msg)
+			elif type=="warning":
+				outputWarning(rd.sourceId, msg)
+			else:
+				raise NotImplementedError(f"No such message type: {type}")
 
 		# associated datalink services and the columns must exist.
 		for dldef in td.iterMeta("_associatedDatalinkService"):
@@ -364,7 +381,6 @@ def validateTables(rd, args):
 				_ = base.resolveId(td.rd,
 					base.getMetaText(dldef, "serviceId"))
 				_ = td.getColumnByName(base.getMetaText(dldef, "idColumn"))
-				_
 
 			except (base.NotFoundError, base.MetaError) as msg:
 				valid = False
