@@ -5,7 +5,7 @@ This comprises helpers and wrappers that do not need gavo.base but for some
 reason or another should be within the dc package.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -83,7 +83,7 @@ from gavo.utils.texttricks import (formatSize, formatFloat,
 	floatRE, dateRE, datetimeRE, identifierPattern, looksLikeURLPat,
 	datetimeToRFC2616,
 	parseDefaultDatetime, parseDefaultDate, parseDefaultTime,
-	parseAccept,
+	parseMediaType, parseAccept,
 	isoTimestampFmt, isoTimestampFmtNoTZ, parseISODT, formatISODT,
 	formatRFC2616Date, parseRFC2616Date, roundToSeconds,
 	getFileStem,
