@@ -6,7 +6,7 @@ constructors.  The Autonodes are used when building something like abstract
 syntax trees in STC or ADQL.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -63,7 +63,7 @@ class AutoNodeType(type):
 
 
 class AutoNode(object, metaclass=AutoNodeType):
-	"""An AutoNode.
+	"""A node in a tree with automatic attributes.
 
 	AutoNodes are explained in AutoNode's metaclass, AutoNodeType.
 
@@ -89,7 +89,7 @@ class AutoNode(object, metaclass=AutoNodeType):
 			"%s=%s"%(name, repr(val))
 			for name, val in self.iterAttributes(skipEmpty=True)))
 
-	def change(self, **kwargs):
+	def change(self, **kwargs) -> 'AutoNode':
 		"""returns a shallow copy of self with constructor arguments in kwargs
 		changed.
 		"""
