@@ -3,11 +3,12 @@ Functions dealing with compilation and introspection of python and
 external code.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+
 from __future__ import annotations
 
 import ast
@@ -117,8 +118,7 @@ class DeferredImport:
 		utils.DeferredImport("wcs", "from astropy import wcs")
 	
 	As a service for static code checking, you'll usually want to repeat
-	the module name, though:
-
+	the module name, though::
 
 		wcs = utils.DeferredImport("wcs", "from astropy import wcs")
 	"""
@@ -694,6 +694,7 @@ def loadPythonModule(fqName: Filename, relativeTo: Optional[Filename]=None
 	This is to keep Python from using the module when someone writes
 	import mod.
 	"""
+	fqName = str(fqName)
 	if relativeTo is not None:
 		fqName = os.path.join(os.path.dirname(relativeTo), fqName)
 
