@@ -7,14 +7,14 @@ This also lets us smuggle in fallbacks for python 3.7 (and we won't go
 further back than that).
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
 from typing import ( #noflake: gratuitous imports for re-exporting
-	Any,
+	AbstractSet, Any,
 	BinaryIO,
 	Callable, ContextManager,
 	Dict,
@@ -35,12 +35,14 @@ from typing import ( #noflake: gratuitous imports for re-exporting
 
 from types import ModuleType  #noflake gratuitous imports for re-exporting
 
+import pathlib
+
 try:
 	from numpy.typing import NDArray
 except ImportError:
 	NDArray = NewType('NDArray', object)  # type: ignore
 
-Filename = str
+Filename = Union[str, pathlib.Path]
 # positions when doing XML parsing
 ParsePosition =  NewType('ParsePosition', object)
 StrOrBytes = Union[str, bytes]
