@@ -42,7 +42,7 @@ For examples of how this is used, see pftf (http://www.tfiu.de/pftf)
 or DaCHS (http://soft.g-vo.org/dachs)
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -52,6 +52,7 @@ import codecs
 import configparser
 import re
 import os
+import pathlib
 import tempfile
 import warnings
 import weakref
@@ -469,13 +470,19 @@ class PathConfigItem(StringConfigItem):
 			if mat.group(1)=='$':
 				return '$'
 			else:
-				return self.parent.get(mat.group(1))
+				return str(self.parent.get(mat.group(1)))
+
 		res = []
 		for p in self._value:
+			if isinstance(p, pathlib.Path):
+				p = str(p)
+
 			if p.startswith("~"):
 				p = os.environ.get("HOME", "")+p[1:]
+
 			if '$' in p:
 				p = re.sub(r"\$(\w+)", resolveReference, p)
+
 			res.append(p)
 		return res
 
@@ -501,11 +508,13 @@ class PathRelativeConfigItem(StringConfigItem):
 	def _getValue(self):
 		if self._value is None:
 			return None
-		if self._value.startswith("~"):
-			return os.environ.get("HOME", "/no_home")+self._value[1:]
+		if str(self._value).startswith("~"):
+			return pathlib.Path(
+				os.environ.get("HOME", "/no_home")+self._value[1:])
 		if self.baseKey:
-			return os.path.join(self.parent.get(self.baseKey), self._value)
-		return self._value
+			return pathlib.Path(
+				os.path.join(self.parent.get(self.baseKey), self._value))
+		return pathlib.Path(self._value)
 	
 	def _setValue(self, val):
 		self._value = val
@@ -520,7 +529,7 @@ class ExpandedPathConfigItem(StringConfigItem):
 		val = StringConfigItem._parse(self, value)
 		if val is not None:
 			val = os.path.expanduser(val)
-		return val
+		return pathlib.Path(val)
 
 
 class _Undefined(object):
