@@ -5,12 +5,11 @@ It's unclear if pyfits coming from astropy is now threadsafe, but I'm hoping
 it might be, so for now I'm making fitslock a no-op.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
-import builtins
 import datetime
 import itertools
 import re
@@ -20,8 +19,6 @@ import warnings
 from contextlib import contextmanager
 
 import numpy
-# temporary measure to shut up astropy's configuration parser.
-builtins._ASTROPY_SETUP_ = True  # type:ignore
 # the following is the original import of pyfits, and only it should be used
 from astropy.io import fits as pyfits # type:ignore
 from astropy.utils.exceptions import AstropyDeprecationWarning # type:ignore
@@ -213,6 +210,7 @@ def replacePrimaryHeaderInPlace(
 
 	gzipped inputs used to be supported here, but they aren't any more.
 	"""
+	fitsName = str(fitsName)
 	if fitsName.endswith(".gz"):
 		raise NotImplementedError("replacePrimaryHeaderInPlace no longer"
 			" supports gzipped files.")
@@ -802,6 +800,19 @@ class WCSAxis(object):
 		limits = self.pixToPhys(1), self.pixToPhys(self.axisLength)
 		return min(limits), max(limits)
 
+	@staticmethod
+	def _bailIfInseparable(header, axisIndex):
+		"""raises a ValueError if axisIndex appears to be entangled
+		with another axis in header.
+		"""
+		headerPattern = re.compile(rf"(PC|CD){axisIndex}_(\d)")
+		entangledWith = {int(headerPattern.match(key).group(2))
+			for key in header
+			if headerPattern.match(key)}
+		if entangledWith and entangledWith!={axisIndex}:	
+			raise ValueError("FITS axis %s appears not separable.  WCSAxis"
+				" cannot handle this."%axisIndex)
+
 	@classmethod
 	def fromHeader(cls,
 			header: pyfits.Header,
@@ -815,11 +826,8 @@ class WCSAxis(object):
 		The axisIndex is 1-based; to get a transform for the axis described
 		by CTYPE1, pass 1 here.
 		"""
-		if ("CD%d_%d"%(axisIndex, axisIndex) in header
-				or "PC%d_%d"%(axisIndex, axisIndex) in header):
-			if not forceSeparable:
-				raise ValueError("FITS axis %s appears not separable.  WCSAxis"
-					" cannot handle this."%axisIndex)
+		if not forceSeparable:
+			cls._bailIfInseparable(header, axisIndex)
 
 		def get(key, default):
 			return header.get("%s%d"%(key, axisIndex), default)
