@@ -6,7 +6,7 @@ by utils.__init__.   Hence, none of these functions are in gavo.api or
 gavo.utils.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -25,8 +25,8 @@ def _normalizeForImage(pixels: NDArray, gamma: float) -> NDArray:
 	"""
 	pixels = numpy.flipud(pixels)
 	pixMax, pixMin = numpy.max(pixels), numpy.min(pixels)
-	return numpy.asarray(numpy.power(
-		(pixels-pixMin)/(pixMax-pixMin), gamma)*255, 'uint8')
+	pixels = numpy.power((pixels-pixMin)/(pixMax-pixMin), gamma)*255
+	return numpy.asarray(pixels, numpy.uint8)
 
 
 def jpegFromNumpyArray(pixels: NDArray, gamma: float=0.25) -> bytes:
