@@ -1,7 +1,7 @@
 """
 OS abstractions and related.
 
-This module contains, in partiular, the interface for having "easy subcommands"
+This module contains, in particular, the interface for having "easy subcommands"
 using argparse.  The idea is to use the exposedFunction decorator on functions
 that should be callable from the command line as subcommands; the functions
 must all have the same signature. For example, if they all took the stuff
@@ -14,7 +14,7 @@ To specify the command line arguments to the function, use Args.  See
 admin.py for an example.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -22,6 +22,7 @@ admin.py for an example.
 
 import argparse
 import contextlib
+import grp
 import os
 import sys
 import tempfile
@@ -226,13 +227,14 @@ def cat(srcF: IO, destF: IO, chunkSize: int=1<<20) -> None:
 
 
 def ensureDir(dirPath: str, *,
-			mode: Optional[int]=None, setGroupTo: Optional[int]=None) -> None:
+			mode: Optional[int]=None,
+			setGroupTo: Optional[Union[str,int]]=None) -> None:
 	"""makes sure that dirPath exists and is a directory.
 
 	If dirPath does not exist, it is created, and its permissions are
 	set to mode with group ownership setGroupTo if those are given.
 
-	setGroupTo must be a numerical gid if given.
+	If given, setGroupTo can be either numerical gid or a group name.
 
 	This function may raise all kinds of os.errors if something goes
 	wrong.  These probably should be handed through all the way to the
@@ -245,6 +247,8 @@ def ensureDir(dirPath: str, *,
 	if mode is not None:
 		os.chmod(dirPath, mode)
 	if setGroupTo:
+		if isinstance(setGroupTo, str):
+			setGroupTo = grp.getgrnam(setGroupTo).gr_gid
 		os.chown(dirPath, -1, setGroupTo)
 
 
