@@ -7,17 +7,20 @@ and you're done.
 All native representation is in rad.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
+from __future__ import annotations
+
 import io
 import math
 import re
 import tempfile
 
+from astropy import units as u  # type: ignore
 import numpy
 
 from gavo.utils import codetricks
@@ -28,9 +31,16 @@ from gavo.utils import texttricks
 from gavo.utils.fitstools import pyfits
 from gavo.utils.mathtricks import DEG
 
+from gavo.utils.dachstypes import (Any, Dict, List, Optional, Sequence,
+	Union, Tuple, TYPE_CHECKING, cast)
+if TYPE_CHECKING:
+	Component = Union[float, u.Quantity]
+	from astropy.coordinates import ( # type: ignore
+		UnitSphericalRepresentation)
+	
 
-_TRAILING_ZEROES = re.compile("0+(\s|$)")
-def removeTrailingZeroes(s):
+_TRAILING_ZEROES = re.compile(r"0+(\s|$)")
+def removeTrailingZeroes(s: str) -> str:
 	"""remove zeroes in front of whitespace or the string end.
 
 	This is used for cosmetics in STC-S strings.
@@ -41,21 +51,6 @@ def removeTrailingZeroes(s):
 	return _TRAILING_ZEROES.sub(r"\1", s)
 
 
-class TwoSBoxes(excs.ExecutiveAction):
-	"""is raised when an SBox is constructed from center and size such that
-	it overlaps the pole.
-	"""
-	def __init__(self, box1, box2):
-		self.box1, self.box2 = box1, box2
-
-
-def _query(conn, query, pars=None):
-	c = conn.cursor()
-	c.execute(query, pars)
-	res = list(c)
-	return res
-
-
 class PgSAdapter(object):
 	"""A base class for objects adapting pgSphere objects.
 
@@ -79,9 +74,10 @@ class PgSAdapter(object):
 	:asSODA(self):
 		returns a representation of the geometry as used in SODA parameters.
 	"""
-	pgType = None
+	pgType: Optional[str] = None
+	checkedAttributes: List[str] = []
 
-	def __eq__(self, other):
+	def __eq__(self, other: Any):
 		if self.__class__!=other.__class__:
 			return False
 		for attName in self.checkedAttributes:
@@ -89,7 +85,7 @@ class PgSAdapter(object):
 				return False
 		return True
 
-	def __ne__(self, other):
+	def __ne__(self, other: Any):
 		return not self==other
 
 	def __repr__(self):
@@ -123,58 +119,71 @@ class SPoint(PgSAdapter):
 	checkedAttributes = ["x", "y"]
 	pattern = re.compile(r"\s*\(\s*([0-9.eNa-]+)\s*,\s*([0-9.eNa-]+)\s*\)")
 
-	def __init__(self, x, y):
-		self.x, self.y = float(x), float(y)
+	def __init__(self, x: Component, y: Component):
+		if isinstance(x, u.Quantity) and isinstance(y, u.Quantity):
+			self.x, self.y = x.to(u.rad).value, y.to(u.rad).value
+		else:
+			self.x, self.y = float(x), float(y)
 
-	def __repr__(self):
+	def __repr__(self) -> str:
 		return "SPoint(%r, %r)"%(self.x, self.y)
 
 	@staticmethod
-	def _adaptToPgSphere(spoint):
+	def _adaptToPgSphere(spoint: SPoint) -> AsIs:
 		return AsIs("spoint '(%.10f,%.10f)'"%(spoint.x, spoint.y))
-	
+
 	@classmethod
-	def _castFromPgSphere(cls, value, cursor):
+	def _castFromPgSphere(cls, value: Optional[str], cursor: Any
+			) -> 'Optional[SPoint]':
 		if value is not None:
-			return cls(*[float(v) for v in cls.pattern.match(value).groups()])
-	
+			return cls(*[float(v)
+				for v in cls.pattern.match(value).groups()]) # type: ignore
+		return None
+
 	@classmethod
-	def fromDegrees(cls, x, y):
-		if x is None or y is None:
-			return None
+	def fromDegreesP(cls, x: Component, y: Component) -> SPoint:
+		"""fromDegrees when it's certain that there's no None x and y.
+		"""
 		return cls(x*DEG, y*DEG)
 
 	@classmethod
-	def fromUnitVector(cls, uvec):
+	def fromDegrees(cls, x: Optional[Component], y: Optional[Component]
+			) -> Optional[SPoint]:
+		if x is None or y is None:
+			return None
+		return cls.fromDegreesP(x, y)
+
+	@classmethod
+	def fromUnitVector(cls, uvec: UnitSphericalRepresentation) -> SPoint:
 		"""returns an SPoint for a 3-unit sphere vector.
 		"""
 		return cls(*mathtricks.cartToSpher(uvec))
 
-	def asCooPair(self):
+	def asCooPair(self) -> Tuple[float, float]:
 		"""returns this point as (long, lat) in degrees.
 		"""
 		return (self.x/DEG, self.y/DEG)
 
-	def asSTCS(self, systemString):
+	def asSTCS(self, systemString: str) -> str:
 		return removeTrailingZeroes(
 			"Position %s %.10f %.10f"%(systemString, self.x/DEG, self.y/DEG))
 
-	def asPgSphere(self):
+	def asPgSphere(self) -> str:
 		return "spoint '(%.10f,%.10f)'"%(self.x, self.y)
 
-	def p(self):   # helps below
+	def p(self) -> str:   # helps below
 		return "(%r, %r)"%(self.x, self.y)
 	
-	def asUnitSphereVec(self):
+	def asUnitSphereVec(self) -> UnitSphericalRepresentation:
 		"""returns self as a triple of cx, cy, cz on the unit sphere.
 		"""
 		return mathtricks.spherToCart(self.x, self.y)
 
-	def asDALI(self):
+	def asDALI(self) -> Tuple[float, float]:
 		return self.asCooPair()
 
 	@classmethod
-	def fromDALI(cls, coos):
+	def fromDALI(cls, coos: Sequence[Component]):
 		return cls.fromDegrees(*coos)
 
 
@@ -185,22 +194,25 @@ class SCircle(PgSAdapter):
 	"""
 	pgType = "scircle"
 	checkedAttributes = ["center", "radius"]
-	pattern = re.compile("<(\([^)]*\))\s*,\s*([0-9.e-]+)>")
+	pattern = re.compile(r"<(\([^)]*\))\s*,\s*([0-9.e-]+)>")
 
-	def __init__(self, center, radius):
+	def __init__(self, center: SPoint, radius: Component):
 		self.center, self.radius = center, float(radius)
 
 	@staticmethod
-	def _adaptToPgSphere(sc):
+	def _adaptToPgSphere(sc: SCircle):
 		return AsIs("scircle '< %s, %r >'"%(sc.center.p(), sc.radius))
 	
 	@classmethod
-	def _castFromPgSphere(cls, value, cursor):
+	def _castFromPgSphere(cls, value: Optional[str], cursor: Any
+			) -> Optional[SCircle]:
 		if value is not None:
-			pt, radius = cls.pattern.match(value).groups()
-			return cls(SPoint._castFromPgSphere(pt, cursor), radius)
+			pt, radius = cls.pattern.match(value).groups() # type: ignore
+			if pt is not None:
+				return cls(SPoint._castFromPgSphere(pt, cursor), radius) # type: ignore
+		return None
 
-	def asDALI(self):
+	def asDALI(self) -> Tuple[float, float, float]:
 		"""returns the "DALI-form" for this circle.
 
 		This is an array containing the center coordinates and the radius
@@ -209,7 +221,8 @@ class SCircle(PgSAdapter):
 		return (self.center.x/DEG, self.center.y/DEG, self.radius/DEG)
 
 	@classmethod
-	def fromDALI(cls, daliSeq):
+	def fromDALI(cls, daliSeq: Sequence[Optional[Component]]
+			) -> Optional[SCircle]:
 		"""returns a circle from its DALI float sequence.
 
 		Any None in daliSeq makes this a None to help with VOTable null value
@@ -217,31 +230,32 @@ class SCircle(PgSAdapter):
 		"""
 		if None in daliSeq:
 			return None
-		ra, dec, radius = [float(s) for s in daliSeq]
-		return cls(SPoint.fromDegrees(ra, dec), radius*DEG)
+		ra, dec, radius = [float(s) for s in daliSeq]  # type: ignore
+		center = SPoint.fromDegreesP(ra, dec)
+		return cls(center, radius*DEG)
 	
 	@classmethod
-	def fromPointSet(cls, points):
+	def fromPointSet(cls, points: Sequence[SPoint]) -> SCircle:
 		"""returns an SCircle covering all the SPoints in points.
 
 		(max radius: pi/2).
 		"""
 		uvecs = [numpy.array(p.asUnitSphereVec()) for p in points]
 		center = sum(uvecs)/len(uvecs)
-		radius = max(center.dot(v) for v in uvecs)
+		radius = max(center.dot(v) for v in uvecs)  # type: ignore
 		return cls(
 			SPoint.fromUnitVector(center),
 			math.acos(radius))
 
-	def asSTCS(self, systemString):
+	def asSTCS(self, systemString: str) -> str:
 		return removeTrailingZeroes("Circle %s %s"%(
 			systemString, self.asSODA()))
 
-	def asPgSphere(self):
+	def asPgSphere(self) -> str:
 		return "scircle '< (%.10f, %.10f), %.10f >'"%(
 			self.center.x, self.center.y, self.radius)
 
-	def asPoly(self, nSegments=32):
+	def asPoly(self, nSegments: int=32) -> SPoly:
 		# approximate the circle with 32 line segments and don't worry about
 		# circles with radii larger than 90 degrees.
 		# We compute the circle around the north pole and then rotate
@@ -249,23 +263,24 @@ class SCircle(PgSAdapter):
 		# circle's center.
 		r = math.sin(self.radius)
 		innerOffset = math.cos(self.radius)
-		rotationMatrix = mathtricks.getRotZ(math.pi/2-self.center.x).matMul(
-			mathtricks.getRotX(math.pi/2-self.center.y))
+		rotationMatrix = mathtricks.getRotZ(math.pi/2-self.center.x
+			)@mathtricks.getRotX(math.pi/2-self.center.y)
 
 		points = []
 		for i in range(nSegments):
 			angle = 2.*i*math.pi/nSegments
 			dx, dy = r*math.sin(angle), r*math.cos(angle)
 			points.append(SPoint(
-				*mathtricks.cartToSpher(rotationMatrix.vecMul((dx, dy, innerOffset)))))
+				*mathtricks.cartToSpher(
+					rotationMatrix@numpy.array([dx, dy, innerOffset]))))
 		return SPoly(points)
 
-	def asSMoc(self, order=6, inclusive=False):
+	def asSMoc(self, order: int=6, inclusive: bool=False):
 		"""returns an SMoc instance for this circle.
 
 		order is the maximum order of the moc returned.
 		"""
-		import healpy
+		import healpy # type: ignore
 		pixels = healpy.query_disc(vec=self.center.asUnitSphereVec(),
 			radius=self.radius,
 			nside=2**order, nest=True, inclusive=inclusive)
@@ -279,22 +294,24 @@ class SPoly(PgSAdapter):
 	"""
 	pgType = "spoly"
 	checkedAttributes = ["points"]
-	pattern = re.compile("\([^)]+\)")
+	pattern = re.compile(r"\([^)]+\)")
 
-	def __init__(self, points):
+	def __init__(self, points: Sequence[SPoint]):
 		self.points = tuple(points)
 
 	@staticmethod
-	def _adaptToPgSphere(spoly):
+	def _adaptToPgSphere(spoly: SPoly) -> AsIs:
 		return AsIs("spoly '{%s}'"%(", ".join(p.p() for p in spoly.points)))
 	
 	@classmethod
-	def _castFromPgSphere(cls, value, cursor):
+	def _castFromPgSphere(cls, value: Optional[str], cursor: Any
+			) -> Optional[SPoly]:
 		if value is not None:
-			return cls([SPoint._castFromPgSphere(ptLit, cursor)
+			return cls([SPoint._castFromPgSphere(ptLit, cursor) # type: ignore
 				for ptLit in cls.pattern.findall(value)])
+		return None
 
-	def asDALI(self):
+	def asDALI(self) -> List[float]:
 		"""returns the DALI form of this polygon.
 
 		This is a sequence of floats of the vertex coordinates in degrees.
@@ -305,7 +322,7 @@ class SPoly(PgSAdapter):
 		return res
 
 	@classmethod
-	def fromDALI(cls, daliSeq):
+	def fromDALI(cls, daliSeq: List[Component]) -> Optional[SPoly]:
 		"""returns a polygon from a DALI float-sequence
 
 		This returns None for empty daliSeqs to help with VOTable NULL value
@@ -317,10 +334,10 @@ class SPoly(PgSAdapter):
 			raise ValueError("Need an even-numbered number (>=6) of floats"
 				" in a DALI polygon representation, got %s floats."%len(daliSeq))
 
-		return cls([SPoint.fromDegrees(*tuple(p))
+		return cls([SPoint.fromDegreesP(*tuple(p))
 			for p in misctricks.grouped(2, daliSeq)])
 
-	def asCooPairs(self):
+	def asCooPairs(self) -> List[Tuple[float, float]]:
 		"""returns the vertices as a sequence of (long, lat) pairs in
 		degrees.
 
@@ -328,18 +345,18 @@ class SPoly(PgSAdapter):
 		"""
 		return [p.asCooPair() for p in self.points]
 
-	def asSTCS(self, systemString):
+	def asSTCS(self, systemString: str) -> str:
 		return removeTrailingZeroes("Polygon %s %s"%(systemString,
 			self.asSODA()))
 
-	def asPgSphere(self):
+	def asPgSphere(self) -> str:
 		return "spoly '{%s}'"%(",".join("(%.10f,%.10f)"%(p.x, p.y)
 			for p in self.points))
 
-	def asPoly(self):
+	def asPoly(self) -> SPoly:
 		return self
 
-	def asSMoc(self, order=6, inclusive=False):
+	def asSMoc(self, order: int=6, inclusive: bool=False) -> SMoc:
 		"""returns an SMoc instance for this polygon.
 
 		order is the maximum order of the moc returned.
@@ -350,7 +367,9 @@ class SPoly(PgSAdapter):
 			nside=2**order, nest=True, inclusive=inclusive)
 		return SMoc.fromCells(order, pixels, maxOrder=order)
 
-	def getCenter(self):
+	def getCenter(self) -> List[float]:
+		# This probably should return an SPoint, but let's first see where
+		# this is used.
 		"""returns an estimate for some sort of center of this polygon.
 
 		This is computed as the mean of the vertices (in unit sphere
@@ -363,7 +382,7 @@ class SPoly(PgSAdapter):
 		vertices = [numpy.array(mathtricks.spherToCart(p.x, p.y))
 			for p in self.points]
 		center = numpy.average(vertices, axis=0)
-		center = mathtricks.normalize3(center)
+		center = center/(center@center)
 		return mathtricks.cartToSpher(center)
 
 
@@ -374,23 +393,26 @@ class SBox(PgSAdapter):
 	"""
 	pgType = "sbox"
 	checkedAttributes = ["corner1", "corner2"]
-	pattern = re.compile("\([^()]+\)")
+	pattern = re.compile(r"\([^()]+\)")
 
-	def __init__(self, corner1, corner2):
+	def __init__(self, corner1: SPoint, corner2: SPoint):
 		self.corner1, self.corner2 = corner1, corner2
 
 	@staticmethod
-	def _adaptToPgSphere(sbox):
+	def _adaptToPgSphere(sbox: SBox) -> AsIs:
 		return AsIs("sbox '(%s, %s)'"%(sbox.corner1.p(), sbox.corner2.p()))
 
 	@classmethod
-	def _castFromPgSphere(cls, value, cursor):
+	def _castFromPgSphere(cls, value: Optional[str], cursor: Any
+			) -> Optional[SBox]:
 		if value is not None:
-			return cls(*[SPoint._castFromPgSphere(ptLit, cursor)
+			return cls(*[SPoint._castFromPgSphere(ptLit, cursor) # type: ignore
 				for ptLit in cls.pattern.findall(value)])
+		return None
 
 	@classmethod
-	def fromSIAPPars(cls, ra, dec, raSize, decSize):
+	def fromSIAPPars(cls, ra: float, dec: float, raSize: float, decSize: float
+			) -> SBox:
 		"""returns an SBox corresponding to what SIAP passes in.
 
 		In particular, all values are in degrees, and a cartesian projection
@@ -418,28 +440,28 @@ class SBox(PgSAdapter):
 		elif bottom<-90:
 			raise TwoSBoxes(
 				cls(
-					SPoint.fromDegrees(minRA, -90),
-					SPoint.fromDegrees(maxRA, top)),
+					SPoint.fromDegreesP(minRA, -90),
+					SPoint.fromDegreesP(maxRA, top)),
 				cls(
-					SPoint.fromDegrees(180+minRA, -90),
-					SPoint.fromDegrees(180+maxRA, top)))
+					SPoint.fromDegreesP(180+minRA, -90),
+					SPoint.fromDegreesP(180+maxRA, top)))
 		elif top>90:
 			raise TwoSBoxes(
 				cls(
-					SPoint.fromDegrees(minRA, bottom),
-					SPoint.fromDegrees(maxRA, 90)),
+					SPoint.fromDegreesP(minRA, bottom),
+					SPoint.fromDegreesP(maxRA, 90)),
 				cls(
-					SPoint.fromDegrees(180+minRA, bottom),
-					SPoint.fromDegrees(180+maxRA, 90)))
-		return cls(SPoint.fromDegrees(minRA, bottom),
-			SPoint.fromDegrees(maxRA, top))
+					SPoint.fromDegreesP(180+minRA, bottom),
+					SPoint.fromDegreesP(180+maxRA, 90)))
+		return cls(SPoint.fromDegreesP(minRA, bottom),
+			SPoint.fromDegreesP(maxRA, top))
 
-	def asSTCS(self, systemString):
+	def asSTCS(self, systemString: str) -> str:
 		return removeTrailingZeroes("PositionInterval %s %s %s"%(systemString,
 			"%.10f %.10f"%(self.corner1.x/DEG, self.corner1.y/DEG),
 			"%.10f %.10f"%(self.corner2.x/DEG, self.corner2.y/DEG)))
 
-	def asPoly(self):
+	def asPoly(self) -> SPoly:
 		x1, y1 = self.corner1.x, self.corner1.y
 		x2, y2 = self.corner2.x, self.corner2.y
 		minX, maxX = min(x1, x2), max(x1, x2)
@@ -451,23 +473,32 @@ class SBox(PgSAdapter):
 			SPoint(maxX, minY)))
 
 	@classmethod
-	def fromDALI(cls, daliSeq):
+	def fromDALI(cls, daliSeq: Sequence[Component]) -> SBox:
 		# see asDALI()
 		return cls(
 			SPoint.fromDALI(daliSeq[:2]),
 			SPoint.fromDALI(daliSeq[2:]))
 	
-	def asDALI(self):
+	def asDALI(self) -> Tuple[float, float, float, float]:
 		# we're cheating a bit here; there's no official DALI representation
 		# yet.  This is for our private xtype x-box
 		return self.corner1.asDALI()+self.corner2.asDALI()
 
 
-# temporary measure to shut up astropy's configuration parser.
-import builtins
-builtins._ASTROPY_SETUP_ = True
+class TwoSBoxes(excs.ExecutiveAction):
+	"""is raised when an SBox is constructed from center and size such that
+	it overlaps the pole.
+	"""
+	def __init__(self, box1: SBox, box2: SBox):
+		self.box1, self.box2 = box1, box2
 
 
+try:
+	import pymoc # type: ignore
+except ImportError:
+	# we're trying to work without pymoc, too
+	pass
+
 class SMoc(PgSAdapter):
 	"""a MOC with a pgsphere interface.
 
@@ -478,7 +509,7 @@ class SMoc(PgSAdapter):
 	pgType = "smoc"
 	checkedAttributes = ["moc"]
 
-	def __init__(self, moc):
+	def __init__(self, moc: pymoc.MOC):
 		self.moc = moc
 		if not hasattr(self.moc, "explicitMaxOrder"):
 			self.moc.explicitMaxOrder = None
@@ -487,14 +518,15 @@ class SMoc(PgSAdapter):
 	_rangeRE = re.compile(r"\s*(\d+)(?:-(\d+))?\s*$")
 
 	@property
-	def maxOrder(self):
+	def maxOrder(self) -> int:
 		if self.moc.explicitMaxOrder is None:
 			return self.moc.order
 		else:
 			return self.moc.explicitMaxOrder
 
 	@classmethod
-	def _parseCells(cls, cellLiteral, firstPos):
+	def _parseCells(cls, cellLiteral: str, firstPos: int
+			) -> List[int]:
 		"""returns a sequence of cells from a MOC cell literal.
 
 		firstPos is the string position of the beginning of cellLiteral
@@ -522,14 +554,14 @@ class SMoc(PgSAdapter):
 		return cells
 
 	@classmethod
-	def fromASCII(cls, literal):
+	def fromASCII(cls, literal: str) -> SMoc:
 		"""returns an SMoc from a quasi-standard ASCII serialisation.
 		"""
-		import pymoc
-
 		# we do the parsing ourselves -- the pymoc interface is too clumsy,
 		# and the rigidity of the parser doesn't look good.
-		seps = list(cls._orderHeadRE.finditer(literal))+[re.search("$", literal)]
+		seps = cast(List[re.Match],
+			list(cls._orderHeadRE.finditer(literal))+[re.search("$", literal)])
+
 		if len(seps)==1:
 			raise ValueError("No order separator visible in MOC literal '%s'"%
 				texttricks.makeEllipsis(literal, 40))
@@ -540,7 +572,8 @@ class SMoc(PgSAdapter):
 		moc = pymoc.MOC()
 		moc.explicitMaxOrder = None
 
-		for openMat, closeMat in codetricks.iterRanges(seps):
+		for openMat, closeMat in cast( # TODO: why do I need this cast?
+				Sequence[Tuple[re.Match, re.Match]], codetricks.iterRanges(seps)):
 			order = int(openMat.group(1))
 			cells = cls._parseCells(
 				literal[openMat.end():closeMat.start()],
@@ -553,20 +586,21 @@ class SMoc(PgSAdapter):
 		return cls(moc)
 	
 	@classmethod
-	def fromCells(cls, order, pixels, maxOrder=None):
+	def fromCells(cls,
+			order: int,
+			pixels: List[int],
+			maxOrder: Optional[int]=None) -> SMoc:
 		"""returns a SMoc instance from a collection of pixels at order.
 
 		Pass maxOrder to set an explicit max order for the resulting MOC.
 		"""
-		import pymoc
-		
 		moc = pymoc.MOC(order=order, cells=pixels)
 		moc.explicitMaxOrder = maxOrder
 		moc.normalize()
 		return cls(moc)
 
 	@staticmethod
-	def _formatASCIIRange(minCell, maxCell):
+	def _formatASCIIRange(minCell: int, maxCell: int) -> str:
 		"""returns a cell literal for a MOC.
 		"""
 		if minCell==maxCell:
@@ -574,7 +608,7 @@ class SMoc(PgSAdapter):
 		else:
 			return "%d-%d"%(minCell, maxCell)
 
-	def asASCII(self):
+	def asASCII(self) -> str:
 		"""returns an ascii serialisation of this MOC.
 		"""
 		# this is essentially the pymoc code, but again saving the file
@@ -582,10 +616,10 @@ class SMoc(PgSAdapter):
 		parts = []
 		for order, cells in self.moc:
 			ranges = []
-			rmin = rmax = None
+			rmin, rmax = -1, -1
 
 			for cell in sorted(cells):
-				if rmin is None:
+				if rmin==-1:
 					rmin = rmax = cell
 				elif rmax==cell-1:
 					rmax = cell
@@ -602,11 +636,10 @@ class SMoc(PgSAdapter):
 		return " ".join(parts)
 
 	@classmethod
-	def fromFITS(cls, literal):
+	def fromFITS(cls, literal: bytes) -> SMoc:
 		"""returns an SMoc from a string containing a FITS-serialised MOC.
 		"""
-		from pymoc.io.fits import read_moc_fits_hdu
-		import pymoc
+		from pymoc.io.fits import read_moc_fits_hdu # type: ignore
 
 		moc = pymoc.MOC()
 		read_moc_fits_hdu(moc,
@@ -614,23 +647,25 @@ class SMoc(PgSAdapter):
 		return cls(moc)
 
 	@staticmethod
-	def _adaptToPgSphere(smoc):
+	def _adaptToPgSphere(smoc: SMoc) -> AsIs:
 		return AsIs("smoc '%s'"%(smoc.asASCII()))
 
 	@classmethod
-	def _castFromPgSphere(cls, value, cursor):
+	def _castFromPgSphere(cls, value: Optional[str], cursor: Any
+			) -> Optional[SMoc]:
 		if value is not None:
 			return cls.fromASCII(value)
+		return None
 
-	def asPoly(self):
+	def asPoly(self) -> SPoly:
 		raise TypeError("MOCs cannot be represented as polygons")
 
-	def asSTCS(self, frame):
+	def asSTCS(self, frame: str) -> str:
 		# no STCS for MOCs, but this is really just for old-style VOTable
 		# serialisation, so let's cheat
 		return "MOC "+self.asASCII()
 
-	def asSMoc(self, order=6):
+	def asSMoc(self, order: int=6) -> SMoc:
 		"""returns a copy of self, normalised for order.
 		"""
 		moc = self.moc.copy()
@@ -641,15 +676,15 @@ class SMoc(PgSAdapter):
 
 		return self.__class__(moc)
 
-	def getPlot(self, **kwargs):
+	def getPlot(self, **kwargs) -> bytes:
 		"""returns a png string with a plot visualising this moc.
 		"""
-		from pymoc.util.plot import plot_moc
+		from pymoc.util.plot import plot_moc # type: ignore
 		with tempfile.NamedTemporaryFile(suffix=".png") as f:
 			plot_moc(self.moc, filename=f.name, projection="moll", **kwargs)
 			return f.read()
 
-	def asFITS(self):
+	def asFITS(self) -> bytes:
 		"""returns a standard FITS (table) representation of this MOC.
 		"""
 		from pymoc.io.fits import write_moc_fits
@@ -659,7 +694,7 @@ class SMoc(PgSAdapter):
 			f.seek(0)
 			return f.read()
 
-	def asDALI(self):
+	def asDALI(self) -> str:
 		"""returns the string representation of this MOC.
 
 		This isn't what DALI actually says at this point, but we suspect
@@ -668,17 +703,17 @@ class SMoc(PgSAdapter):
 		return self.asASCII()
 	
 	@classmethod
-	def fromDALI(cls, literal):
+	def fromDALI(cls, literal: str) -> SMoc:
 		"""returns an SMoc from a MOC string.
 
 		(see asDALI)
 		"""
 		return cls.fromASCII(literal)
 
-	def asNUNIQs(self):
+	def asNUNIQs(self) -> List[int]:
 		"""returns a list of integers usable as nuniqs.
 		"""
-		res = []
+		res: List[int] = []
 		for order, cells in self.moc:
 			shiftedOrder = 4 * (4 ** order)
 			res.extend(pix+shiftedOrder for pix in cells)
@@ -689,13 +724,24 @@ try:
 	import psycopg2
 	from psycopg2.extensions import (register_adapter, AsIs, register_type,
 		new_type)
+	if TYPE_CHECKING:
+		from psycopg2.extensions import connection as Connection
 
-	_getPgSClass = codetricks.buildClassResolver(PgSAdapter, list(globals().values()),
-		key=lambda obj: obj.pgType, default=PgSAdapter)
+	def _query(conn: Connection,
+			query: str,
+			pars: Optional[Dict[str, Any]]=None) -> List[Tuple]:
+		c = conn.cursor()
+		c.execute(query, pars)
+		return list(c)
+
+	_getPgSClass = codetricks.buildClassResolver(
+		PgSAdapter,
+		list(globals().values()),
+		key=lambda obj: obj.pgType, default=PgSAdapter) # type: ignore
 
 
-	def preparePgSphere(conn):
-		if hasattr(psycopg2, "_pgsphereLoaded"):
+	def preparePgSphere(conn: Connection) -> None:
+		if hasattr(psycopg2, "_pgsphereLoaded"): # type: ignore
 			return
 		try:
 			oidmap = _query(conn,
@@ -709,13 +755,12 @@ try:
 					register_adapter(cls, cls._adaptToPgSphere)
 					register_type(
 						new_type((oid,), "spoint", cls._castFromPgSphere))
-				psycopg2._pgsphereLoaded = True
+				psycopg2._pgsphereLoaded = True # type: ignore
 			conn.commit()
 		except:
-			psycopg2._pgsphereLoaded = False
+			psycopg2._pgsphereLoaded = False # type: ignore
 
 except ImportError:  # pragma: no cover
-	raise
 	# psycopg2 not installed.  Since preparsePgSphere can only be
 	# called from code depending on psycopg2, there's no harm if
 	# we don't define it.
