@@ -12,33 +12,39 @@ that during serialization the instance document's root element contains
 the namespace mapping (and the schema locations) required.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
 import io
-
-from xml.etree import ElementTree
+import types
+# This is utils.ElementTree, which is what DaCHS-internal code
+# should use to avoid multiple etrees having to interact:
+from xml.etree import ElementTree #noflake: re-export
 
 from gavo.utils import autonode
 from gavo.utils import excs
 from gavo.utils import misctricks
 from gavo.utils import texttricks
 
+from gavo.utils.dachstypes import (Any, BinaryIO, Callable, Dict,
+	FrozenSet, IO, Iterator, List, Optional, Set, Tuple, Type, Union)
+# note on type annotation: mypy 1.0.1 doesn't cope with attributes
+# introduced by metaclasses.  Most of the type ignore pragmas in here
+# work around that deficiency and can perhaps be dropped with a
+# metaclass-aware mypy.
+
+
 class Error(Exception):
 	pass
 
-
 class ChildNotAllowed(Error):
 	pass
 
 
-# you probably can't just change encoding here, as DaCHS assumes UTF-8
-# essentially almost everywhere.  So, consider the pretense legacy.
-encoding = "utf-8"
-XML_HEADER = texttricks.bytify('<?xml version="1.0" encoding="%s"?>'%encoding)
+XML_HEADER = b'<?xml version="1.0" encoding="utf-8"?>'
 
 
 class _Autoconstructor(autonode.AutoNodeType):
@@ -48,18 +54,18 @@ class _Autoconstructor(autonode.AutoNodeType):
 	attributes, on the other, it will instantiate itself when indexed
 	-- that we want for convenient stan-like notation.
 	"""
-	def __init__(cls, name, bases, dict):
+	def __init__(cls, name: str, bases: Tuple[type, ...], dict: Dict):
 		autonode.AutoNodeType.__init__(cls, name, bases, dict)
 		if hasattr(cls, "_childSequence") and cls._childSequence is not None:
 			cls._allowedChildren = set(cls._childSequence)
 		else:
-			cls._childSequence = None
+			cls._childSequence: Optional[List[str]] = None
 
-	def __getitem__(cls, items):
+	def __getitem__(cls, items: List[Any]) -> autonode.AutoNode:
 		return cls()[items]
 
 
-class Stub(object):
+class Stub:
 	"""A sentinel class for embedding objects not yet existing into
 	stanxml trees.
 
@@ -72,35 +78,35 @@ class Stub(object):
 	name_ = "stub"
 	text_ = None
 
-	def __init__(self, dest):
+	def __init__(self, dest: str) -> None:
 		self.dest = dest
 
-	def __repr__(self):
+	def __repr__(self) -> str:
 		return "%s(%s)"%(self.__class__.__name__, repr(self.dest))
 
-	def __eq__(self, other):
+	def __eq__(self, other: Any) -> bool:
 		return self.dest==getattr(other, "dest", Stub)
 	
-	def __ne__(self, other):
+	def __ne__(self, other: Any) -> bool:
 		return not self==other
 
-	def __hash__(self):
+	def __hash__(self) -> int:
 		return hash(self.dest)
 
-	def isEmpty(self):
+	def isEmpty(self) -> bool:
 		return False
 
-	def shouldBeSkipped(self):
+	def shouldBeSkipped(self) -> bool:
 		return False
 
-	def getChildDict(self):
+	def getChildDict(self) -> Dict:
 		return {}
 
 	def iterAttNames(self):
 		if False:
 			yield
 
-	def apply(self, func):
+	def apply(self, func: Callable) -> None:
 		"""does nothing.
 
 		Stubs don't have what Element.apply needs, so we don't even pretend.
@@ -108,7 +114,7 @@ class Stub(object):
 		return
 
 
-class Element(object, metaclass=_Autoconstructor):
+class Element(metaclass=_Autoconstructor):
 	"""An element for serialization into XML.
 
 	This is loosely modelled after t.w.template stan.
@@ -146,33 +152,30 @@ class Element(object, metaclass=_Autoconstructor):
 	Elements cannot harbor mixed content (or rather, there is only
 	one piece of text).
 	"""
-
 	name_ = None
 	_a_id = None
 	_prefix = ""
-	_additionalPrefixes = frozenset()
+	_additionalPrefixes: FrozenSet[str] = frozenset()
 	_mayBeEmpty = False
 	_local = False
 	_stringifyContent = False
 
+	_localAttrs: Union[Tuple, List[Tuple[str, str]]] = ()
+
 	# should probably do this in the elements needing it (quite a lot of them
 	# do, however...)
 	_name_a_xsi_type = "xsi:type"
 
-	# for type dispatching in addChild.
-	_generator_t = type((x for x in ()))
-
 	# see _setupNode below for __init__
 
-	def __getitem__(self, children):
+	def __getitem__(self, children: Any) -> "Element":
 		self.addChild(children)
 		return self
 
-	def __call__(self, **kw):
+	def __call__(self, **kw: Any) -> "Element":
 		if not kw:
 			return self
 	
-		# XXX TODO: namespaced attributes?
 		for k, v in kw.items():
 			# Only allow setting attributes already present
 			getattr(self, k)
@@ -182,43 +185,47 @@ class Element(object, metaclass=_Autoconstructor):
 	def __iter__(self):
 		raise NotImplementedError("Element instances are not iterable.")
 
-	def __bool__(self):
+	def __bool__(self) -> bool:
 		return self.isEmpty()
 
-	def _setupNodeNext(self, cls):
+	def _setupNodeNext(self, cls) -> None:
 		try:
-			pc = super(cls, self)._setupNode
+			pc = super(cls, self)._setupNode  #type: ignore
 		except AttributeError:
 			pass
 		else:
 			pc()
 
-	def _setupNode(self):
-		self._isEmptyCache = None
-		self._children = []
+	def _setupNode(self) -> None:
+		self._isEmptyCache: Optional[bool] = None
+		self._children: List[Any] = []
 		self.text_ = ""
 		if self.name_ is None:
 			self.name_ = self.__class__.__name__.split(".")[-1]
 		self._setupNodeNext(Element)
 
-	def _makeAttrDict(self):
+	def _makeAttrDict(self) -> Dict[str, str]:
 		res = {}
 		for name, attName in self.iterAttNames():
 			if getattr(self, name, None) is not None:
 				res[attName] = str(getattr(self, name))
+
+		for name, value in self._localAttrs:
+			res[name] = value
+
 		return res
 
-	def _iterChildrenInSequence(self):
+	def _iterChildrenInSequence(self) -> Iterator[Any]:
 		cDict = self.getChildDict()
-		for cName in self._childSequence:
+		for cName in self._childSequence:  #type: ignore
 			if cName in cDict:
 				for c in cDict[cName]:
 					yield c
 
-	def bailIfBadChild(self, child):
-		if (self._childSequence is not None
-				and getattr(child, "name_", None) not in self._allowedChildren
-				and type(child) not in self._allowedChildren):
+	def bailIfBadChild(self, child: Any) -> None:
+		if (self._childSequence is not None  # type: ignore
+				and getattr(child, "name_", None) not in self._allowedChildren # type: ignore
+				and type(child) not in self._allowedChildren):  # type: ignore
 			raise ChildNotAllowed("No %s children in %s"%(
 				getattr(child, "name_", "text"), self.name_))
 
@@ -236,7 +243,7 @@ class Element(object, metaclass=_Autoconstructor):
 	# t.w.template compatibility
 	clone = deepcopy
 
-	def addChild(self, child):
+	def addChild(self, child: Any) -> None:
 		"""adds child to the list of children.
 
 		Child may be an Element, a string, or a list or tuple of Elements and
@@ -258,7 +265,7 @@ class Element(object, metaclass=_Autoconstructor):
 			self.bailIfBadChild(child)
 			self._children.append(child)
 
-		elif isinstance(child, (list, tuple, self._generator_t)):
+		elif isinstance(child, (list, tuple, types.GeneratorType)):
 			for c in child:
 				self.addChild(c)
 
@@ -282,7 +289,7 @@ class Element(object, metaclass=_Autoconstructor):
 			raise Error("%s element %s cannot be added to %s node"%(
 				type(child), repr(child), self.name_))
 
-	def isEmpty(self):
+	def isEmpty(self) -> bool:
 		"""returns true if the current node has no non-empty children and no
 		non-whitespace text content.
 		"""
@@ -299,7 +306,7 @@ class Element(object, metaclass=_Autoconstructor):
 
 		return self._isEmptyCache
 
-	def shouldBeSkipped(self):
+	def shouldBeSkipped(self) -> bool:
 		"""returns true if the current node should be part of an output.
 
 		That is true if it is either non-empty or _mayBeEmpty is true.
@@ -310,44 +317,48 @@ class Element(object, metaclass=_Autoconstructor):
 			return False
 		return self.isEmpty()
 
-	def iterAttNames(self):
+	def iterAttNames(self) -> Iterator[Tuple[str, str]]:
 		"""iterates over the defined attribute names of this node.
 		
 		Each element returned is a pair of the node attribute name and the
 		xml name (which may be translated via _a_name_<att>
 		"""
-		for name, default in self._nodeAttrs:
+		for name, default in self._nodeAttrs:  # type: ignore
 			xmlName = getattr(self, "_name_a_"+name, name)
 			yield name, xmlName
 
-	def addAttribute(self, attName, attValue):
-		"""adds attName, attValue to this Element's attributes when instantiated.
+	def addAttribute(self, attName: str, attValue: str) -> None:
+		"""adds attName, attValue as a local attribute/value pair.
 
-		You cannot add _a_<attname> attributes to instances.  Thus, when
-		in a pinch, use this.
+		This is for when you have, if you will, "non-namespace" attributes
+		(most of the time, xml namespace declarations).
+
+		If you set the same name multiple times, the last value set will win.
 		"""
-		attName = str(attName)
-		if not hasattr(self, attName):
-			self._nodeAttrs.append((attName, attValue))
-		setattr(self, attName, attValue)
+		if self._localAttrs==():
+			self._localAttrs = [(attName, attValue)]
+		else:
+			self._localAttrs.append((attName, attValue))  # type: ignore
 
-	def iterChildrenOfType(self, type):
+	def iterChildrenOfType(self, type: _Autoconstructor
+			) -> Iterator[_Autoconstructor]:
 		"""iterates over all children having type.
 		"""
 		for c in self._children:
 			if isinstance(c, type):
 				yield c
 
-	def iterChildren(self):
+	def iterChildren(self) -> Iterator[Any]:
 		return iter(self._children)
 
-	def getChildDict(self):
-		cDict = {}
+	def getChildDict(self) -> Dict[str, Any]:
+		cDict: Dict[str, List] = {}
 		for c in self._children:
 			cDict.setdefault(c.name_, []).append(c)
 		return cDict
 	
-	def iterChildrenWithName(self, elName):
+	def iterChildrenWithName(self, elName: str
+			) -> Iterator[Any]:
 		"""iterates over children whose element name is elName.
 
 		This always does a linear search through the children and hence
@@ -357,24 +368,23 @@ class Element(object, metaclass=_Autoconstructor):
 			if c.name_==elName:
 				yield c
 		
-	def _getChildIter(self):
-		if self._childSequence is None:
+	def _getChildIter(self) -> Iterator[Any]:
+		if self._childSequence is None:  # type: ignore
 			return iter(self._children)
 		else:
 			return self._iterChildrenInSequence()
 
-	def apply(self, func):
+	def apply(self, func: Callable) -> Optional[Any]:
 		"""calls func(node, text, attrs, childIter).
 
-		This is a building block for tree traversals; the expectation is that
-		func does something like func(node, text, attrDict, childSequence).
+		This is a building block for tree traversals.
 		"""
 		try:
 			if self.shouldBeSkipped():
-				return
+				return None
 			attrs = self._makeAttrDict()
-			return func(self, self.text_,
-				attrs, self._getChildIter())
+			return func(
+				self, self.text_, attrs, self._getChildIter())
 		except Error:
 			raise
 		except Exception:
@@ -386,10 +396,10 @@ class Element(object, metaclass=_Autoconstructor):
 			raise
 
 	def render(self,
-			prefixForEmpty=None,
-			includeSchemaLocation=True,
-			xmlDecl=False,
-			prolog=None):
+			prefixForEmpty: Optional[str] = None,
+			includeSchemaLocation: bool = True,
+			xmlDecl: bool = False,
+			prolog: str = "") -> bytes:
 		"""returns this and its children as a string.
 		"""
 		f = io.BytesIO()
@@ -398,11 +408,11 @@ class Element(object, metaclass=_Autoconstructor):
 			prefixForEmpty=prefixForEmpty,
 			xmlDecl=xmlDecl,
 			includeSchemaLocation=includeSchemaLocation,
-			prolog=texttricks.bytify(prolog))
+			prolog=prolog)
 		return f.getvalue()
 
 
-class NSRegistry(object):
+class NSRegistry:
 	"""A container for a registry of namespace prefixes to namespaces.
 
 	This is used to have fixed namespace prefixes (IMHO the only way
@@ -410,12 +420,15 @@ class NSRegistry(object):
 	class is never instantiated.  It is used through the module-level
 	method registerPrefix.
 	"""
-	_registry = {}
-	_reverseRegistry = {}
-	_schemaLocations = {}
+	_registry: Dict[str, str] = {}
+	_reverseRegistry: Dict[str, str] = {}
+	_schemaLocations: Dict[str, Optional[str]] = {}
 
 	@classmethod
-	def registerPrefix(cls, prefix, ns, schemaLocation):
+	def registerPrefix(cls,
+			prefix: str,
+			ns: str,
+			schemaLocation: Optional[str]) -> None:
 		if prefix in cls._registry:
 			if ns!=cls._registry[prefix]:
 				raise ValueError("Prefix %s is already allocated for namespace %s"%
@@ -425,7 +438,7 @@ class NSRegistry(object):
 		cls._schemaLocations[prefix] = schemaLocation
 
 	@classmethod
-	def getPrefixForNS(cls, ns):
+	def getPrefixForNS(cls, ns:str) -> str:
 		try:
 			return cls._reverseRegistry[ns]
 		except KeyError:
@@ -435,7 +448,7 @@ class NSRegistry(object):
 				" the right module?")
 
 	@classmethod
-	def getNSForPrefix(cls, prefix):
+	def getNSForPrefix(cls, prefix:str) -> str:
 		try:
 			return cls._registry[prefix]
 		except KeyError:
@@ -445,7 +458,11 @@ class NSRegistry(object):
 				" the right module?")
 	
 	@classmethod
-	def _iterNSAttrs(cls, prefixes, prefixForEmpty, includeSchemaLocation):
+	def _iterNSAttrs(cls,
+			prefixes: set[str],
+			prefixForEmpty: Optional[str],
+			includeSchemaLocation: bool
+			) -> Iterator[Tuple[str, str]]:
 		"""iterates over pairs of (attrName, attrVal) for declaring
 		prefixes.
 		"""
@@ -470,39 +487,27 @@ class NSRegistry(object):
 			yield "xsi:schemaLocation", " ".join(schemaLocations)
 
 	@classmethod
-	def addNamespaceDeclarationsETree(cls, root, prefixes, prefixForEmpty=None,
-			includeSchemaLocation=True):
-		"""adds xmlns declarations for prefixes to the etree node root.
-
-		With stanxml and the global-prefix scheme, xmlns declarations
-		only come at the root element; thus, root should indeed be root
-		rather than some random element.
-
-		Deprecated, don't use ElementTree with stanxml any more.
-		"""
-		for attName, attVal in cls._iterNSAttrs(prefixes, prefixForEmpty,
-				includeSchemaLocation):
-			root.attrib[attName] = attVal
-
-	@classmethod
-	def addNamespaceDeclarations(cls, root, prefixes, prefixForEmpty=None,
-			includeSchemaLocation=True):
+	def addNamespaceDeclarations(cls,
+			root: Element,
+			prefixes: Set[str],
+			prefixForEmpty: Optional[str] = None,
+			includeSchemaLocation: bool = True) -> None:
 		"""adds xmlns declarations for prefixes to the stanxml node root.
 
 		With stanxml and the global-prefix scheme, xmlns declarations
 		only come at the root element; thus, root should indeed be root
 		rather than some random element.
 		"""
-		for attName, attVal in cls._iterNSAttrs(prefixes, prefixForEmpty,
-				includeSchemaLocation):
+		for attName, attVal in cls._iterNSAttrs(
+				prefixes, prefixForEmpty, includeSchemaLocation):
 			root.addAttribute(attName, attVal)
 
 	@classmethod
-	def getPrefixInfo(cls, prefix):
+	def getPrefixInfo(cls, prefix: str) -> Tuple[str, Optional[str]]:
 		return (cls._registry[prefix], cls._schemaLocations[prefix])
 
 	@classmethod
-	def getSchemaForNS(self, ns):
+	def getSchemaForNS(self, ns: str) -> Optional[str]:
 		try:
 			return self._schemaLocations[self._reverseRegistry[ns]]
 		except KeyError:
@@ -515,7 +520,7 @@ class NSRegistry(object):
 registerPrefix = NSRegistry.registerPrefix
 getPrefixInfo = NSRegistry.getPrefixInfo
 
-def schemaURL(xsdName):
+def schemaURL(xsdName: str) -> str:
 	"""returns the URL to the local mirror of the schema xsdName.
 
 	This is used by the various xmlstan clients to make schemaLocations.
@@ -529,7 +534,7 @@ registerPrefix("xsi","http://www.w3.org/2001/XMLSchema-instance",  None)
 xsiPrefix = frozenset(["xsi"])
 
 
-class NillableMixin(object):
+class NillableMixin:
 	"""An Element mixin making the element XSD nillable.
 
 	This element will automatically have an xsi:nil="true" attribute
@@ -538,23 +543,25 @@ class NillableMixin(object):
 	This overrides apply, so the mixin must be before the base class in
 	the inheritance list.
 	"""
+# typing TODO: Mixins are nasty; we would need to define a Protocol,
+# which seems over the top here.  Let's see if they come up with something
+# smarter and meanwhile sprinkle ignore-s.
 	_mayBeEmpty = True
 
-	def apply(self, func):
-		attrs = self._makeAttrDict()
-		if self.text_:
-			return Element.apply(self, func)
+	def apply(self, func: Callable) -> Any:
+		if self.text_:  # type: ignore
+			return Element.apply(self, func) # type: ignore
 		else:
-			attrs = self._makeAttrDict()
+			attrs = self._makeAttrDict()  # type: ignore
 			attrs["xsi:nil"] = "true"
-			self._additionalPrefixes = self._additionalPrefixes|set(["xsi"])
+			self._additionalPrefixes |= {"xsi"}  # type: ignore
 			return func(self, "", attrs, ())
 
-	def isEmpty(self):
+	def isEmpty(self) -> bool:
 		return False
 
 
-def escapePCDATA(val):
+def escapePCDATA(val: str) -> str:
 	return (val
 		).replace("&", "&amp;"
 		).replace('<', '&lt;'
@@ -562,15 +569,50 @@ def escapePCDATA(val):
 		).replace("\0", "&x00;")
 
 
-def escapeAttrVal(val):
+def escapeAttrVal(val: str) -> str:
 	return '"%s"'%(escapePCDATA(val).replace('"', '&quot;'))
 
 
-def _makeVisitor(outputFile, prefixForEmpty):
+class WriteBuffer:
+	"""a very simple write buffer, collecting strings and handing them
+	through to an actual files in suitably large chunks.
+
+	This ought to help make our XML writing to twisted requests a bit
+	more wire-efficient.
+
+	The raw argument of course can't be Any, it needs to be a
+	binary-writable thing.  I'm not giving a proper type because
+	what's here most of the time is a twisted web Request.  The
+	incomplete implementation of an io writer is also why this
+	class exists in the first place.
+	"""
+	def __init__(self, raw: Any, bufSize: int=8192):
+		self.raw, self.bufSize = raw, 8192
+		self.curSize = 0
+		self.buffer: List[bytes] = []
+	
+	def write(self, data):
+		self.curSize += len(data)
+		self.buffer.append(data)
+		if self.curSize>=self.bufSize:
+			self.flush()
+	
+	def flush(self):
+		self.raw.write(b"".join(self.buffer))
+		self.curSize, self.buffer = 0, []
+
+
+def _makeVisitor(
+		outputFile: Union[IO, WriteBuffer],
+		prefixForEmpty: Optional[str]) -> Callable:
 	"""returns a function writing nodes to outputFile.
 	"""
 	
-	def visit(node, text, attrs, childIter):
+	def visit(
+			node: Any,
+			text: str,
+			attrs: Dict[str, str],
+			childIter: Iterator) -> None:
 		attrRepr = " ".join(sorted("%s=%s"%(k, escapeAttrVal(attrs[k]))
 			for k in attrs))
 		if attrRepr:
@@ -609,8 +651,14 @@ def _makeVisitor(outputFile, prefixForEmpty):
 	return visit
 
 
-def xmlwrite(root, outputFile, prefixForEmpty=None, nsRegistry=NSRegistry,
-		xmlDecl=True, includeSchemaLocation=True, prolog=None):
+def xmlwrite(
+	root: Element,
+	outputFile: BinaryIO,
+	prefixForEmpty: Optional[str] = None,
+	nsRegistry: Type[NSRegistry] = NSRegistry,
+	xmlDecl: bool = True,
+	includeSchemaLocation: bool = True,
+	prolog: Optional[str] = None) -> None:
 	"""writes an xmlstan tree starting at root to outputFile.
 
 	prefixForEmpty is a namespace URI that should have no prefix at all.
@@ -619,15 +667,23 @@ def xmlwrite(root, outputFile, prefixForEmpty=None, nsRegistry=NSRegistry,
 	"""
 	# since namespaces only enter here through prefixes, I just need to
 	# figure out which ones are used.
-	prefixesUsed = set()
+	prefixesUsed: Set[str] = set()
 
-	def collectPrefixes(node, text, attrs, childIter,
-			prefixesUsed=prefixesUsed):
+	def collectPrefixes(
+			node: Element,
+			text: str,
+			attrs: Dict[str, str],
+			childIter:  Iterator[Any],
+			prefixesUsed: Set[str] = prefixesUsed) -> None:
 		prefixesUsed |= node._additionalPrefixes
 		prefixesUsed.add(node._prefix)
 		for child in childIter:
 			child.apply(collectPrefixes)
 
+	# put a buffer between us and the output; if we don't twisted
+	# will produce one little chunk per tag in http chunked encoding.
+	buffered = WriteBuffer(outputFile)
+
 	root.apply(collectPrefixes)
 	# An incredibly nasty hack for VOTable generation; we need a better
 	# way to handle with the 1.1/1.2 namespaces: Root may declare it
@@ -637,29 +693,35 @@ def xmlwrite(root, outputFile, prefixForEmpty=None, nsRegistry=NSRegistry,
 			includeSchemaLocation)
 
 	if xmlDecl:
-		outputFile.write(b"<?xml version='1.0' encoding='utf-8'?>\n")
+		buffered.write(b"<?xml version='1.0' encoding='utf-8'?>\n")
 
 	if prolog:
-		outputFile.write(texttricks.bytify(prolog)+b"\n")
+		buffered.write(texttricks.bytify(prolog)+b"\n")
 
-	root.apply(_makeVisitor(outputFile, prefixForEmpty))
+	try:
+		root.apply(_makeVisitor(buffered, prefixForEmpty))
+	finally:
+		buffered.flush()
 
 
-def xmlrender(tree, prolog=None, prefixForEmpty=None):
+def xmlrender(
+	tree: Element,
+	prolog: str = "",
+	prefixForEmpty: Optional[Any] = None) -> bytes:
 	"""returns a unicode object containing tree in serialized forms.
 
-	tree can be any object with a render method or some sort of string.
-	If it's a byte string, it must not contain any non-ASCII.
+	tree should be a stanxml.Element.  Anything else will be phased out.
 
 	If prolog is given, it must be a string that will be prepended to the
 	serialization of tree.  The way ElementTree currently is implemented,
 	you can use this for xml declarations or stylesheet processing
 	instructions.
 	"""
+	# Historically, we have accepted all kind of mess in here.  In the
+	# end, we only want to retain option 1 of the following selection.
+	# TODO: nuke the rest
 	if hasattr(tree, "render"):
 		res = tree.render(prefixForEmpty=prefixForEmpty)
-	elif hasattr(tree, "getchildren"):  # hopefully an xml.etree Element
-		res = ElementTree.tostring(tree)
 	elif isinstance(tree, bytes):
 		res = tree.decode("utf-8")
 	elif isinstance(tree, str):
