@@ -2,7 +2,7 @@
 Formatting, text manipulation, string constants, and such.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -12,37 +12,34 @@ import calendar
 import datetime
 import functools
 import os
+import pathlib
 import quopri
 import random
 import re
 import string
 import time
 from email import utils as emailutils
+from email import message as emailmessage
 
 from gavo.utils.dachstypes import (Any, Filename, Generator,
 	Optional, StrToStrMap, TextIO, Tuple, Union)
 
-try:
-	# old astropy
-	from astropy.coordinates import angle_utilities as angle_formats # type: ignore
-except ImportError:
-	from astropy.coordinates.angles import formats as angle_formats # type: ignore
-
+from gavo.imp import angle_formats
 from gavo.utils import codetricks
 from gavo.utils import misctricks
 from gavo.utils.excs import SourceParseError, StructureError
 
 floatRE = r"[+-]?(?:\d+\.?\d*|\.\d+)(?:[eE][+-]?\d+)?"
-dateRE = re.compile("\d\d\d\d-\d\d-\d\d$")
-datetimeRE = re.compile("\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\dZ?$")
-identifierPattern = re.compile("[A-Za-z_][A-Za-z0-9_]*$")
+dateRE = re.compile(r"\d\d\d\d-\d\d-\d\d$")
+datetimeRE = re.compile(r"\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\dZ?$")
+identifierPattern = re.compile(r"[A-Za-z_][A-Za-z0-9_]*$")
 isoTimestampFmt = "%Y-%m-%dT%H:%M:%SZ"
 isoTimestampFmtNoTZ = "%Y-%m-%dT%H:%M:%S"
-entityrefPat = re.compile("&([^;])+;")
-looksLikeURLPat = re.compile("[a-z]{2,5}://")
+entityrefPat = re.compile(r"&([^;])+;")
+looksLikeURLPat = re.compile(r"[a-z]{2,5}://")
 
 
-# file names that don't cause headaches in URLs and are otherwise reasonable
+# file names that don't cause headaches in URLs and are otherwise reasonably
 # benign (so, let's disallow shell metachars while we're at it).
 _SAFE_FILENAME = re.compile("[,-:=@-Z_a-z{}~-]+$")
 
@@ -73,7 +70,7 @@ def makeEllipsis(aStr: str, maxLen: int=60, ellChars: str="...") -> str:
 
 	Cropped strings are returned with an ellipsis marker.
 	"""
-	aStr = re.sub("\s+", " ", aStr)
+	aStr = re.sub(r"\s+", " ", aStr)
 	if len(aStr)>maxLen:
 		return aStr[:maxLen-len(ellChars)]+ellChars
 	return aStr
@@ -87,7 +84,7 @@ def makeLeftEllipsis(aStr: str, maxLen: int=60):
 	>>> makeLeftEllipsis("0123456789"*2, 11)
 	'...23456789'
 	"""
-	aStr = re.sub("\s+", " ", aStr)
+	aStr = re.sub(r"\s+", " ", aStr)
 	if len(aStr)>maxLen:
 		return "..."+aStr[-maxLen+3:]
 	return aStr
@@ -168,24 +165,34 @@ def formatSimpleTable(
 
 @codetricks.document
 def getRelativePath(
-		fullPath: str,
-		rootPath: str,
-		liberalChars: bool=True) -> str:
-	"""returns rest if fullPath has the form rootPath/rest and raises an
-	exception otherwise.
+		fullPath: Filename,
+		rootPath: Filename,
+		liberalChars: bool=True) -> Filename:
+	"""returns rest if fullPath has the form rootPath/rest and raises a
+	ValueError otherwise.
+
+	This accepts either strings or pathlib.Path-s and returns an object of the
+	type of fullPath (pathlib functionality since 2.9.3).
 
 	Pass ``liberalChars=False`` to make this raise a ValueError when
 	URL-dangerous characters (blanks, amperands, pluses, non-ASCII, and
 	similar) are present in the result.  This is mainly for products.
 	"""
-	if not fullPath.startswith(rootPath):
-		raise ValueError(
-			"Full path %s does not start with resource root %s"%(fullPath, rootPath))
-	res = fullPath[len(rootPath):].lstrip("/")
-	if not liberalChars and not _SAFE_FILENAME.match(res):
+	if isinstance(fullPath, pathlib.PurePath):
+		res: Filename = fullPath.relative_to(rootPath)
+	else:
+		if not fullPath.startswith(str(rootPath)):
+			raise ValueError(f"{fullPath} does not start with {rootPath}")
+		res = os.path.relpath(fullPath, str(rootPath))
+		# backwards compatibility hack
+		if res==".":
+			res = ""
+
+	if not liberalChars and not _SAFE_FILENAME.match(str(res)):
 		raise ValueError("File path '%s' contains characters known to"
 			" the DaCHS authors to be hazardous in URLs.  Please defuse the name"
 			" before using it for published names (or see howDoI)."%res)
+
 	return res
 
 
@@ -226,7 +233,7 @@ def fixIndentation(code: str, newIndent: str, governingLine: int=0) -> str:
 	reserved, codeLines = codeLines[:governingLine], codeLines[governingLine:]
 	while codeLines:
 		if codeLines[0].strip():
-			if mat := re.match("^\s*", codeLines[0]):
+			if mat := re.match(r"^\s*", codeLines[0]):
 				firstIndent = mat.group()
 				break
 		else:
@@ -679,7 +686,7 @@ def parseDefaultTime(literal: Optional[Union[str, datetime.time]]
 	"""
 	if literal is None or isinstance(literal, datetime.time):
 		return literal
-	# as long as we're builing on top of time, we can't do fractional seconds
+	# as long as we're building on top of time, we can't do fractional seconds
 	return datetime.time(*time.strptime(literal, '%H:%M:%S')[3:6])
 
 
@@ -938,6 +945,9 @@ def defuseFileName(fName: Union[Filename, bytes], replaceSlash: bool=True):
 	a string that's guaranteed to be ASCII only.  With replaceSlash=False,
 	it can also double as a reasonable asciification.
 	"""
+	if isinstance(fName, pathlib.Path):
+		fName = str(fName)
+
 	nameBytes = fName.encode("utf-8") if isinstance(fName, str) else fName
 	res = quopri.encodestring(nameBytes
 		).translate(DEFUSE_NONPRINTABLE
@@ -976,6 +986,36 @@ def parseAccept(aString: str) -> StrToStrMap:
 	return res
 
 
+def parseMediaType(mediaType: str) -> Tuple[str, dict]:
+	"""returns a pair of basic, lowercased media type and a parameter
+	dictionary for an RFC 2045 media type.
+
+	Except for lowercasing the type, normalisation follows whatever the mail
+	package does, which is probably too little.
+
+	Regrettably, EmailMessage's parsing code silently falls back to text/plain
+	when it cannot make out anything.  We can't use that, so we have extra
+	logic that doubts text/plain and raises a ValueError if it suspects
+	that's a hallucination.
+
+	>>> parseMediaType("application/x-VOTable+xml;serialisation=tabledata; content=datalink")
+	('application/x-votable+xml', {'serialisation': 'tabledata', 'content': 'datalink'})
+	>>> parseMediaType("not a media type at all.")
+	Traceback (most recent call last):
+	ValueError: Cannot parse media type 'not a media type at all.'
+	"""
+	# yeah, it's weird to construct an email message here, but it's what
+	# the purgers of the cgi module wanted
+	m = emailmessage.EmailMessage()
+	m.add_header("content-type", mediaType)
+	baseType = "{}/{}".format(m.get_content_maintype(), m.get_content_subtype())
+	if baseType=="text/plain":
+		if not mediaType.lower().startswith(baseType):
+			raise ValueError(f"Cannot parse media type '{mediaType}'")
+
+	return baseType, dict(m.get("content-type").params)
+
+
 if __name__=="__main__": # pragma: no cover
 	import doctest
 	doctest.testmod()
