@@ -12,11 +12,14 @@ In fact, we use a couple of extensions:
 		blobs at some point, but right now we simply don't touch it.
 	- vexpr-float, -text, -date, -mjd -- vizier-like expressions coming in from
 		the web.  These are always strings.
+	- pql-X as vexpr, just for DALI-like arguments
 	- raw -- handed right through, whatever it is.  For target formats that
 		can't do this, usually strings are used.
 	- unicode -- this is TEXT in the database, but while normal text will
 	  be rendered as byte strings in VOTables (with non-ASCII-characters
 	  replaced by ?), unicode will become an array of unicodeChars.
+	- geometry types (see utils.pgsphere)
+	-
 
 This module contains a base class and the VOTable type system conversion,
 as the VOTable module (that should not depend on base) depends on it.
@@ -26,7 +29,7 @@ when code can rely on gavo.base; this module should be considered an
 implementation detail.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -38,16 +41,17 @@ implementation detail.
 import re
 
 from gavo.utils import excs
+from gavo.utils.dachstypes import (Dict, Optional, Tuple, Union)
 
 
 class ConversionError(excs.Error):
 	pass
 
 
-class FromSQLConverter(object):
+class FromSQLConverter:
 	"""is an abstract base class for type converters from the SQL type system.
 
-	Implementing classes have to provide a dict simpleMap mapping sql type
+	Implementing classes have to provide a dict simpleMap mapping SQL type
 	strings to target types, and a method mapComplex that receives a type
 	and a length (both strings, derived from SQL array types) and either
 	returns None (no matching type) or the target type.
@@ -57,11 +61,15 @@ class FromSQLConverter(object):
 	"""
 	_charTypes = set(["character varying", "varchar", "character", "char"])
 
-	def _parseSQLDim(self, subscript):
+	simpleMap: Dict = {}
+
+	typeSystem = "<not overridden>"
+
+	def _parseSQLDim(self, subscript: str) -> str:
 		"""returns a VOTable-style arraysize specifiation for a SQL subscript
 		expression.
 		"""
-		parts = re.findall("[\[(](\d*|\*)[\])]", subscript)
+		parts = re.findall(r"[\[(](\d*|\*)[\])]", subscript)
 		# In VOTable, we can only have * on the last dimension; we'll error out
 		# for now if we are supposed to make some other type
 		for part in parts[:-1]:
@@ -74,7 +82,9 @@ class FromSQLConverter(object):
 
 		return "x".join(parts)
 
-	def convert(self, sqlType):
+	def convert(self, sqlType: str) -> Union[
+			Tuple[str, Optional[str], Optional[str]], # VOTable type, arrsz, xtype
+			Optional[str]]: # Most other type systems have just a single string
 		res = None
 		if sqlType in self.simpleMap:
 			res = self.simpleMap[sqlType]
@@ -92,7 +102,7 @@ class FromSQLConverter(object):
 			raise ConversionError("No %s type for %s"%(self.typeSystem, sqlType))
 		return res
 
-	def mapComplex(self, type, length):
+	def mapComplex(self, type: str, length: str):
 		return
 
 
@@ -135,7 +145,8 @@ class ToVOTableConverter(FromSQLConverter):
 		"jsonb": ("char", "*", "json"),
 	}
 
-	def mapComplex(self, type, length):
+	def mapComplex(self, type: str, length: Optional[str]
+			) -> Tuple[str, Optional[str], Optional[str]]:
 		if length=='':
 			length = '*'
 
@@ -192,7 +203,7 @@ class FromVOTableConverter(object):
 		("raw", '1'): "raw",
 	}
 
-	xtypeMap = {
+	xtypeMap: Dict[str, str] = {
 		"adql:POINT": "spoint",
 		"adql:REGION": "spoly",
 		"adql:TIMESTAMP": "timestamp",
@@ -206,8 +217,11 @@ class FromVOTableConverter(object):
 		"jsonb": "json",
 	}
 
-	def convert(self, type, arraysize, xtype=None):
-		if self.xtypeMap.get(xtype):
+	def convert(self,
+			type: str,
+			arraysize: Optional[str],
+			xtype: Optional[str] = None) -> str:
+		if xtype is not None and xtype in self.xtypeMap:
 			return self.xtypeMap[xtype]
 		if arraysize=="1" or arraysize=="" or arraysize is None:
 			arraysize = "1"
@@ -216,7 +230,7 @@ class FromVOTableConverter(object):
 		else:
 			return self.mapComplex(type, arraysize)
 
-	def mapComplex(self, type, arraysize):
+	def mapComplex(self, type: str, arraysize: str) -> str:
 		if arraysize=="*":
 			arraysize = ""
 		if type=="char":
