@@ -2,7 +2,7 @@
 xmlstan elements of VOTable.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -19,6 +19,7 @@ from gavo.votable import common
 
 
 NAMESPACES = {
+	"1.5": "http://www.ivoa.net/xml/VOTable/v1.3",
 	"1.4": "http://www.ivoa.net/xml/VOTable/v1.3",
 	"1.3": "http://www.ivoa.net/xml/VOTable/v1.3",
 	"1.2": "http://www.ivoa.net/xml/VOTable/v1.2",
@@ -27,7 +28,7 @@ NAMESPACES = {
 
 # WARNING: if you change the default version of the VOTables generated,
 # you'll probably have to adapt them in resources/xsl, too.
-registerPrefix("vot", NAMESPACES["1.3"], schemaURL("VOTable-1.4.xsd"))
+registerPrefix("vot", NAMESPACES["1.3"], schemaURL("VOTable.xsd"))
 registerPrefix("vot2", NAMESPACES["1.2"], schemaURL("VOTable-1.2.xsd"))
 registerPrefix("vot1", NAMESPACES["1.1"], schemaURL("VOTable-1.1.xsd"))
 
@@ -425,7 +426,7 @@ class VOTable(_MIVOTElements):
 
 	class VOTABLE(_VOTElement):
 		_a_ID = None
-		_a_version = "1.4"
+		_a_version = "1.5"
 		_prefix = "vot"
 		_supressedPrefix = "vot"
 		_mayBeEmpty = True
