@@ -6,7 +6,7 @@ module using some of this; on the long run, it should probably be
 integrated here.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -73,23 +73,23 @@ class UWSResource(resource.Resource):
 		self.renderer = renderer
 		resource.Resource.__init__(self)
 
-	def _deliverError(self, flr, request, httpCode=200):
+	def _deliverError(self, flr, request, httpStatus=200):
 		# auth requests and redirects handled by normal dc methods.
 		if isinstance(flr.value, (svcs.Authenticate, svcs.SeeOther)):
 			return weberrors.renderDCErrorPage(flr, request)
 
 		else:
 			if isinstance(flr.value, uws.JobNotFound):
-				httpCode = 404
+				httpStatus = 404
 			else:
 				base.ui.notifyFailure(flr)
-			return dali.serveDALIError(request, flr.value, httpCode)
+			return dali.serveDALIError(request, flr.value, httpStatus=httpStatus)
 
 	def render(self, request):
 		try:
 			return resource.Resource.render(self, request)
 		except Exception as ex:
-			return self._deliverError(failure.Failure(ex), request, httpCode=400)
+			return self._deliverError(failure.Failure(ex), request, httpStatus=400)
 
 
 class JoblistResource(UWSResource):
