@@ -2,7 +2,7 @@
 Common code for the nevow based interface.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -36,7 +36,7 @@ static.File.contentTypes[".rd"] = "application/x-gavo-descriptor+xml"
 static.File.contentTypes[".f90"] = "text/x-fortran"
 static.File.contentTypes[".skyglow"] = "text/plain"
 static.File.contentTypes[".fitstable"] = "application/fits"
-static.File.contentTypes[".fits"] = "image/fits"
+static.File.contentTypes[".fits"] = "application/fits"
 # this one is for co-operation with ifpages
 static.File.contentTypes[".shtml"] = "text/nevow-template"
 
@@ -218,7 +218,7 @@ class CommonRenderers(object):
 		in [general].  Behaviour for undefined config items is undefined.
 		"""
 		configKey = tag.children[0].strip()
-		mat = re.match("\[([^]]*)\](.*)", configKey)
+		mat = re.match(r"\[([^]]*)\](.*)", configKey)
 		if mat:
 			content = base.getConfig(mat.group(1), mat.group(2))
 		else:
@@ -314,6 +314,8 @@ class Request(formal.Request):
 	  you're actually dealing with non-utf-8 data (and then you should
 	  probably use request.files).
 	"""
+	defaultContentType = "text/html;charset=utf-8"
+
 	def __init__(self, *args, **kwargs):
 		server.Request.__init__(self, *args, **kwargs)
 		self.files, self.args = {}, {}
