@@ -16,7 +16,7 @@ logically belong (and where they should be documented in the renderer
 docstrings) is that we don't want docutils imports all over the place.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -56,7 +56,7 @@ class _Example(base.MetaMixin):
 		self.setMetaParent(exMeta)
 		self.original = exMeta
 		self.title = base.getMetaText(self.original, "title", propagate=False)
-		self.htmlId = re.sub("\W", "", self.title)
+		self.htmlId = re.sub(r"\W", "", self.title)
 
 	def _getTranslatedHTML(self):
 		rawHTML = self.original.getContent("html")
