@@ -6,15 +6,16 @@ somehow acquire input (typically, nevow contexts) and then format
 the result for the user.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
+import functools
 import types
 import os
-import urllib.request, urllib.parse, urllib.error
+import urllib.parse
 
 from twisted.internet import threads
 from twisted.python import log
@@ -26,8 +27,11 @@ from gavo import base
 from gavo import svcs
 from gavo import rsc
 from gavo import utils
+from gavo.base import meta
+from gavo.base import typedmeta
 from gavo.formal import nevowc
 from gavo.protocols import creds
+from gavo.protocols import vocabularies
 from gavo.web import common
 from gavo.web import htmltable
 from gavo.web import weberrors
@@ -36,6 +40,15 @@ from gavo.web import weberrors
 __docformat__ = "restructuredtext en"
 
 
+@functools.cache
+def getRelationshipTypes():
+	try:
+		return vocabularies.get_vocabulary("voresource/relationship_type")
+	except base.ReportableError:
+		# that's from get_vocabulary
+		return {"terms": {}}
+
+
 class RDBlocked(Exception):
 	"""is raised when a ResourceDescriptor is blocked due to maintenance
 	and caught by the root resource..
@@ -72,6 +85,12 @@ class GavoRenderMixin(common.CommonRenderers):
 	* rd <rdId> -- makes the referenced RD the current data (this is
 	  not too useful right now, but it lets you check of the existence
 	  of RDs already)
+	
+	Other:
+
+	* dcRelations -- sets the current data to a dictionary of DataCite
+	  relationships of the current resource.  That's probably only
+	  interesting in the default sidebar.
 	"""
 	_sidebar = svcs.loadSystemTemplate("sidebar.html").load()
 	_footer = svcs.loadSystemTemplate("footer.html").load()
@@ -309,6 +328,42 @@ class GavoRenderMixin(common.CommonRenderers):
 				return None
 		return _
 
+	def data_dcRelations(self, request, tag):
+		"""makes [{anchor, href, relname}] for the datacite relationships
+		of the current data.
+		"""
+		relationships = []
+
+		metaCarrier = self.rd
+		# for now, we *only* use relationships on the main RD.  If we
+		# ever want to make this overridable in individual services,
+		# se if there's a service attribute and use that as a metaCarrier
+		# if available.  And then do away with the direct checks in
+		# meta_ below and use normal iterMeta(., propagate=True)
+		if self.macroPackage:
+			expand = self.macroPackage.expand
+		else:
+			expand = lambda v: v
+
+		# I'm peeking directly into meta_ here because the loop is
+		# actually executed quite a number of time and there is no
+		# reason for parsing the keys here anyway.
+		for key in meta.getKeysForClass(typedmeta.RelatedResourceMeta):
+			if key in metaCarrier.meta_:
+				for m in metaCarrier.iterMeta(key):
+					key = key[0].upper()+key[1:]
+					try:
+						relationships.append({
+							"relname": getRelationshipTypes()["terms"
+								].get(key, {"label": key})["label"],
+							"anchor": expand(m.getContent()),
+							"href": expand(base.makeLinkForIvoid(str(m.getMeta("ivoId"))))})
+					except ValueError:
+						# presumably can't make a link from ivoId
+						pass
+
+		return relationships
+
 
 class HTMLResultRenderMixin(object):
 	"""is a mixin with render functions for HTML tables and associated
@@ -801,7 +856,7 @@ class ServiceBasedPage(ResourceBasedPage):
 		# Do not upcall to this if you override getChild.
 		if name==b"":
 			raise svcs.WebRedirect(str(request.URLPath().parent()))
-		raise svcs.UnknownURI("%s has no child resources"%repr(self.name))
+		raise svcs.UnknownURI(f"{self.name} has no child resources")
 
 
 if __name__=="__main__": # pragma: no cover
