@@ -2,7 +2,7 @@
 A renderer for Data to HTML/stan
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -109,14 +109,19 @@ _registerHTMLMF(_simbadMapperFactory)
 
 
 def _bibcodeMapperFactory(colDesc):
-	if colDesc["displayHint"].get("type")!="bibcode":
+	if not (colDesc["displayHint"].get("type")=="bibcode"
+			or colDesc["ucd"].startswith("meta.bib.bibcode")):
 		return
 	def coder(data):
 		if data:
 			for item in data.split(","):
-				urlBibcode = urllib.parse.quote(item.strip())
-				adsLink = f"https://ui.adsabs.harvard.edu/abs/{urlBibcode}/abstract"
-				yield T.a(href=adsLink)[item.strip()]
+				item = item.strip()
+				if utils.couldBeABibcode(item):
+					urlBibcode = urllib.parse.quote(item.strip())
+					adsLink = f"https://ui.adsabs.harvard.edu/abs/{urlBibcode}/abstract"
+					yield T.a(href=adsLink)[item.strip()]
+				else:
+					yield item
 				yield " "
 		else:
 			yield ""
