@@ -2,7 +2,7 @@
 Infrastructure pages.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -11,6 +11,7 @@ Infrastructure pages.
 import datetime
 import io
 import os
+import pathlib
 import time
 
 import pkg_resources
@@ -28,6 +29,7 @@ from gavo import registry
 from gavo import svcs
 from gavo import utils
 from gavo.base import meta
+from gavo.base import sqlsupport
 from gavo.formal import nevowc
 from gavo.protocols import vocabularies
 from gavo.utils import stanxml
@@ -340,10 +342,9 @@ class StaticServer(resource.Resource):
 	"""
 	def __init__(self):
 		resource.Resource.__init__(self)
-		self.userPath = utils.ensureOneSlash(
-			os.path.abspath(os.path.join(base.getConfig("webDir"), "nv_static")))
-		self.systemPath = utils.ensureOneSlash(
-			os.path.abspath(pkg_resources.resource_filename('gavo', "resources/web")))
+		self.userPath = (base.getConfig("webDir") / "nv_static").resolve()
+		self.systemPath = pathlib.Path(
+			pkg_resources.resource_filename('gavo', "resources/web")).resolve()
 
 	def render(self, request):
 		raise svcs.UnknownURI("What did you expect here?")
@@ -352,20 +353,21 @@ class StaticServer(resource.Resource):
 		relPath = "/".join(request.popSegments(name))
 		associatedRD = base.caches.getRD(registry.SERVICELIST_ID)
 
+		# check in the user resources and our built-in files.
 		for basePath in [self.userPath, self.systemPath]:
-			path = os.path.abspath(basePath+relPath)
-			if os.path.exists(path):
-				if not path.startswith(basePath):
-					raise svcs.ForbiddenURI(
-						"%s is not located in a published static directory"%path)
-				return common.compwrap(StaticFile(path, associatedRD))
+			try:
+				path = (basePath / relPath).resolve(strict=True)
+			except FileNotFoundError:
+				# try next prefix
+				continue
 
-		if os.path.exists(path):
-			if not path.startswith(self.systemPath):
+			if not str(path).startswith(str(basePath)):
 				raise svcs.ForbiddenURI(
-					"%s is not located in the static system directory"%path)
+					f"{path} is not located in a published static directory")
 			return common.compwrap(StaticFile(path, associatedRD))
 
+		# built-in magic for files we don't have in the Debian
+		# package.
 		if relPath in DEBIAN_FALLBACKS:
 			res = DEBIAN_FALLBACKS[relPath]
 			if isinstance(res, str):
@@ -446,7 +448,7 @@ class RobotsTxt(resource.Resource):
 	def _getContent(self):
 		content = self.builtin
 		try:
-			with open(os.path.join(base.getConfig("webDir"), "robots.txt"),
+			with open(base.getConfig("webDir") / "robots.txt",
 					encoding="utf-8") as f:
 				content = content+"\n"+f.read()
 		except IOError:
@@ -470,7 +472,8 @@ class ServiceUnavailable(resource.Resource):
 		request.setResponseCode(503)
 		request.setHeader("retry-after", "3600")
 
-		with open(os.path.join(base.getConfig("stateDir"), "MAINT"),
+		with open(
+				base.getConfig("stateDir") / "MAINT",
 				"r", encoding="utf-8") as f:
 			try:
 				maintText = f.read()
@@ -492,14 +495,14 @@ class ServiceUnavailable(resource.Resource):
 class ACMEChallenge(resource.Resource):
 	"""A resource for letsencrypt and friends.
 	"""
-	acmeChallengeDir = os.path.join(base.getConfig("statedir"), "acme-challenge")
+	acmeChallengeDir = base.getConfig("stateDir") / "acme-challenge"
 
 	def render(self, request):
 		# nobody has a reason to request a directory here
 		raise svcs.ForbiddenURI("No listing of challenges")
 	
 	def getChild(self, name, request):
-		fullPath = os.path.join(self.acmeChallengeDir, utils.debytify(name))
+		fullPath = self.acmeChallengeDir / utils.debytify(name)
 		if os.path.exists(fullPath):
 			return static.File(fullPath)
 		raise svcs.UnknownURI("No such challenge known here")
@@ -515,19 +518,55 @@ class WellKnown(resource.Resource):
 		if name==b"acme-challenge":
 			return ACMEChallenge()
 
-		raise svcs.UnknownURI("Only ACME supported in .well-known")
+		elif name==b"security.txt":
+			return StaticFile(
+				base.getPathForDistFile("templates/security.txt"),
+				base.caches.getRD("//services"))
+
+		raise svcs.UnknownURI(f"{name} not supported in .well-known")
 
 
 class CurReaders(resource.Resource):
 	"""A page returning an approximate number of clients served currently.
+
+	This is actually surprisingly hard to compute or even estimate; and for
+	what this is really intended to do -- indicate a safe time to restart --,
+	running async operations, for instance, don't really matter, as they
+	can be resumed.
+
+	So, what I do to figure out how much is going on: look at how many
+	database connections are taken from the untrusted and table connection
+	pools (which are what the server uses).
 	"""
 	def render(self, request):
+		nCheckedOut = 0
+		for manager in [
+				sqlsupport.getUntrustedConn,
+				sqlsupport.getWritableUntrustedConn,
+				sqlsupport.getTableConn,
+				sqlsupport.getWritableTableConn]:
+			if manager.pool:
+				nCheckedOut += len(manager.pool[0]._used)
+
 		request.setHeader("content-type", "text/plain")
-# TODO: the following must be amended for IPv6
-		hostsActive = len([1
-			for r in reactor.getReaders()
-			if hasattr(r, "getHost") and r.getHost().host!='0.0.0.0'])
-		return utils.bytify(str(hostsActive)+"\n")
+		return utils.bytify(str(nCheckedOut)+"\n")
+
+
+class BlockMessage(resource.Resource):
+	"""A resource handed out universally to people on web.root.BLOCKED_HOSTS.
+	"""
+	def render(self, request):
+		try:
+			with open(
+					base.getConfig("stateDir") / "block-message.txt",
+					encoding="utf-8") as f:
+				content = f.read()
+		except IOError:
+			content = ("Sorry, you're blocked.\nThere should be an explanation,"
+				" but alas, there is none.\n")
+
+		request.setHeader("content-type", "text/plain; charset=utf-8")
+		return content.encode("utf-8")
 
 
 class Teapot(resource.Resource):
@@ -617,7 +656,7 @@ class ThirdPartyCachePage(resource.Resource):
 			b"https://aladin.u-strasbg.fr/AladinLite/api/v2/latest/aladin.min.css",
 	}
 
-	cacheDir = os.path.join(base.getConfig("cacheDir"), "3rdparty")
+	cacheDir = base.getConfig("cacheDir") / "3rdparty"
 
 	def render(self, request):
 		raise svcs.UnknownURI("No representation here")
@@ -626,7 +665,7 @@ class ThirdPartyCachePage(resource.Resource):
 		"""returns the name of a file the resource name would be
 		cached under.
 		"""
-		return os.path.join(self.cacheDir, utils.debytify(name))
+		return self.cacheDir / utils.debytify(name)
 
 	def getFromCache(self, name):
 		"""returns a resource serving name if it's alreading in the cache,
