@@ -3,7 +3,7 @@
 Renderers that take services "as arguments".
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -19,6 +19,7 @@ from twisted.web.template import tags as T
 from gavo import base
 from gavo import formal
 from gavo import registry
+from gavo import stc
 from gavo import svcs
 from gavo import utils
 from gavo.web import common
@@ -62,8 +63,8 @@ def makeBibTeXForMetaCarrier(mc):
 			# fall back to something made from the creation date
 			# (which at least won't change between invocations)
 			label = utils.intToFunnyWord(
-				int(re.sub("[^\d]+", "", creationDate.isoformat())))
-	label = "vo:"+re.sub("[^\w]", "_", label)
+				int(re.sub(r"[^\d]+", "", creationDate.isoformat())))
+	label = "vo:"+re.sub(r"[^\w]", "_", label)
 
 	authors = " and ".join(m.getContent()
 				for m in mc.iterMeta("creator.name",
@@ -413,6 +414,23 @@ class RendExplainer(object):
 			" into a HiPS client or click it when you have a javascript-"
 			"enabled browser."]
 
+	@classmethod
+	def _explain_biblinks_json(cls, service):
+		return T.transparent["an endpoint for harvesting pairs of"
+			" of bibliographic identifiers and dataset-related URLs."
+			"  This is probably not interestering to you unless you"
+			" are the ADS.  Retrieve the data from ",
+			T.a(href=service.getURL("biblinks.json"))[
+				service.getURL("biblinks.json")],
+			"."]
+
+	@classmethod
+	def _explain_edition(cls, service):
+		return T.transparent["a version of a document.  See ",
+			T.a(href=service.getURL("edition"))[
+				service.getURL("edition")],
+			" for details."]
+
 	@classmethod
 	def _explainEverything(cls, service):
 		return T.transparent["a renderer with some custom access method that"
@@ -460,6 +478,33 @@ class ServiceInfoRenderer(MetaRenderer, utils.IdManagerMixin):
 				T.xml(note.getContent(targetFormat="html"))
 			for note in sorted(self.rawFootnotes, key=lambda n: n.tag)]]
 
+	@template.renderer
+	def spectral_coverage(self, request, tag):
+		intervals = list(self.describingRD.iterMeta("coverage.spectral"))
+		for m in intervals:
+			lower, upper = map(float, m.getContent().split())
+			# turn from Joule to eV
+			lower = utils.formatFloat(lower/base.ELECTRON_CHARGE)
+			upper = utils.formatFloat(upper/base.ELECTRON_CHARGE)
+			tag[f"{lower} {upper} eV", T.br]
+		return tag
+
+	@staticmethod
+	def _mjdToJYear(mjd):
+		return utils.formatFloat(
+				stc.dateTimeToJYear(
+					stc.mjdToDateTime(float(mjd))))
+
+	@template.renderer
+	def temporal_coverage(self, request, tag):
+		intervals = list(self.describingRD.iterMeta("coverage.temporal"))
+		for m in intervals:
+			lower, upper = m.getContent().split()
+			lower = self._mjdToJYear(lower)
+			upper = self._mjdToJYear(upper)
+			tag[f"{lower} {upper}", T.br]
+		return tag
+
 	def data_internalpath(self, request, tag):
 		return "%s/%s"%(self.service.rd.sourceId, self.service.id)
 
@@ -649,7 +694,7 @@ class TableNoteRenderer(MetaRenderer):
 	def noteHTML(self, request, tag):
 		return T.xml(self.renderedNote)
 
-	loader = formal.XMLString("""
+	loader = formal.XMLString(r"""
 <html xmlns="xmlns=http://www.w3.org/1999/xhtml"
 	xmlns:n="http://nevow.com/ns/nevow/0.1">
 <head>
@@ -733,13 +778,61 @@ class CoverageRenderer(MetaRenderer):
 class EditionRenderer(MetaRenderer):
 	"""A renderer representing a (tutorial-like) text document.
 
-	Not sure yet what I'll do when people actually retrieve these.
-
 	This must have a meta accessURL with the document URI.  It may
 	have a sourceURL meta giving the VCS URI.
 	"""
 	name = "edition"
 
+	@template.renderer
+	def title(self, request, tag):
+		return tag["Editions of “{}”".format(
+			base.getMetaText(self.metaCarrier, "title", default="unnamed document"))]
+	
+	@template.renderer
+	def datameta(self, request, tag):
+		return self._doRenderMeta(request, tag, carrier=tag.slotData)
+
+	@template.renderer
+	def ifdatameta(self, metaName):
+		def _(request, tag):
+			if tag.slotData.getMeta(metaName) is not None:
+				return tag
+			else:
+				return ""
+		return _
+
+	@template.renderer
+	def accurl(self, request, tag):
+		url = base.getMetaText(tag.slotData, "accessURL")
+		return tag(href=url, class_="access-url")[url]
+
+	def data_editions(self, request, tag):
+		return self.metaCarrier.publications or []
+
+	defaultLoader =  common.doctypedStan(
+		T.html[
+			T.head[
+				T.title(render="title"),
+				T.transparent(render="commonhead"),
+			],
+			T.body(render="withsidebar")[
+				T.h1(render="title"),
+				T.p["The following editions of this document are available:"],
+				formal.addNevowAttributes(T.ul, data="editions")(render="sequence")[
+					formal.addNevowAttributes(T.li, pattern="item")[
+						T.a(render="accurl"),
+						T.transparent(render="ifdatameta locTitle") [
+							" with a localised title ",
+							T.em(render="datameta")["locTitle"]],
+						T.transparent(render="ifdatameta languageCode")[
+							"  written in the BCP 47 language ",
+							T.em(class_="langcode", render="datameta")["languageCode"]],
+					".",
+					T.transparent(render="ifdatameta sourceURL") [
+						"  Source code available at ",
+						T.transparent(render="datameta")["sourceURL"]]]
+		]]])
+
 
 class ExternalRenderer(grend.ServiceBasedPage):
 	"""A renderer redirecting to an external resource.
@@ -786,6 +879,9 @@ class RDInfoPage(grend.CustomTemplateMixin, grend.ResourceBasedPage):
 				and not t.hasProperty("internal")),
 			key=lambda t: t.id)
 
+	def data_otherrecs(self, request, tag):
+		return self.rd.resRecs
+
 	def data_clientRdId(self, request, tag):
 		return self.rd.sourceId
 
