@@ -2,7 +2,7 @@
 A renderer that queries a single field in a service.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -58,7 +58,10 @@ class QPRenderer(grend.HTMLResultRenderMixin,
 
 	To adjust the results' appreance, you can override resultline (for when
 	there's just one result row) and resulttable (for when there is more
-	than one result row) templates.
+	than one result row) templates.  In the templates, you can retrieve
+	the input parameter's value as the inPar data, for instance, like this::
+
+		<n:invisible n:data="inPar" n:render="string"/>
 	"""
 	name = "qp"
 	queryValue = None
@@ -112,7 +115,7 @@ class QPRenderer(grend.HTMLResultRenderMixin,
 	def _handleError(self, flr, request):
 		# all errors except svcs.Errors (which are handled by weberror)
 		# are translated to 404s (and logged).
-		if isinstance(flr.value, base.ExecutiveAction):
+		if isinstance(flr.value, (base.ExecutiveAction, svcs.Error)):
 			return flr
 
 		base.ui.notifyFailure(flr, "A core raised '%s' into a qp renderer."
@@ -120,6 +123,9 @@ class QPRenderer(grend.HTMLResultRenderMixin,
 		raise svcs.UnknownURI("The query initiated by your URL failed,"
 			" yielding a message '%s'."%flr.getErrorMessage())
 
+	def data_inPar(self, request, tag):
+		return self.queryValue
+
 	def getChild(self, name, request):
 		# if we're here, we are the responsible resource and just stuff
 		# the remaining segments into the query value
