@@ -2,13 +2,14 @@
 The root resource of the data center.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
 import functools
+import gc
 import os
 import re
 import time
@@ -34,6 +35,13 @@ from gavo.web import weberrors
 
 from gavo.svcs import (UnknownURI, WebRedirect)
 
+
+# you can add IP addresses here of hosts that will receive
+# state/block-message.txt rather than using up our resources.
+# Remain friendly in block-message.txt
+BLOCKED_HOSTS = set()
+
+
 def _escape(s):
 	"""helps formatDefaultLog.
 	"""
@@ -88,8 +96,7 @@ def makeFaviconPNG():
 	This is used mainly for SAMP logos at them moment.
 	"""
 	from gavo.utils import imgtools
-	imgPath = os.path.join(
-		base.getConfig("webDir"), "nv_static", "logo_medium.png")
+	imgPath = base.getConfig("webDir") / "nv_static" / "logo_medium.png"
 	if not os.path.exists(imgPath):
 		imgPath = base.getPathForDistFile("web/img/logo_medium.png")
 	return static.Data(
@@ -182,10 +189,15 @@ class ArchiveService(nevowc.TemplatedPage):
 
 	def __init__(self):
 		nevowc.TemplatedPage.__init__(self)
-		self.maintFile = os.path.join(base.getConfig("stateDir"), "MAINT")
+		self.maintFile = base.getConfig("stateDir") / "MAINT"
 		self.rootSegments = tuple(s for s in
 			base.getConfig("web", "nevowRoot").split("/") if s)
 		self.rootLen = len(self.rootSegments)
+		# At least in python 3.11, we need to occasionally call gc.collect()
+		# manually or cruft accumulates.  We're doing that every 10
+		# requests that do RD operations for now.  Debugging what's
+		# going wrong would of course be preferable...
+		self.rdRequestsSinceLastGC = 0
 
 	@classmethod
 	def addRedirect(cls, key, destination):
@@ -287,9 +299,12 @@ class ArchiveService(nevowc.TemplatedPage):
 	def getChild(self, name, request):
 		segments = request.popSegments(name)
 
+		if request.host.host in BLOCKED_HOSTS:
+			return ifpages.BlockMessage()
+
 		if False:
 			from gavo.helpers import testtricks
-			testtricks.memdebug(common.Request)
+			testtricks.memdebug(base.Structure)
 
 		request.setHeader(b"server", base.SERVER_SOFTWARE)
 
@@ -306,8 +321,9 @@ class ArchiveService(nevowc.TemplatedPage):
 			if upgrader:
 				return upgrader
 		
-		if ((segments and segments[0]!='static')
-				and os.path.exists(self.maintFile)):
+		if ((segments
+					and segments[0]!='static')
+				and self.maintFile.exists()):
 			return ifpages.ServiceUnavailable()
 		if self.rootSegments:
 			if segments[:self.rootLen]!=self.rootSegments:
@@ -338,6 +354,11 @@ class ArchiveService(nevowc.TemplatedPage):
 		if name in self.mappings:
 			segments[:1] = self.mappings[name]
 
+		self.rdRequestsSinceLastGC += 1
+		if self.rdRequestsSinceLastGC>10:
+			gc.collect()
+			self.rdRequestsSinceLastGC = 0
+
 		try:
 			res, postpath = self._locateResourceBasedChild(request, segments)
 			request.pushBackSegments(postpath)
