@@ -4,7 +4,7 @@ Support for IVOA DAL and registry protocols.
 DALI-specified protocols are in dalirender, UWS/async is in asyncrender.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -171,7 +171,8 @@ class DALRenderer(grend.ServiceBasedPage):
 			# remote side has gone away -- avoid triggering ugly errors
 			return
 
-		return dali.serveDALIError(request, errmsg, httpStatus, queryStatus)
+		return dali.serveDALIError(request, errmsg,
+			httpStatus=httpStatus, queryStatus=queryStatus)
 
 	def _formatOutput(self, data, request, stream=True):
 		if isinstance(data, tuple):
@@ -547,8 +548,6 @@ class SSAPRenderer(UnifiedDALRenderer):
 			data.contributingMetaCarriers.append(self.service)
 
 			data.setMeta("_type", "results")
-			data.addMeta("_votableRootAttributes",
-				'xmlns:ssa="http://www.ivoa.net/xml/DalSsap/v1.0"')
 			data.addMeta("info", "SSAP",
 				infoName="SERVICE_PROTOCOL",
 				infoValue=self.version)
