@@ -2,7 +2,7 @@
 Tests for active tags within RDs (and friends).
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -72,8 +72,8 @@ class ReplayMacroTest(testhelpers.VerboseTest):
 
 	def testStandardMacrosAvailable(self):
 		res = base.parseFromString(rscdef.DataDescriptor,
-			"""<data><STREAM id="foo"><table id="test">
-			<column name="x\\upper{\col}"/></table></STREAM>
+			r"""<data><STREAM id="foo"><table id="test">
+			<column name="x\upper{\col}"/></table></STREAM>
 			<FEED source="foo" col="abc"/></data>""")
 		self.assertEqual(res.tables[0].columns[0].name, "xABC")
 
@@ -176,10 +176,12 @@ class ReplayMacroTest(testhelpers.VerboseTest):
 				<table id="y">
 					<FEED source="switch" use="two"/></table></data>""")
 		self.assertEqual(res.tables[0].id, "x")
-		self.assertEqual(len(res.tables[0].columns), 1)
-		self.assertEqual(res.tables[0].columns[0].name, "fromone")
-		self.assertEqual(len(res.tables[1].columns), 1)
-		self.assertEqual(res.tables[1].columns[0].name, "fromtwo")
+		self.assertEqual(
+			testhelpers.pickSingle(res.tables[0].columns).name,
+			"fromone")
+		self.assertEqual(
+			testhelpers.pickSingle(res.tables[1].columns).name,
+			"fromtwo")
 	
 
 class NestedTest(testhelpers.VerboseTest):
@@ -219,10 +221,13 @@ class NestedTest(testhelpers.VerboseTest):
 		res = base.parseFromString(rscdesc.RD,
 			r"""<resource schema="test"><STREAM id="cols">
 					<LOOP listItems="x y">
-					<events><column name="\item"/></events></LOOP></STREAM>
-				<table id="foo"><FEED source="cols"/></table></resource>""")
+					<events><column name="\\mode\\+_\item"/></events></LOOP></STREAM>
+				<table id="foo">
+					<FEED source="cols" mode="a" reexpand="True"/>
+					<FEED source="cols" mode="b" reexpand="True"/>
+				</table></resource>""")
 		self.assertEqual([c.name for c in res.tables[0]],
-			['x', 'y'])
+			['a_x', 'a_y', 'b_x', 'b_y'])
 
 	def testLoopExpansion(self):
 		res = base.parseFromString(rscdesc.RD,
@@ -234,6 +239,31 @@ class NestedTest(testhelpers.VerboseTest):
 		self.assertEqual([c.name for c in res.tables[0]],
 			['x', 'y'])
 
+	def testFeedInLoop(self):
+		res = base.parseFromString(rscdesc.RD,
+			r"""<resource schema="test">
+					<STREAM id="cols">
+						<column name="\foo"/>
+						<column name="err_\foo"/>
+					</STREAM>
+					<STREAM id="coords">
+						<LOOP listItems="x y">
+							<events passivate="True">
+								<FEED source="cols" foo="\\mode\\+\item" reexpand="True"/>
+							</events>
+						</LOOP>
+					</STREAM>
+					<STREAM id="modes">
+						<FEED source="coords" mode="a" reexpand="True"/>
+						<FEED source="coords" mode="b" reexpand="True"/>
+					</STREAM>
+					<table id="abc">
+						<FEED source="modes"/>
+					</table>
+				</resource>""")
+		self.assertEqual([c.name for c in res.tables[0]],
+			['ax', 'err_ax', 'ay', 'err_ay', 'bx', 'err_bx', 'by', 'err_by'])
+
 
 class PruneTest(testhelpers.VerboseTest):
 	def testWithName(self):
@@ -573,32 +603,46 @@ class LoopTest(testhelpers.VerboseTest):
 					<events>
 						<LOOP listItems="a b">
 							<events>
-								<meta name="gob\\item\\+\item">present</meta>
+								<meta name="gob\item\\+\\item">present</meta>
 							</events>
 						</LOOP>
 					</events>
 				</LOOP>
 			</resource>""")
 		self.assertEqual(set(rd.meta_.keys()),
-			set(['gob1b', 'gob1a', 'gob2b', 'gob2a']))
+			set(['gobb1', 'goba1', 'gobb2', 'goba2']))
 
-	def testReexpandedStream(self):
+	def testStreamReexpansion(self):
 		rd = base.parseFromString(rscdesc.RD,
 			r"""<resource schema="data">
-				<STREAM id="pm">
-					<meta name="\\item\\+\thing\+_plus">Present</meta>
-					<meta name="\\item\\+\thing\+_minus">Present</meta>
-				</STREAM>
-				<LOOP listItems="a b" reexpand="True">
-					<events>
-						<meta name="\item">Present</meta>
-						<FEED source="pm" thing="klotz"/>
+			<STREAM id="plusminus">
+				<column name="\prefix\+\basecol\+_error_m"/>
+				<column name="\prefix\+\basecol\+_error_p"/>
+			</STREAM>
+			<table id="t">
+				<LOOP listItems="l1_ l2_">
+					<events passivate="True">
+						<FEED source="plusminus" basecol="dist" prefix="\item"
+							reexpand="True"/>
 					</events>
 				</LOOP>
+			</table>
 			</resource>""")
-		self.assertEqual(set(rd.meta_.keys()),
-			set(['bklotz_plus', 'bklotz_minus', 'b', 'aklotz_plus',
-				'aklotz_minus', 'a']))
+		self.assertEqual(
+			[c.name for c in rd.getById("t").columns], [
+				"l1_dist_error_m", "l1_dist_error_p",
+				"l2_dist_error_m", "l2_dist_error_p"])
+
+	def testBrokenCodeItemMeta(self):
+		self.assertRaisesWithMsg(
+			base.BadCode,
+			"At IO:'<resource schema=\"data\"> <LOOP><codeItems>return = 3<...', (2, 31): Bad source code in function (invalid syntax (<LOOP at <internally built>>, line 2))",
+			base.parseFromString,
+			(rscdesc.RD,
+			r"""<resource schema="data">
+				<LOOP><codeItems>return = 3</codeItems>
+					<events/></LOOP></resource>"""))
+
 
 class RDBasedTest(testhelpers.VerboseTest):
 	def setUp(self):
