@@ -6,7 +6,7 @@ DaCHS-defined meta makers and data functions (at least where concerned
 with parameters contained in SODA).
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -119,6 +119,14 @@ class _MetaMakerTestData(testhelpers.TestResource):
 			<property key="staticData">data</property>
 			<meta name="description">A sample processing service</meta>
 			<datalinkCore>
+				<descriptorGenerator procDef="//soda#fromStandardPubDID">
+					<setup>
+						<code>
+							def addExtras(descriptor):
+								descriptor.bogosity = len(descriptor.pubDID)
+						</code>
+					</setup>
+				</descriptorGenerator>
 				<metaMaker>
 					<code>
 					ik = MS(InputKey, name="format", type="text",
@@ -170,6 +178,28 @@ class _MetaMakerTestData(testhelpers.TestResource):
 							contentQualifier="#something")
 					</code>
 				</metaMaker>
+				<metaMaker>
+					<code>
+						if descriptor.pubDID!='ivo://x-testing/~?data/a.imp':
+							return
+						yield ExternalProcLinkDef(
+							descriptor.pubDID, [
+								MS(InputKey, name="QUERY", type="text",
+									content_="SELECT * FROM some_table_name"),
+								MS(InputKey, name="LANG", type="text",
+									content_="ADQL")],
+							"http://dc.g-vo.org/tap/sync",
+							"GAVO TAP",
+							"TAP service with an alternative representation of the data",
+							standardId="ivo://ivoa.net/std/tap")
+					</code>
+				</metaMaker>
+				<metaMaker semantics="#testing">
+					<code>
+						yield descriptor.makeLink(
+							f"http://bogosity.code/{descriptor.bogosity}")
+					</code>
+				</metaMaker>
 				<dataFunction procDef="//soda#generateProduct"/>
 			</datalinkCore>
 			<publish render="dlmeta" sets="ivo_managed"/>
@@ -258,6 +288,33 @@ class DatalinkMetaMakerTest(testhelpers.VerboseTest):
 			tree.xpath("//PARAM[@name='accessURL']")[0].get("value"),
 			"http://localhost:8080/data/test/foo/dlget")
 
+	def testAddExtras(self):
+		rows = self.serviceResult[1].xpath("//TR[TD[6]='#testing']")
+		self.assertEqual(rows[0][1].text, "http://bogosity.code/28")
+		self.assertEqual(rows[1][1].text, "http://bogosity.code/28")
+
+	def testExternalProc(self):
+		tree = testhelpers.pickSingle(
+			self.serviceResult[1].xpath("RESOURCE[@utype='adhoc:service']["
+				"INFO[@name='title']/@value='GAVO TAP']"))
+		self.assertEqual(
+			testhelpers.pickSingle(tree.xpath("PARAM[@name='standardID']")
+				).get("value"),
+			"ivo://ivoa.net/std/tap")
+		self.assertEqual(
+			testhelpers.pickSingle(tree.xpath("PARAM[@ucd='meta.ref.url']")
+				).get("value"),
+			"http://dc.g-vo.org/tap/sync")
+		self.assertEqual(
+			testhelpers.pickSingle(tree.xpath("GROUP/PARAM[@name='QUERY']")
+				).get("value"),
+			"SELECT * FROM some_table_name")
+
+		refLine = self.serviceResult[1].xpath("//TR[TD='TAP service with an alternative representation of the data']")[0]
+		self.assertEqual(
+			refLine[2].text,  # svc_id
+			tree.get("ID"))
+
 	def testCapability(self):
 		intfEl = self.serviceResult[3].xpath("//interface")[0]
 		self.assertEqual(
@@ -368,6 +425,7 @@ class DatalinkMetaMakerTest(testhelpers.VerboseTest):
 			"core still lives and references s")
 
 
+	
 class _MetaMakerTestRows(testhelpers.TestResource):
 	resources = [
 		("serviceResult", _metaMakerTestData)]
@@ -386,9 +444,10 @@ class DatalinkMetaRowsTest(testhelpers.VerboseTest):
 		("serviceResult", _metaMakerTestData)]
 
 	def testAllLinks(self):
-		self.assertEqual(len(self.rows), 15)
-		for r in list(self.rows.values()):
-			self.assertEqual(len(r), 1)
+		self.assertEqual(len(self.rows), 17)
+		for key, r in list(self.rows.items()):
+			if key[1]!="#proc":
+				self.assertEqual(len(r), 1)
 	
 	def testAllWithId(self):
 		self.assertEqual(set(r[0] for r in self.rows),
@@ -427,6 +486,7 @@ class DatalinkMetaRowsTest(testhelpers.VerboseTest):
 	def testSemantics(self):
 		self.assertEqual(set(r[1] for r in self.rows),
 			set(['#proc', '#this', '#sibling', '#calibration', '#preview',
+				"#testing",
 				"http://dc.g-vo.org/datalink#other",
 				'http://www.g-vo.org/dl#related',
 				'http://www.g-vo.org/dl#unrelated',
@@ -568,11 +628,12 @@ class TimestampTest(testhelpers.VerboseTest):
 		# this sucks -- xtype=interval and xtype=timestamp is mutually
 		# exclusive at this point.  I invent something.
 		pe = tree.xpath("//GROUP[@name='inputParams']/PARAM[@name='t']")
-		self.assertEqual(len(pe), 1)
-		self.assertEqual(pe[0].get("xtype"), "timestamp-interval")
-		self.assertEqual(pe[0].get("arraysize"), "19x2")
-		self.assertEqual(pe[0].get("datatype"), "char")
-		self.assertEqual(pe[0].get("value"),
+		param = testhelpers.pickSingle(pe)
+		self.assertEqual(param.get("xtype"), "timestamp-interval")
+		self.assertEqual(param.get("arraysize"), "19x2")
+		self.assertEqual(param.get("datatype"), "char")
+		self.assertEqual(
+			param.get("value"),
 			"2020-01-01T00:00:002021-01-01T00:00:00")
 
 	def testTimestampsParsed(self):
@@ -589,9 +650,8 @@ class TimestampTest(testhelpers.VerboseTest):
 		s, metadata = votable.loads(data[1])
 		res = list(metadata.iterDicts(s))
 
-		self.assertEqual(len(res), 1)
 		self.assertEqual(
-			(res[0]["ID"], res[0]["error_message"], res[0]["semantics"]),
+			(testhelpers.pickSingle(res)["ID"], res[0]["error_message"], res[0]["semantics"]),
 			("bogus", "NotFoundFault: No bogus data here", "#this"))
 
 	def testTitleMetaInInfo(self):
