@@ -2,7 +2,7 @@
 Some tests for the database interface and the surrounding helper code.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -494,14 +494,14 @@ class TestQueryExpands(TestWithTableCreation):
 	def testExpandedQuery(self):
 		self.table.connection.execute(
 			self.table.expand(
-				"insert into \qName (\colNames) values (-133.0)"))
+				r"insert into \qName (\colNames) values (-133.0)"))
 		self.assertEqual(
 			len(list(self.table.iterQuery([self.tableDef.getColumnByName("foo")],
 				"foo=-133.0"))),
 			1)
 
 	def testBadMacroRaises(self):
-		self.assertRaises(base.MacroError, self.table.expand, "\monkmacrobad")
+		self.assertRaises(base.MacroError, self.table.expand, r"\monkmacrobad")
 
 
 class NewlyCreatedTest(testhelpers.VerboseTest):
@@ -646,7 +646,7 @@ class DerivedViewsDroppingTest(testhelpers.VerboseTest):
 	def testViewScriptRunning(self):
 		with testhelpers.testFile("viewdropping.rd",
 				inDir=base.getConfig("inputsDir"),
-				content="""<resource schema="test">
+				content=r"""<resource schema="test">
 					<table id="to_derive" onDisk="True">
 						<column name="dist"/>
 					</table>
