@@ -4,7 +4,7 @@ Tests for fits helpers
 For FITS binary table tests see formatstest.py
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -434,12 +434,20 @@ class WCSFromHeaderTest(testhelpers.VerboseTest):
 
 	def testMultiDRejected(self):
 		hdr = self._baseHdr.copy()
-		hdr["PC2_2"] = 23.3
+		hdr["PC2_1"] = 23.3
 		self.assertRaisesWithMsg(ValueError,
 			"FITS axis 2 appears not separable.  WCSAxis cannot handle this.",
 			fitstools.WCSAxis.fromHeader,
 			(hdr, 2))
 
+	def testSelfCDOk(self):
+		hdr = self._baseHdr.copy()
+		hdr["CD2_2"] = 148.75
+		hdr["CD1_1"] = 23.3
+		self.assertEqual(
+			fitstools.WCSAxis.fromHeader(hdr, 2).physToPix(23),
+			4527033.940000127)
+
 	def testNameInferenceCtype(self):
 		ax = fitstools.WCSAxis.fromHeader(self._baseHdr, 1)
 		self.assertEqual(ax.name, "WAVELEN_1")
