@@ -19,8 +19,8 @@ export COVERAGE_FILE=testrunner.cov
 $COVERAGE run --source gavo -m gavo.user.cli test -T120 ALL
 kill $srvpid
 
-export COVERAGE_FILE=docgen.cov
-$COVERAGE run --source gavo -m gavo.user.cli gendoc refdoc > /dev/null
+export COVERAGE_FILE=../tests/docgen.cov
+(cd ../docs; $COVERAGE run --source gavo -m gavo.user.cli gendoc refdoc > /dev/null)
 
 export -n COVERAGE_FILE
 $COVERAGE combine unittests.cov regressiontests.cov testrunner.cov\
