@@ -3,7 +3,7 @@
 Tests for grammars and their helpers.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -16,6 +16,8 @@ import os
 import struct
 import unittest
 
+from astropy import table
+
 from gavo.helpers import testhelpers
 
 from gavo import base
@@ -27,6 +29,7 @@ from gavo.grammars import binarygrammar
 from gavo.grammars import columngrammar
 from gavo.grammars import common
 from gavo.grammars import fitsprodgrammar
+from gavo.grammars import fitstablegrammar
 from gavo.grammars import odbcgrammar
 from gavo.grammars import pdsgrammar
 from gavo.grammars import regrammar
@@ -88,8 +91,10 @@ class PredefinedRowfilterTest(testhelpers.VerboseTest):
 		for source in dd.iterSources(None):
 			for row in dd.grammar.parse(source, None):
 				res.add(row["prodtblPreview"])
-		self.assertEqual(res, set(['prefoo/ZGF0YS9hLmltcA==',
-			'prefoo/ZGF0YS9iLmltcA==']))
+
+		self.assertEqual(set(str(p) for p in res),
+			set(['prefoo/ZGF0YS9hLmltcA==',
+				'prefoo/ZGF0YS9iLmltcA==']))
 
 
 class SequencedRowfilterTest(testhelpers.VerboseTest):
@@ -275,11 +280,29 @@ class EmbeddedGrammarTest(testhelpers.VerboseTest):
 		self.assertEqual(getCleaned(rd.dds[0].grammar.parse(None)),
 			[{'y': 2, 'x': 1}, {'y': 2, 'x': 2}])
 
+	def testErrorBehaviour(self):
+		from gavo import rscdesc
+		rd = base.parseFromString(rscdesc.RD,
+			"""<resource schema="test">
+				<table id="bla"><column name="foo" type="text"/></table>
+				<data id="fake"><embeddedGrammar>
+				<iterator><code>
+					for prefix in ["vor", "nach"]:
+						self.location = f"{self.sourceToken}, {prefix}"
+						yield {"foo": prefix+self.sourceToken}
+						UndefinedName
+				</code></iterator></embeddedGrammar><make table="bla"/>
+				</data></resource>""")
+		self.assertRaisesWithMsg(base.SourceParseError,
+			'At sicht, vor: NameError("name \'UndefinedName\' is not defined")',
+			rsc.makeData,
+			(rd.dds[0],), forceSource="sicht")
+
 
 class KVGrammarTest(testhelpers.VerboseTest):
 	def testSimple(self):
 		grammar = base.parseFromString(rscdef.getGrammar("keyValueGrammar"),
-			'<keyValueGrammar commentPattern="--.*?\*/" enc="utf-8"/>')
+			r'<keyValueGrammar commentPattern="--.*?\*/" enc="utf-8"/>')
 		with TestFile("a=b\nc=2 --nothing*/\n"
 				"wonkö:Närd".encode("utf-8")) as f:
 			rec = list(grammar.parse(f))[0]
@@ -412,8 +435,7 @@ class ColumnGrammarTest(testhelpers.VerboseTest):
 			'</columnGrammar>')
 		with TestFile("#Anfang\nMitte\n#Ende") as f:
 			res = list(g.parse(f))
-		self.assertEqual(len(res), 1)
-		self.assertEqual(res[0]['a'], 'Mitte')
+		self.assertEqual(testhelpers.pickSingle(res)['a'], 'Mitte')
 
 	def testWithoutComment(self):
 		g = base.parseFromString(columngrammar.ColumnGrammar,
@@ -435,6 +457,16 @@ class ColumnGrammarTest(testhelpers.VerboseTest):
 			{"a": "Döde"},
 			{"a": "ɑΒ"}])
 
+	def testEndMarker(self):
+		grammar = base.parseFromString(rscdef.getGrammar("columnGrammar"),
+			"""<columnGrammar endMarker="schlussjetzt  $">
+				<colDefs>a: 1-4</colDefs></columnGrammar>""")
+		with TestFile("schluss jetzt\nschlussjetzt nicht\n"
+				"  schlussjetzt wieder nicht\nschlussjetzt  \ndasnicht\n") as f:
+			recs = getCleaned(grammar.parse(f))
+			self.assertEqual(recs,
+				[{'a': 'schl'}, {'a': 'schl'}, {'a': 'sc'}])
+
 
 class BinaryRecordTest(testhelpers.VerboseTest):
 	def testTypes(self):
@@ -536,6 +568,30 @@ class FITSProdGrammarTest(testhelpers.VerboseTest):
 		self.assertEqual(d["__HDUS"][0].data[0][0], 7896.0)
 
 
+class FITSTableGrammarTest(testhelpers.VerboseTest):
+	def testBasic(self):
+		t = table.Table([
+			table.MaskedColumn([1, 0], mask=[False, True]),
+			table.MaskedColumn([2.0, 0], mask=[False, True]),
+			table.MaskedColumn(['foo', ''], mask=[False, True])],
+			names=["an_int", "a_float", "a_string"],
+			dtype=["i4", "f8", str])
+
+		with utils.sandbox():
+			t.write("tmp.fits", format="fits")
+			grammar = base.parseFromString(fitstablegrammar.FITSTableGrammar,
+				"<fitsTableGrammar/>")
+			res = list(grammar.parse("tmp.fits"))
+
+			self.assertEqual(res[0]["an_int"], 1)
+			self.assertEqual(res[0]["a_float"], 2.0)
+			self.assertEqual(res[0]["a_string"], 'foo')
+
+			self.assertEqual(res[1]["an_int"], None)
+			self.assertEqual(res[1]["a_float"], None)
+			self.assertEqual(res[1]["a_string"], None)
+
+
 class ReGrammarTest(testhelpers.VerboseTest):
 	def testBadInputRejection(self):
 		grammar = base.parseFromString(regrammar.REGrammar,
@@ -656,8 +712,9 @@ class PDSGrammarTest(testhelpers.VerboseTest):
 				# PyPDS probably missing, skip this test
 				return
 
-		self.assertEqual(len(recs), 1)
-		self.assertEqual(recs[0]["PRODUCT_ID"], '"S1_00237390711"')
+		self.assertEqual(
+			testhelpers.pickSingle(recs)["PRODUCT_ID"],
+			'"S1_00237390711"')
 
 
 class ContextGrammarTest(testhelpers.VerboseTest):
@@ -857,7 +914,7 @@ class HDF5GrammarTest(testhelpers.VerboseTest):
 
 class _SampleUnionGrammar(testhelpers.TestResource):
 	def make(self, _):
-		return base.parseFromString(uniongrammar.UnionGrammar, """
+		return base.parseFromString(uniongrammar.UnionGrammar, r"""
 			<unionGrammar>
 				<handles pattern="\.txt$">
 					<reGrammar names="a,b,c"/>
