@@ -2,7 +2,7 @@
 Tests for our low-level VOTable interface.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -100,8 +100,7 @@ class TrivialParseTest(testhelpers.VerboseTest):
 	def testTrivialWatchlist(self):
 		res = list(votable.parseBytes("<VOTABLE/>",
 			watchset=[V.VOTABLE]))
-		self.assertEqual(len(res), 1)
-		self.assertTrue(isinstance(res[0], V.VOTABLE))
+		self.assertTrue(isinstance(testhelpers.pickSingle(res), V.VOTABLE))
 	
 	def testTrivialWithNamespace(self):
 		res = list(votable.parseBytes(
@@ -1274,8 +1273,8 @@ class VOTableDataValidationTest(testhelpers.VerboseTest):
 		vot = V.VOTABLE[V.RESOURCE[votable.DelayedTable(
 			V.TABLE[V.FIELD(datatype="char", arraysize="*", xtype="json")],
 			[['"🄁"']], V.TABLEDATA)]]
-		self.assertTrue(b"Will not produce utf-8 surrogates"
-			in votable.asBytes(vot))
+		content = votable.asBytes(vot)
+		self.assertTrue(b"Will not produce utf-8 surrogates" in content)
 
 
 class ArraySizeValidationTest(testhelpers.VerboseTest):
@@ -1362,8 +1361,7 @@ class WeirdTablesTest(testhelpers.VerboseTest):
 		dest.seek(0)
 		data, metadata = votable.load(dest)
 		self.assertEqual(len(data), 20)
-		self.assertEqual(len(data[0]), 1)
-		self.assertEqual(len(data[0][0]), 1000)
+		self.assertEqual(len(testhelpers.pickSingle(data[0])), 1000)
 
 	def testUnknownAttributesFails(self):
 		it = votable.parseBytes('<VOTABLE><RESOURCE class="upper"/></VOTABLE>')
@@ -1480,7 +1478,7 @@ class StanXMLTest(testhelpers.VerboseTest):
 		vot = V.VOTABLE[
 			V.INFO(name="QUERY_STATUS", value="ERROR")["Nothing: µ-test"]]
 		ser = vot.render()
-		self.assertEqual(ser, b'<VOTABLE version="1.4" xmlns="http://www.ivoa.net/xml/VOTable/v1.3" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.ivoa.net/xml/VOTable/v1.3 http://vo.ari.uni-heidelberg.de/docs/schemata/VOTable-1.4.xsd"><INFO name="QUERY_STATUS" value="ERROR">Nothing: \xc2\xb5-test</INFO></VOTABLE>')
+		self.assertEqual(ser, b'<VOTABLE version="1.5" xmlns="http://www.ivoa.net/xml/VOTable/v1.3" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.ivoa.net/xml/VOTable/v1.3 http://vo.ari.uni-heidelberg.de/docs/schemata/VOTable.xsd"><INFO name="QUERY_STATUS" value="ERROR">Nothing: \xc2\xb5-test</INFO></VOTABLE>')
 
 
 class _BOMB(V._VOTElement):
