@@ -2,7 +2,7 @@
 Tests for RD metadata management.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -22,7 +22,6 @@ from gavo.base import metavalidation
 from gavo.formal import nevowc
 from gavo.registry import builders
 from gavo.web import common as webcommon
-from gavo.web import metarender
 
 
 class KeyTest(testhelpers.VerboseTest):
@@ -190,7 +189,7 @@ class SequenceTest(testhelpers.VerboseTest):
 		m.addMeta("creator.name", "grub")
 		m.addMeta("creator.name", "grob")
 		self.assertRaisesWithMsg(meta.MetaCardError,
-			"getContent not allowed for sequence meta items (key creator.name)",
+			"getContent not allowed for sequence meta items ([<meta grub>, <meta grob>]). (key creator.name)",
 			meta.getMetaText,
 			(m, "creator.name"))
 
@@ -418,9 +417,9 @@ def parseMetaXML(src):
 	return base.parseFromString(_MetaCarrier, "<m>"+src+"</m>")
 
 
-def getMetaHTMLFor(rdMetaXML, metaName):
+def getMetaHTMLFor(rdMetaXML, metaName, macroPackage=None):
 	mc = parseMetaXML(rdMetaXML)
-	builder = webcommon.HTMLMetaBuilder()
+	builder = webcommon.HTMLMetaBuilder(macroPackage)
 	return nevowc.flattenSync(mc.buildRepr(metaName, builder)).decode("utf-8")
 
 
@@ -456,12 +455,12 @@ class SpecialsTest(testhelpers.VerboseTest):
 			"olds", foo="x")
 
 	def testLinks(self):
-		html = getMetaHTMLFor("""<meta>
-			_related: http://anythi.ng
+		html = getMetaHTMLFor(r"""<meta>
+			_related: \internallink{foo/bar/baz}
 			_related.title: Link 1
-			</meta>""", "_related")
+			</meta>""", "_related", macroPackage=testhelpers.getTestRD())
 		self.assertEqual(html,
-			'<a href="http://anythi.ng">Link 1</a>')
+			'<a href="http://localhost:8080/foo/bar/baz">Link 1</a>')
 
 	def testNews(self):
 		m = parseMetaXML("""<meta name="_news" date="2009-03-06" author="MD"
@@ -567,13 +566,13 @@ class SpecialsTest(testhelpers.VerboseTest):
 			'<ul class="metaEnum"><li class="metaItem"><span class="plainmeta">Example, J.F.</span></li><li class="metaItem"><img class="metalogo" src="http://imgs.example.org/logo.png" alt="[Logo]"/></li></ul>')
 
 
-
 class DatetimeSpecialsTest(testhelpers.VerboseTest):
 	def testBasic(self):
 		m = parseMetaXML("""<meta>_dataUpdated:1992-12-23T12:11:30Z</meta>""")
 		self.assertEqual(m.getMeta("_dataUpdated").getContent("text"),
 			"1992-12-23T12:11:30Z")
 
+
 def getRadioMeta():
 	m = base.MetaMixin()
 	m.addMeta("radio", "on")
@@ -716,14 +715,6 @@ class XMLTest(testhelpers.VerboseTest):
 		self.assertEqual(str(mc.getMeta("radio.freq")), "90.9")
 		self.assertEqual(str(mc.getMeta("radio.unit")), "MHz")
 
-	def testLink(self):
-		"""tests for working recognition of link-typed metas.
-		"""
-		mc = parseMetaXML('<meta name="_related" title="a link">'
-			'http://foo.bar</meta>')
-		self.assertEqual(mc.getMeta("_related").getContent("html"),
-			'<a href="http://foo.bar">a link</a>')
-	
 	def testRst(self):
 		mc = parseMetaXML('<meta name="bla" format="rst">A\n'
 			'  text that is somewhat indented\n'
@@ -740,15 +731,6 @@ class XMLTest(testhelpers.VerboseTest):
 		self.assertEqual(mc.getMeta("bla").children[0].content,
 			'\nIndented\n\tmess')
 	
-	def testCustomMetaIndentation(self):
-		mc = parseMetaXML('\t\t<meta name="note" tag="6">\n'
-			'\t\t\tA zero for the number of used images indicates that all images\n'
-			'\t\t\thave some "problem" (such as overexposure).\n'
-			'\t\t</meta>')
-		self.assertEqual(mc.getMeta("note").children[0].content,
-			'\nA zero for the number of used images indicates that all images\n'
-			'have some "problem" (such as overexposure).')
-
 	def testAllEmpty(self):
 		parseMetaXML("<meta/>")
 		# assertion: just parses
@@ -917,7 +899,7 @@ class MacroExpansionText(testhelpers.VerboseTest):
 			self.assertEqual(
 				str(exc),
 				"On <Table definition of test.test>: getContent not allowed"
-				" for sequence meta items")
+				" for sequence meta items ([<meta eins>, <meta zwei>]).")
 			self.assertEqual(
 				str(exc.pos),
 				'IO:\'<resource schema="test"> <table id="test"> <LOOP> <co...\', line 3')
@@ -953,6 +935,21 @@ class MacroExpansionText(testhelpers.VerboseTest):
 		self.assertEqual(t.params.getColumnByName("y").value, "eins and zwei")
 
 
+class ModelValidatorParseTest(testhelpers.VerboseTest):
+	def testBadAssertion(self):
+
+		class Silly(base.Structure):
+			metaModel = "somemeta(gnork)"
+			name_ = "silly"
+		instance = base.parseFromString(Silly, "<silly/>")
+
+		self.assertRaisesWithMsg(
+			base.StructureError,
+			"Invalid meta model on Silly: Expected end of text, found '('  (at char 8), (line:1, col:9)",
+			metavalidation._validateMetaCarrier,
+			(instance,))
+
+
 class ModelValidationTest(testhelpers.VerboseTest):
 	def setUp(self):
 		self.model = metavalidation.parseModel(self.modelDesc)
@@ -1050,6 +1047,35 @@ class ExistsAtomicValidationTest(ModelValidationTest):
 		self.assertValidates(m)
 
 
+class ProdTypeVocabularyValidationTest(ModelValidationTest):
+	modelDesc = "productTypeServed(? voc:product-type)"
+
+	def testGoodSingleTerm(self):
+		m = _MetaCarrier(None)
+		m.setMeta("productTypeServed", "image")
+		self.assertValidates(m)
+
+	def testGoodMultipleTermsBad(self):
+		m = _MetaCarrier(None)
+		m.addMeta("productTypeServed", "image")
+		m.addMeta("productTypeServed", "balkon")
+		self.assertFailures(m, ["productTypeServed meta only admits values from product-type, but 'balkon' is not in there."])
+
+	def testEmptyOk(self):
+		m = _MetaCarrier(None)
+		self.assertValidates(m)
+
+	def testCombination(self):
+		m = _MetaCarrier(None)
+		model = metavalidation.parseModel("productTypeServed(! voc:product-type)")
+
+		self.assertRaisesWithMsg(
+			base.MetaValidationError,
+			"Meta structure on _MetaCarrier item (within <internally built>) did not validate: Meta key productTypeServed missing",
+			model.validate,
+			(m,))
+
+
 class StructureValidationTest(testhelpers.VerboseTest):
 	def setUp(self):
 		class Foo(_MetaCarrier):
@@ -1111,7 +1137,17 @@ class _NonComputer(meta.ComputedMetaMixin):
 		return "abc"+"x"
 
 
+class _PublishedRD(testhelpers.TestResource):
+	def make(self, deps):
+		from gavo.registry import publication
+		rd = base.caches.getRD("data/cores")
+		publication.updateServiceList([rd], keepTimestamp=True)
+		return rd
+
+
 class ComputedMetaTest(testhelpers.VerboseTest):
+	resources = [("publishedRD", _PublishedRD())]
+
 	def testBasic(self):
 		f = _NonComputer()
 		self.assertEqual(f.getMeta("computed").getContent(), "abcx")
@@ -1130,6 +1166,16 @@ class ComputedMetaTest(testhelpers.VerboseTest):
 		copy.copyMetaFrom(original)
 		self.assertEqual(str(copy.getMeta("computed")), "abcx")
 
+	def testPublishedIdentifier(self):
+		svc = self.publishedRD.getById("scs")
+		self.assertEqual({m.getContent()
+			for m in svc.iterMeta("published_identifier")}, {
+				'ivo://x-testing/data/cores/cstest',
+				'ivo://x-testing/data/cores/dl',
+				'ivo://x-testing/data/cores/scs',
+				'ivo://x-testing/data/cores/pc',
+				'ivo://x-testing/data/cores/singlearg'})
+
 
 class DeprecatedTest(testhelpers.VerboseTest):
 	def testHierarchicalRenaming(self):
@@ -1166,88 +1212,6 @@ class DeprecatedTest(testhelpers.VerboseTest):
 			["foo", "quux"])
 
 
-class BibTeXTest(testhelpers.VerboseTest):
-	def testBasic(self):
-		mc = parseMetaXML("<meta>creator.name: Thor, A.U.\n"
-			"creator:\ncreator.name: Bohr, N.\ntitle:On GAVO's role"
-			" in Writing DaCHS &amp; \\\\expandafter\ncreationDate: 1984-08-05\n"
-			"referenceURL:http://purl.org/pub/thor+bohr\n"
-			"shortName:testres\n"
-			"doi:10.542/testing</meta>")
-		self.assertEqual(metarender.makeBibTeXForMetaCarrier(mc),
-			r"""@MISC{vo:testres,
-  year=1984,
-  title={On {GAVO}'s role in Writing {DaCHS} \& \\expandafter},
-  author={Thor, A.U. and Bohr, N.},
-  url={http://purl.org/pub/thor+bohr},
-  howpublished={{VO} resource provided by the Unittest Suite},
-  doi = {10.542/testing}
-}
-""")
-
-	def testIncompleteMetaErrors(self):
-		mc = parseMetaXML("<meta>creator.name: Thor, A.U.</meta>")
-		self.assertRaisesWithMsg(base.MetaError,
-			"On <test meta carrier>: No meta item creationDate",
-			metarender.makeBibTeXForMetaCarrier,
-			(mc,))
-	
-	def testIdFallback1(self):
-		mc = parseMetaXML("<meta>creator.name: Thor, A.U.\n"
-			"creationDate: 1984-08-05T13:45:23\ntitle: Ulysses\n"
-			"referenceURL:http://purl.org/pub/thor\n</meta>")
-		self.assertTrue(metarender.makeBibTeXForMetaCarrier(mc
-			).startswith("@MISC{vo:eiabtliumabb,"))
-
-	def testTableNameMangling(self):
-		mc = parseMetaXML("<meta>creator.name: Thor, A.U.\n"
-			"creationDate: 1984-08-05T13:45:23\ntitle: ToC\n"
-			"referenceURL:http://purl.org/pub/thor\n</meta>")
-		mc.getQName = lambda: "mySchema.someTable"
-		self.assertTrue(metarender.makeBibTeXForMetaCarrier(mc
-			).startswith("@MISC{vo:mySchema_someTable,"))
-
-	def testNoAuthor(self):
-		mc = parseMetaXML("<meta>creationDate: 1984-08-05T13:45:23\n"
-			"title: Der hessische Landbote\n"
-			"referenceURL:http://purl.org/pub/thor\n</meta>")
-		# The fallback to the DC creator is probably a bad idea here.
-		# Well, any other behaviour is going to be still more painful.
-		self.assertTrue(
-			"author={Could be same as contact.name},"
-			in metarender.makeBibTeXForMetaCarrier(mc))
-
-
-class RightsTest(testhelpers.VerboseTest):
-	def testCopyrightMaps(self):
-		self.assertEqual(
-			getMetaHTMLFor(
-				'<meta name="copyright">Free to use</meta>',
-				"rights"),
-			'<span class="plainmeta">Free to use</span>')
-
-	def testCombiningSynonyms(self):
-		self.assertEqual(
-			getMetaHTMLFor(
-				'<meta name="_copyright">Do not dare touch this</meta>'
-				'<meta name="copyright">Free to use</meta>'
-				'<meta name="rights" format="rst">http://xkcd.com/2228/</meta>',
-				"rights"),
-			'<ul class="metaEnum"><li class="metaItem"><span class="plainmeta">'
-			'Do not dare touch this</span></li>'
-			'<li class="metaItem"><span class="plainmeta">Free to use</span></li>'
-			'<li class="metaItem"><p><a class="reference external"'
-			' href="http://xkcd.com/2228/">http://xkcd.com/2228/</a></p>'
-			'\n</li></ul>')
-
-	def testRenamingHappens(self):
-		self.assertRaisesWithMsg(
-			base.NoMetaKey,
-			"On <test meta carrier>: No meta item copyright",
-			getMetaHTMLFor,
-			('<meta name="_copyright">Do not dare touch this</meta>', "copyright"))
-
-
 class MetaStreamFormatTest(testhelpers.VerboseTest):
 	def testDefaultFormat(self):
 		res = getMetaHTMLFor("""<meta>
