@@ -2,7 +2,7 @@
 Some unit tests not yet fitting anywhere else.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -562,6 +562,28 @@ Cheers,
 		self.assertTrue("To: gnubbel@somewhere.org" in res)
 		self.assertTrue(re.search("Date: .*GMT", res))
 
+	def testIvoidExternal(self):
+		self.assertEqual(
+			base.makeLinkForIvoid("ivo://leibniz-kis/chrotel/q/epn_core"),
+			"https://dc.g-vo.org/LP/ivo://leibniz-kis/chrotel/q/epn_core")
+	
+	def testIvoidMalformed(self):
+		self.assertRaisesWithMsg(
+			ValueError,
+			"Not an ivoid: ''",
+			base.makeLinkForIvoid,
+			("",))
+	
+	def testIvoidInternalNonResolving(self):
+		self.assertEqual(
+			base.makeLinkForIvoid("ivo://x-testing/toss/q/line_tap"),
+			"https://dc.g-vo.org/LP/ivo://x-testing/toss/q/line_tap")
+
+	def testIvoidInternalResolving(self):
+		self.assertEqual(
+			base.makeLinkForIvoid("ivo://x-testing/data/cores/dl"),
+			"http://localhost:8080/data/cores/dl/info")
+
 
 import lxml
 from gavo.helpers import testtricks
@@ -637,10 +659,12 @@ class PgValidatorTest(testhelpers.VerboseTest):
 
 from gavo.user import validation
 
-class GavoTableValTest(testhelpers.VerboseTest):
+class TableValTest(testhelpers.VerboseTest):
+	resources = [("ignored", tresc.csTestTable)]
+
 	class defaultArgs:
 		compareDB = False
-
+	
 	def _getRdWithTable(self, columns):
 		rd = base.parseFromString(rscdesc.RD,
 			"""<resource schema="__test"><table id="totest" onDisk="True">
@@ -675,6 +699,20 @@ class GavoTableValTest(testhelpers.VerboseTest):
 			"<column name='foo' ucd='em.IR'/>"),
 			"[WARNING] testing/q: Column __test.totest.foo: UCD em.IR not accepted by astropy (Secondary word 'em.IR' is not valid as a primary word).\n")
 
+	def testTAPSchemaValidation(self):
+		td = testhelpers.getTestRD().getById("csdata")
+		prev = td.columns[0].ucd
+		try:
+			td.columns[0].ucd = "blabla"
+			self.assertEqual(
+				list(validation.validateTable(td, True)), [
+					('warning', 'Column test.csdata.alpha: UCD blabla not'
+						" accepted by astropy (Unknown word 'blabla')."),
+					('error',
+						"Table test.csdata: Column alpha: tap_schema ucd!='blabla'")])
+		finally:
+			td.columns[0].ucd = prev
+
 
 from gavo.web import examplesrender
 
@@ -831,7 +869,7 @@ class VOTableColGenTest(testhelpers.VerboseTest):
 		table = api.TableForDef(api.resolveCrossId("data/test#abcd"))
 		inF = io.BytesIO(votable.getAsVOTable(table))
 		self.assertEqual(
-			mkrd.getColumnXML(mkrd.iterColAttrsVOTable(inF)),
+			mkrd.getColumnXML(mkrd.iterColAttrsVOTable(inF, 1)),
 			'<column name="a" type="text"\n'
   		'  ucd=""\n'
   		'  description="Some weirdness"\n'
@@ -1024,7 +1062,7 @@ class HipsGenTest(testhelpers.VerboseTest):
 hips_creator         = Gandalf, W.
 bib_reference        = Gallilei, G., in prep.
 #obs_regime          = Waveband keyword (Radio
-em_max               = 1.9864458571489288e-18
+em_max               = 6.621486190496428e-06
 """)
 
 
@@ -1040,7 +1078,7 @@ class BiblinksImportTest(testhelpers.VerboseTest):
 		
 		self.assertEqual(len(created), 4)
 		self.assertEqual(set(r["bib_ref"] for r in created
-			if r["dataset_ref"]=='http://reg.g-vo.org/rr/q/lp/x-testing/data/testdata/haslinks'),
+			if r["dataset_ref"]=='http://reg.g-vo.org/LP/x-testing/data/testdata/haslinks'),
 			{'2015A&C....10...88D', '2005ASPC..347...29T'})
 		self.assertEqual(set(r["relationship"] for r in created),
 			{"Cites", "IsSupplementedBy"})
@@ -1057,5 +1095,46 @@ class BiblinksImportTest(testhelpers.VerboseTest):
 		return self.testImport()
 
 
+from gavo.rscdef import regtest
+
+_EMPTY_VOTABLE = b"""
+<VOTABLE version="1.4" xmlns="http://www.ivoa.net/xml/VOTable/v1.3">
+	<RESOURCE type="results">
+		<TABLE name="result">
+			<FIELD ID="hipno"  arraysize="*" datatype="char" name="hipno" ucd="ID_MAIN"/>
+			<DATA><BINARY><STREAM encoding="base64"/></BINARY>
+			</DATA></TABLE></RESOURCE></VOTABLE>"""
+
+_ERROR_VOTABLE = b"""
+<VOTABLE version="1.4" xmlns="http://www.ivoa.net/xml/VOTable/v1.3">
+	<RESOURCE type="results">
+		<INFO ID="Error" name="Error" value="Field SR: 'fromp' is not a valid literal for SR"/></RESOURCE></VOTABLE>"""
+
+
+class RegTestTest(testhelpers.VerboseTest):
+	def testGetRowEmptyResponse(self):
+		t = api.makeStruct(regtest.RegTest, title="Testtest")
+		t.data = _EMPTY_VOTABLE
+		self.assertEqual(t.getVOTableRows(), [])
+
+	def testGetRowErrorResponse(self):
+		t = api.makeStruct(regtest.RegTest, title="Testtest")
+		t.data = _ERROR_VOTABLE
+		self.assertRaisesWithMsg(
+			AssertionError,
+			"Response did not contain a TABLE -- is it an error message?",
+			t.getVOTableRows,
+			())
+
+	def testGetFirstErrorResponse(self):
+		t = api.makeStruct(regtest.RegTest, title="Testtest")
+		t.data = _ERROR_VOTABLE
+		self.assertRaisesWithMsg(
+			AssertionError,
+			"Response did not contain a TABLE -- is it an error message?",
+			t.getFirstVOTableRow,
+			())
+
+
 if __name__=="__main__":
 	testhelpers.main(KVLMakeTest)
