@@ -1,8 +1,8 @@
 """
-Tests pertaining to the parsing system
+Tests pertaining to the data parsing system
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -29,6 +29,29 @@ from gavo.rsc import dumping
 import tresc
 
 
+class RowValidationTest(testhelpers.VerboseTest):
+	def testOptional(self):
+		rd = testhelpers.getTestRD()
+		recDef = rd.getTableDefById("valSpec")
+		rec = {}
+		try:
+			recDef.validateRow(rec)
+		except base.ValidationError as ex:
+			self.assertEqual(ex.colName, "a_num")
+		rec["enum"] = "abysimal"
+		self.assertRaises(base.ValidationError, recDef.validateRow,
+			rec)
+		rec["a_num"] = 14
+		self.assertTrue(recDef.validateRow(rec)==None)
+	
+	def testBadSchema(self):
+		self.assertRaisesWithMsg(base.StructureError,
+			'At IO:\'<resource schema="a;drop table foo"/>\', (1, 37):'
+			' DaCHS schema attributes must be valid python identifiers',
+			base.parseFromString,
+			(rscdesc.RD, '<resource schema="a;drop table foo"/>'))
+
+
 @contextlib.contextmanager
 def _inputFile(fName, content):
 	with open(fName, "w") as f:
