@@ -2,7 +2,7 @@
 Tests having to do with the registry code.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -378,8 +378,7 @@ class SIAPCapabilityTest(testhelpers.VerboseTest):
 	
 	def testImageSizeAtomic(self):
 		children = self.txNTree[1].xpath("//maxImageSize")
-		self.assertEqual(len(children), 1)
-		self.assertEqual(children[0].text, "3000")
+		self.assertEqual(testhelpers.pickSingle(children).text, "3000")
 
 	def testTestQuery(self):
 		tqEl = self.txNTree[1].xpath("/capability/testQuery")[0]
@@ -450,11 +449,11 @@ class SIAP2RecordTest(testhelpers.VerboseTest, testhelpers.XSDTestMixin):
 	def testValid(self):
 		self.assertValidates(self.txNTree[0])
 
-	def testDerivedFromTAP(self):
+	def testDerivedServedByObscore(self):
 		self.assertEqual(
 			self.txNTree[1].xpath("content/relationship["
-				"relationshipType='IsDerivedFrom']/relatedResource")[0].text,
-			"ivo://x-testing/tap")
+				"relationshipType='IsServedBy']/relatedResource/@ivo-id")[0],
+			"ivo://x-testing/__system__/obscore/obscore")
 	
 	def testSpatialCoverage(self):
 		self.assertEqual(
@@ -567,57 +566,11 @@ class _DocRec(testhelpers.TestResource):
 		base.setConfig("ivoa", "registerAlternative", "True")
 		try:
 			rd = base.parseFromString(rscdesc.RD,
-				"""<resource schema="test">
+				rf"""<resource schema="test">
 					<macDef name="waran">Warning: Waran</macDef>
 					<meta name="description">We should not have any backslashes
 						here: \waran.</meta>
-					<resRec id="rec">
-						<meta>
-							creationDate: 2010-10-10T10:10:10Z
-							_dataUpdated: 2015-10-10T09:09:09
-							resType: document
-							status: active
-							title: a test document
-							subject: testing
-							referenceURL: http://bar/doc/fancy
-							identifier: ivo://foo.bar
-							uses: Sample useful resource
-							uses.ivoId: ivo://useful/sample1
-							uses: Crapola junkyard bad example
-							uses.ivoId: ivo://crapola/work
-							_news: Cool date
-							_news.date: 2010-10-10
-							_news.role: silly
-							_news: Cooler date
-							_news.date: 2011-11-11
-							tableset: //users#groups
-							tableset: data/test#typesTable
-						</meta>
-
-						<publish render="edition" sets="ivo_managed,local">
-							<meta>
-								accessURL: http://foo.us/fancy/doc
-								sourceURL: svn://foo.us/docs/src/fancy
-							</meta>
-						</publish>
-
-						<publish render="edition" sets="ivo_managed">
-							<meta>
-								accessURL: http://foo.fr/ycnaf/cod
-								sourceURL: git+ssh://foo.fr/scod/src/ycnaf
-								languageCode: fr
-							</meta>
-						</publish>
-
-						<publish render="edition" sets="ivo_managed">
-							<meta>
-								accessURL: http://localhost:8080/fancy/doc
-								languageCode: de
-								locTitle: Ja, geh zu.
-							</meta>
-						</publish>
-
-					</resRec>
+					{tresc.A_RES_REC}
 				</resource>""")
 
 			return getGetRecordResponse(rd.getById("rec"))
@@ -640,8 +593,7 @@ class DocumentResTest(testhelpers.VerboseTest, testtricks.XSDTestMixin):
 	def testLocTitle(self):
 		els = self.srcAndTree[1].xpath(
 			"//capability[languageCode='de']/locTitle")
-		self.assertEqual(len(els), 1)
-		self.assertEqual(els[0].text, "Ja, geh zu.")
+		self.assertEqual(testhelpers.pickSingle(els).text, "Ja, geh zu.")
 
 	def testAccessRenderedURLs(self):
 		urls = self.srcAndTree[1].xpath("//interface[@role='rendered']/accessURL")
@@ -656,8 +608,7 @@ class DocumentResTest(testhelpers.VerboseTest, testtricks.XSDTestMixin):
 
 	def testRelationships(self):
 		relations = self.srcAndTree[1].xpath("//relationship")
-		self.assertEqual(len(relations), 1)
-		t = relations[0].xpath("relationshipType")[0]
+		t = testhelpers.pickSingle(relations).xpath("relationshipType")[0]
 
 		self.assertEqual(t.text, "Cites")
 		ids = relations[0].xpath("relatedResource")
@@ -666,11 +617,10 @@ class DocumentResTest(testhelpers.VerboseTest, testtricks.XSDTestMixin):
 
 	def testNewsRenderingWithRole(self):
 		sillyDates = self.srcAndTree[1].xpath("//date[@role='silly']")
-		self.assertEqual(len(sillyDates), 1)
-		self.assertEqual(sillyDates[0].text, "2010-10-10")
+		self.assertEqual(testhelpers.pickSingle(sillyDates).text, "2010-10-10")
 	
 	def testNewsRenderingDefaultRole(self):
-		upDates = self.srcAndTree[1].xpath("//date[@role='updated']")
+		upDates = self.srcAndTree[1].xpath("//date[@role='Updated']")
 		# there's two updates: one from news, another from the _dataUpdated meta
 		self.assertEqual(len(upDates), 2)
 		self.assertEqual(
@@ -728,20 +678,11 @@ class DataPublicationMetaTest(testhelpers.VerboseTest):
 			list,
 			(self.resg.parse(rd),))
 
-	_minimalMeta = """
-			<meta>
-				title:x
-				creationDate:2020-03-05T12:13:14Z
-				description:z
-				subject:a
-				referenceURL:b
-			</meta>"""
-
 	def testIterDataTable(self):
-		rd = base.parseFromString(rscdesc.RD, """<resource schema="data">
-			%s
+		rd = base.parseFromString(rscdesc.RD, f"""<resource schema="data">
+			{tresc.MINIMAL_META}
 			<table id="ronk"><register sets="ivo_managed,local"/>
-			</table></resource>"""%self._minimalMeta)
+			</table></resource>""")
 		recs = list(self.resg.parse(rd))
 
 		self.assertEqual(len(recs), 6)
@@ -768,12 +709,12 @@ class DataPublicationMetaTest(testhelpers.VerboseTest):
 		self.assertEqual(sets, {"ivo_managed", "local"})
 
 	def testIterDataData(self):
-		rd = base.parseFromString(rscdesc.RD, """<resource schema="data">
-			%s
+		rd = base.parseFromString(rscdesc.RD, f"""<resource schema="data">
+			{tresc.MINIMAL_META}
 			<table id="ronk"/><table id="funk"/>
 			<data id="ronkcoll"><register sets="ivo_managed,local"/>
 			<make table="ronk"/><make table="funk"/>
-			</data></resource>"""%self._minimalMeta)
+			</data></resource>""")
 		rd.sourceId = "data/test"
 		recs = list(self.resg.parse(rd))
 
@@ -821,6 +762,20 @@ class DataPublicationMetaTest(testhelpers.VerboseTest):
 			[],
 			"registrytest._PublishedData.clean failed?")
 
+	def testContentLevelChecked(self):
+		rec = base.parseFromString(nonservice.ResRec,
+			f"""<resRec>
+				{tresc.MINIMAL_META}
+				<meta>
+				resType: document
+				contentLevel: University
+				</meta></resRec>""")
+		self.assertRaisesWithMsg(
+			base.MetaValidationError,
+			"Meta structure on ResRec item (within IO:'<resRec> <meta> title:x creationDate:2020-03-05T12:13...', line 1) did not validate: contentLevel meta only admits values from voresource/content_level, but 'University' is not in there.",
+			base.validateStructure,
+			(rec,))
+
 
 class _ResobGrammarResponse(testhelpers.TestResource):
 	"""A resource giving the records the grammar in registry.publication
@@ -847,9 +802,9 @@ class ResobGrammarTest(testhelpers.VerboseTest):
 	def testSubjects(self):
 		self.assertEqual([r for r in self.tables["subjects"]
 				if r["sourceRD"]!="__system__/tap"], [
-			{'sourceRD': 'data/ssatest', 'resId': 's', 'subject': 'Testing'},
-			{'sourceRD': 'data/ssatest', 'resId': 'c', 'subject': 'Testing'},
-			{'resId': 'hcdtest', 'sourceRD': 'data/ssatest', 'subject': 'Testing'}])
+			{'sourceRD': 'data/ssatest', 'resId': 's', 'subject': 'testing'},
+			{'sourceRD': 'data/ssatest', 'resId': 'c', 'subject': 'testing'},
+			{'resId': 'hcdtest', 'sourceRD': 'data/ssatest', 'subject': 'testing'}])
 	
 	def testAuthors(self):
 		self.assertEqual(
@@ -1223,8 +1178,7 @@ class TablePublicationRecordTest(testhelpers.VerboseTest):
 	def testTAPCapabilityPresent(self):
 		auxCaps = self.tree.xpath(
 			"capability[@standardID='ivo://ivoa.net/std/TAP#aux']")
-		self.assertEqual(len(auxCaps), 1)
-		auxCap = auxCaps[0]
+		auxCap = testhelpers.pickSingle(auxCaps)
 		self.assertEqual(auxCap.xpath("interface[@role='std']/accessURL")[0].text,
 			"http://localhost:8080/tap")
 
@@ -1242,8 +1196,9 @@ class TablePublicationRecordTest(testhelpers.VerboseTest):
 	
 	def testMirrorURL(self):
 		mirrors = self.tree.xpath("//mirrorURL")
-		self.assertEqual(len(mirrors), 1)
-		self.assertEqual(mirrors[0].text, "http://whereever.el.se/overview")
+		self.assertEqual(
+			testhelpers.pickSingle(mirrors).text,
+			"http://whereever.el.se/overview")
 
 	def testXtypeRendered(self):
 		dt = self.tree.xpath("//column[name='whereitsat']/dataType")[0]
@@ -1306,8 +1261,7 @@ class DataGetRecordTest(testhelpers.VerboseTest, testtricks.XSDTestMixin):
 	
 	def testUtypeDeclared(self):
 		utypeChildren = self.srcAndTree[1].xpath("//schema/utype")
-		self.assertEqual(len(utypeChildren), 1)
-		self.assertEqual(utypeChildren[0].text,
+		self.assertEqual(testhelpers.pickSingle(utypeChildren).text,
 			"ivo://some.mad.stuff/datamodel#schema-2.0")
 
 	def testTablesetPresent(self):
@@ -1332,10 +1286,9 @@ class BiblinkRecordTest(testhelpers.VerboseTest):
 
 	def testCapability(self):
 		caps = self.tree.xpath("//capability[@standardID="
-			"'ivo://ivoa.net/std/BibVO#biblink-harvest-0.1']")
-		self.assertEqual(len(caps), 1)
+			"'ivo://ivoa.net/std/BibVO#biblink-harvest-1.0']")
 		self.assertEqual(
-			caps[0].xpath("interface")[0].get(
+			testhelpers.pickSingle(caps).xpath("interface")[0].get(
 				"{http://www.w3.org/2001/XMLSchema-instance}type"),
 			"vs:ParamHTTP")
 
@@ -1357,8 +1310,12 @@ _NMDB_SVC_LIT = """
 	<meta name="subject">Cosmic ray</meta>
 	<meta name="creator.name">Steigies, C.</meta>
 	<meta name="creator">
-		<meta name="name">Wilbanks, J.</meta>
-		<meta name="altIdentifier">orcid:0000-0002-4510-0385</meta>
+		<meta name="name">Wilbanks, J.
+		<meta name="altIdentifier">http://orcid.org/0000-0002-4510-0385</meta>
+		</meta>
+	</meta>
+	<meta name="contributor">
+		<meta name="altIdentifier">http://orcid.org/blabla</meta>Joe Cool
 	</meta>
 	<service id="run" allowed="external">
 		<nullCore/>
@@ -1394,9 +1351,13 @@ class ExternalRecordTest(testhelpers.VerboseTest, testtricks.XSDTestMixin):
 
 	def testCreatorAltIdentifier(self):
 		self.assertEqual(
-			self.srcAndTree[1].xpath("//creator/altIdentifier")[0].text,
-			"orcid:0000-0002-4510-0385")
+			self.srcAndTree[1].xpath("//creator/name/@altIdentifier")[0],
+			"http://orcid.org/0000-0002-4510-0385")
 
+	def testContributorAltIdentifier(self):
+		self.assertEqual(
+			self.srcAndTree[1].xpath("//contributor/@altIdentifier")[0],
+			"http://orcid.org/blabla")
 
 class ForeignIdentifiersRejectedTest(testhelpers.VerboseTest):
 	def testExternalIdsSuppressed(self):
@@ -1411,8 +1372,9 @@ class ForeignIdentifiersRejectedTest(testhelpers.VerboseTest):
 
 		grammar = base.makeStruct(publication.RDRscRecGrammar)
 		res = [p for (d, p) in grammar.parse(rd) if d=='resources']
-		self.assertEqual(len(res), 1)
-		self.assertEqual(res[0]["ivoid"], 'ivo://x-testing/funky/town/run')
+		self.assertEqual(
+			testhelpers.pickSingle(res)["ivoid"],
+			'ivo://x-testing/funky/town/run')
 
 	def testXUnregisteredForLocalIsOk(self):
 		base.caches.clearForName("data/test")
@@ -1496,8 +1458,8 @@ class WithDatalinkTest(testhelpers.VerboseTest):
 		dlURLs = self.srcAndTree[1].xpath(
 			"//capability[@standardID='ivo://ivoa.net/std/SODA#sync-1.0']"
 			"/interface/accessURL")
-		self.assertEqual(len(dlURLs), 1)
-		self.assertTrue(dlURLs[0].text.endswith("/data/ssatest/dl/dlget"))
+		self.assertTrue(
+			testhelpers.pickSingle(dlURLs).text.endswith("/data/ssatest/dl/dlget"))
 
 	def testSODAParameters(self):
 		parentPath = ("//capability[@standardID="
@@ -1524,15 +1486,14 @@ class WithDatalinkTest(testhelpers.VerboseTest):
 		dlURLs = self.srcAndTree[1].xpath(
 			"//capability[@standardID='ivo://ivoa.net/std/DataLink#links-1.1']"
 			"/interface/accessURL")
-		self.assertEqual(len(dlURLs), 1)
-		self.assertTrue(dlURLs[0].text.endswith("/data/ssatest/dl/dlmeta"))
+		self.assertTrue(
+			testhelpers.pickSingle(dlURLs).text.endswith("/data/ssatest/dl/dlmeta"))
 
 	def testDatalinkResultType(self):
 		dlTypes = self.srcAndTree[1].xpath(
 			"//capability[@standardID='ivo://ivoa.net/std/DataLink#links-1.1']"
 			"/interface/resultType")
-		self.assertEqual(len(dlTypes), 1)
-		self.assertEqual(dlTypes[0].text,
+		self.assertEqual(testhelpers.pickSingle(dlTypes).text,
 			"application/x-votable+xml;content=datalink")
 
 	def testDatalinkParameters(self):
@@ -1722,8 +1683,7 @@ class ListIdentifiersTest(testhelpers.VerboseTest):
 	def testSetSpec(self):
 		sets = self.tree.xpath(
 			"//header[identifier='ivo://x-testing']/setSpec")
-		self.assertEqual(len(sets), 1)
-		self.assertEqual(sets[0].text, "ivo_managed")
+		self.assertEqual(testhelpers.pickSingle(sets).text, "ivo_managed")
 
 
 class OAIParameterTest(testhelpers.VerboseTest):
@@ -1775,6 +1735,10 @@ class _OtherServiceRRTree(testhelpers.TestResource):
 				<meta name="referenceURL">http://faked</meta>
 				<meta name="description">All Faked</meta>
 				<meta name="subject">Fakes</meta>
+
+				<meta name="productTypeServed">image</meta>
+				<meta name="productTypeServed">polarized-spectrum</meta>
+
 				<dbCore id="adcore" queriedTable="data/test#adqltable"/>
 				<service id="web" core="adcore">
 					<meta name="title">Web service</meta>
@@ -1870,6 +1834,11 @@ class OtherServiceTest(testhelpers.VerboseTest):
 	def testTitle(self):
 		self.assertEqual(self.treeAndRecs[2][0]["title"], "SCS service")
 
+	def testProductType(self):
+		self.assertEqual(
+			[el.text for el in self.treeAndRecs[0].xpath("//productTypeServed")],
+			['image', 'polarized-spectrum'])
+
 
 class TAPDateBumpingTest(testhelpers.VerboseTest):
 	def setUp(self):
@@ -1889,10 +1858,10 @@ class TAPDateBumpingTest(testhelpers.VerboseTest):
 	def testTAPTimestampConstantWithoutADQL(self):
 		# data/cores has no adql-published tables in it: no TAP record
 		publication.updateServiceList([base.caches.getRD("data/cores")])
-		updated = [getattr(o, "getFullId", lambda: "junk")()
+		updated = [getattr(o, "getFullId", lambda: "no-full-id-sorry")()
 			for o in oaiinter.getMatchingResobs(
 				{"from": datetime.date.today().isoformat()})]
-		self.assertTrue('data/seeded#hooray' in updated)
+		self.assertTrue('data/cores#pc' in updated)
 		self.assertFalse('__system__/tap#run' in updated)
 
 
