@@ -2,7 +2,7 @@
 Tests for the various structures in rscdef.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -158,11 +158,38 @@ class ColumnTest(testhelpers.VerboseTest):
 			(rscdef.Column, '<column unit="d"/>'))
 
 	def testNoMetaParent(self):
-		t = testhelpers.getTestRD().getById("noname")
-		self.assertRaises(base.StructureError,
-			t.getColumnByName("alpha").setMetaParent,
+		t = testhelpers.getTestRD().getById("pythonscript")
+		self.assertRaisesWithMsg(base.StructureError,
+			"Columns may not have meta parents.",
+			t.getColumnByName("counter").setMetaParent,
 			(t,))
 
+	def testTAPSchemaMissing(self):
+		col = base.makeStruct(rscdef.Column, name="u")
+		self.assertEqual(list(col.validateAgainstTAPSchema({})),
+			['Column u missing in TAP_SCHEMA'])
+
+	def testTAPSchemaUCD(self):
+		col = base.makeStruct(rscdef.Column, name="u")
+		self.assertEqual(list(col.validateAgainstTAPSchema(
+			{"name": "u", "ucd": "meta.cryptic", "datatype": "float",
+				"xtype": None, "unit": "", "arraysize": None})),
+			["Column u: tap_schema ucd!=''"])
+
+	def testTAPSchemaXType(self):
+		col = base.makeStruct(rscdef.Column, name="u", type="spoint")
+		self.assertEqual(list(col.validateAgainstTAPSchema(
+			{"name": "u", "ucd": "", "datatype": "double",
+				"xtype": "", "unit": "", "arraysize": "2"})),
+			["Column u: tap_schema xtype!='point'"])
+
+	def testTAPSchemaType(self):
+		col = base.makeStruct(rscdef.Column, name="u", type="char[2][*]")
+		self.assertEqual(list(col.validateAgainstTAPSchema(
+			{"name": "u", "ucd": "", "datatype": "char",
+				"xtype": None, "unit": "", "arraysize": "2"})),
+			["Column u: tap_schema arraysize!='2x*'"])
+
 
 class IsScalarTest(testhelpers.VerboseTest,
 		metaclass=testhelpers.SamplesBasedAutoTest):
@@ -327,8 +354,7 @@ class TableDefTest(testhelpers.VerboseTest):
 			'<column name="one" type="text"/><column name="one"/>'
 			'</table>')
 		fields = list(t)
-		self.assertEqual(len(fields), 1)
-		self.assertEqual(fields[0].type, "real")
+		self.assertEqual(testhelpers.pickSingle(fields).type, "real")
 
 	def testPrimary(self):
 		t = base.parseFromString(rscdef.TableDef, '<table id="test" primary="a">'
@@ -401,6 +427,12 @@ class TableDefTest(testhelpers.VerboseTest):
 		self.assertEqual(fk.destTableName, "test.adqltable")
 		self.assertEqual(fk.isADQLKey, True)
 
+	def testTAPName(self):
+		t = base.parseFromString(rscdef.TableDef, '<table id="test">'
+			'<column name="MagA"/>'
+			'</table>')
+		self.assertEqual(t.getColumnByName("MagA").getTAPName(), "maga")
+
 	def testSTCCopy(self):
 		t0 = base.parseFromString(rscdef.TableDef, '<table id="test">'
 			'<stc>Position ICRS "a" "b"</stc>'
@@ -536,6 +568,9 @@ class QuotedColumnNameTest(testhelpers.VerboseTest):
 		self.assertTrue(msgs.events[0][1][0].startswith(
 			"Column name 'references' coincides"))
 
+	def testTAPName(self):
+		self.assertEqual(self.td.columns[2].getTAPName(), '"Harmless"')
+
 
 class _ResTestTable(testhelpers.TestResource):
 	def make(self, ignored):
