@@ -2,7 +2,7 @@
 Tests for the various structures in rscdef.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -19,6 +19,7 @@ from gavo import base
 from gavo import rsc
 from gavo import rscdef
 from gavo import rscdesc #noflake: for registration
+from gavo import utils
 from gavo.utils import DEG
 
 
@@ -334,6 +335,77 @@ class ApplyTest(testhelpers.VerboseTest):
 			 <code>result["d"] = @foo</code>
 			</apply>""",))
 
+	def testLateAliasOverwrites(self):
+		rowmakerDef = base.parseFromString(rscdef.RowmakerDef, """<rowmaker>
+				<apply id="t">	
+					<setup>
+						<par key="foo" alias="bar" late="True">"knall"</par>
+					</setup>
+					<code>
+						result[foo] = vars["in"]
+					</code>
+				</apply>
+				<apply procDef="t">
+					<bind name="bar">"fall"</bind>
+				</apply>
+			</rowmaker>""")
+		tab = rsc.TableForDef(base.makeStruct(rscdef.TableDef, columns=[]))
+		rmk = rowmakerDef.compileForTableDef(tab.tableDef)
+		self.assertEqual(
+			rmk({"in": "x"}, None),
+			{'knall': 'x', 'fall': 'x'})
+
+	def testEarlyAliasOverwrites(self):
+		rowmakerDef = base.parseFromString(rscdef.RowmakerDef, """<rowmaker>
+				<apply id="t">	
+					<setup>
+						<par key="foo" alias="bar">"knall"</par>
+					</setup>
+					<code>
+						result[foo] = vars["in"]
+					</code>
+				</apply>
+				<apply procDef="t">
+					<bind name="bar">"fall"</bind>
+				</apply>
+			</rowmaker>""")
+		tab = rsc.TableForDef(base.makeStruct(rscdef.TableDef, columns=[]))
+		rmk = rowmakerDef.compileForTableDef(tab.tableDef)
+		self.assertEqual(
+			rmk({"in": "x"}, None),
+			{'knall': 'x', 'fall': 'x'})
+
+	def testComplexAliasing(self):
+		rmk = base.parseFromString(rscdef.RowmakerDef, """<rowmaker>
+			<apply procDef="//siap2#setMeta" id="foo">
+				<bind key="instrument">"tubelet"</bind>
+				<bind key="obs_collection">"testing"</bind>
+			</apply></rowmaker>""")
+		tab = rsc.TableForDef(base.makeStruct(rscdef.TableDef, columns=[]))
+		tab.tableDef.addMeta("facility", "GAVO")
+		rmk = rmk.compileForTableDef(tab.tableDef)
+		res = rmk({"prodtblAccref": "ab", "prodtblTable": None,
+			"prodtblMime": None, "prodtblFsize": None}, tab.tableDef)
+
+		# this means that the facility_name default has been overwritten
+		self.assertEqual(res["instrument_name"], "tubelet")
+		# this means that the getMeta defaults are still in place
+		self.assertEqual(res["facility_name"], "GAVO")
+
+	def testNoEarlyVar(self):
+		proc = base.parseFromString(
+			rscdef.RowmakerDef, """<rowmaker>
+			<apply procDef="//procs#dictMap" id="foo">
+				<bind key="mapping">@mapping</bind>
+				<bind key="key">"testing"</bind>
+			</apply></rowmaker>""")
+		td = base.makeStruct(rscdef.TableDef, columns=[])
+		self.assertRaisesWithMsg(
+			base.StructureError,
+			utils.EqualingRE(r"At .*/procs.rd, line 238: You cannot use @-references in non-late parameter bindings.  But '@mapping' \(bound to mapping\) looks like an @-reference, so I give up."),
+			proc.compileForTableDef,
+			(td,))
+
 
 class VarTest(testhelpers.VerboseTest):
 	"""tests for rowmaker variables.
