@@ -5,7 +5,7 @@ There's a lot of tests like these in productstest, but for the old,
 atomic parameter-style dlget stuff.  This should go ca. 2017.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -78,7 +78,7 @@ class SODAElementTest(testhelpers.VerboseTest):
 				</datalinkCore>
 			</service>""")
 		res = trialhelpers.runSvcWith(svc, "form", {"ID": rscdef.getStandardPubDID(
-			"data/b.imp"), "ignored": 0.4})
+			"data/b.imp"), "ignored": "0.4"})
 		self.assertEqual(b"".join(res.iterData()), b'alpha: 03 34 33.45'
 			b'\ndelta: 42 34 59.7\nobject: michael\nembargo: 2003-12-31\n')
 
@@ -87,7 +87,7 @@ class SODAElementTest(testhelpers.VerboseTest):
 				allowed="dlget">
 			<datalinkCore>
 				<dataFunction procDef="//soda#generateProduct">
-					<bind name="requireMimes">["image/fits"]</bind></dataFunction>
+					<bind name="requireMimes">["application/fits"]</bind></dataFunction>
 					<metaMaker><code>yield MS(InputKey, name="ignored")</code></metaMaker>
 				</datalinkCore>
 			</service>""")
@@ -95,7 +95,7 @@ class SODAElementTest(testhelpers.VerboseTest):
 			"Field PUBDID: Document type not supported: text/plain",
 			trialhelpers.runSvcWith,
 			(svc, "form", {"ID": rscdef.getStandardPubDID("data/b.imp"),
-				"ignored": 0.5}))
+				"ignored": "0.5"}))
 
 	def testAccrefPrefixDeny(self):
 		svc = api.parseFromString(svcs.Service, """<service id="foo"
@@ -160,7 +160,7 @@ class SODAElementTest(testhelpers.VerboseTest):
 			"Internal Error: a first data function did not create data.",
 			trialhelpers.runSvcWith,
 			(svc, "form", {"ID": rscdef.getStandardPubDID("data/b.imp"),
-				"ignored": 0.4}))
+				"ignored": "0.4"}))
 
 	def testProductsMogrifier(self):
 		svc = api.parseFromString(svcs.Service, """<service id="foo">
@@ -405,7 +405,7 @@ class SODAFITSTest(testhelpers.VerboseTest):
 				"COO_3": ["3753 3755"],
 				}))
 
-		self.assertEqual(mime, "image/fits")
+		self.assertEqual(mime, "application/fits")
 		hdr = fitstools.readPrimaryHeaderQuick(io.BytesIO(data))
 		self.assertEqual(hdr["NAXIS1"], 11)
 		self.assertEqual(hdr["NAXIS2"], 7)
@@ -432,7 +432,7 @@ class SODAFITSTest(testhelpers.VerboseTest):
 				"DEC": ["30.39845 +Inf"],
 				}))
 
-		self.assertEqual(mime, "image/fits")
+		self.assertEqual(mime, "application/fits")
 		hdr = fitstools.readPrimaryHeaderQuick(io.BytesIO(data))
 		self.assertEqual(hdr["NAXIS1"], 8)
 		self.assertEqual(hdr["NAXIS2"], 7)
@@ -458,7 +458,7 @@ class SODAFITSTest(testhelpers.VerboseTest):
 				"COO_3": ["3753 3755"],
 				}))
 
-		self.assertEqual(mime, "image/fits")
+		self.assertEqual(mime, "application/fits")
 		hdr = fitstools.readPrimaryHeaderQuick(io.BytesIO(data))
 		self.assertEqual(hdr["NAXIS1"], 4)
 		self.assertEqual(hdr["NAXIS2"], 2)
@@ -483,7 +483,7 @@ class SODAFITSTest(testhelpers.VerboseTest):
 					" 359.359 30.985"]
 				}))
 
-		self.assertEqual(mime, "image/fits")
+		self.assertEqual(mime, "application/fits")
 		with io.BytesIO(data) as f:
 			hdr = fitstools.readPrimaryHeaderQuick(f)
 		self.assertEqual(hdr["NAXIS1"], 4)
@@ -520,7 +520,7 @@ class SODAFITSTest(testhelpers.VerboseTest):
 				"CIRCLE": ["359.36 30.984 0.0004"]
 				}))
 
-		self.assertEqual(mime, "image/fits")
+		self.assertEqual(mime, "application/fits")
 		hdr = fitstools.readPrimaryHeaderQuick(io.BytesIO(data))
 # TODO: I'd really expect NAXIS1==NAXIS2, but I cannot find a mistake
 # in my reasoning up to here.  Hm.
@@ -615,7 +615,7 @@ class SODAFITSTest(testhelpers.VerboseTest):
 			"ID": rscdef.getStandardPubDID("data/excube.fits"),
 				"PIXEL_1": ["4 4"],
 				"PIXEL_3": ["2 2"]}))
-		self.assertEqual(mime, "image/fits")
+		self.assertEqual(mime, "application/fits")
 		self.assertTrue(b"NAXIS1  =                    1" in data)
 		self.assertTrue(b"NAXIS2  =                    7" in data)
 		self.assertTrue(b"NAXIS3  =                    1" in data)
@@ -869,8 +869,7 @@ class SODA_POLYGONMetaTest(testhelpers.VerboseTest, metaclass=testhelpers.Sample
 
 	def _runTest(self, sample):
 		keys = self._produceKeys(sample)
-		self.assertEqual(len(keys), 1)
-		key = keys[0]
+		key = testhelpers.pickSingle(keys)
 		self.assertEqual(key.type, "double precision(*)")
 		self.assertEqual(key.values.max,
 			self.truth[sample[0]])
@@ -909,8 +908,7 @@ class SODA_CIRCLEMetaTest(testhelpers.VerboseTest, metaclass=testhelpers.Samples
 
 	def _runTest(self, sample):
 		keys = self._produceKeys(sample)
-		self.assertEqual(len(keys), 1)
-		key = keys[0]
+		key = testhelpers.pickSingle(keys)
 		self.assertEqual(key.type, "double precision(3)")
 		self.assertEqual(key.values.max,
 			self.truth[sample[0]])
@@ -983,7 +981,7 @@ class SODAPOSTest(testhelpers.VerboseTest):
 					"POS": ["RANGE 359.359 359.36 30.9845 30.985"],
 					}))
 
-		self.assertEqual(mime, "image/fits")
+		self.assertEqual(mime, "application/fits")
 		hdr = fitstools.readPrimaryHeaderQuick(io.BytesIO(data))
 		self.assertEqual(hdr["NAXIS1"], 4)
 		self.assertEqual(hdr["NAXIS2"], 2)
@@ -1087,7 +1085,7 @@ class SODACFITSTest(testhelpers.VerboseTest):
 				"POLYGON": ["168.2464 22.2147 168.2454 22.2147 168.2464 22.2149"]
 				}))
 
-		self.assertEqual(mime, "image/fits")
+		self.assertEqual(mime, "application/fits")
 		with pyfits.open(io.BytesIO(data)) as hdus:
 			hdr = hdus[1].header
 			self.assertEqual(hdr["NAXIS1"], 4)
@@ -1110,7 +1108,7 @@ class SODACFITSTest(testhelpers.VerboseTest):
 				"CIRCLE": ["168.2455 22.2161 0.0002"]
 				}))
 
-		self.assertEqual(mime, "image/fits")
+		self.assertEqual(mime, "application/fits")
 		hdr = pyfits.open(io.BytesIO(data))[1].header
 # TODO: I'd really expect NAXIS1==NAXIS2, but I cannot find a mistake
 # in my reasoning up to here.  Hm.
@@ -1293,7 +1291,7 @@ class SDMDatalinkTest(testhelpers.VerboseTest):
 			self.runService, ({
 				"ID": ['ivo://test.inv/test1'],
 				"responseFORMAT": ["text/plain"],
-				"BAND": "23 "},))
+				"BAND": ["23 "]},))
 
 	def testOriginalCalibOk(self):
 		mime, payload = self.runService(
@@ -1380,8 +1378,8 @@ class SDMResultTest(testhelpers.VerboseTest):
 
 	def testSpecGroup(self):
 		self.assertEqual(
-			self.tree.xpath("//GROUP[@utype='spec:Spectrum.Data']"
-				"/FIELDref[@utype='spec:Spectrum.Data.SpectralAxis.Value']"
+			self.tree.xpath("//GROUP[@utype='spec:spectrum.data']"
+				"/FIELDref[@utype='spec:spectrum.data.spectralaxis.value']"
 				)[0].get("ref"),
 			"spectral")
 
