@@ -2,7 +2,7 @@
 Some tests around the SSAP infrastructure.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -211,7 +211,7 @@ class CoreQueriesTest(_WithSSATableTest, metaclass=testhelpers.SamplesBasedAutoT
 #20
 		({"WILDTARGET": "b??g*"},
 			["test2"]),
-		({"WILDTARGET": "\*"},
+		({"WILDTARGET": "\\*"},
 			[]),
 		({"WILDTARGET": "[br][oa]*"},
 			["test2", "test3"]),
@@ -237,7 +237,7 @@ class CoreQueriesTest(_WithSSATableTest, metaclass=testhelpers.SamplesBasedAutoT
 			['test1', 'test3', 'test2']),
 		({"COLLECTION": "ssa Test set"},
 			[]),
-		({"FORMAT": "application/x-votable+xml,image/fits"},
+		({"FORMAT": "application/x-votable+xml,application/fits"},
 			["test1", "test2", "test3"]),
 		({"WAVECALIB": "any"},
 			['test1', 'test3', 'test2']),
@@ -297,7 +297,7 @@ class MetaKeyTest(_WithSSATableTest):
 # these are like CoreQueries except they exercise custom logic
 	def testTOP(self):
 		res = self.runService("s",
-			_mkd(REQUEST="queryData", TOP=1))
+			_mkd(REQUEST="queryData", TOP="1"))
 		self.assertEqual(len(res.getPrimaryTable()), 1)
 
 	def testMAXREC(self):
@@ -344,8 +344,7 @@ class SSAMetaResponseTest(testhelpers.VerboseTest):
 	
 	def testSizeInput(self):
 		matches = self.meta[1].xpath("//PARAM[@name='INPUT:SIZE']")
-		self.assertEqual(len(matches), 1)
-		par = matches[0]
+		par = testhelpers.pickSingle(matches)
 		self.assertEqual(par[0].text,
 			"Size of the region of interest around POS")
 
@@ -434,10 +433,6 @@ class SSATableTest(testhelpers.VerboseTest):
 
 	resources = [("docAndTree", _renderedSSAResponse)]
 
-	def testSillyNSDeclPresent(self):
-		self.assertTrue(b'xmlns:ssa="http://www.ivoa.net/xml/DalSsap/v1.0"'
-			in self.docAndTree[0])
-	
 	def testOverflowFlagged(self):
 		infoEl = self.docAndTree[1].xpath(
 			"//RESOURCE/INFO[@name='QUERY_STATUS']")[0]
