@@ -2,7 +2,7 @@
 Generic tests for the STC library.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -152,43 +152,43 @@ class OtherCoordIntervalTest(testhelpers.VerboseTest):
 
 	def testSpecInterval(self):
 		ast = stcsast.parseSTCS("SpectralInterval 23 45 unit Hz")
-		self.assertEqual(len(ast.freqAs), 1)
-		self.assertEqual(ast.freqAs[0].frame.refPos.standardOrigin, None)
-		self.assertEqual(ast.freqAs[0].lowerLimit, 23.0)
-		self.assertEqual(ast.freqAs[0].upperLimit, 45.0)
+		freqA = testhelpers.pickSingle(ast.freqAs)
+		self.assertEqual(freqA.frame.refPos.standardOrigin, None)
+		self.assertEqual(freqA.lowerLimit, 23.0)
+		self.assertEqual(freqA.upperLimit, 45.0)
 
 	def testRedshiftInterval(self):
 		ast = stcsast.parseSTCS("RedshiftInterval VELOCITY 2 4")
-		self.assertEqual(len(ast.redshiftAs), 1)
-		self.assertEqual(ast.redshiftAs[0].frame.type,
+		redshiftA = testhelpers.pickSingle(ast.redshiftAs)
+		self.assertEqual(redshiftA.frame.type,
 			"VELOCITY")
-		self.assertEqual(ast.redshiftAs[0].lowerLimit, 2.0)
-		self.assertEqual(ast.redshiftAs[0].upperLimit, 4.0)
+		self.assertEqual(redshiftA.lowerLimit, 2.0)
+		self.assertEqual(redshiftA.upperLimit, 4.0)
 		self.assertEqual(ast.redshift.unit, "km")
 		self.assertEqual(ast.redshift.velTimeUnit, "s")
 
 	def testStartTime(self):
 		ast = stcsast.parseSTCS("StartTime TT MJD24000.5")
-		self.assertEqual(len(ast.timeAs), 1)
-		self.assertEqual(ast.timeAs[0].upperLimit, None)
-		self.assertEqual(ast.timeAs[0].lowerLimit,
+		timeA = testhelpers.pickSingle(ast.timeAs)
+		self.assertEqual(timeA.upperLimit, None)
+		self.assertEqual(timeA.lowerLimit,
 			datetime.datetime(1924, 8, 3, 12, 0))
 
 	def testStopTime(self):
 		ast = stcsast.parseSTCS("StopTime TT MJD24000.5")
-		self.assertEqual(len(ast.timeAs), 1)
-		self.assertEqual(ast.timeAs[0].lowerLimit, None)
-		self.assertEqual(ast.timeAs[0].upperLimit,
+		timeA = testhelpers.pickSingle(ast.timeAs)
+		self.assertEqual(timeA.lowerLimit, None)
+		self.assertEqual(timeA.upperLimit,
 			datetime.datetime(1924, 8, 3, 12, 0))
 
 
 class SpaceCoordIntervalTest(testhelpers.VerboseTest):
 	def testSimple2D(self):
 		ast = stcsast.parseSTCS("PositionInterval ICRS 12.25 23.75 13.5 25.0")
-		self.assertEqual(ast.areas[0].frame.refPos.standardOrigin, None)
-		self.assertEqual(len(ast.areas), 1)
-		self.assertEqual(ast.areas[0].lowerLimit, (12.25, 23.75))
-		self.assertEqual(ast.areas[0].upperLimit, (13.5, 25.0))
+		area = testhelpers.pickSingle(ast.areas)
+		self.assertEqual(area.frame.refPos.standardOrigin, None)
+		self.assertEqual(area.lowerLimit, (12.25, 23.75))
+		self.assertEqual(area.upperLimit, (13.5, 25.0))
 		self.assertTrue(ast.place is not None)
 	
 	def test2DWithError(self):
@@ -198,16 +198,16 @@ class SpaceCoordIntervalTest(testhelpers.VerboseTest):
 
 	def testSimple3D(self):
 		ast = stcsast.parseSTCS("PositionInterval ICRS CART3 1 2 3")
-		self.assertEqual(len(ast.areas), 1)
-		self.assertEqual(ast.areas[0].lowerLimit, (1.0, 2.0, 3.0))
-		self.assertEqual(ast.areas[0].upperLimit, None)
+		area = testhelpers.pickSingle(ast.areas)
+		self.assertEqual(area.lowerLimit, (1.0, 2.0, 3.0))
+		self.assertEqual(area.upperLimit, None)
 
 	def test3DWithError(self):
 		ast = stcsast.parseSTCS("PositionInterval ICRS CART3 1 2 3 4 5 6"
 			" Error 0.25 0.5 0.75")
-		self.assertEqual(len(ast.areas), 1)
-		self.assertEqual(ast.areas[0].lowerLimit, (1.0, 2.0, 3.0))
-		self.assertEqual(ast.areas[0].upperLimit, (4.0, 5.0, 6.0))
+		area = testhelpers.pickSingle(ast.areas)
+		self.assertEqual(area.lowerLimit, (1.0, 2.0, 3.0))
+		self.assertEqual(area.upperLimit, (4.0, 5.0, 6.0))
 		self.assertEqual(ast.place.error.values, ((.25, .5, .75),))
 	
 	def testWithPosition(self):
