@@ -2,7 +2,7 @@
 Tests for rsc.XTable
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -17,6 +17,8 @@ from gavo import rsc
 from gavo import rscdef
 from gavo import rscdesc
 from gavo import svcs
+from gavo.protocols import tap
+from gavo.user import validation
 from gavo.stc import dm
 
 import tresc
@@ -252,8 +254,8 @@ class DBTableTest(tresc.TestWithDBConnection):
 		try:
 			self.conn.execute("alter table testing.xy drop column x")
 			self.assertRaisesWithMsg(base.DataError,
-				"Table testing.xY: mismatching name of x (DB: y); from"
-				" column y on: No matches in DB",
+				'Table testing.xY: mismatching name of x (DB: y); column y:'
+				' No matches in DB',
 				table.ensureOnDiskMatches,
 				())
 		finally:
@@ -320,16 +322,27 @@ class DBTableTest(tresc.TestWithDBConnection):
 
 	def testOnDiskHiddenMatchAndDelimited(self):
 		rd = base.parseFromString(rscdesc.RD, r"""<resource schema="test">
-			<table id="withhidden" onDisk="True">
+			<meta>
+				description: Forget it
+				creationDate: 2021-01-10T10:00:00
+				subject: galaxies
+			</meta>
+			<table id="withhidden" onDisk="True" adql="True">
 				<column name="quoted/colA"/>
 				<column name="colB" hidden="True"/>
 			</table></resource>""")
 		td = rd.getById("withhidden")
+		tap.publishToTAP(rd, self.conn)
 		table = rsc.TableForDef(td, connection=self.conn, nometa=True, create=True)
+		self.conn.commit()
+
 		try:
-			table.ensureOnDiskMatches() # will raise an exception if the test fails
+			self.assertEqual([],
+				list(validation.validateTable(table.tableDef, True)))
 		finally:
+			tap.unpublishFromTAP(rd, self.conn)
 			table.drop()
+			self.conn.commit()
 
 	def testWithMethodIndex(self):
 		rd = base.parseFromString(rscdesc.RD, r"""<resource schema="test">
