@@ -8,7 +8,7 @@ There's more TAP/UWS related tests in test_tap.py; these require a
 running reactor and are based on trial.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -36,6 +36,7 @@ from gavo.protocols import taprunner
 from gavo.protocols import uws
 from gavo.protocols import uwsactions
 from gavo.registry import capabilities
+from gavo.web import tap_uploads
 from gavo.web import vosi
 
 import adqltest
@@ -380,7 +381,7 @@ class CapabilitiesTest(testhelpers.VerboseTest, testtricks.XSDTestMixin):
 			["http://localhost:8080/tap", "http://localhost:8080/tap"])
 
 
-class TAPSchemaTest(testhelpers.VerboseTest):
+class TAPSchemaQueryTest(testhelpers.VerboseTest):
 	def setUp(self):
 		self.jobId = _makeATAPJob({
 				"query": ["SELECT TOP 1 * FROM TAP_SCHEMA.tables"],
@@ -399,6 +400,156 @@ class TAPSchemaTest(testhelpers.VerboseTest):
 		self.assertTrue('<RESOURCE type="results">' in res)
 
 
+class _TAPSchemaContent(testhelpers.TestResource):
+	def make(self, deps):
+		rd = base.parseFromString(rscdesc.RD, """
+			<resource schema="schm" resdir="data">
+			<meta name="schema-rank">4000</meta>
+			<meta name="utype">ivo://ivoa.net/std/cool-model</meta>
+			<meta name="description">We actually don't test for empty
+				schema descriptions yet</meta>
+
+			<table id="nontap" onDisk="True">
+				<column name="col1" description="A column that should not appear
+					in tap_schema because its table isn't ADQL-visible"/>
+			</table>
+
+			<table id="boring" adql="True">
+				<column name="col1" type="spoly"/>
+				<column name="quoted/select" type="integer[20]"
+					utype="omg:array"/>
+				<viewStatement>
+					(it's fake)
+				</viewStatement>
+			</table>
+
+			<table id="tbl" adql="True" nrows="27">
+				<meta name="table-rank">1</meta>
+				<meta name="utype">adhoc:an.example.table</meta>
+				<meta name="description">An example table</meta>
+				<foreignKey source="col1,ivoa" inTable="boring" dest="col1,select"/>
+				<column name="col1" ucd="src.sample" utype="adhoc:tbl.col1"
+					description="A column with a UCD of src.sample"
+					unit="km/s"/>
+				<column name="ivoa">
+					<property name="std">1</property>
+				</column>
+			</table>
+			</resource>""")
+	
+		rows = {}
+		tapRD = base.caches.getRD("//tap")
+
+		# tables is a dispatching grammar
+		dd = tapRD.getById("importTablesFromRD")
+		for dest, row in dd.grammar.parse(rd):
+			del row["parser_"]
+			rows.setdefault(dest, []).append(row)
+
+		for ddId in ["importDMsFromRD", "importColumnsFromRD",
+				"importFkeysFromRD", "importGroupsFromRD"]:
+			dd = tapRD.getById(ddId)
+			rows[dd.makes[0].table.id
+				] = list(dd.grammar.parse(rd))
+		
+		return rows
+
+
+class TAPSchemaCreationTest(testhelpers.VerboseTest):
+	resources = [("rows", _TAPSchemaContent())]
+
+	def testSchemaRow(self):
+		row = testhelpers.pickSingle(self.rows["schemas"])
+		self.assertEqual(row, {
+			'schema_description': "We actually don't test for"
+				" empty schema descriptions yet",
+			'schema_index': '4000',
+			'schema_name': 'schm',
+			'schema_utype': 'ivo://ivoa.net/std/cool-model'})
+
+	def testRankedTable(self):
+		rows = [r for r in self.rows["tables"] if r["table_name"]=="schm.tbl"]
+		self.assertEqual(testhelpers.pickSingle(rows), {'nrows': 27,
+			'schema_name': 'schm',
+			'sourceRD': 'temporary',
+			'table_description': 'An example table',
+			'table_index': '1',
+			'table_name': 'schm.tbl',
+			'table_type': 'table',
+			'table_utype': 'adhoc:an.example.table'})
+
+	def testNonADQLTable(self):
+		for r in self.rows["tables"]:
+			if "nontap" in r["table_name"]:
+				raise AssertionError("non-ADQL table in tap_schema?")
+	
+	def testPlainTable(self):
+		row = testhelpers.pickSingle(
+			[r for r in self.rows["tables"] if r["table_name"]=="schm.boring"])
+		self.assertEqual(row, {'nrows': None,
+			'schema_name': 'schm',
+			'sourceRD': 'temporary',
+			'table_description': "We actually don't test for empty schema"
+				" descriptions yet",
+			'table_index': None,
+			'table_name': 'schm.boring',
+			'table_type': 'view',
+			'table_utype': None})
+
+	def testColumnUtype(self):
+		row = testhelpers.pickSingle([r for r in self.rows["columns"]
+			if r["utype"]=="adhoc:tbl.col1"])
+		self.assertEqual(row["datatype"], "float")
+		self.assertEqual(row["column_name"], "col1")
+		self.assertEqual(row["ucd"], "src.sample")
+		self.assertEqual(row["utype"], "adhoc:tbl.col1")
+		self.assertEqual(row["unit"], "km/s")
+		self.assertEqual(row["description"],
+			"A column with a UCD of src.sample")
+
+	def testXtype(self):
+		row = testhelpers.pickSingle([r for r in self.rows["columns"]
+			if r["xtype"]=="polygon"])
+		self.assertEqual(row["column_name"], "col1")
+		self.assertEqual(row["xtype"], "polygon")
+		self.assertEqual(row["datatype"], "double")
+		self.assertEqual(row["arraysize"], "*")
+	
+	def testQuotedColname(self):
+		row = testhelpers.pickSingle([r for r in self.rows["columns"]
+			if r["utype"]=="omg:array"])
+		self.assertEqual(str(row["column_name"]), '"select"')
+	
+	def testSizedArray(self):
+		row = testhelpers.pickSingle([r for r in self.rows["columns"]
+			if r["utype"]=="omg:array"])
+		self.assertEqual(row["arraysize"], '20')
+		self.assertEqual(row["datatype"], "int")
+		self.assertEqual(row["size"], 20)
+
+	def testStd(self):
+		row = testhelpers.pickSingle([r for r in self.rows["columns"]
+			if r["column_name"]=="ivoa"])
+		self.assertEqual(row["std"], 1)
+
+	def testKeys(self):
+		# Update the grammar to be dispatching and then this test, too
+		rows = self.rows["keys"]
+		self.assertEqual(len(rows), 3)
+		self.assertEqual(rows[0]["from_table"], "schm.tbl")
+		self.assertEqual(rows[0]["target_table"], "schm.boring")
+
+		self.assertEqual(rows[1]["from_column"], "col1")
+		self.assertEqual(rows[1]["target_column"], "col1")
+
+		self.assertEqual(rows[2]["from_column"], "ivoa")
+		# TODO: select should probably be quoted here.
+		self.assertEqual(rows[2]["target_column"], "select")
+
+		self.assertEqual(len(set(r["key_id"] for r in rows)), 1)
+
+# todo?: groups (but then it's been ages since we've last heard of it)
+
 class EvilUploadTest(testhelpers.VerboseTest):
 	def _runWithUpload(self, query, uploadContent):
 		jobId = _makeATAPJob({
@@ -507,7 +658,6 @@ class SimpleRunnerTest(testhelpers.VerboseTest):
 				if job.phase not in runningPhases:
 					break
 			else:
-				import pdb;pdb.Pdb(nosigint=True).set_trace()
 				raise AssertionError("Job does not finish.  Your machine cannot be"
 					" *that* slow?")
 
@@ -665,8 +815,8 @@ class JoinMetaTest(testhelpers.VerboseTest):
 
 	def testDatalinkServiceMentioned(self):
 		svcDefinition = self.tree.xpath("//RESOURCE[@type='meta']")
-		self.assertEqual(len(svcDefinition), 1)
-		idid = svcDefinition[0].xpath("GROUP/PARAM[@name='ID']")[0].get("ref")
+		idid = testhelpers.pickSingle(svcDefinition
+			).xpath("GROUP/PARAM[@name='ID']")[0].get("ref")
 		self.assertEqual(
 			self.tree.xpath("//FIELD[@ID='%s']"%idid)[0].get("name"),
 			"ssa_pubDID")
@@ -705,8 +855,10 @@ class DataOriginTest(testhelpers.VerboseTest):
 	# them all.
 
 	def testAllIVOIDsPresent(self):
-		self.assertEqual(self._getINFOs("ivoid"),
-			set(["ivo://x-testing/__system__/tap/run",
+		self.assertEqual(self._getINFOs("ivoid"), set([
+				'ivo://x-testing/tap',
+				'ivo://x-testing/data/ssatest/c',
+				'ivo://x-testing/data/ssatest/s',
 				"ivo://x-testing/data/ssatest/hcdtest"]))
 
 	def testPublisher(self):
@@ -799,7 +951,7 @@ class TAPTableMetaTest(testhelpers.VerboseTest):
 		self.assertEqual(
 			base.getMetaText(res, "description"),
 			"The IVOA-defined obscore table, containing generic metadata for\n"
-			"datasets within this datacenter.")
+			"datasets within this data centre.")
 	
 	def testSingleSourceTableParamsCopied(self):
 		res = adqlglue.runTAPQuery(
@@ -914,16 +1066,16 @@ class TAPPublicationTest(testhelpers.VerboseTest):
 			"Sundry test tables and other fancy stuff")
 		self.assertEqual(res["utype"],
 			"http://www.g-vo.org/DaCHS")
-		self.assertEqual(res["schema_index"], None)
+		self.assertEqual(res["schema_index"], 100)
 
 	def testTablesPublished(self):
 		res = list(self.conn.queryToDicts(
 			"select * from tap_schema.tables where table_name='test.csdata'"))
-		self.assertEqual(len(res), 1)
-		self.assertEqual(res[0]["table_type"], "table")
+		row = testhelpers.pickSingle(res)
+		self.assertEqual(row["table_type"], "table")
 		# I suppose we ought to create some way (properties?) to set table_index
-		self.assertEqual(res[0]["table_index"], None)
-		self.assertEqual(res[0]["nrows"], 10)
+		self.assertEqual(row["table_index"], None)
+		self.assertEqual(row["nrows"], 10)
 
 	def testColumnsPublished(self):
 		res = list(self.conn.query(
@@ -1028,5 +1180,154 @@ class WeirdTypesTAPTest(testhelpers.VerboseTest):
 			"10x*")
 
 
+class UserUploadCleanupTest(testhelpers.VerboseTest):
+	def testCleanup(self):
+		with base.getWritableAdminConn() as conn:
+			with open(base.getConfig("inputsDir") / "data" / "vizier_votable.vot",
+					"rb") as f:
+				tap_uploads.uploadUserTable(
+					conn, "glue", f, "testuser")
+				f.seek(0)
+				tap_uploads.uploadUserTable(
+					conn, "flu", f, "testuser")
+			conn.execute("UPDATE tap_user.tables SET expiry=CURRENT_TIMESTAMP"
+				" WHERE user_name='testuser' AND table_name='flu'")
+
+		try:
+			cleanup = base.resolveCrossId("//tap_user#expire")
+			cleanup.runImmediate()
+			with base.getTableConn() as conn:
+				still_there = [r[0] for r in
+					conn.query("select table_name from tap_user.tables"
+						" where user_name='testuser'")]
+				self.assertEqual(still_there, ["glue"])
+
+				q = base.UnmanagedQuerier(conn)
+				self.assertEqual(
+					q.getTableType("tap_user._testuser_glue"), "BASE TABLE")
+				self.assertEqual(q.getTableType("tap_user._testuser_flu"), None)
+		finally:
+			with base.getWritableAdminConn() as conn:
+				conn.execute("DROP TABLE IF EXISTS tap_user._testuser_glue")
+				conn.execute("DROP TABLE IF EXISTS tap_user._testuser_flu")
+				conn.execute("DELETE FROM tap_user.tables WHERE user_name='testuser'")
+
+
+class _UploadedTables(testhelpers.TestResource):
+	resources = [("conn", tresc.dbConnection)]
+
+	def make(self, deps):
+		conn = deps["conn"]
+		with open(base.getConfig("inputsDir") / "data" / "vizier_votable.vot",
+				"rb") as f:
+			tap_uploads.uploadUserTable(
+				conn, "glue", f, "anonymous")
+		return conn
+
+	def clean(self, conn):
+		createdTables = ['_anonymous_glue']
+		conn.execute("DELETE FROM tap_user.tables WHERE"
+			" physical_name in ({})".format(",".join(repr(t) for t in createdTables)))
+		for t in createdTables:
+			conn.execute(f"DROP TABLE tap_user.{t}")
+
+
+class UserUploadResolutionTest(testhelpers.VerboseTest):
+	resources = [("uploaded", _UploadedTables()),
+		("conn", tresc.dbConnection)]
+
+	def testInfoGetterSuccess(self):
+		getter = adqlglue.DaCHSFieldInfoGetter(forUser="anonymous")
+		infos = getter.getInfosFor("tap_user.glue")
+		col = infos[1]
+		self.assertEqual(col[0].name, "_RAJ2000")
+		self.assertEqual(col[1].ucd, "pos.eq.ra;meta.main")
+
+	def testInfoGetterWrongUser(self):
+		getter = adqlglue.DaCHSFieldInfoGetter(forUser="testing")
+		self.assertRaisesWithMsg(utils.NotFoundError,
+			"table 'tap_user.glue' could not be located in your uploaded tables",
+			getter.getInfosFor,
+			("tap_user.glue",))
+
+	def testADQLMorphing(self):
+		res = list(
+			adqlglue.runTAPQuery("WITH a AS"
+				" (SELECT * from tap_user.glue where mag<1)"
+				" SELECT b.tyc2 from tap_user.glue as b join a using (iras)",
+			2, self.conn, [], 20, False))
+		self.assertEqual(res, [{'TYC2': '1266-01416-1'}])
+
+
+class _UploadedTablesUsers(testhelpers.TestResource):
+	resources = [("conn", tresc.dbConnection)]
+
+	def make(self, deps):
+		conn = deps["conn"]
+		with open(base.getConfig("inputsDir") / "data" / "vizier_votable.vot",
+				"rb") as f:
+			tap_uploads.uploadUserTable(
+				conn, "glue", f, "anonymous")
+	
+		with open(base.getConfig("inputsDir") / "data" / "vizier_votable.vot",
+				"rb") as f:
+			tap_uploads.uploadUserTable(
+				conn, "honk", f, "testing")
+
+		with open(base.getConfig("inputsDir") / "data" / "vizier_votable.vot",
+				"rb") as f:
+			tap_uploads.uploadUserTable(
+				conn, "glue", f, "george")
+
+		with open(base.getConfig("inputsDir") / "data" / "vizier_votable.vot",
+				"rb") as f:
+			tap_uploads.uploadUserTable(
+				conn, "flu", f, "testing")
+
+		return conn
+
+	def clean(self, conn):
+		createdTables = [
+			'_anonymous_glue', "_testing_honk", "_george_glue", "_testing_flu"]
+		conn.execute("DELETE FROM tap_user.tables WHERE"
+			" physical_name in ({})".format(",".join(repr(t) for t in createdTables)))
+		for t in createdTables:
+			conn.execute(f"DROP TABLE tap_user.{t}")
+
+
+TAP_SERVICE = base.resolveCrossId("//tap#run")
+
+class TableListingTest(testhelpers.VerboseTest):
+	resources = [("tables", _UploadedTablesUsers())]
+	
+	def getForUser(self, username="testting", password="testing"):
+		req = trialhelpers.FakeRequest("")
+		req.user, req.password = username, password
+		tap_uploads.PersistentUploadRenderer(req, TAP_SERVICE
+			).render(req)
+		return testhelpers.getXMLTree(req.accumulator, debug=False)
+
+	def testTablesListingAnonymous(self):
+		self.assertEqual(
+			self.getForUser("").xpath("//INFO[@name='QUERY_STATUS']")[0].text,
+			"No persistent tables listing for anonymous users.")
+
+	def testTablesListingAuthUser(self):
+		tree = self.getForUser("testing")
+		self.assertEqual(tree.xpath("schema/name")[0].text,  "tap_user")
+		tables = tree.xpath("schema/table")
+		# this asserts there's no tables from george or from anonymous
+		self.assertEqual(len(tables), 2)
+
+		table_names = {t.xpath("name")[0].text for t in tables}
+		self.assertEqual(table_names, {"tap_user.honk", "tap_user.flu"})
+
+		ucds = set(n.text for n in tree.xpath("//ucd"))
+		self.assertEqual(ucds, {'pos.eq.ra;meta.main', 'meta.id;meta.main',
+			'pos.eq.dec;meta.main', 'pos.angDistance', 'phot.color',
+			'pos.pm;pos.eq.ra', 'pos.eq.dec', 'meta.id;src', 'pos.eq.ra',
+			'phot.mag;em.opt.V', 'pos.pm;pos.eq.dec'})
+
+
 if __name__=="__main__":
 	testhelpers.main(JobMetaTest)
