@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Twisted trial-based tests for TAP and UWS.
 
@@ -8,7 +7,7 @@ The real big integration tap tests use the HTTP interface and are in
 an external resource package taptest.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -16,6 +15,7 @@ an external resource package taptest.
 
 import atexit
 import datetime
+import io
 import re
 import os
 import time
@@ -625,4 +625,132 @@ class SimpleAsyncTest(TAPRenderTest):
 		).addCallback(startJob)
 
 
+class PersistentUploadTest(TAPRenderTest):
+	with open(
+			base.getConfig("inputsDir") / "data" / "vizier_votable.vot",
+			"rb") as f:
+		someVOT = io.BytesIO(f.read())
+
+	def assert404Status(self, res):
+		self.assertEqual(res[1].code, 404)
+		return res
+
+	def testNoTable(self):
+		return self.assertGETHasStrings("tap/user_tables", {}, [
+			'name="QUERY_STATUS"',
+			"No persistent tables listing for anonymous users.<",])
+	
+	def testTablesListing(self):
+		return self.assertGETHasStrings("tap/user_tables", {}, [
+			"<vtm:tableset",
+			'xmlns:vtm="http://www.ivoa.net/xml/VOSITables/v1.0"',
+			"<name>tap_user</name>",
+			"A schema containing users' uploads"], rm=trialhelpers.addCreds)
+
+	def testBadTableName(self):
+		return self.assertGETHasStrings("tap/user_tables/666", {}, [
+			'name="QUERY_STATUS"',
+			'>You must give a valid upload table name here<']
+			).addCallback(self.assert404Status)
+
+	def testNoPosting(self):
+		return self.assertPOSTHasStrings("tap/user_tables/test_upload", {}, [
+			'name="QUERY_STATUS"',
+			"This resource cannot respond to the HTTP 'POST' method</INFO>"])
+
+	def testNoMediaType(self):
+		def makePUT(request):
+			request.setPayload(self.someVOT, "application/octet-stream")
+			request.method = "PUT"
+
+		return self.assertPOSTHasStrings(
+			f"{_B}user_tables/test_upload", {}, [
+				"Do not know how to ingest application/octet-stream tables<",
+				"if in doubt, use application/x-votable+xml"
+			], rm=makePUT)
+
+	def testSuccessfulUpload(self):
+
+		def makePUT(request):
+			request.setPayload(self.someVOT, base.votableType)
+			request.method = "PUT"
+
+		def makeDELETE(request):
+			request.method = "DELETE"
+
+		def assertCleanedUp(result):
+			self.assertEqual(result[0], b'Dropped user table test_upload\n')
+			self.assertEqual(result[1].code, 200)
+
+			with base.getTableConn() as conn:
+				self.assertEqual(list(conn.query(
+					"select user_name, table_name"
+					" from tap_user.tables where user_name='anonymous' and"
+					" table_name='test_upload'")), [])
+				self.assertEqual(base.UnmanagedQuerier(conn).getTableType(
+					"tap_user._anonymous_test_upload"), None)
+			return result
+
+		def tryDELETE(result):
+			return self.assertGETHasStrings(
+				f"{_B}user_tables/test_upload", {}, [
+				"Dropped user table test_upload"], rm=makeDELETE
+			).addCallback(assertCleanedUp)
+
+		def tryGET(result):
+			self.assertEqual(result[1].code, 200)
+			with base.getTableConn() as conn:
+				self.assertEqual(
+					list(conn.query("select color from tap_user._anonymous_test_upload"
+						" where tyc2='1265-01176-1'")),
+					[(0.25,)])
+			return self.assertGETHasStrings(
+				f"{_B}user_tables/test_upload", {}, [
+				"<name>tap_user.test_upload</name>",
+				"<column><name>tyc2</name>"]
+				).addCallback(tryDELETE)
+
+		return self.assertPOSTHasStrings(
+			f"{_B}user_tables/test_upload", {}, [
+				"Query this table as tap_user.test_upload",
+			], rm=makePUT
+			).addCallback(tryGET)
+			
+	def testCreateAs(self):
+		def makeDELETE(request):
+			request.method = "DELETE"
+
+		def cleanup(res):
+			return self.assertGETHasStrings(f"{_B}user_tables/fromquery", {}, [
+				"Dropped user table fromquery"], rm=makeDELETE)
+
+		def tryQuery(res):
+			return self.assertGETHasStrings(
+				f"{_B}sync", {"LANG": "ADQL",
+					"QUERY": "select * from tap_user.fromquery"}, [
+					'<STREAM encoding="base64">QAAAAA==</STREAM>'])
+
+		return self.assertGETHasStrings(
+			f"{_B}sync", {"LANG": "ADQL",
+				"QUERY": "CREATE TABLE tap_user.fromquery AS"
+					" SELECT alpha FROM test.adql WHERE alpha<3"}, [
+				'<DESCRIPTION>A sample RA</DESCRIPTION>', # metadata present
+				'<STREAM encoding="base64"></STREAM>',  # empty VOTable
+				]).addCallback(tryQuery
+				).addBoth(cleanup)
+	
+	def testNoReservedWords(self):
+		def makePUT(request):
+			request.setPayload(self.someVOT, base.votableType)
+			request.method = "PUT"
+
+		return self.assertPOSTHasStrings(
+			f"{_B}user_tables/table", {}, [
+				'value="ERROR"',
+				"'table' cannot be used as an upload table name (which must be"
+				" regular ADQL identifiers, in particular not ADQL reserved words)."
+			], rm=makePUT)
+
+			
+
 atexit.register(trialhelpers.provideRDData("test", "ADQLTest"))
