@@ -2,7 +2,7 @@
 Tests for various parts of the server infrastructure, using trial.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -22,6 +22,7 @@ from gavo import api
 from gavo import base
 from gavo import utils
 from gavo.web import ifpages
+from gavo.web import root
 
 base.DEBUG = True
 
@@ -222,7 +223,7 @@ class PathResolutionTest(trialhelpers.ArchiveTest):
 		with trialhelpers.tempConfig(
 				("web", "root", "__system__/adql/query/form")):
 			return self.assertGETHasStrings("", {},
-				["<title>ADQL Query</title>"])
+				["<title>Unittest Suite ADQL Query</title>"])
 
 
 class MaintResponsesTest(trialhelpers.ArchiveTest):
@@ -418,7 +419,6 @@ class TemplatingTest(trialhelpers.ArchiveTest):
 		return self._assertTemplateRendersTo(
 			'<div><p>This is there</p><p n:render="borken">This is not</p></div>',
 			{}, [
-				"<p>This is there</p>",
 				"Exception while flattening",
 				"Missing render method",
 				": borken"
@@ -530,7 +530,7 @@ class IfPagesTest(trialhelpers.ArchiveTest):
 
 	def testNoOddWellKnownChildren(self):
 		return self.assertGETHasStrings(".well-known/unknown", {},
-			">Only ACME supported in .well-known")
+			[">b'unknown' not supported in .well-known"])
 
 	def testChallenge(self):
 		challengeName = utils.getRandomString(20)
@@ -579,7 +579,7 @@ class IfPagesTest(trialhelpers.ArchiveTest):
 			return res
 
 		return self.assertGETHasStrings("static/null", {},
-			[">Only plain files are served here"]).addBoth(cleanup)
+			[">/dev/null is not located "]).addBoth(cleanup)
 
 	def testDebianFallback(self):
 		jqDist = os.path.join(
@@ -602,6 +602,11 @@ class IfPagesTest(trialhelpers.ArchiveTest):
 		return self.assertGETHasStrings("static/help.shtml", {},
 			["<title>Unittest Suite Site Help</title>"])
 
+	def testSecurityTxt(self):
+		self.assertGETHasStrings(".well-known/security.txt", {}, [
+			"PGP SIGNATURE",
+			"Canonical: https://dc.g-vo.org/.well-known/security.txt"])
+
 
 class MetaRenderTest(trialhelpers.ArchiveTest):
 	def testMacroExpanded(self):
@@ -641,6 +646,19 @@ class MetaRenderTest(trialhelpers.ArchiveTest):
 				"available through this site's TAP service",
 				"url: <strong>http://localhost:8080/tap"])
 
+	def testRelationships(self):
+		return self.assertGETHasStrings("/data/cores/pc/info", {}, [
+			"is derived from</span>", # relationship names translated
+			'href="http://localhost:8080/tableinfo/test.abcd"', # ref URL fetch
+			'">Some Service', # titles rendered
+			'<a href="http://just.any.old.url">DataCite be damned', #_related support
+			'<a href="https://dc.g-vo.org/LP/ivo://org', # Landing page URLs
+			])
+
+	def testRelatedMacroExpanded(self):
+		return self.assertGETHasStrings("/data/cores/pc/info", {}, [
+			"http://localhost:8080/foo/bar/baz"])
+
 
 class MetaPagesTest(trialhelpers.ArchiveTest):
 	def testGetRR404(self):
@@ -921,7 +939,7 @@ class TestUserUWS(trialhelpers.ArchiveTest):
 			self.assertTrue(
 				b'<uws:parameter id="opim">3.0</uws:parameter>' in result[0], "opim")
 			self.assertTrue(
-				b'<uws:parameter id="powers">[1, 2, 3]</uws:parameter>' in result[0],
+				b'<uws:parameter id="powers">1 2 3</uws:parameter>' in result[0],
 				"powers")
 			self.assertTrue(
 				b'<uws:parameter id="responseformat">application/x-votable+xml'
@@ -1055,7 +1073,7 @@ class TestUWSAuth(trialhelpers.ArchiveTest):
 
 		def assertAuthenticatedAccess(result, jobURL):
 			self.assertTrue(b"<uws:ownerId>testing</uws:ownerId>" in result[0])
-			self.assertTrue(b"[1, 2, 3]</uws:parame" in result[0])
+			self.assertTrue(b"1 2 3</uws:parame" in result[0])
 			return trialhelpers.runQuery(self.renderer, "GET",
 				jobURL, {},
 			).addCallback(assertNoGeneralAccess, jobURL)
@@ -1130,12 +1148,6 @@ class UserUWSJoblistTest(trialhelpers.ArchiveTest):
 		).addCallback(postOther)
 
 
-def _addCreds(request, user="testing", password="testing"):
-	# testhelpers sets up testing in the users table
-	request.user = user
-	request.password = password
-
-
 def _requestCompression(request):
 	request.requestHeaders.addRawHeader("Accept-Encoding", "gzip")
 
@@ -1158,7 +1170,7 @@ class LogInOutTest(trialhelpers.ArchiveTest):
 			return res
 
 		return self.assertStatus("/login", 303, {"nextURL": ["/rompp"]},
-			rm=_addCreds
+			rm=trialhelpers.addCreds
 		).addCallback(assertNextURL)
 
 	def testLogoutAuth(self):
@@ -1168,7 +1180,7 @@ class LogInOutTest(trialhelpers.ArchiveTest):
 		return self.assertGETHasStrings("/login", {"nextURL": ["/rompp"],
 				"relog": ["True"]}, [
 			"<title>Unittest Suite -- Authentication Required</title>"],
-			rm=_addCreds,
+			rm=trialhelpers.addCreds,
 		).addCallback(assertStatus)
 
 	def testLogoutNonauth(self):
@@ -1185,7 +1197,7 @@ class LogInOutTest(trialhelpers.ArchiveTest):
 	def testLogoutNoNextURL(self):
 		return self.assertGETHasStrings("/login", {"relog": ["True"]}, [
 			"<p>The resource you are trying to access is protected, and "],
-			rm=_addCreds)
+			rm=trialhelpers.addCreds)
 
 
 class AuthcheckTest(trialhelpers.ArchiveTest):
@@ -1224,7 +1236,7 @@ class AuthcheckTest(trialhelpers.ArchiveTest):
 	def testLimitToOk(self):
 		return self.assertGETHasStrings("/data/cores/keepout/capabilities", {},
 			["<accessURL"],
-			rm=_addCreds
+			rm=trialhelpers.addCreds
 			).addCallback(self._assert200
 			).addCallback(self._assertHasNoChallenge
 			).addCallback(self._assertAuthUser("testing"))
@@ -1232,7 +1244,7 @@ class AuthcheckTest(trialhelpers.ArchiveTest):
 	def testLimitToBadPass(self):
 		return self.assertGETHasStrings("/data/cores/keepout/capabilities", {},
 			["access is protected"],
-			rm=lambda req:_addCreds(req, password="wrong")
+			rm=lambda req:trialhelpers.addCreds(req, password="wrong")
 			).addCallback(self._assert401
 			).addCallback(self._assertHasChallenge
 			).addCallback(self._assertNoAuthUser)
@@ -1240,7 +1252,7 @@ class AuthcheckTest(trialhelpers.ArchiveTest):
 	def testLimitToBadUser(self):
 		return self.assertGETHasStrings("/data/cores/keepout/capabilities", {},
 			["access is protected"],
-			rm=lambda req:_addCreds(req, user="other", password="other")
+			rm=lambda req:trialhelpers.addCreds(req, user="other", password="other")
 			).addCallback(self._assert401
 			).addCallback(self._assertHasChallenge
 			).addCallback(self._assertNoAuthUser)
@@ -1255,7 +1267,7 @@ class AuthcheckTest(trialhelpers.ArchiveTest):
 	def testOpenBadPass(self):
 		return self.assertGETHasStrings("/data/cores/coltest/capabilities", {},
 			["<accessURL"],
-			rm=lambda req:_addCreds(req, password="wrong")
+			rm=lambda req:trialhelpers.addCreds(req, password="wrong")
 			).addCallback(self._assert401
 			).addCallback(self._assertHasChallenge
 			).addCallback(self._assertNoAuthUser)
@@ -1263,7 +1275,7 @@ class AuthcheckTest(trialhelpers.ArchiveTest):
 	def testOpenGoodPass(self):
 		return self.assertGETHasStrings("/data/cores/coltest/capabilities", {},
 			["<accessURL"],
-			rm=lambda req:_addCreds(req, user="other", password="other")
+			rm=lambda req:trialhelpers.addCreds(req, user="other", password="other")
 			).addCallback(self._assert200
 			).addCallback(self._assertHasNoChallenge
 			).addCallback(self._assertAuthUser("other"))
@@ -1271,7 +1283,7 @@ class AuthcheckTest(trialhelpers.ArchiveTest):
 	def testOnTAP(self):
 		return self.assertGETHasStrings("/tap/capabilities", {},
 			["<accessURL"],
-			rm=lambda req:_addCreds(req, user="other", password="other")
+			rm=lambda req:trialhelpers.addCreds(req, user="other", password="other")
 			).addCallback(self._assert200
 			).addCallback(self._assertHasNoChallenge
 			).addCallback(self._assertAuthUser("other"))
@@ -1286,7 +1298,7 @@ class AuthcheckTest(trialhelpers.ArchiveTest):
 	def testHeaderAdded(self):
 		return self.assertGETHasStrings("/data/cores/keepout/form", {},
 			['<form id="genForm" action="data/cores/keepout/form"'],
-			rm=_addCreds).addCallback(
+			rm=trialhelpers.addCreds).addCallback(
 				self._assertAuthUser("testing"))
 
 
@@ -1370,7 +1382,7 @@ class CachingTest(trialhelpers.ArchiveTest):
 			return self.assertGETHasStrings("data/seeded/hooray/info",
 				{}, [
 					"<title>Information on Service 'Injection testing'</title>"],
-				rm=_addCreds
+				rm=trialhelpers.addCreds
 			).addCallback(self.assertWithUserNotFromCache, 0)
 		return res
 
@@ -1378,7 +1390,7 @@ class CachingTest(trialhelpers.ArchiveTest):
 		return self.assertGETHasStrings("data/seeded/hooray/info",
 			{}, [
 				"<title>Information on Service 'Injection testing'</title>"],
-			rm=_addCreds
+			rm=trialhelpers.addCreds
 		).addCallback(self.assertWithUserNotFromCache, 1)
 
 	
@@ -1496,3 +1508,28 @@ class SchemaTest(trialhelpers.ArchiveTest):
 		return self.assertGETHasStrings("schemata", {}, [
 			'<a href="http://vo.ari.uni-heidelberg.de/docs/schemata/OAI-PMH'
 			'.xsd">oai</a>'])
+
+
+class BlockingTest(trialhelpers.ArchiveTest):
+	def testNoMessageBlocking(self):
+		root.BLOCKED_HOSTS.add("this.is.no.ip.address")
+		def cleanup(res):
+			root.BLOCKED_HOSTS.pop()
+			return res
+
+		return self.assertGETHasStrings("", {}, [
+			"Sorry, you're blocked.\nThere should"]).addBoth(cleanup)
+
+	def testWithMessageBlocking(self):
+		root.BLOCKED_HOSTS.add("this.is.no.ip.address")
+		msgFile = api.getConfig("stateDir") / "block-message.txt"
+		with open(msgFile, "w") as f:
+			f.write("Call us for good advice: ++49 6221 54 1837\n")
+
+		def cleanup(res):
+			root.BLOCKED_HOSTS.pop()
+			os.unlink(msgFile)
+			return res
+
+		return self.assertGETHasStrings("", {}, [
+			"all us for good advice: ++49 6221 54 1837\n"]).addBoth(cleanup)
