@@ -18,6 +18,8 @@ gavo.utils.misctricks
 gavo.utils.ostricks
 gavo.utils.pgsphere
 gavo.utils.plainxml
+gavo.utils.serializers
+gavo.utils.stanxml
 gavo.utils.texttricks"
 
 cd ..
