@@ -2,7 +2,7 @@
 Tests for ADQL user defined functions and Region expressions.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -400,8 +400,7 @@ class LegacyMotionTest(testhelpers.VerboseTest):
 			" where 1=Contains("
 			"ivo_apply_pm(ra, dec, 0.01, -0.01, 23),"
 			"CIRCLE('ICRS', 23.735, -12.48, 0.01))")
-		self.assertEqual(len(res.rows), 1)
-		self.assertEqual(res.rows[0]["testgroup"], "jd")
+		self.assertEqual(testhelpers.pickSingle(res.rows)["testgroup"], "jd")
 
 	def testTransformBadarg(self):
 		self.assertRaisesWithMsg(
@@ -496,8 +495,7 @@ class EpochPropTest(testhelpers.VerboseTest):
 			"SELECT ra, dec from test.ufuncex where testgroup='intervals'"
 			" AND distance(ivo_epoch_prop_pos(12, 0, NULL, -120000, 0,"
 			" NULL, 2000, 2060), point(ra, dec-20))<0.001")
-		self.assertEqual(len(res.rows), 1)
-		self.assertEqual(res.rows[0]["ra"], 10.)
+		self.assertEqual(testhelpers.pickSingle(res.rows)["ra"], 10.)
 
 	def testPropPosMsg(self):
 		self.assertRaisesWithMsg(base.ValidationError,
@@ -508,6 +506,47 @@ class EpochPropTest(testhelpers.VerboseTest):
 				" from test.ufuncex where testgroup='intervals'"
 				" ORDER BY dec",))
 
+	def testPropNoPM(self):
+		res = self.querier.queryADQL(
+			"SELECT TOP 1"
+			" ivo_epoch_prop(10, 12, NULL, NULL, NULL, NULL, 2000, 2020) as prop"
+			" from tap_schema.tables")
+		self.assertEqual(res.rows[0]["prop"],
+			[10.0, 12.000000000000002, None, None, None, None])
+
+	def testPropNoPMDec(self):
+		res = self.querier.queryADQL(
+			"SELECT TOP 1"
+			" ivo_epoch_prop(10, 12, 3, NULL, NULL, NULL, 2000, 2020) as prop"
+			" from tap_schema.tables")
+		self.assertEqual(res.rows[0]["prop"],
+			[10.0, 12.000000000000002, 3, None, None, None])
+
+	def testPropNoPMRA(self):
+		res = self.querier.queryADQL(
+			"SELECT TOP 1"
+			" ivo_epoch_prop(10, 12, NULL, 3, NULL, NULL, 2000, 2020) as prop"
+			" from tap_schema.tables")
+		self.assertAlmostEqualVector(res.rows[0]["prop"],
+			[10.0, 12., None, None, None, None])
+
+	def testPropNoRV(self):
+		res = self.querier.queryADQL(
+			"SELECT TOP 1"
+			" ivo_epoch_prop(10, 12, 4, 2, 5, NULL, 2000, 2020) as prop"
+			" from tap_schema.tables")
+		self.assertAlmostEqualVector(res.rows[0]["prop"],
+			[10.000011359341114, 12.000027777777, 3.999999999999,
+				2.000000206100, 4.99999991755, None])
+
+	def testPropNoParallax(self):
+		res = self.querier.queryADQL(
+			"SELECT TOP 1"
+			" ivo_epoch_prop(10, 12, NULL, 2, 5, 4000, 2000, 2020) as prop"
+			" from tap_schema.tables")
+		self.assertAlmostEqualVector(res.rows[0]["prop"],
+			[10.000011359341022, 12.000027777777323, None,
+			2.0000001733735733, 4.999999835741298, None])
 
 class IntervalQueryTest(testhelpers.VerboseTest,
 		metaclass=testhelpers.SamplesBasedAutoTest):
@@ -554,9 +593,9 @@ class HistogramTest(testhelpers.VerboseTest):
 			"SELECT ivo_histogram(ra, 20., 60., 2) as ras,"
 			"  ivo_histogram(dec, -20., 81., 10) as decs from test.ufuncex"
 			"  WHERE testgroup IN ('jd', 'intervals')")
-		self.assertEqual(len(res.rows), 1)
-		self.assertEqual(res.rows[0]["ras"], [1, 1, 1, 0])
-		self.assertEqual(res.rows[0]["decs"],
+		row = testhelpers.pickSingle(res.rows)
+		self.assertEqual(row["ras"], [1, 1, 1, 0])
+		self.assertEqual(row["decs"],
 			[0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0])
 
 	def testAlias(self):
@@ -589,8 +628,7 @@ class TransformTest(testhelpers.VerboseTest):
 		res = self.querier.queryADQL(
 			"SELECT ivo_geom_transform('ICRS', 'GALACTIC', p) as ps, p"
 			"  from test.ufuncex where testgroup='jd'")
-		self.assertEqual(len(res.rows), 1)
-		ps = res.rows[0]["ps"]
+		ps = testhelpers.pickSingle(res.rows)["ps"]
 
 		# XXX TODO: figure out where the precision goes -- this should have
 		# about 13 s.f. rather than just 8.
@@ -602,12 +640,11 @@ class TransformTest(testhelpers.VerboseTest):
 		res = self.querier.queryADQL(
 			"SELECT gavo_transform('GALACTIC', 'FK4', p) as ps, p"
 			"  from test.ufuncex where testgroup='jd'")
-		self.assertEqual(len(res.rows), 1)
 		self.assertEqual(
 			res.tableDef.getColumnByName("ps").type,
 			"spoint")
 
-		ps = res.rows[0]["ps"]
+		ps = testhelpers.pickSingle(res.rows)["ps"]
 		self.assertAlmostEqual(ps.x/utils.DEG, 162.4867829887925)
 		self.assertAlmostEqual(ps.y/utils.DEG, 15.59041298060233)
 		self.assertEqual(res.tableDef.getColumnByName("ps").ucd, "pos.eq")
