@@ -2,7 +2,7 @@
 Tests for event propagation and user interaction.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -120,9 +120,9 @@ class MiscCLITest(testhelpers.VerboseTest):
 				expectedStdout=lambda txt:
 					re.match(b'.*/tmp/bad.xml -- valid\n$', txt))
 
-	def testShowDD(self):
+	def testShowDDGood(self):
 		self.assertOutput(cli.main, argList=["show", "dds", "//tap"],
-			expectedStdout="importTablesFromRD\nimportDMsFromRD\nimportColumnsFromRD\nimportGroupsFromRD\nimportFkeysFromRD\ncreateSchema*\ncreateJobTable*\n")
+			expectedStdout="importTablesFromRD\nimportDMsFromRD\nimportColumnsFromRD\nimportGroupsFromRD\nimportFkeysFromRD\ncreateSchema*\nrefresh\ncreateJobTable*\n")
 
 	def testShowDDBad(self):
 		self.assertOutput(cli.main, argList=["show", "dds", "//nonex"],
@@ -167,7 +167,8 @@ class MiscCLITest(testhelpers.VerboseTest):
 
 	def testUpdateTAPSchema(self):
 		self.assertOutput(cli.main, argList=["adm", "updateTAP"],
-			expectedStdout="", expectedStderr="")
+			expectedStdout="",
+			expectedStderr=lambda output: "Starting refresh\nDone refresh, read 0\n")
 
 
 class _CatalogTestResource(tresc.FileResource):
@@ -652,8 +653,8 @@ class DropTest(testhelpers.VerboseTest):
 
 class ValidationTest(testhelpers.VerboseTest):
 	def testValidUserconfig(self):
-		base.caches.clearForName(rscdesc.USERCONFIG_RD_PATH)
-		with testtricks.testFile(rscdesc.USERCONFIG_RD_PATH+".rd",
+		base.caches.clearForName(str(rscdesc.USERCONFIG_RD_PATH))
+		with testtricks.testFile(rscdesc.USERCONFIG_RD_PATH.with_suffix(".rd"),
 				"""<resource schema="test"><STREAM id="foo"><column name="abc"/>
 				</STREAM></resource>"""):
 			self.assertOutput(cli.main, argList=["val", "%"],
@@ -661,8 +662,8 @@ class ValidationTest(testhelpers.VerboseTest):
 				expectedStdout='% -- OK\n')
 
 	def testInvalidUserconfig(self):
-		base.caches.clearForName(rscdesc.USERCONFIG_RD_PATH)
-		with testtricks.testFile(rscdesc.USERCONFIG_RD_PATH+".rd",
+		base.caches.clearForName(str(rscdesc.USERCONFIG_RD_PATH))
+		with testtricks.testFile(rscdesc.USERCONFIG_RD_PATH.with_suffix(".rd"),
 				"""<resource schema="test"><STREAM id="foo"><column name="abc">
 				</STREAM></resource>"""):
 			self.assertOutput(cli.main, argList=["val", "%"],
