@@ -3,7 +3,7 @@
 Tests for correct interpretation of vizier-type expressions.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -322,7 +322,7 @@ class PatternsSQLGenerTest(_SQLGenerTest):
 		("=s*ta?", "foo ~ %(foo0)s", {"foo0": "^s.*ta.$"}),
 		("~a+b*", "foo ~* %(foo0)s", {"foo0": r"^a\+b.*$"}),
 # 5
-		("!a+b\*", "foo !~ %(foo0)s", {"foo0": r"^a\+b\\.*$"}),
+		(r"!a+b\*", "foo !~ %(foo0)s", {"foo0": r"^a\+b\\.*$"}),
 		("!~[a-z]", "foo !~* %(foo0)s", {"foo0": r"^[a-z]$"}),
 		("!~[a-z]", "foo !~* %(foo0)s", {"foo0": r"^[a-z]$"}),
 		("=| a,b,a|b", "foo IN (%(foo0)s, %(foo1)s)",
@@ -460,7 +460,7 @@ class StringQueryTest(testhelpers.VerboseTest,
 			("<b", 10),
 			(">b", 1),
 #10
-			("== \it", 1),
+			(r"== \it", 1),
 			("== B*", 1),
 			("=~ a", 2),
 			("=~ x33+4", 1),
@@ -478,8 +478,8 @@ class StringQueryTest(testhelpers.VerboseTest,
 			("~*+*", 1),
 			("~*|*", 1),
 #25
-			("~\*", 1),
-			("!\*", 10),
+			(r"~\*", 1),
+			(r"!\*", 10),
 			("~B*", 3),
 			("= B*", 2),
 			("~B?", 1),
