@@ -2,7 +2,7 @@
 Some tests for votable production.
 """
 
-#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2024, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -23,7 +23,6 @@ from gavo import rscdesc
 from gavo import utils
 from gavo import votable
 from gavo.formats import votableread, votablewrite, getFormatted
-from gavo.svcs import outputdef
 from gavo.utils import ElementTree
 from gavo.utils import pgsphere
 
@@ -489,8 +488,7 @@ class ParamNullValueTest(VOTableRenderTest):
 
 	def _getParamFor(self, colDef):
 		pars = self._getParamsFor(colDef)
-		self.assertEqual(len(pars), 1)
-		return pars[0]
+		return testhelpers.pickSingle(pars)
 
 	def _assertDeclaredNull(self, colDef, nullLiteral):
 		par = self._getParamFor(colDef)
@@ -736,6 +734,24 @@ class ValuesParsedTest(testhelpers.VerboseTest):
 		self.assertEqual(opts[1].title, "yes")
 
 
+class ValuesWrittenTest(VOTableRenderTest):
+	def testMinMaxSerialised(self):
+		tree = testhelpers.getXMLTree(self._getAsVOTable(
+			'<column name="x" type="integer"><values min="-2" max="5000"/>'
+			'</column>',
+				rows=[{"x": 2}]), debug=False)
+		self.assertEqual(tree.xpath("//VALUES/MIN/@value")[0], "-2")
+		self.assertEqual(tree.xpath("//VALUES/MAX/@value")[0], "5000")
+
+	def testNoEmptyMinMax(self):
+		tree = testhelpers.getXMLTree(self._getAsVOTable(
+			'<column name="x" type="integer"><values min=""/>'
+			'</column>',
+				rows=[{"x": 2}]), debug=False)
+		self.assertEqual(len(tree.xpath("//VALUES/MIN")), 0)
+		self.assertEqual(len(tree.xpath("//VALUES/MAX")), 0)
+
+
 class GroupWriteTest(testhelpers.VerboseTest):
 	def testEmptyGroup(self):
 		tree = _getVOTTreeForTable(
@@ -743,11 +759,11 @@ class GroupWriteTest(testhelpers.VerboseTest):
 			' description="A meaningless group"/>'
 				'</table>')
 		res = tree.findall("RESOURCE/TABLE/GROUP")
-		self.assertEqual(len(res), 1)
-		self.assertEqual(res[0].attrib["ucd"], "empty.group")
-		self.assertEqual(res[0].attrib["utype"], "testing:silly")
-		self.assertEqual(res[0].attrib["name"], "tg")
-		self.assertEqual(res[0].find("DESCRIPTION").text,
+		grp = testhelpers.pickSingle(res)
+		self.assertEqual(grp.attrib["ucd"], "empty.group")
+		self.assertEqual(grp.attrib["utype"], "testing:silly")
+		self.assertEqual(grp.attrib["name"], "tg")
+		self.assertEqual(grp.find("DESCRIPTION").text,
 			"A meaningless group")
 	
 	def testRefs(self):
@@ -766,16 +782,16 @@ class GroupWriteTest(testhelpers.VerboseTest):
 		self.assertEqual(_getElementByID(table, refs[1]).attrib["name"], "y")
 
 		refs = [el.attrib["ref"] for el in g.findall("PARAMref")]
-		self.assertEqual(len(refs), 1)
-		self.assertEqual(_getElementByID(table, refs[0]).attrib["value"], "4")
+		self.assertEqual(
+			_getElementByID(table, testhelpers.pickSingle(refs)).attrib["value"],
+			"4")
 
 	def testLocalParam(self):
 		tree = _getVOTTreeForTable(
 			'<table><group><param name="u" type="integer">5</param></group>'
 				'</table>')
 		pars = tree.findall("RESOURCE/TABLE/GROUP/PARAM")
-		self.assertEqual(len(pars), 1)
-		self.assertEqual(pars[0].attrib["value"], "5")
+		self.assertEqual(testhelpers.pickSingle(pars).attrib["value"], "5")
 
 	def testRecursive(self):
 		tree = _getVOTTreeForTable(
@@ -919,71 +935,11 @@ class SingleSTCEmbedTest(testhelpers.VerboseTest):
 				]:
 			self.assertTrue(len(tree.xpath(path))==1, "%s not found"%path)
 
-	def testColumnCopy(self):
-		table = _getTableWithSimpleSTC()
-		td = table.tableDef.copy(None)
-		# do another copy so the original copy gets un-referenced
-		# and we catch if we still touch their columns.
-		td = base.makeStruct(outputdef.OutputTableDef,
-			columns=[
-				td.getByName("alpha"),
-				td.getByName("delta"),
-				td.getByName("tca"),])
-		table = rsc.TableForDef(td, rows=table.rows)
-
-		tree = testhelpers.getXMLTree(
-			votablewrite.getAsVOTable(table), debug=False)
-		for path in [
-				"RESOURCE/COOSYS/FIELDref[@utype='votable:LonLatPoint-lat']",
-				"RESOURCE/COOSYS/FIELDref[@utype='votable:LonLatPoint-lon']",
-				"//COOSYS[@epoch='J2015.0']"
-				]:
-			self.assertTrue(len(tree.xpath(path))==1, f"missing {path}")
-
 	def testTIMESYSRendered(self):
 		tsRef = self.tree.xpath("//FIELD[@name='tca']/@ref")[0]
 		self.assertEqual("GEOCENTER",
 			self.tree.xpath(f"RESOURCE/TIMESYS[@ID='{tsRef}']/@refposition")[0])
 
-	def _assertUtypeResolves(self, utype, fieldname):
-		ref = self.tree.xpath(f"RESOURCE/COOSYS/FIELDref[@utype='{utype}']"
-			)[0].get("ref")
-		self.assertEqual(fieldname,
-			self.tree.xpath(f"//*[@ID='{ref}']")[0].get("name"))
-
-	def testLong(self):
-		self._assertUtypeResolves("votable:LonLatPoint-lon", "alpha")
-
-	def testLat(self):
-		self._assertUtypeResolves("votable:LonLatPoint-lat", "delta")
-
-	def testDist(self):
-		self._assertUtypeResolves("votable:LonLatPoint-dist", "dist")
-
-	def testPmra(self):
-		self._assertUtypeResolves("votable:ProperMotion-lon", "pmra")
-
-	def testPmde(self):
-		self._assertUtypeResolves("votable:ProperMotion-lat", "pmdec")
-
-	def testRV(self):
-		self._assertUtypeResolves("votable:ProperMotion-rv", "rv")
-
-	def testCopying(self):
-		# this is actually a test of OutputTableDef's adoption of its
-		# column and params
-		td = _getTableWithSimpleSTC().tableDef
-		td = base.makeStruct(outputdef.OutputTableDef,
-			columns=td.columns, params=td.params, id="copy")
-		table = rsc.TableForDef(td)
-		tree = testhelpers.getXMLTree(
-			votablewrite.getAsVOTable(table), debug=False)
-
-		ref = tree.xpath("RESOURCE/COOSYS/"
-			"FIELDref[@utype='votable:ProperMotion-lon']")[0].get("ref")
-		self.assertEqual("pmra",
-			self.tree.xpath(f"//*[@ID='{ref}']")[0].get("name"))
-	
 	def testCoreference(self):
 		td = base.parseFromString(rscdef.TableDef, """
 			<table>
@@ -1063,18 +1019,6 @@ class TwinSTCEmbedTest(testhelpers.VerboseTest):
 		self.assertEqual(self.tree.xpath(
 			"//FIELD[@name='ep']")[0].get("ref"), destId)
 
-	def testCOOSYSEpochColref(self):
-		epref = self.tree.xpath(
-			"RESOURCE/COOSYS/FIELDref[@utype='votable:CustomRefLocation-epoch']"
-			)[0].get("ref")
-		self.assertEqual("ep",
-			self.tree.xpath(f"//*[@ID='{epref}']")[0].get("name"))
-
-	def testPosRef(self):
-		self.assertEqual(
-			self.tree.xpath("//FIELDref[@ref='pos']")[0].get("utype"),
-			"stc:AstroCoords.Position2D.Value2")
-
 	def testPosXtype(self):
 		posfield = self.tree.xpath("//FIELD[@name='pos']")[0]
 		self.assertEqual(posfield.get("xtype"), "point")
@@ -1092,16 +1036,6 @@ class TwinSTCEmbedTest(testhelpers.VerboseTest):
 			self.tree.xpath("//*[@ID='%s']"%sysname)[0].get("system"),
 			"ICRS")
 
-	def testNewStyleRefs(self):
-		latRef = self.tree.xpath("RESOURCE/COOSYS[@system='ICRS']"
-			"/FIELDref[@utype='votable:LonLatPoint-lat']")[0].get("ref")
-		self.assertEqual("dec",
-			self.tree.xpath(f"//*[@ID='{latRef}']")[0].get("name"))
-		glonRef = self.tree.xpath("RESOURCE/COOSYS[@system='galactic']"
-			"/FIELDref[@utype='votable:LonLatPoint-lon']")[0].get("ref")
-		self.assertEqual("gal_l",
-			self.tree.xpath(f"//*[@ID='{glonRef}']")[0].get("name"))
-
 	def testEpochSyntax(self):
 		self.assertEqual("J2000.3",
 			self.tree.xpath("//COOSYS[@system='galactic']")[0].get("epoch"))
@@ -1238,7 +1172,7 @@ class HackMetaTest(testhelpers.VerboseTest):
 		res = votablewrite.getAsVOTable(table,
 			votablewrite.VOTableContext(suppressNamespace=True))
 		self.assertTrue(
-			b'VOTABLE version=\"1.4\"'
+			b'VOTABLE version=\"1.5\"'
 			b" malformed mess xmlns:crazy='http://forget.this'"
 			in res)
 
@@ -1384,14 +1318,13 @@ class VOTableSTCTest(testhelpers.VerboseTest, testhelpers.XSDTestMixin):
 	def testTIMESYSReference(self):
 		tsID = self.litAndTree[1].xpath("//FIELD[@ID='epoch']")[0].get("ref")
 		systems = self.litAndTree[1].xpath("//*[@ID='%s']"%tsID)
-		self.assertEqual(len(systems), 1)
-		self.assertEqual(systems[0].tag, "TIMESYS")
-		self.assertEqual(systems[0].get("timeorigin"), "2450000.5")
+		sys = testhelpers.pickSingle(systems)
+		self.assertEqual(sys.tag, "TIMESYS")
+		self.assertEqual(sys.get("timeorigin"), "2450000.5")
 	
 	def testCOOSYSMeta(self):
 		systems = self.litAndTree[1].xpath("RESOURCE/COOSYS")
-		self.assertEqual(len(systems), 1)
-		system = systems[0]
+		system = testhelpers.pickSingle(systems)
 		self.assertEqual(system.get("system"), "ICRS")
 		self.assertEqual(system.get("epoch"), "J2000.0")
 
@@ -1411,28 +1344,6 @@ class VOTableSTCTest(testhelpers.VerboseTest, testhelpers.XSDTestMixin):
 	def testValid(self):
 		self.assertValidates(self.litAndTree[0])
 
-	def testParamRef(self):
-		paramRef = self.litAndTree[1].xpath(
-			"RESOURCE/COOSYS/PARAMref[@utype='votable:LonLatPoint-lon']"
-			)[0].get("ref")
-		self.assertEqual(
-			self.litAndTree[1].xpath(f"//PARAM[@ID='{paramRef}']")[0].get("name"),
-			"ra")
-
-	def testFieldRef(self):
-		ref = self.litAndTree[1].xpath(
-			"RESOURCE/COOSYS/FIELDref[@utype='votable:LonLatPoint-dist']"
-			)[0].get("ref")
-		self.assertEqual(
-			self.litAndTree[1].xpath(f"//FIELD[@ID='{ref}']")[0].get("name"),
-			"dist")
-	
-	def testAllUtypes(self):
-		self.assertEqual(set(self.litAndTree[1].xpath("RESOURCE/COOSYS//@utype")),
-			{'votable:LonLatPoint-lon', 'votable:LonLatPoint-lat',
-			'votable:LonLatPoint-dist', 'votable:ProperMotion-lon',
-			'votable:ProperMotion-lat', 'votable:ProperMotion-rv'})
-
 
 class _ConstantSTCTable(testhelpers.TestResource):
 	def make(self, ignored):
